@@ -1,18 +1,30 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2001-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2001-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #ifndef _NV_LINUX_H_
 #define _NV_LINUX_H_
 
 #include "nvstatus.h"
-#include "nv-misc.h"
 #include "nv.h"
 #include "nv-ioctl-numa.h"
 #include "conftest.h"
@@ -22,7 +34,7 @@
 #include "nv-mm.h"
 #include "os-interface.h"
 #include "nv-timer.h"
-#include "nv-caps.h"
+#include "nv-time.h"
 
 #define NV_KERNEL_NAME "Linux"
 
@@ -100,6 +112,30 @@
 
 #include <linux/sched.h>            /* suser(), capable() replacement   */
 
+#include <linux/random.h>           /* get_random_bytes()               */
+
+#if defined(NV_LINUX_DMA_BUF_H_PRESENT)
+#include <linux/dma-buf.h>
+#endif
+
+#if defined(NV_DRM_AVAILABLE)
+#if defined(NV_DRM_DRM_DEVICE_H_PRESENT)
+#include <drm/drm_device.h>
+#endif
+
+#if defined(NV_DRM_DRM_DRV_H_PRESENT)
+#include <drm/drm_drv.h>
+#endif
+
+#if defined(NV_DRM_DRMP_H_PRESENT)
+#include <drm/drmP.h>
+#endif
+
+#if defined(NV_DRM_DRM_GEM_H_PRESENT)
+#include <drm/drm_gem.h>
+#endif
+#endif /* NV_DRM_AVAILABLE */
+
 /*
  * sched.h was refactored with this commit (as part of Linux 4.11)
  *   2017-03-03  1827adb11ad26b2290dc9fe2aaf54976b2439865
@@ -128,13 +164,6 @@
 #include <linux/pm_runtime.h>       /* pm_runtime_*                     */
 #include <linux/fdtable.h>          /* files_fdtable, etc               */
 
-
-
-
-
-
-
-
 #include <asm/div64.h>              /* do_div()                         */
 #if defined(NV_ASM_SYSTEM_H_PRESENT)
 #include <asm/system.h>             /* cli, sli, save_flags             */
@@ -143,6 +172,11 @@
 #include <asm/uaccess.h>            /* access_ok                        */
 #include <asm/page.h>               /* PAGE_OFFSET                      */
 #include <asm/pgtable.h>            /* pte bit definitions              */
+#include <asm/bitops.h>             /* __set_bit()                      */
+
+#if defined(NV_LINUX_TIME_H_PRESENT)
+#include <linux/time.h>             /* FD_SET()                         */
+#endif
 
 #include "nv-list-helpers.h"
 
@@ -191,11 +225,8 @@ static inline uid_t __kuid_val(uid_t uid)
 #if defined(NV_LINUX_EFI_H_PRESENT)
 #include <linux/efi.h>              /* efi_enabled                      */
 #endif
-#if defined(NV_LINUX_SCREEN_INFO_H_PRESENT)
-#include <linux/screen_info.h>      /* screen_info                      */
-#else
-#include <linux/tty.h>              /* screen_info                      */
-#endif
+
+#include <linux/fb.h>               /* fb_info struct                   */
 
 #if !defined(CONFIG_PCI)
 #warning "Attempting to build driver for a platform with no PCI support!"
@@ -234,7 +265,7 @@ NV_STATUS nvos_forward_error_to_cray(struct pci_dev *, NvU32,
 #undef NV_SET_PAGES_UC_PRESENT
 #endif
 
-#if !defined(NVCPU_FAMILY_ARM) && !defined(NVCPU_PPC64LE)
+#if !defined(NVCPU_AARCH64) && !defined(NVCPU_PPC64LE)
 #if !defined(NV_SET_MEMORY_UC_PRESENT) && !defined(NV_SET_PAGES_UC_PRESENT)
 #error "This driver requires the ability to change memory types!"
 #endif
@@ -272,12 +303,11 @@ NV_STATUS nvos_forward_error_to_cray(struct pci_dev *, NvU32,
 #endif
 
 #if defined(DEBUG) && defined(CONFIG_KGDB) && \
-    defined(NVCPU_FAMILY_ARM)
+    defined(NVCPU_AARCH64)
 #include <asm/kgdb.h>
 #endif
 
-#if (defined(NVCPU_X86) || defined(NVCPU_X86_64)) && \
-  !defined(NV_XEN_SUPPORT_FULLY_VIRTUALIZED_KERNEL)
+#if defined(NVCPU_X86_64) && !defined(NV_XEN_SUPPORT_FULLY_VIRTUALIZED_KERNEL)
 #define NV_ENABLE_PAT_SUPPORT
 #endif
 
@@ -318,25 +348,6 @@ extern int nv_pat_mode;
 #define NV_CONFIG_PREEMPT_RT 1
 #endif
 
-#if defined(NVCPU_X86)
-#ifndef write_cr4
-#define write_cr4(x) __asm__ ("movl %0,%%cr4" :: "r" (x));
-#endif
-
-#ifndef read_cr4
-#define read_cr4()                                  \
- ({                                                 \
-      unsigned int __cr4;                           \
-      __asm__ ("movl %%cr4,%0" : "=r" (__cr4));     \
-      __cr4;                                        \
-  })
-#endif
-
-#ifndef wbinvd
-#define wbinvd() __asm__ __volatile__("wbinvd" ::: "memory");
-#endif
-#endif /* defined(NVCPU_X86) */
-
 #if defined(NV_WRITE_CR4_PRESENT)
 #define NV_READ_CR4()       read_cr4()
 #define NV_WRITE_CR4(cr4)   write_cr4(cr4)
@@ -357,12 +368,12 @@ extern int nv_pat_mode;
 #define register_hotcpu_notifier register_cpu_notifier
 #endif
 
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+#if defined(NVCPU_X86_64)
 #if !defined(pmd_large)
 #define pmd_large(_pmd) \
     ((pmd_val(_pmd) & (_PAGE_PSE|_PAGE_PRESENT)) == (_PAGE_PSE|_PAGE_PRESENT))
 #endif
-#endif /* defined(NVCPU_X86) || defined(NVCPU_X86_64) */
+#endif /* defined(NVCPU_X86_64) */
 
 #define NV_PAGE_COUNT(page) \
   ((unsigned int)page_count(page))
@@ -420,24 +431,9 @@ extern int nv_pat_mode;
 
 extern NvBool nvos_is_chipset_io_coherent(void);
 
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+#if defined(NVCPU_X86_64)
 #define CACHE_FLUSH()  asm volatile("wbinvd":::"memory")
 #define WRITE_COMBINE_FLUSH() asm volatile("sfence":::"memory")
-#elif defined(NVCPU_FAMILY_ARM)
-#if defined(NVCPU_ARM)
-    static inline void nv_flush_cache_cpu(void *info)
-    {
-        __cpuc_flush_kern_all();
-    }
-#define CACHE_FLUSH()            nv_flush_cache_cpu(NULL)
-#define CACHE_FLUSH_ALL()        on_each_cpu(nv_flush_cache_cpu, NULL, 1)
-#if defined(NV_OUTER_FLUSH_ALL_PRESENT)
-#define OUTER_FLUSH_ALL()               outer_flush_all()
-#endif
-#if defined(CONFIG_OUTER_CACHE)
-#define OUTER_FLUSH_RANGE(start, end)   outer_flush_range((start),(end))
-#endif
-#define WRITE_COMBINE_FLUSH()    { dsb(); outer_sync(); }
 #elif defined(NVCPU_AARCH64)
     static inline void nv_flush_cache_cpu(void *info)
     {
@@ -453,16 +449,23 @@ extern NvBool nvos_is_chipset_io_coherent(void);
 #define CACHE_FLUSH()            nv_flush_cache_cpu(NULL)
 #define CACHE_FLUSH_ALL()        on_each_cpu(nv_flush_cache_cpu, NULL, 1)
 #define WRITE_COMBINE_FLUSH()    mb()
-#endif
 #elif defined(NVCPU_PPC64LE)
 #define CACHE_FLUSH()            asm volatile("sync;  \n" \
                                               "isync; \n" ::: "memory")
 #define WRITE_COMBINE_FLUSH()    CACHE_FLUSH()
 #endif
 
-#if defined(NVCPU_FAMILY_ARM) || defined(NVCPU_PPC64LE)
+typedef enum
+{
+    NV_MEMORY_TYPE_SYSTEM,      /* Memory mapped for ROM, SBIOS and physical RAM. */
+    NV_MEMORY_TYPE_REGISTERS,
+    NV_MEMORY_TYPE_FRAMEBUFFER,
+    NV_MEMORY_TYPE_DEVICE_MMIO, /* All kinds of MMIO referred by NVRM e.g. BARs and MCFG of device */
+} nv_memory_type_t;
+
+#if defined(NVCPU_AARCH64) || defined(NVCPU_PPC64LE)
 #define NV_ALLOW_WRITE_COMBINING(mt)    1
-#elif (defined(NVCPU_X86) || defined(NVCPU_X86_64))
+#elif defined(NVCPU_X86_64)
 #if defined(NV_ENABLE_PAT_SUPPORT)
 #define NV_ALLOW_WRITE_COMBINING(mt)    \
     ((nv_pat_mode != NV_PAT_MODE_DISABLED) && \
@@ -651,14 +654,35 @@ static NvBool nv_numa_node_has_memory(int node_id)
         free_pages(ptr, order);                      \
     }
 
+#if defined(PAGE_KERNEL_NOENC)
+#if defined(__pgprot_mask)
+#define NV_PAGE_KERNEL_NOCACHE_NOENC __pgprot_mask(__PAGE_KERNEL_NOCACHE)
+#elif defined(default_pgprot)
+#define NV_PAGE_KERNEL_NOCACHE_NOENC default_pgprot(__PAGE_KERNEL_NOCACHE)
+#elif defined( __pgprot)
+#define NV_PAGE_KERNEL_NOCACHE_NOENC __pgprot(__PAGE_KERNEL_NOCACHE)
+#else
+#error "Unsupported kernel!!!"
+#endif
+#endif
+
 static inline NvUPtr nv_vmap(struct page **pages, NvU32 page_count,
-    NvBool cached)
+                             NvBool cached, NvBool unencrypted)
 {
     void *ptr;
     pgprot_t prot = PAGE_KERNEL;
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
-    prot = cached ? PAGE_KERNEL : PAGE_KERNEL_NOCACHE;
-#elif defined(NVCPU_FAMILY_ARM)
+#if defined(NVCPU_X86_64)
+#if defined(PAGE_KERNEL_NOENC)
+    if (unencrypted)
+    {
+        prot = cached ? PAGE_KERNEL_NOENC : NV_PAGE_KERNEL_NOCACHE_NOENC;
+    }
+    else
+#endif
+    {
+        prot = cached ? PAGE_KERNEL : PAGE_KERNEL_NOCACHE;
+    }
+#elif defined(NVCPU_AARCH64)
     prot = cached ? PAGE_KERNEL : NV_PGPROT_UNCACHED(PAGE_KERNEL);
 #endif
     /* All memory cached in PPC64LE; can't honor 'cached' input. */
@@ -734,17 +758,9 @@ static inline dma_addr_t nv_phys_to_dma(struct device *dev, NvU64 pa)
 #define NV_PCI_RESOURCE_SIZE(pci_dev, bar)  pci_resource_len(pci_dev, (bar))
 #define NV_PCI_RESOURCE_FLAGS(pci_dev, bar) pci_resource_flags(pci_dev, (bar))
 
-#if defined(NVCPU_X86)
-#define NV_PCI_RESOURCE_VALID(pci_dev, bar)                                     \
-    ((NV_PCI_RESOURCE_START(pci_dev, bar) != 0) &&                              \
-     (NV_PCI_RESOURCE_SIZE(pci_dev, bar) != 0) &&                               \
-     (!((NV_PCI_RESOURCE_FLAGS(pci_dev, bar) & PCI_BASE_ADDRESS_MEM_TYPE_64) && \
-       ((NV_PCI_RESOURCE_START(pci_dev, bar) >> PAGE_SHIFT) > 0xfffffULL))))
-#else
 #define NV_PCI_RESOURCE_VALID(pci_dev, bar)                                     \
     ((NV_PCI_RESOURCE_START(pci_dev, bar) != 0) &&                              \
      (NV_PCI_RESOURCE_SIZE(pci_dev, bar) != 0))
-#endif
 
 #define NV_PCI_DOMAIN_NUMBER(pci_dev) (NvU32)pci_domain_nr(pci_dev->bus)
 #define NV_PCI_BUS_NUMBER(pci_dev)    (pci_dev)->bus->number
@@ -1039,46 +1055,44 @@ static inline vm_fault_t nv_insert_pfn(struct vm_area_struct *vma,
 extern void *nvidia_stack_t_cache;
 
 /*
- * On Linux, when a kmem cache is created, a new sysfs entry is created
- * for the same unless it's merged with an existing cache. Upstream Linux
- * kernel commit 3b7b314053d021601940c50b07f5f1423ae67e21 (version 4.12+)
- * made cache destruction asynchronous which creates a race between cache
- * destroy and create. A new cache created with attributes as
- * a previous cache, which is scheduled for destruction, can try to create
- * a sysfs entry with the same conflicting name. Upstream Linux kernel
- * commit d50d82faa0c964e31f7a946ba8aba7c715ca7ab0 (4.18) fixes this issue
- * by cleaning up sysfs entry within slab_mutex, so the entry is deleted
- * before a cache with the same attributes could be created.
+ * On Linux, when a kmem cache is created, a new sysfs entry is created for the
+ * same unless it's merged with an existing cache. Upstream Linux kernel commit
+ * 3b7b314053d021601940c50b07f5f1423ae67e21 (version 4.12+) made cache
+ * destruction asynchronous which creates a race between cache destroy and
+ * create. A new cache created with attributes as a previous cache, which is
+ * scheduled for destruction, can try to create a sysfs entry with the same
+ * conflicting name. Upstream Linux kernel commit
+ * d50d82faa0c964e31f7a946ba8aba7c715ca7ab0 (4.18) fixes this issue by cleaning
+ * up sysfs entry within slab_mutex, so the entry is deleted before a cache with
+ * the same attributes could be created.
  *
- * To workaround this kernel issue we take two steps:
- * - Create unmergeable caches: a kmem_cache with a constructor is
- *   unmergeable. So, we define an empty contructor for the same.
- *   We need to create unmergeable caches to control all kmem destory
- *   calls so we can flush.
- * - Issue flush_scheduled_work() after kmem_cache_destroy() to make cache
- *   destruction immediate and avoid race between destroy and create as
- *   described above.
+ * To workaround this kernel issue, we take two steps:
+ * - Create unmergeable caches: a kmem_cache with a constructor is unmergeable.
+ *   So, we define an empty contructor for the same. Creating an unmergeable
+ *   cache ensures that the kernel doesn't generate an internal name and always
+ *   uses our name instead.
+ *
+ * - Generate a unique cache name by appending the current timestamp (ns). We
+ *   wait for the timestamp to increment by at least one to ensure that we do
+ *   not hit a name conflict in cache create -> destroy (async) -> create cycle.
  */
 #if defined(NV_KMEM_CACHE_HAS_KOBJ_REMOVE_WORK) && !defined(NV_SYSFS_SLAB_UNLINK_PRESENT)
 static inline void nv_kmem_ctor_dummy(void *arg)
 {
     (void)arg;
 }
-#define NV_KMEM_CACHE_DESTROY_FLUSH flush_scheduled_work
 #else
 #define nv_kmem_ctor_dummy NULL
-#define NV_KMEM_CACHE_DESTROY_FLUSH()
 #endif
 
 #define NV_KMEM_CACHE_CREATE(name, type)    \
-    kmem_cache_create(name, sizeof(type), 0, 0, nv_kmem_ctor_dummy)
+    nv_kmem_cache_create(name, sizeof(type), 0)
 
 /* The NULL pointer check is required for kernels older than 4.3 */
 #define NV_KMEM_CACHE_DESTROY(kmem_cache)   \
     if (kmem_cache != NULL)                 \
     {                                       \
         kmem_cache_destroy(kmem_cache);     \
-        NV_KMEM_CACHE_DESTROY_FLUSH();      \
     }
 
 #define NV_KMEM_CACHE_ALLOC(kmem_cache)     \
@@ -1110,7 +1124,7 @@ static inline void *nv_kmem_cache_zalloc(struct kmem_cache *k, gfp_t flags)
 static inline int nv_kmem_cache_alloc_stack(nvidia_stack_t **stack)
 {
     nvidia_stack_t *sp = NULL;
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+#if defined(NVCPU_X86_64)
     sp = NV_KMEM_CACHE_ALLOC(nvidia_stack_t_cache);
     if (sp == NULL)
         return -ENOMEM;
@@ -1123,7 +1137,7 @@ static inline int nv_kmem_cache_alloc_stack(nvidia_stack_t **stack)
 
 static inline void nv_kmem_cache_free_stack(nvidia_stack_t *stack)
 {
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+#if defined(NVCPU_X86_64)
     if (stack != NULL)
     {
         NV_KMEM_CACHE_FREE(stack, nvidia_stack_t_cache);
@@ -1131,7 +1145,7 @@ static inline void nv_kmem_cache_free_stack(nvidia_stack_t *stack)
 #endif
 }
 
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+#if defined(NVCPU_X86_64)
 /*
  * RAM is cached on Linux by default, we can assume there's
  * nothing to be done here. This is not the case for the
@@ -1174,14 +1188,16 @@ typedef struct nv_alloc_s {
     struct device     *dev;
     atomic_t       usage_count;
     struct {
-        NvBool contig   : 1;
-        NvBool guest    : 1;
-        NvBool zeroed   : 1;
-        NvBool aliased  : 1;
-        NvBool user     : 1;
-        NvBool node0    : 1;
-        NvBool peer_io  : 1;
-        NvBool physical : 1;
+        NvBool contig      : 1;
+        NvBool guest       : 1;
+        NvBool zeroed      : 1;
+        NvBool aliased     : 1;
+        NvBool user        : 1;
+        NvBool node0       : 1;
+        NvBool peer_io     : 1;
+        NvBool physical    : 1;
+        NvBool unencrypted : 1;
+        NvBool coherent    : 1;
     } flags;
     unsigned int   cache_type;
     unsigned int   num_pages;
@@ -1191,8 +1207,8 @@ typedef struct nv_alloc_s {
     unsigned int   pid;
     struct page  **user_pages;
     NvU64         guest_id;             /* id of guest VM */
-    NvBool        is_coherent;
     void          *import_priv;
+    struct sg_table *import_sgt;
 } nv_alloc_t;
 
 /**
@@ -1297,6 +1313,14 @@ nv_dma_maps_swiotlb(struct device *dev)
        * whether swiotlb could possibly be used (outside of swiotlb=force).
        */
   #endif
+
+    /*
+     * Commit 2017-11-07 d7b417fa08d ("x86/mm: Add DMA support for
+     * SEV memory encryption") forces SWIOTLB to be enabled when AMD SEV 
+     * is active in all cases.
+     */
+    if (os_sev_enabled)
+        swiotlb_in_use = NV_TRUE;
 #endif
 
     return swiotlb_in_use;
@@ -1317,6 +1341,7 @@ typedef struct nv_dma_submap_s {
     NvU32 page_count;
     NvU32 sg_map_count;
     struct sg_table sgt;
+    NvBool imported;
 } nv_dma_submap_t;
 
 typedef struct nv_dma_map_s {
@@ -1324,6 +1349,7 @@ typedef struct nv_dma_map_s {
     NvU64 page_count;
     NvBool contiguous;
     NvU32 cache_type;
+    struct sg_table *import_sgt;
 
     union
     {
@@ -1394,6 +1420,12 @@ struct os_work_queue {
     nv_kthread_q_t nvk;
 };
 
+/* Linux version of the opaque type used for os_wait_*() */
+struct os_wait_queue {
+    struct completion q;
+};
+
+
 
 
 
@@ -1444,6 +1476,19 @@ struct nv_dma_device {
     NvBool nvlink;
 };
 
+
+
+
+
+
+
+
+
+
+
+
+
+
 /* linux-specific version of old nv_state_t */
 /* this is a general os-specific state structure. the first element *must* be
    the general state structure, for the generic unix-based code */
@@ -1459,6 +1504,23 @@ typedef struct nv_linux_state_s {
     /* IBM-NPU info associated with this GPU */
     nv_ibmnpu_info_t *npu;
 
+
+
+
+
+
+    /* NUMA node information for the platforms where GPU memory is presented
+     * as a NUMA node to the kernel */
+    struct {
+        /* NUMA node id >=0  when the platform supports GPU memory as NUMA node
+         * otherwise it holds the value of NUMA_NO_NODE */
+        NvS32 node_id;
+
+        /* NUMA online/offline status for platforms that support GPU memory as
+         * NUMA node */
+        atomic_t status;
+    } numa_info;
+
     nvidia_stack_t *sp[NV_DEV_STACK_COUNT];
 
     char registry_keys[NV_MAX_REGISTRY_KEYS_LENGTH];
@@ -1499,8 +1561,6 @@ typedef struct nv_linux_state_s {
 
     struct msix_entry *msix_entries;
 
-    atomic_t msix_ref_cnt;
-
     NvU64 numa_memblock_size;
 
     struct {
@@ -1543,6 +1603,8 @@ typedef struct nv_linux_state_s {
 
 
 
+
+
     /* Lock serializing ISRs for different SOC vectors */
     nv_spinlock_t soc_isr_lock;
 
@@ -1661,7 +1723,6 @@ typedef enum
 {
     NV_FOPS_STACK_INDEX_MMAP,
     NV_FOPS_STACK_INDEX_IOCTL,
-    NV_FOPS_STACK_INDEX_PROCFS,
     NV_FOPS_STACK_INDEX_COUNT
 } nvidia_entry_point_index_t;
 
@@ -1674,14 +1735,11 @@ typedef struct
     struct semaphore fops_sp_lock[NV_FOPS_STACK_INDEX_COUNT];
     nv_alloc_t *free_list;
     void *nvptr;
-    void *proc_data;
-    void *data;
-    nvidia_event_t *event_head, *event_tail;
-    NvBool event_pending;
+    nvidia_event_t *event_data_head, *event_data_tail;
+    NvBool dataless_event_pending;
     nv_spinlock_t fp_lock;
     wait_queue_head_t waitqueue;
     nv_kthread_q_item_t deferred_close_q_item;
-    off_t off;
     NvU32 *attached_gpus;
     size_t num_attached_gpus;
     nv_alloc_mapping_context_t mmap_context;
@@ -1711,6 +1769,45 @@ static inline nv_linux_file_private_t *nv_get_nvlfp_from_nvfp(nv_file_private_t
 #define NV_ATOMIC_DEC(data)             atomic_dec(&(data))
 #define NV_ATOMIC_DEC_AND_TEST(data)    atomic_dec_and_test(&(data))
 
+static inline struct kmem_cache *nv_kmem_cache_create(const char *name, unsigned int size,
+                                                      unsigned int align)
+{
+    char *name_unique;
+    struct kmem_cache *cache;
+ 
+#if defined(NV_KMEM_CACHE_HAS_KOBJ_REMOVE_WORK) && !defined(NV_SYSFS_SLAB_UNLINK_PRESENT)
+    size_t len;
+    NvU64 tm_ns = nv_ktime_get_raw_ns();
+
+    /*
+     * Wait for timer to change at least once. This ensures
+     * that the name generated below is always unique.
+     */
+    while (tm_ns == nv_ktime_get_raw_ns());
+    tm_ns = nv_ktime_get_raw_ns();
+
+    /* 20 is the max length of a 64-bit integer printed in decimal */
+    len = strlen(name) + 20 + 1;
+    name_unique = kzalloc(len, GFP_KERNEL);
+    if (!name_unique)
+        return NULL;
+
+    if (snprintf(name_unique, len, "%s-%llu", name, tm_ns) >= len)
+    {
+        WARN(1, "kmem cache name too long: %s\n", name);
+        kfree(name_unique);
+        return NULL;
+    }
+#else
+    name_unique = (char *)name;
+#endif
+    cache = kmem_cache_create(name_unique, size, align, 0, nv_kmem_ctor_dummy);
+    if (name_unique != name)
+        kfree(name_unique);
+ 
+    return cache;
+}
+
 
 #if defined(CONFIG_PCI_IOV)
 #define NV_PCI_SRIOV_SUPPORT
@@ -1908,9 +2005,12 @@ static inline NvU32 nv_default_irq_flags(nv_state_t *nv)
     #define NV_GET_UNUSED_FD_FLAGS(flags)  (-1)
 #endif
 
-
-#if defined(NV_BACKLIGHT_DEVICE_REGISTER_PRESENT)
-#include <linux/backlight.h>
+#if defined(NV_SET_CLOSE_ON_EXEC_PRESENT)
+    #define NV_SET_CLOSE_ON_EXEC(fd, fdt) __set_close_on_exec(fd, fdt)
+#elif defined(NV_LINUX_TIME_H_PRESENT) && defined(FD_SET)
+    #define NV_SET_CLOSE_ON_EXEC(fd, fdt) FD_SET(fd, fdt->close_on_exec)
+#else
+    #define NV_SET_CLOSE_ON_EXEC(fd, fdt) __set_bit(fd, fdt->close_on_exec)
 #endif
 
 #define MODULE_BASE_NAME "nvidia"
@@ -1924,19 +2024,96 @@ NvS32 nv_request_soc_irq(nv_linux_state_t *, NvU32, nv_soc_irq_type_t, NvU32, Nv
 
 
 
+
 static inline void nv_mutex_destroy(struct mutex *lock)
 {
-
-
-
-
-
-
-
-
-
     mutex_destroy(lock);
-
 }
 
+static inline NvBool nv_platform_supports_numa(nv_linux_state_t *nvl)
+{
+    return nvl->numa_info.node_id != NUMA_NO_NODE;
+}
+
+static inline int nv_get_numa_status(nv_linux_state_t *nvl)
+{
+    if (!nv_platform_supports_numa(nvl))
+    {
+        return NV_IOCTL_NUMA_STATUS_DISABLED;
+    }
+
+    return NV_ATOMIC_READ(nvl->numa_info.status);
+}
+
+static inline int nv_set_numa_status(nv_linux_state_t *nvl, int status)
+{
+    if (!nv_platform_supports_numa(nvl))
+    {
+        return -EINVAL;
+    }
+
+    NV_ATOMIC_SET(nvl->numa_info.status, status);
+    return 0;
+}
+
+typedef enum
+{
+    NV_NUMA_STATUS_DISABLED             = 0,
+    NV_NUMA_STATUS_OFFLINE              = 1,
+    NV_NUMA_STATUS_ONLINE_IN_PROGRESS   = 2,
+    NV_NUMA_STATUS_ONLINE               = 3,
+    NV_NUMA_STATUS_ONLINE_FAILED        = 4,
+    NV_NUMA_STATUS_OFFLINE_IN_PROGRESS  = 5,
+    NV_NUMA_STATUS_OFFLINE_FAILED       = 6,
+    NV_NUMA_STATUS_COUNT
+} nv_numa_status_t;
+
+#if defined(NV_LINUX_PLATFORM_DEVICE_H_PRESENT)
+#include <linux/platform_device.h>
+#endif
+
+#if defined(NV_LINUX_MUTEX_H_PRESENT)
+#include <linux/mutex.h>
+#endif
+
+#if defined(NV_LINUX_RESET_H_PRESENT)
+#include <linux/reset.h>
+#endif
+
+#if defined(NV_LINUX_DMA_BUF_H_PRESENT)
+#include <linux/dma-buf.h>
+#endif
+
+#if defined(NV_LINUX_GPIO_H_PRESENT)
+#include <linux/gpio.h>
+#endif
+
+#if defined(NV_LINUX_OF_GPIO_H_PRESENT)
+#include <linux/of_gpio.h>
+#endif
+
+#if defined(NV_LINUX_OF_DEVICE_H_PRESENT)
+#include <linux/of_device.h>
+#endif
+
+#if defined(NV_LINUX_OF_PLATFORM_H_PRESENT)
+#include <linux/of_platform.h>
+#endif
+
+#if defined(NV_LINUX_INTERCONNECT_H_PRESENT)
+#include <linux/interconnect.h>
+#endif
+
+#if defined(NV_LINUX_PM_RUNTIME_H_PRESENT)
+#include <linux/pm_runtime.h>
+#endif
+
+#if defined(NV_LINUX_CLK_H_PRESENT)
+#include <linux/clk.h>
+#endif
+
+#if defined(NV_LINUX_CLK_PROVIDER_H_PRESENT)
+#include <linux/clk-provider.h>
+#endif
+
 #endif  /* _NV_LINUX_H_ */
