@@ -1,11 +1,24 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2013-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2013-2019 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 //
@@ -18,15 +31,9 @@
 // Forward references, to break circular header file dependencies:
 struct UvmOpsUvmEvents;
 
-//
-// TODO (bug 1359805): This should all be greatly simplified. It is still
-// carrying along a lot of baggage from when RM depended on UVM. Now that
-// direction is reversed: RM is independent, and UVM depends on RM.
-//
 #if defined(NVIDIA_UVM_ENABLED)
 
 // We are in the UVM build system, for a Linux target.
-#include "uvmtypes.h"
 #include "uvm_linux.h"
 
 #else
@@ -273,30 +280,12 @@ NV_STATUS nvUvmInterfaceMemoryAllocSys(uvmGpuAddressSpaceHandle vaSpace,
 /*******************************************************************************
     nvUvmInterfaceGetP2PCaps
 
-    Obtain the P2P capabilities between the devices with the given gpu address
-    space handles.
-
-    This function accepts gpu address space handles, both are required.
-
-    If indirect access is reported, the contents of peerIds are undefined.
+    Obtain the P2P capabilities between two devices.
 
     Arguments:
-        vaSpace1[IN]               - first gpu address space handle
-        vaSpace1[IN]               - second gpu address space handle
-        p2pCapsParams members:
-        peerLink[OUT]              - type of the link enabled between the peers
-        peerIds[OUT]               - peer ids between given pair of peers
-        writeSupported[OUT]        - p2p writes between peers are supported
-        readSupported[OUT]         - p2p reads between peers are supported
-        atomicSupported[OUT]       - p2p atomics between peers are supported
-        totalLinkLineRateMBps[OUT] - Maximum unidirectional bandwidth between
-                                     the peers in megabytes per second
-        directAccess[OUT]          - if a valid peer link is returned, this
-                                     field tells whether there is a direct link
-                                     between the devices or they communicate
-                                     through an intermediate device such as
-                                     a npu. If indirect access is reported link
-                                     must be NVLink2 or greater.
+        device1[IN]         - Device handle of the first GPU (required)
+        device2[IN]         - Device handle of the second GPU (required)
+        p2pCapsParams [OUT] - P2P capabilities between the two GPUs
 
     Error codes:
         NV_ERR_INVALID_ARGUMENT
@@ -305,8 +294,8 @@ NV_STATUS nvUvmInterfaceMemoryAllocSys(uvmGpuAddressSpaceHandle vaSpace,
           code,because it is not very informative.
 
 */
-NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
-                                   uvmGpuAddressSpaceHandle vaSpace2,
+NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuDeviceHandle device1,
+                                   uvmGpuDeviceHandle device2,
                                    UvmGpuP2PCapsParams * p2pCapsParams);
 
 /*******************************************************************************
@@ -316,8 +305,8 @@ NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
     PMA object handle is required for page allocation.
 
     Arguments:
-        vaSpace [IN]        - VA space handle allocated in
-                              nvUvmInterfaceAddressSpaceCreate
+        device [IN]         - Device handle allocated in
+                              nvUvmInterfaceDeviceCreate
         pPma [OUT]          - Pointer to PMA object
         pPmaPubStats [OUT]  - Pointer to UvmPmaStatistics object
 
@@ -327,7 +316,7 @@ NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
           Unexpected error. We try hard to avoid returning this error
           code,because it is not very informative.
 */
-NV_STATUS nvUvmInterfaceGetPmaObject(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceGetPmaObject(uvmGpuDeviceHandle device,
                                      void **pPma,
                                      const UvmPmaStatistics **pPmaPubStats);
 
@@ -589,7 +578,7 @@ NV_STATUS nvUvmInterfaceQueryCaps(uvmGpuDeviceHandle device,
       NV_ERR_GENERIC
       NV_ERR_NO_MEMORY
 */
-NV_STATUS nvUvmInterfaceQueryCopyEnginesCaps(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceQueryCopyEnginesCaps(uvmGpuDeviceHandle device,
                                              UvmGpuCopyEnginesCaps *caps);
 
 /*******************************************************************************
@@ -597,14 +586,10 @@ NV_STATUS nvUvmInterfaceQueryCopyEnginesCaps(uvmGpuAddressSpaceHandle vaSpace,
 
     Return various gpu info, refer to the UvmGpuInfo struct for details.
     If no gpu matching the uuid is found, an error will be returned.
-*/
 
-/*
     On Ampere+ GPUs, pGpuClientInfo contains SMC information provided by the
     client regarding the partition targeted in this operation.
-*/
 
-/*
     Error codes:
       NV_ERR_GENERIC
       NV_ERR_INSUFFICIENT_RESOURCES
@@ -664,33 +649,42 @@ NV_STATUS nvUvmInterfaceUnsetPageDirectory(uvmGpuAddressSpaceHandle vaSpace);
 /*******************************************************************************
     nvUvmInterfaceDupAllocation
 
-    Duplicate an allocation represented by a physical handle.
-    Duplication means: the physical handle will be duplicated from src vaspace
-    to dst vaspace and a new mapping will be created in the dst vaspace.
+    Duplicate the given allocation in a different VA space.
+
+    The physical handle backing the source allocation is duplicated in
+    the GPU device associated with the destination VA space, and a new mapping
+    is created in that VA space.
+
+    The input allocation can be located in sysmem (i.e. allocated using
+    nvUvmInterfaceMemoryAllocSys) or vidmem (i.e. allocated using
+    nvUvmInterfaceMemoryAllocFB). If located in vidmem, duplication across
+    GPUs is not supported.
+
+    For duplication of physical memory use nvUvmInterfaceDupMemory.
 
     Arguments:
-        hPhysHandle[IN]          - Handle representing the phys allocation.
-        srcVaspace[IN]           - Pointer to source vaSpace object
-        srcAddress[IN]           - Offset of the gpu mapping in source vaspace.
-        dstVaspace[IN]           - Pointer to destination vaSpace object
-        dstAddress[OUT]          - Offset of the gpu mapping in destination
-                                   vaspace.
-        bPhysHandleValid[IN]     - Whether the client has provided the handle
-                                   for source allocation.
-                                   If True; hPhysHandle will be used.
-                                   Else; ops will find out the handle using
-                                   srcVaspace and srcAddress
+        srcVaSpace[IN]  - Source VA space.
+        srcAddress[IN]  - GPU VA in the source VA space. The provided address
+                          should match one previously returned by
+                          nvUvmInterfaceMemoryAllocFB or
+                          nvUvmInterfaceMemoryAllocSys.
+        dstVaSpace[IN]  - Destination VA space where the new mapping will be
+                          created.
+        dstAddress[OUT] - Pointer to the GPU VA in the destination VA space.
 
     Error codes:
-      NV_ERROR
-      NV_ERR_INVALID_ARGUMENT
+      NV_ERR_INVALID_ARGUMENT - If any of the inputs is invalid, or the source
+                                and destination VA spaces are identical.
+      NV_ERR_OBJECT_NOT_FOUND - If the input allocation is not found in under
+                                the provided VA space.
+      NV_ERR_NO_MEMORY        - If there is no memory to back the duplicate,
+                                or the associated metadata.
+      NV_ERR_NOT_SUPPORTED    - If trying to duplicate vidmem across GPUs.
 */
-NV_STATUS nvUvmInterfaceDupAllocation(NvHandle hPhysHandle,
-                                      uvmGpuAddressSpaceHandle srcVaspace,
+NV_STATUS nvUvmInterfaceDupAllocation(uvmGpuAddressSpaceHandle srcVaSpace,
                                       NvU64 srcAddress,
-                                      uvmGpuAddressSpaceHandle dstVaspace,
-                                      NvU64 *dstAddress,
-                                      NvBool bPhysHandleValid);
+                                      uvmGpuAddressSpaceHandle dstVaSpace,
+                                      NvU64 *dstAddress);
 
 /*******************************************************************************
     nvUvmInterfaceDupMemory
@@ -699,7 +693,7 @@ NV_STATUS nvUvmInterfaceDupAllocation(NvHandle hPhysHandle,
     about the allocation.
 
     Arguments:
-        vaSpace[IN]                     - VA space linked to a client and a device under which
+        device[IN]                      - Device linked to a client under which
                                           the phys memory needs to be duped.
         hClient[IN]                     - Client owning the memory.
         hPhysMemory[IN]                 - Phys memory which is to be duped.
@@ -711,7 +705,7 @@ NV_STATUS nvUvmInterfaceDupAllocation(NvHandle hPhysHandle,
       NV_ERR_NOT_SUPPORTED      - If the allocation is not a physical allocation.
       NV_ERR_OBJECT_NOT_FOUND   - If the allocation is not found in under the provided client.
 */
-NV_STATUS nvUvmInterfaceDupMemory(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceDupMemory(uvmGpuDeviceHandle device,
                                   NvHandle hClient,
                                   NvHandle hPhysMemory,
                                   NvHandle *hDupMemory,
@@ -720,18 +714,18 @@ NV_STATUS nvUvmInterfaceDupMemory(uvmGpuAddressSpaceHandle vaSpace,
 /*******************************************************************************
     nvUvmInterfaceFreeDupedAllocation
 
-    Free the lallocation represented by the physical handle used to create the
+    Free the allocation represented by the physical handle used to create the
     duped allocation.
 
     Arguments:
-        vaspace[IN]              - Pointer to source vaSpace object
+        device[IN]               - Device handle used to dup the memory.
         hPhysHandle[IN]          - Handle representing the phys allocation.
 
     Error codes:
       NV_ERROR
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceFreeDupedHandle(uvmGpuAddressSpaceHandle vaspace,
+NV_STATUS nvUvmInterfaceFreeDupedHandle(uvmGpuDeviceHandle device,
                                         NvHandle hPhysHandle);
 
 /*******************************************************************************
@@ -740,14 +734,14 @@ NV_STATUS nvUvmInterfaceFreeDupedHandle(uvmGpuAddressSpaceHandle vaspace,
     Gets FB information from RM.
 
     Arguments:
-        vaspace[IN]       - Pointer to source vaSpace object
+        device[IN]        - GPU device handle
         fbInfo [OUT]      - Pointer to FbInfo structure which contains
                             reservedHeapSize & heapSize
     Error codes:
       NV_ERROR
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuDeviceHandle device,
                                   UvmGpuFbInfo * fbInfo);
 
 /*******************************************************************************
@@ -756,14 +750,14 @@ NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuAddressSpaceHandle vaSpace,
     Gets ECC information from RM.
 
     Arguments:
-        vaspace[IN]       - Pointer to source vaSpace object
+        device[IN]        - GPU device handle
         eccInfo [OUT]     - Pointer to EccInfo structure
 
     Error codes:
       NV_ERROR
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceGetEccInfo(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceGetEccInfo(uvmGpuDeviceHandle device,
                                    UvmGpuEccInfo * eccInfo);
 
 /*******************************************************************************
@@ -1005,20 +999,20 @@ void nvUvmInterfaceDeRegisterUvmOps(void);
 /*******************************************************************************
     nvUvmInterfaceP2pObjectCreate
 
-    This API creates an NV50_P2P object for the gpus with the given address
-    space handles and returns the handle to the object.
+    This API creates an NV50_P2P object for the GPUs with the given device
+    handles, and returns the handle to the object.
 
     Arguments:
-        vaSpace1[IN]        - first gpu address space handle
-        vaSpace1[IN]        - second gpu address space handle
-        hP2pObject[OUT]     - handle to the created P2p object.
+        device1[IN]        - first GPU device handle
+        device2[IN]        - second GPU device handle
+        hP2pObject[OUT]    - handle to the created P2p object.
 
     Error codes:
       NV_ERR_INVALID_ARGUMENT
       NV_ERR_OBJECT_NOT_FOUND : If device object associated with the uuids aren't found.
 */
-NV_STATUS nvUvmInterfaceP2pObjectCreate(uvmGpuAddressSpaceHandle vaSpace1,
-                                        uvmGpuAddressSpaceHandle vaSpace2,
+NV_STATUS nvUvmInterfaceP2pObjectCreate(uvmGpuDeviceHandle device1,
+                                        uvmGpuDeviceHandle device2,
                                         NvHandle *hP2pObject);
 
 /*******************************************************************************
@@ -1080,8 +1074,9 @@ NV_STATUS nvUvmInterfaceGetExternalAllocPtes(uvmGpuAddressSpaceHandle vaSpace,
 /*******************************************************************************
     nvUvmInterfaceRetainChannel
 
-    Validates and returns information about the user's channel. The state is
-    refcounted and must be released by calling nvUvmInterfaceReleaseChannel.
+    Validates and returns information about the user's channel and its resources
+    (local CTX buffers + global CTX buffers). The state is refcounted and must be
+    released by calling nvUvmInterfaceReleaseChannel.
 
     Arguments:
         vaSpace[IN]               - vaSpace handle.
@@ -1096,6 +1091,7 @@ NV_STATUS nvUvmInterfaceGetExternalAllocPtes(uvmGpuAddressSpaceHandle vaSpace,
         NV_ERR_INVALID_ARGUMENT : If the parameter/s are invalid.
         NV_ERR_OBJECT_NOT_FOUND : If the object associated with the handle isn't found.
         NV_ERR_INVALID_CHANNEL : If the channel verification fails.
+        NV_ERR_INSUFFICIENT_RESOURCES : If no memory available to store the resource information.
  */
 NV_STATUS nvUvmInterfaceRetainChannel(uvmGpuAddressSpaceHandle vaSpace,
                                       NvHandle hClient,
@@ -1103,27 +1099,6 @@ NV_STATUS nvUvmInterfaceRetainChannel(uvmGpuAddressSpaceHandle vaSpace,
                                       void **retainedChannel,
                                       UvmGpuChannelInstanceInfo *channelInstanceInfo);
 
-/*******************************************************************************
-    nvUvmInterfaceRetainChannelResources
-
-    Returns information about channel resources (local CTX buffers + global CTX buffers).
-    Also, it refcounts the memory descriptors associated with the resources.
-
-    Arguments:
-        retainedChannel[IN]       - Channel pointer returned by nvUvmInterfaceRetainChannel
-        channelResourceInfo[OUT]  - This should be a buffer which can fit at least
-                                    UvmGpuChannelInstanceInfo::resourceCount UvmGpuChannelResourceInfo
-                                    entries. The channel resource information will be written in this
-                                    buffer. See nv_uvm_types.h for details.
-
-    Error codes:
-        NV_ERR_INVALID_ARGUMENT : If the parameter/s are invalid.
-        NV_ERR_OBJECT_NOT_FOUND : If the object associated with the handle isn't found.
-        NV_ERR_INSUFFICIENT_RESOURCES : If no memory available to store the resource information.
- */
-NV_STATUS nvUvmInterfaceRetainChannelResources(void *retainedChannel,
-                                               UvmGpuChannelResourceInfo *channelResourceInfo);
-
 /*******************************************************************************
     nvUvmInterfaceBindChannelResources
 
@@ -1146,26 +1121,10 @@ NV_STATUS nvUvmInterfaceBindChannelResources(void *retainedChannel,
 /*******************************************************************************
     nvUvmInterfaceReleaseChannel
 
-    Releases state retained by nvUvmInterfaceRetainChannel. If
-    nvUvmInterfaceBindChannelResources has been called, you must release the
-    resources by calling nvUvmInterfaceReleaseChannelResources before calling
-    this function.
+    Releases state retained by nvUvmInterfaceRetainChannel.
  */
 void nvUvmInterfaceReleaseChannel(void *retainedChannel);
 
-/*******************************************************************************
-    nvUvmInterfaceReleaseChannelResources
-
-    Release refcounts on the memory descriptors associated with the resources.
-    Also, frees the memory descriptors if refcount reaches zero.
-
-    Arguments:
-        descriptors[IN]         - The call expects the input buffer of size(NvP64) * descriptorCount initialized
-                                  with the descriptors returned by nvUvmInterfaceRetainChannelResources as input.
-        descriptorCount[IN]     - The count of descriptors to be released.
- */
-void nvUvmInterfaceReleaseChannelResources(NvP64 *resourceDescriptors, NvU32 descriptorCount);
-
 /*******************************************************************************
     nvUvmInterfaceStopChannel
 
@@ -1220,14 +1179,198 @@ NV_STATUS nvUvmInterfaceGetChannelResourcePtes(uvmGpuAddressSpaceHandle vaSpace,
     will log the fault, notify the clients and then trigger RC on the channel.
 
     Arguments:
-        vaSpace[IN]                     -  The vaSpace where the fault happened.
-        pFaultPacket[IN]                -  The opaque pointer from UVM that will be later
-                                           converted to a MMU_FAULT_PACKET type.
+        device[IN]                     -  The device where the fault happened.
+        pFaultPacket[IN]               -  The opaque pointer from UVM that will be later
+                                          converted to a MMU_FAULT_PACKET type.
     Error codes:
         NV_ERR_INVALID_ARGUMENT         - Invalid parameter/s is passed.
         NV_ERR_NOT_SUPPORTED            - Functionality is not supported.
 */
-NV_STATUS nvUvmInterfaceReportNonReplayableFault(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceReportNonReplayableFault(uvmGpuDeviceHandle device,
                                                  const void *pFaultPacket);
 
+/*******************************************************************************
+    nvUvmInterfacePagingChannelAllocate
+
+    In SR-IOV heavy, this function requests the allocation of a paging channel
+    (i.e. a privileged CE channel) bound to a specified copy engine. Unlike
+    channels allocated via nvUvmInterfaceChannelAllocate, the caller cannot push
+    methods to a paging channel directly, but instead relies on the
+    nvUvmInterfacePagingChannelPushStream API to do so.
+
+    SR-IOV heavy only. The implementation of this interface can acquire
+    RM or GPU locks.
+
+    Arguments:
+        device[IN]       - device under which the paging channel will be allocated
+        allocParams[IN]  - structure with allocation settings
+        channel[OUT]     - pointer to the allocated paging channel handle
+        channelInfo[OUT] - structure filled with channel information
+
+   Error codes:
+        NV_ERR_INVALID_ARGUMENT         - Invalid parameter/s is passed.
+        NV_ERR_NO_MEMORY                - Not enough memory to allocate
+                                          paging channel/shadow notifier.
+        NV_ERR_NOT_SUPPORTED            - SR-IOV heavy mode is disabled.
+
+ */
+NV_STATUS nvUvmInterfacePagingChannelAllocate(uvmGpuDeviceHandle device,
+                                              const UvmGpuPagingChannelAllocParams *allocParams,
+                                              UvmGpuPagingChannelHandle *channel,
+                                              UvmGpuPagingChannelInfo *channelInfo);
+
+/*******************************************************************************
+    nvUvmInterfacePagingChannelDestroy
+
+    This function destroys a given paging channel.
+
+    SR-IOV heavy only. The implementation of this interface can acquire
+    RM or GPU locks.
+
+    Arguments:
+        channel[IN] - paging channel handle.
+*/
+void nvUvmInterfacePagingChannelDestroy(UvmGpuPagingChannelHandle channel);
+
+/*******************************************************************************
+
+    nvUvmInterfacePagingChannelsMap
+
+    Map a guest allocation in the address space associated with all the paging
+    channels allocated under the given device.
+
+    SR-IOV heavy only. The implementation of this interface can acquire
+    RM or GPU locks.
+
+    Arguments:
+        srcVaSpace[IN]  - VA space handle used to allocate the input pointer
+                          srcAddress.
+        srcAddress[IN]  - virtual address returned by nvUvmInterfaceMemoryAllocFB
+                          or nvUvmInterfaceMemoryAllocSys. The entire allocation
+                          backing this guest VA is mapped.
+        device[IN]      - device under which paging channels were allocated
+        dstAddress[OUT] - a virtual address that is valid (i.e. is mapped) in
+                          all the paging channels allocated under the given vaSpace.
+        
+   Error codes:
+        NV_ERR_INVALID_ARGUMENT         - Invalid parameter/s is passed.
+        NV_ERR_NOT_SUPPORTED            - SR-IOV heavy mode is disabled.
+*/
+NV_STATUS nvUvmInterfacePagingChannelsMap(uvmGpuAddressSpaceHandle srcVaSpace,
+                                          UvmGpuPointer srcAddress,
+                                          uvmGpuDeviceHandle device,
+                                          NvU64 *dstAddress);
+
+/*******************************************************************************
+
+    nvUvmInterfacePagingChannelsUnmap
+
+    Unmap a VA returned by nvUvmInterfacePagingChannelsMap.
+
+    SR-IOV heavy only. The implementation of this interface can acquire
+    RM or GPU locks.
+
+    Arguments:
+        srcVaSpace[IN] - VA space handle that was passed to prevous mapping.
+        srcAddress[IN] - virtual address that was passed to prevous mapping.
+        device[IN]     - device under which paging channels were allocated.
+ */
+void nvUvmInterfacePagingChannelsUnmap(uvmGpuAddressSpaceHandle srcVaSpace,
+                                       UvmGpuPointer srcAddress,
+                                       uvmGpuDeviceHandle device);
+
+
+/*******************************************************************************
+    nvUvmInterfacePagingChannelPushStream
+
+    Used for remote execution of the passed methods; the UVM driver uses this
+    interface to ask the vGPU plugin to execute certain HW methods on its
+    behalf. The callee should push the methods in the specified order i.e. is
+    not allowed to do any reordering.
+
+    The API is asynchronous. The UVM driver can wait on the remote execution by
+    inserting a semaphore release method at the end of the method stream, and
+    then loop until the semaphore value reaches the completion value indicated
+    in the release method.
+
+    The valid HW methods that can be passed by the UVM driver follow; the source
+    functions listed contain the exact formatting (encoding) of the HW method
+    used by the UVM driver for Ampere.
+
+      - TLB invalidation targeting a VA range. See
+        uvm_hal_volta_host_tlb_invalidate_va.
+
+      - TLB invalidation targeting certain levels in the page tree (including
+        the possibility of invalidating everything).
+        See uvm_hal_pascal_host_tlb_invalidate_all.
+
+      - Replayable fault replay. See uvm_hal_volta_replay_faults.
+
+      - Replayable fault cancellation targeting a guest virtual address. See
+        uvm_hal_volta_cancel_faults_va
+
+      - Membar, scoped to device or to the entire system. See
+        uvm_hal_pascal_host_membar_gpu and uvm_hal_pascal_host_membar_sys
+
+      - Host semaphore acquire, see uvm_hal_turing_host_semaphore_acquire. The
+        virtual address specified in the semaphore operation must lie within a
+        buffer previously mapped by nvUvmInterfacePagingChannelsMap.
+
+      - CE semaphore release, see uvm_hal_pascal_ce_semaphore_release. The
+        virtual address specified in the semaphore operation must lie within a
+        buffer previously mapped by nvUvmInterfacePagingChannelsMap.
+
+      - 64 bits-wide memset, see uvm_hal_kepler_ce_memset_8. The destination
+        address is a physical address in vidmem.
+
+      - No-op, see uvm_hal_kepler_host_noop. Used to store the source buffer
+        of a memcopy method within the input stream itself.
+
+      - Memcopy, see uvm_hal_kepler_ce_memcopy. The destination address is a
+        physical address in vidmem. The source address is an offset within
+        methodStream, in bytes, indicating the location of the (inlined) source
+        buffer. The copy size does not exceed 4KB.
+
+      - CE semaphore release with timestamp, see
+        uvm_hal_kepler_ce_semaphore_timestamp. The virtual address specified in
+        the semaphore operation must lie within a buffer previously mapped by
+        nvUvmInterfacePagingChannelsMap.
+
+      - CE semaphore reduction, see uvm_hal_kepler_ce_semaphore_reduction_inc.
+        The virtual address specified in the semaphore operation must lie within
+        a buffer previously mapped by nvUvmInterfacePagingChannelsMap.
+
+    Only invoked in SR-IOV heavy mode.
+
+    NOTES:
+      - This function uses a pre-allocated stack per paging channel
+        (stored in the UvmGpuPagingChannel object)
+      - This function DOES NOT acquire the RM API or GPU locks. That is because
+        it is called during fault servicing, which could produce deadlocks.
+      - Concurrent calls to this function using channels under same device are not
+        allowed due to:
+          a. pre-allocated stack
+          b. the fact that internal RPC infrastructure doesn't acquire GPU lock.
+        Therefore, locking is the caller's responsibility.
+      - This function DOES NOT sleep (does not allocate memory or acquire locks)
+        so it can be invoked while holding a spinlock.
+
+    Arguments:
+        channel[IN]          - paging channel handle obtained via
+                               nvUvmInterfacePagingChannelAllocate
+
+        methodStream[IN]     - HW methods to be pushed to the paging channel.
+
+        methodStreamSize[IN] - Size of methodStream, in bytes. The maximum push
+                               size is 128KB.
+
+        
+   Error codes:
+        NV_ERR_INVALID_ARGUMENT         - Invalid parameter/s is passed.
+        NV_ERR_NOT_SUPPORTED            - SR-IOV heavy mode is disabled.
+*/
+NV_STATUS nvUvmInterfacePagingChannelPushStream(UvmGpuPagingChannelHandle channel,
+                                                char *methodStream,
+                                                NvU32 methodStreamSize);
+
 #endif // _NV_UVM_INTERFACE_H_
