@@ -1,11 +1,24 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2014-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2014-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 //
@@ -77,11 +90,11 @@ typedef unsigned long long UvmGpuPointer;
 // The following typedefs serve to explain the resources they point to.
 // The actual resources remain RM internal and not exposed.
 //
-typedef struct uvmGpuSession_tag      *uvmGpuSessionHandle;      // gpuSessionHandle
-typedef struct uvmGpuDevice_tag       *uvmGpuDeviceHandle;       // gpuDeviceHandle
-typedef struct uvmGpuAddressSpace_tag *uvmGpuAddressSpaceHandle; // gpuAddressSpaceHandle
-typedef struct uvmGpuChannel_tag      *uvmGpuChannelHandle;      // gpuChannelHandle
-typedef struct uvmGpuCopyEngine_tag   *uvmGpuCopyEngineHandle;   // gpuObjectHandle
+typedef struct uvmGpuSession_tag       *uvmGpuSessionHandle;       // gpuSessionHandle
+typedef struct uvmGpuDevice_tag        *uvmGpuDeviceHandle;        // gpuDeviceHandle
+typedef struct uvmGpuAddressSpace_tag  *uvmGpuAddressSpaceHandle;  // gpuAddressSpaceHandle
+typedef struct uvmGpuChannel_tag       *uvmGpuChannelHandle;       // gpuChannelHandle
+typedef struct uvmGpuCopyEngine_tag    *uvmGpuCopyEngineHandle;    // gpuObjectHandle
 
 typedef struct UvmGpuMemoryInfo_tag
 {
@@ -119,12 +132,21 @@ typedef struct UvmGpuMemoryInfo_tag
     NvProcessorUuid uuid;
 } UvmGpuMemoryInfo;
 
-// Note: Global resources might have to be shared for all channels
-// under a single GPU's VA space. If the resource is global, the same
-// resourceDescriptor might be returned for more than one channel under
-// that VA space. The RM-internal reference count is incremented each
-// time it's returned.
-// Local resources are 1:1 with a channel.
+// Some resources must share the same virtual mappings across channels. A mapped
+// resource must be shared by a channel iff:
+//
+// 1) The channel belongs to a TSG (UvmGpuChannelInstanceInfo::bTsgChannel is
+//    NV_TRUE).
+//
+// 2) The channel is in the same TSG as all other channels sharing that mapping
+//    (UvmGpuChannelInstanceInfo::tsgId matches among channels).
+//
+// 3) The channel is in the same GPU address space as the other channels
+//    sharing that mapping.
+//
+// 4) The resource handle(s) match those of the shared mapping
+//    (UvmGpuChannelResourceInfo::resourceDescriptor and
+//    UvmGpuChannelResourceInfo::resourceId).
 typedef struct UvmGpuChannelResourceInfo_tag
 {
     // Out: Ptr to the RM memDesc of the channel resource.
@@ -133,10 +155,6 @@ typedef struct UvmGpuChannelResourceInfo_tag
     // Out: RM ID of the channel resource.
     NvU32 resourceId;
 
-    // Out: Set to NV_TRUE, if the resource is global.
-    //      Set to NV_FALSE, if the resource is local.
-    NvBool globalResource;
-
     // Out: Alignment needed for the resource allocation.
     NvU64 alignment;
 
@@ -144,6 +162,13 @@ typedef struct UvmGpuChannelResourceInfo_tag
     UvmGpuMemoryInfo resourceInfo;
 } UvmGpuChannelResourceInfo;
 
+typedef struct UvmGpuPagingChannelInfo_tag
+{
+    // Pointer to a shadown buffer mirroring the contents of the error notifier
+    // for the paging channel
+    NvNotification    *shadowErrorNotifier;
+} UvmGpuPagingChannelInfo;
+
 typedef enum
 {
     UVM_GPU_CHANNEL_ENGINE_TYPE_GR = 1,
@@ -161,6 +186,10 @@ typedef struct UvmGpuChannelInstanceInfo_tag
     //      Set to NV_FALSE, if the instance is in vidmem.
     NvBool sysmem;
 
+    // Out: Hardware runlist ID.
+    NvU32 runlistId;
+
+    // Out: Hardware channel ID.
     NvU32 chId;
 
     // Out: NV_TRUE if the channel belongs to a subcontext or NV_FALSE if it
@@ -179,8 +208,7 @@ typedef struct UvmGpuChannelInstanceInfo_tag
     // Out: Maximum number of subcontexts in the TSG to which the channel belongs
     NvU32 tsgMaxSubctxCount;
 
-    // Out: Info of channel resources (local CTX buffers + global CTX buffers)
-    // associated with the channel.
+    // Out: Info of channel resources associated with the channel.
     UvmGpuChannelResourceInfo resourceInfo[UVM_GPU_CHANNEL_MAX_RESOURCES];
 
     // Out: Number of valid entries in resourceInfo array.
@@ -220,12 +248,6 @@ typedef struct UvmGpuChannelResourceBindParams_tag
 
     // In: Starting VA at which the channel resource is mapped.
     NvU64 resourceVa;
-
-    // In: Set to NV_TRUE, if the resource is global.
-    //     Set to NV_FALSE, if the resource is local.
-    // Note: This is a temporary parameter. NV2080_CTRL_CMD_GR_BIND_CHANNEL_CONTEXT should figure
-    // out if the resourceDescriptor belongs to a Global/Local CTX resource. #Bug - 1732494
-    NvBool globalResource;
 } UvmGpuChannelResourceBindParams;
 
 typedef struct UvmGpuChannelInfo_tag
@@ -238,7 +260,10 @@ typedef struct UvmGpuChannelInfo_tag
 
     // The errorNotifier is filled out when the channel hits an RC error.
     NvNotification    *errorNotifier;
+
+    NvU32              hwRunlistId;
     NvU32              hwChannelId;
+
     volatile unsigned *dummyBar1Mapping;
 
     // These values are filled by nvUvmInterfaceCopyEngineAlloc. The work
@@ -281,6 +306,13 @@ typedef struct UvmGpuChannelAllocParams_tag
     NvU32 engineIndex;
 } UvmGpuChannelAllocParams;
 
+typedef struct UvmGpuPagingChannelAllocParams_tag
+{
+    // Index of the LCE engine the channel will be bound to, a zero-based offset
+    // from NV2080_ENGINE_TYPE_COPY0.
+    NvU32 engineIndex;
+} UvmGpuPagingChannelAllocParams;
+
 // The max number of Copy Engines supported by a GPU.
 // The gpu ops build has a static assert that this is the correct number.
 #define UVM_COPY_ENGINE_COUNT_MAX 10
@@ -331,16 +363,20 @@ typedef struct UvmGpuCopyEnginesCaps_tag
 
 typedef enum
 {
-    UVM_PEER_LINK_TYPE_NONE,
-    UVM_PEER_LINK_TYPE_PCIE,
-    UVM_PEER_LINK_TYPE_NVLINK_1,
-    UVM_PEER_LINK_TYPE_NVLINK_2,
-    UVM_PEER_LINK_TYPE_NVLINK_3,
-} UVM_PEER_LINK_TYPE;
+    UVM_LINK_TYPE_NONE,
+    UVM_LINK_TYPE_PCIE,
+    UVM_LINK_TYPE_NVLINK_1,
+    UVM_LINK_TYPE_NVLINK_2,
+    UVM_LINK_TYPE_NVLINK_3,
+
+
+
+
+} UVM_LINK_TYPE;
 
 typedef struct UvmGpuCaps_tag
 {
-    NvU32    sysmemLink;            // UVM_PEER_LINK_TYPE
+    NvU32    sysmemLink;            // UVM_LINK_TYPE
     NvU32    sysmemLinkRateMBps;    // See UvmGpuP2PCapsParams::totalLinkLineRateMBps
     NvBool   numaEnabled;
     NvU32    numaNodeId;
@@ -433,14 +469,14 @@ typedef enum
     UvmRmGpuCachingTypeDefault = 0,
     UvmRmGpuCachingTypeForceUncached = 1,
     UvmRmGpuCachingTypeForceCached = 2,
-    UvmRmGpuCachingTypeCount = 3 
+    UvmRmGpuCachingTypeCount = 3
 } UvmRmGpuCachingType;
 
 // UVM GPU format types
 typedef enum {
    UvmRmGpuFormatTypeDefault = 0,
    UvmRmGpuFormatTypeBlockLinear = 1,
-   UvmRmGpuFormatTypeCount = 2 
+   UvmRmGpuFormatTypeCount = 2
 } UvmRmGpuFormatType;
 
 // UVM GPU Element bits types
@@ -504,7 +540,7 @@ typedef struct UvmGpuP2PCapsParams_tag
     // the GPUs are direct peers.
     NvU32 peerIds[2];
 
-    // Out: UVM_PEER_LINK_TYPE
+    // Out: UVM_LINK_TYPE
     NvU32 p2pLink;
 
     // Out: optimalNvlinkWriteCEs[i] contains gpu[i]'s optimal CE for writing to
@@ -568,12 +604,6 @@ typedef struct UvmGpuInfo_tag
     // Compute class; *_COMPUTE_*, e.g. KEPLER_COMPUTE_A
     NvU32 computeClass;
 
-    // Fault buffer class; *_FAULT_BUFFER_*, e.g. MAXWELL_FAULT_BUFFER_A
-    NvU32 faultBufferClass;
-
-    // Access counter notification buffer class; ACCESS_COUNTER_NOTIFY_BUFFER
-    NvU32 accessCounterBufferClass;
-
     // Set if GPU supports TCC Mode & is in TCC mode.
     NvBool gpuInTcc;
 
@@ -793,7 +823,7 @@ typedef struct UvmGpuFaultInfo_tag
         // here the non-replayable faults that need to be handled by UVM
         void* shadowBufferAddress;
 
-        // Eecution context for the queue associated with the fault buffer
+        // Execution context for the queue associated with the fault buffer
         void* shadowBufferContext;
 
         // Fault buffer size
@@ -808,6 +838,15 @@ typedef struct UvmGpuFaultInfo_tag
     NvHandle faultBufferHandle;
 } UvmGpuFaultInfo;
 
+typedef struct UvmGpuPagingChannel_tag
+{
+    struct gpuDevice  *device;
+    NvNotification    *errorNotifier;
+    NvHandle          channelHandle;
+    NvHandle          errorNotifierHandle;
+    void              *pushStreamSp;
+} UvmGpuPagingChannel, *UvmGpuPagingChannelHandle;
+
 typedef struct UvmGpuAccessCntrInfo_tag
 {
     // Register mappings obtained from RM
@@ -886,5 +925,8 @@ typedef UvmGpuChannelInstanceInfo gpuChannelInstanceInfo;
 typedef UvmGpuChannelResourceBindParams gpuChannelResourceBindParams;
 typedef UvmGpuFbInfo gpuFbInfo;
 typedef UvmGpuEccInfo gpuEccInfo;
+typedef UvmGpuPagingChannel *gpuPagingChannelHandle;
+typedef UvmGpuPagingChannelInfo gpuPagingChannelInfo;
+typedef UvmGpuPagingChannelAllocParams gpuPagingChannelAllocParams;
 
 #endif // _NV_UVM_TYPES_H_
