@@ -23,9 +23,7 @@
 
 #define NVKMS_MAIN_LAYER                      0
 #define NVKMS_OVERLAY_LAYER                   1
-#define NVKMS_MAX_LAYERS_PER_HEAD             2
-#define NVKMS_ALL_LAYERS_MASK                (NVBIT(NVKMS_MAIN_LAYER) | \
-                                              NVBIT(NVKMS_OVERLAY_LAYER))
+#define NVKMS_MAX_LAYERS_PER_HEAD             8
 
 #define NVKMS_MAX_PLANES_PER_SURFACE          3
 
@@ -40,6 +38,7 @@ typedef NvU32 NvKmsSurfaceHandle;
 typedef NvU32 NvKmsFrameLockHandle;
 typedef NvU32 NvKmsDeferredRequestFifoHandle;
 typedef NvU32 NvKmsSwapGroupHandle;
+typedef NvU32 NvKmsVblankSyncObjectHandle;
 
 struct NvKmsSize {
     NvU16 width;
@@ -75,30 +74,34 @@ struct NvKmsMatrix {
 };
 
 typedef enum {
-    NVKMS_CONNECTOR_TYPE_DP      = 0,
-    NVKMS_CONNECTOR_TYPE_VGA     = 1,
-    NVKMS_CONNECTOR_TYPE_DVI_I   = 2,
-    NVKMS_CONNECTOR_TYPE_DVI_D   = 3,
-    NVKMS_CONNECTOR_TYPE_ADC     = 4,
-    NVKMS_CONNECTOR_TYPE_LVDS    = 5,
-    NVKMS_CONNECTOR_TYPE_HDMI    = 6,
-    NVKMS_CONNECTOR_TYPE_USBC    = 7,
-    NVKMS_CONNECTOR_TYPE_UNKNOWN = 8,
-    NVKMS_CONNECTOR_TYPE_MAX     = NVKMS_CONNECTOR_TYPE_UNKNOWN,
+    NVKMS_CONNECTOR_TYPE_DP             = 0,
+    NVKMS_CONNECTOR_TYPE_VGA            = 1,
+    NVKMS_CONNECTOR_TYPE_DVI_I          = 2,
+    NVKMS_CONNECTOR_TYPE_DVI_D          = 3,
+    NVKMS_CONNECTOR_TYPE_ADC            = 4,
+    NVKMS_CONNECTOR_TYPE_LVDS           = 5,
+    NVKMS_CONNECTOR_TYPE_HDMI           = 6,
+    NVKMS_CONNECTOR_TYPE_USBC           = 7,
+    NVKMS_CONNECTOR_TYPE_DSI            = 8,
+    NVKMS_CONNECTOR_TYPE_DP_SERIALIZER  = 9,
+    NVKMS_CONNECTOR_TYPE_UNKNOWN        = 10,
+    NVKMS_CONNECTOR_TYPE_MAX            = NVKMS_CONNECTOR_TYPE_UNKNOWN,
 } NvKmsConnectorType;
 
 static inline
 const char *NvKmsConnectorTypeString(const NvKmsConnectorType connectorType)
 {
     switch (connectorType) {
-    case NVKMS_CONNECTOR_TYPE_DP:    return "DP";
-    case NVKMS_CONNECTOR_TYPE_VGA:   return "VGA";
-    case NVKMS_CONNECTOR_TYPE_DVI_I: return "DVI-I";
-    case NVKMS_CONNECTOR_TYPE_DVI_D: return "DVI-D";
-    case NVKMS_CONNECTOR_TYPE_ADC:   return "ADC";
-    case NVKMS_CONNECTOR_TYPE_LVDS:  return "LVDS";
-    case NVKMS_CONNECTOR_TYPE_HDMI:  return "HDMI";
-    case NVKMS_CONNECTOR_TYPE_USBC:  return "USB-C";
+    case NVKMS_CONNECTOR_TYPE_DP:              return "DP";
+    case NVKMS_CONNECTOR_TYPE_VGA:             return "VGA";
+    case NVKMS_CONNECTOR_TYPE_DVI_I:           return "DVI-I";
+    case NVKMS_CONNECTOR_TYPE_DVI_D:           return "DVI-D";
+    case NVKMS_CONNECTOR_TYPE_ADC:             return "ADC";
+    case NVKMS_CONNECTOR_TYPE_LVDS:            return "LVDS";
+    case NVKMS_CONNECTOR_TYPE_HDMI:            return "HDMI";
+    case NVKMS_CONNECTOR_TYPE_USBC:            return "USB-C";
+    case NVKMS_CONNECTOR_TYPE_DSI:             return "DSI";
+    case NVKMS_CONNECTOR_TYPE_DP_SERIALIZER:   return "DP-SERIALIZER";
     default: break;
     }
     return "Unknown";
@@ -109,7 +112,8 @@ typedef enum {
     NVKMS_CONNECTOR_SIGNAL_FORMAT_LVDS    = 1,
     NVKMS_CONNECTOR_SIGNAL_FORMAT_TMDS    = 2,
     NVKMS_CONNECTOR_SIGNAL_FORMAT_DP      = 3,
-    NVKMS_CONNECTOR_SIGNAL_FORMAT_UNKNOWN = 4,
+    NVKMS_CONNECTOR_SIGNAL_FORMAT_DSI     = 4,
+    NVKMS_CONNECTOR_SIGNAL_FORMAT_UNKNOWN = 5,
     NVKMS_CONNECTOR_SIGNAL_FORMAT_MAX     =
       NVKMS_CONNECTOR_SIGNAL_FORMAT_UNKNOWN,
 } NvKmsConnectorSignalFormat;
@@ -198,14 +202,9 @@ struct NvKmsScalingUsageBounds {
 struct NvKmsUsageBounds {
     struct {
         NvBool usable;
-        NvU64 supportedSurfaceMemoryFormats NV_ALIGN_BYTES(8);
-    } base;
-
-    struct {
-        NvBool usable;
-        NvU64 supportedSurfaceMemoryFormats NV_ALIGN_BYTES(8);
         struct NvKmsScalingUsageBounds scaling;
-    } overlay;
+        NvU64 supportedSurfaceMemoryFormats NV_ALIGN_BYTES(8);
+    } layer[NVKMS_MAX_LAYERS_PER_HEAD];
 };
 
 /*
@@ -228,10 +227,39 @@ struct NvKmsCscMatrix {
         { 0, 0, 0x10000, 0 }        \
     }}
 
+/*!
+ * A color key match bit used in the blend equations and one can select the src
+ * or dst Color Key when blending. Assert key bit means match, de-assert key
+ * bit means nomatch.
+ *
+ * The src Color Key means using the key bit from the current layer, the dst
+ * Color Key means using key bit from the previous layer composition stage. The
+ * src or dst key bit will be inherited by blended pixel for the preparation of
+ * next blending, as dst Color Key.
+ *
+ * src: Forward the color key match bit from the current layer pixel to next layer
+ * composition stage.
+ *
+ * dst: Forward the color key match bit from the previous composition stage
+ * pixel to next layer composition stage.
+ *
+ * disable: Forward “1” to the next layer composition stage as the color key.
+ */
+enum NvKmsCompositionColorKeySelect {
+    NVKMS_COMPOSITION_COLOR_KEY_SELECT_DISABLE = 0,
+    NVKMS_COMPOSITION_COLOR_KEY_SELECT_SRC,
+    NVKMS_COMPOSITION_COLOR_KEY_SELECT_DST,
+};
+
+#define NVKMS_COMPOSITION_NUMBER_OF_COLOR_KEY_SELECTS 3
+
 /*!
  * Composition modes used for surfaces in general.
  * The various types of composition are:
  *
+ * Opaque: source pixels are opaque regardless of alpha,
+ * and will occlude the destination pixel.
+ *
  * Alpha blending: aka opacity, which could be specified
  * for a surface in its entirety, or on a per-pixel basis.
  *
@@ -240,41 +268,48 @@ struct NvKmsCscMatrix {
  * Premultiplied: alpha already applied to source pixel,
  * so it only counter-weighs the destination pixel.
  *
- * Opaque: source pixels are opaque regardless of alpha,
- * and will occlude the destination pixel.
- *
  * Color keying: use a color key structure to decide
  * the criteria for matching and compositing.
  * (See NVColorKey below.)
  */
-enum NvKmsCompositionMode {
+enum NvKmsCompositionBlendingMode {
+    /*!
+     * Modes that use no other parameters.
+     */
+    NVKMS_COMPOSITION_BLENDING_MODE_OPAQUE,
+
+    /*!
+     * Mode that ignores both per-pixel alpha provided
+     * by client and the surfaceAlpha, makes source pixel
+     * totally transparent.
+     */
+    NVKMS_COMPOSITION_BLENDING_MODE_TRANSPARENT,
+
     /*!
      * Modes that use per-pixel alpha provided by client,
      * and the surfaceAlpha must be set to 0.
      */
-    NVKMS_COMPOSITION_MODE_PREMULT_ALPHA,
-    NVKMS_COMPOSITION_MODE_NON_PREMULT_ALPHA,
-
-    /*!
-     * Modes that use no other parameters.
-     */
-    NVKMS_COMPOSITION_MODE_OPAQUE,
-
-    /*!
-     * These modes use the colorKey structure.
-     */
-    NVKMS_COMPOSITION_MODE_SOURCE_COLOR_KEYING,
-    NVKMS_COMPOSITION_MODE_DESTINATION_COLOR_KEYING,
+    NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_ALPHA,
+    NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_ALPHA,
 
     /*!
      * These use both the surface-wide and per-pixel alpha values.
      * surfaceAlpha is treated as numerator ranging from 0 to 255
      * of a fraction whose denominator is 255.
      */
-    NVKMS_COMPOSITION_MODE_PREMULT_SURFACE_ALPHA,
-    NVKMS_COMPOSITION_MODE_NON_PREMULT_SURFACE_ALPHA,
+    NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_SURFACE_ALPHA,
+    NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_SURFACE_ALPHA,
 };
 
+static inline NvBool
+NvKmsIsCompositionModeUseAlpha(enum NvKmsCompositionBlendingMode mode)
+{
+    return mode == NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_ALPHA ||
+           mode == NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_ALPHA ||
+           mode == NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_SURFACE_ALPHA ||
+           mode == NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_SURFACE_ALPHA;
+}
+
 /*!
  * Abstract description of a color key.
  *
@@ -291,12 +326,65 @@ typedef struct {
     NvBool matchA, matchR, matchG, matchB;
 } NVColorKey;
 
+/*!
+ * Describes the composition parameters for the single layer.
+ */
 struct NvKmsCompositionParams {
-    enum NvKmsCompositionMode compMode;
-    union {
-        NvU8 surfaceAlpha;  /* Applies to all pixels of entire surface */
-        NVColorKey colorKey;
-    };
+    enum NvKmsCompositionColorKeySelect colorKeySelect;
+    NVColorKey colorKey;
+    /*
+     * It is possible to assign different blending mode for match pixels and
+     * nomatch pixels. blendingMode[0] is used to blend a pixel with the color key
+     * match bit "0", and blendingMode[1] is used to blend a pixel with the color
+     * key match bit "1".
+     *
+     * But because of the hardware restrictions match and nomatch pixels can
+     * not use blending mode PREMULT_ALPHA, NON_PREMULT_ALPHA,
+     * PREMULT_SURFACE_ALPHA, and NON_PREMULT_SURFACE_ALPHA at once.
+     */
+    enum NvKmsCompositionBlendingMode blendingMode[2];
+    NvU8 surfaceAlpha; /* Applies to all pixels of entire surface */
+    /*
+     * Defines the composition order. A smaller value moves the layer closer to
+     * the top (away from the background). No need to pick consecutive values,
+     * requirements are that the value should be different for each of the
+     * layers owned by the head and the value for the main layer should be
+     * the greatest one.
+     *
+     * Cursor always remains at the top of all other layers, this parameter
+     * has no effect on cursor. NVKMS assigns default depth to each of the
+     * supported layers, by default depth of the layer is calculated as
+     * (NVKMS_MAX_LAYERS_PER_HEAD - index of the layer). If depth is set to
+     * '0' then default depth value will get used.
+     */
+    NvU8 depth;
+};
+
+/*!
+ * Describes the composition capabilities supported by the hardware for
+ * cursor or layer. It describes supported the color key selects and for each
+ * of the supported color key selects it describes supported blending modes
+ * for match and nomatch  pixles.
+ */
+struct NvKmsCompositionCapabilities {
+
+    struct {
+        /*
+         * A bitmask of the supported blending modes for match and nomatch
+         * pixels. It should be the bitwise 'or' of one or more
+         * NVBIT(NVKMS_COMPOSITION_BLENDING_MODE_*) values.
+         */
+        NvU32 supportedBlendModes[2];
+    } colorKeySelect[NVKMS_COMPOSITION_NUMBER_OF_COLOR_KEY_SELECTS];
+
+    /*
+     * A bitmask of the supported color key selects.
+     *
+     * It should be the bitwise 'or' of one or more
+     * NVBIT(NVKMS_COMPOSITION_COLOR_KEY_SELECT_*)
+     * values.
+     */
+    NvU32 supportedColorKeySelects;
 };
 
 /*!
@@ -384,4 +472,9 @@ typedef enum {
     NVKMS_MEMORY_NISO = 1,
 } NvKmsMemoryIsoType;
 
+typedef struct {
+    NvBool coherent;
+    NvBool noncoherent;
+} NvKmsDispIOCoherencyModes;
+
 #endif /* NVKMS_API_TYPES_H */
