@@ -96,25 +96,27 @@ struct NvKmsKapiDisplayMode {
     char name[NVKMS_KAPI_MODE_NAME_LEN];
 };
 
-typedef enum NvKmsKapiLayerTypeRec {
-    NVKMS_KAPI_LAYER_PRIMARY = 0,
-    NVKMS_KAPI_LAYER_CURSOR  = 1,
-    NVKMS_KAPI_LAYER_OVERLAY = 2,
-    NVKMS_KAPI_LAYER_MAX     = 3,
-} NvKmsKapiLayerType;
+#define NVKMS_KAPI_LAYER_MAX 8
+
+#define NVKMS_KAPI_LAYER_INVALID_IDX           0xff
+#define NVKMS_KAPI_LAYER_PRIMARY_IDX              0
 
 struct NvKmsKapiDeviceResourcesInfo {
 
     NvU32 numHeads;
+    NvU32 numLayers[NVKMS_KAPI_MAX_HEADS];
 
     NvU32        numConnectors;
     NvKmsKapiConnector connectorHandles[NVKMS_KAPI_MAX_CONNECTORS];
 
     struct {
         NvU32 validCursorCompositionModes;
-        NvU32 validOverlayCompositionModes;
+        NvU64 supportedCursorSurfaceMemoryFormats;
 
-        NvU16 validLayerRRTransforms;
+        struct {
+            NvU16 validRRTransforms;
+            NvU32 validCompositionModes;
+        } layer[NVKMS_KAPI_LAYER_MAX];
 
         NvU32 minWidthInPixels;
         NvU32 maxWidthInPixels;
@@ -129,6 +131,8 @@ struct NvKmsKapiDeviceResourcesInfo {
         NvU32 hasVideoMemory;
 
         NvU8  genericPageKind;
+
+        NvBool  supportsSyncpts;
     } caps;
 
     NvU64 supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_MAX];
@@ -178,10 +182,28 @@ struct NvKmsKapiStaticDisplayInfo {
 
 };
 
+struct NvKmsKapiSyncpt {
+
+    /*!
+     * Possible syncpt use case in kapi.
+     * For pre-syncpt, use only id and value
+     * and for post-syncpt, use only fd.
+     */
+    NvBool  preSyncptSpecified;
+    NvU32   preSyncptId;
+    NvU32   preSyncptValue;
+
+    NvBool  postSyncptRequested;
+};
+
 struct NvKmsKapiLayerConfig {
     struct NvKmsKapiSurface *surface;
-    struct NvKmsCompositionParams compParams;
+    struct {
+        enum NvKmsCompositionBlendingMode compMode;
+        NvU8 surfaceAlpha;
+    } compParams;
     struct NvKmsRRParams rrParams;
+    struct NvKmsKapiSyncpt syncptParams;
 
     NvU8 minPresentInterval;
     NvBool tearing;
@@ -189,7 +211,7 @@ struct NvKmsKapiLayerConfig {
     NvU16 srcX, srcY;
     NvU16 srcWidth, srcHeight;
 
-    NvU16 dstX, dstY;
+    NvS16 dstX, dstY;
     NvU16 dstWidth, dstHeight;
 };
 
@@ -204,6 +226,21 @@ struct NvKmsKapiLayerRequestedConfig {
     } flags;
 };
 
+struct NvKmsKapiCursorRequestedConfig {
+    struct NvKmsKapiSurface *surface;
+    struct {
+        enum NvKmsCompositionBlendingMode compMode;
+        NvU8 surfaceAlpha;
+    } compParams;
+
+    NvS16 dstX, dstY;
+
+    struct {
+        NvBool surfaceChanged : 1;
+        NvBool dstXYChanged   : 1;
+    } flags;
+};
+
 struct NvKmsKapiHeadModeSetConfig {
     /*
      * DRM distinguishes between the head state "enabled" (the specified
@@ -237,6 +274,8 @@ struct NvKmsKapiHeadRequestedConfig {
         NvBool modeChanged     : 1;
     } flags;
 
+    struct NvKmsKapiCursorRequestedConfig cursorRequestedConfig;
+
     struct NvKmsKapiLayerRequestedConfig
         layerRequestedConfig[NVKMS_KAPI_LAYER_MAX];
 };
@@ -247,6 +286,20 @@ struct NvKmsKapiRequestedModeSetConfig {
         headRequestedConfig[NVKMS_KAPI_MAX_HEADS];
 };
 
+struct NvKmsKapiLayerReplyConfig {
+    int postSyncptFd;
+};
+
+struct NvKmsKapiHeadReplyConfig {
+    struct NvKmsKapiLayerReplyConfig
+        layerReplyConfig[NVKMS_KAPI_LAYER_MAX];
+};
+
+struct NvKmsKapiModeSetReplyConfig {
+    struct NvKmsKapiHeadReplyConfig
+        headReplyConfig[NVKMS_KAPI_MAX_HEADS];
+};
+
 struct NvKmsKapiEventDisplayChanged {
     NvKmsKapiDisplay display;
 };
@@ -257,7 +310,7 @@ struct NvKmsKapiEventDynamicDisplayConnected {
 
 struct NvKmsKapiEventFlipOccurred {
     NvU32 head;
-    NvKmsKapiLayerType layer;
+    NvU32 layer;
 };
 
 struct NvKmsKapiDpyCRC32 {
@@ -821,6 +874,7 @@ struct NvKmsKapiFunctionsTable {
     (
         struct NvKmsKapiDevice *device,
         const struct NvKmsKapiRequestedModeSetConfig *requestedConfig,
+        struct NvKmsKapiModeSetReplyConfig *replyConfig,
         const NvBool commit
     );
 
@@ -831,7 +885,7 @@ struct NvKmsKapiFunctionsTable {
      *
      * \param  [in]  head     A head returned by getDeviceResourcesInfo().
      *
-     * \param  [in]  layer    A layer type.
+     * \param  [in]  layer    A layer index.
      *
      * \param  [out] pending  Return TRUE if head has pending flip for
      *                        given layer.
@@ -842,7 +896,7 @@ struct NvKmsKapiFunctionsTable {
     (
         const struct NvKmsKapiDevice *device,
         const NvU32 head,
-        const NvKmsKapiLayerType layer,
+        const NvU32 layer,
         NvBool *pending
     );
 
@@ -941,6 +995,24 @@ struct NvKmsKapiFunctionsTable {
         NvU64 *pPages
     );
 
+    /*
+     * Import SGT as a memory handle.
+     *
+     * \param [in] device  A device allocated using allocateDevice().
+     *
+     * \param [in] sgt     SGT pointer.
+     * \param [in] gem     GEM pointer that pinned SGT, to be refcounted.
+     *
+     * \param [in] limit   Size, in bytes, of the memory backed by the SGT.
+     *
+     * \return A valid memory handle on success, NULL on failure.
+     */
+    struct NvKmsKapiMemory*
+    (*getSystemMemoryHandleFromSgt)(struct NvKmsKapiDevice *device,
+                                    NvP64 sgt,
+                                    NvP64 gem,
+                                    NvU32 limit);
+
     /*
      * Import dma-buf in the memory handle.
      *
