@@ -74,11 +74,6 @@ extern "C" {
 #endif
 #endif // __cplusplus
 
-#define NVRM_64 1
-#if defined(NV_64_BITS)
-#define NVRM_TRUE64 1
-#endif
-
  /***************************************************************************\
 |*                                 Typedefs                                  *|
  \***************************************************************************/
@@ -377,8 +372,8 @@ do {                                                                           \
 /*!
  * Helper macro to sub two 32 aligned 64 bit numbers on 64 bit processor.
  *
- * @param[in]       pSrc1   Pointer to NvU64_ALIGN32 scource 1 structure.
- * @param[in]       pSrc2   Pointer to NvU64_ALIGN32 scource 2 structure.
+ * @param[in]       pSrc1   Pointer to NvU64_ALIGN32 source 1 structure.
+ * @param[in]       pSrc2   Pointer to NvU64_ALIGN32 source 2 structure.
  * @param[in/out]   pDst    Pointer to NvU64_ALIGN32 dest. structure.
  */
 #define NvU64_ALIGN32_ADD(pDst, pSrc1, pSrc2)                                 \
@@ -394,8 +389,8 @@ do {                                                                          \
 /*!
  * Helper macro to sub two 32 aligned 64 bit numbers on 64 bit processor.
  *
- * @param[in]       pSrc1   Pointer to NvU64_ALIGN32 scource 1 structure.
- * @param[in]       pSrc2   Pointer to NvU64_ALIGN32 scource 2 structure.
+ * @param[in]       pSrc1   Pointer to NvU64_ALIGN32 source 1 structure.
+ * @param[in]       pSrc2   Pointer to NvU64_ALIGN32 source 2 structure.
  * @param[in/out]   pDst    Pointer to NvU64_ALIGN32 dest. structure.
  */
 #define NvU64_ALIGN32_SUB(pDst, pSrc1, pSrc2)                                  \
@@ -570,6 +565,8 @@ typedef NvS32 S032;
 
     #define NV_ATTRIBUTE_UNUSED
 
+    #define NV_FORMAT_PRINTF(_f, _a)
+
 #else // ! defined(_MSC_VER)
 
     #if defined(__GNUC__)
@@ -671,6 +668,18 @@ typedef NvS32 S032;
         #define NV_ATTRIBUTE_UNUSED
     #endif
 
+    /*
+     * Functions decorated with NV_FORMAT_PRINTF(f, a) have a format string at
+     * parameter number 'f' and variadic arguments start at parameter number 'a'.
+     * (Note that for C++ methods, there is an implicit 'this' parameter so
+     * explicit parameters are numbered from 2.)
+     */
+    #if defined(__GNUC__)
+        #define NV_FORMAT_PRINTF(_f, _a) __attribute__((format(printf, _f, _a)))
+    #else
+        #define NV_FORMAT_PRINTF(_f, _a)
+    #endif
+
 #endif  // defined(_MSC_VER)
 
 /*!
@@ -778,7 +787,7 @@ typedef NvUFXP64                                                   NvUFXP52_12;
  * not fit the destination type.  It's caller responsibility to ensure that such
  * situations will not occur.
  *
- * Some conversions perform some commonly preformed tasks other than just
+ * Some conversions perform some commonly performed tasks other than just
  * bit-shifting:
  *
  * - _SCALED:
@@ -864,6 +873,10 @@ typedef NvUFXP64                                                   NvUFXP52_12;
             NV_TYPES_FXP_FRACTIONAL_MSB_ONE, ((NvSFXP##x##_##y) (fxp))) ?     \
             1 : 0))
 
+#define NV_TYPES_SFXP_X_Y_TO_FLOAT32(x, y, fxp)                               \
+    ((NvF32) NV_TYPES_SFXP_X_Y_TO_S32(x, y, (fxp)) +                          \
+        ((NvF32) DRF_NUM(_TYPES, _FXP, _FRACTIONAL((x), (y)),                 \
+            ((NvSFXP##x##_##y) (fxp))) / (NvF32) (1 << (y))))
 
 //
 // 64-bit Signed FXP:
@@ -961,6 +974,8 @@ typedef NvSFXP24_8              NvTemp;
                                 NV_TYPES_SFXP_X_Y_TO_S32(24,8,(nvt))
 #define NV_TYPES_NV_TEMP_TO_CELSIUS_ROUNDED(nvt)                              \
                                 NV_TYPES_SFXP_X_Y_TO_S32_ROUNDED(24,8,(nvt))
+#define NV_TYPES_NV_TEMP_TO_CELSIUS_FLOAT(nvt)                                \
+                                NV_TYPES_SFXP_X_Y_TO_FLOAT32(24,8,(nvt))
 
 /*!
  * Macro for NvType -> number of bits conversion
