@@ -42,12 +42,18 @@
 #include <drm/drm_atomic.h>
 #include <drm/drm_atomic_helper.h>
 
+#if defined(NV_LINUX_NVHOST_H_PRESENT) && defined(CONFIG_TEGRA_GRHOST)
+#include <linux/nvhost.h>
+#endif
+
 static void nv_drm_plane_destroy(struct drm_plane *plane)
 {
+    struct nv_drm_plane *nv_plane = to_nv_plane(plane);
+
     /* plane->state gets freed here */
     drm_plane_cleanup(plane);
 
-    nv_drm_free(plane);
+    nv_drm_free(nv_plane);
 }
 
 static inline void
@@ -57,16 +63,127 @@ plane_req_config_disable(struct NvKmsKapiLayerRequestedConfig *req_config)
     req_config->flags.surfaceChanged = NV_TRUE;
 }
 
+static inline void
+cursor_req_config_disable(struct NvKmsKapiCursorRequestedConfig *req_config)
+{
+    req_config->surface = NULL;
+    req_config->flags.surfaceChanged = NV_TRUE;
+}
+
 static void
+cursor_plane_req_config_update(struct drm_plane *plane,
+                               struct drm_plane_state *plane_state,
+                               struct NvKmsKapiCursorRequestedConfig *req_config)
+{
+    struct nv_drm_plane *nv_plane = to_nv_plane(plane);
+    struct NvKmsKapiCursorRequestedConfig old_config = *req_config;
+    struct nv_drm_device *nv_dev = to_nv_device(plane->dev);
+    struct nv_drm_plane_state *nv_drm_plane_state =
+        to_nv_drm_plane_state(plane_state);
+
+    if (plane_state->fb == NULL) {
+        cursor_req_config_disable(req_config);
+        return;
+    }
+
+    *req_config = (struct NvKmsKapiCursorRequestedConfig) {
+        .surface = to_nv_framebuffer(plane_state->fb)->pSurface,
+
+        .dstX = plane_state->crtc_x,
+        .dstY = plane_state->crtc_y,
+    };
+
+#if defined(NV_DRM_ALPHA_BLENDING_AVAILABLE)
+    if (plane->blend_mode_property != NULL && plane->alpha_property != NULL) {
+
+        switch (plane_state->pixel_blend_mode) {
+            case DRM_MODE_BLEND_PREMULTI:
+                req_config->compParams.compMode =
+                    NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_SURFACE_ALPHA;
+                break;
+            case DRM_MODE_BLEND_COVERAGE:
+                req_config->compParams.compMode =
+                    NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_SURFACE_ALPHA;
+                break;
+            default:
+                /*
+                 * We should not hit this, because
+                 * plane_state->pixel_blend_mode should only have values
+                 * registered in
+                 * __nv_drm_plane_create_alpha_blending_properties().
+                 */
+                WARN_ON("Unsupported blending mode");
+                break;
+
+        }
+
+        req_config->compParams.surfaceAlpha =
+            plane_state->alpha >> 8;
+
+    } else if (plane->blend_mode_property != NULL) {
+
+        switch (plane_state->pixel_blend_mode) {
+            case DRM_MODE_BLEND_PREMULTI:
+                req_config->compParams.compMode =
+                    NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_ALPHA;
+                break;
+            case DRM_MODE_BLEND_COVERAGE:
+                req_config->compParams.compMode =
+                    NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_ALPHA;
+                break;
+            default:
+                /*
+                 * We should not hit this, because
+                 * plane_state->pixel_blend_mode should only have values
+                 * registered in
+                 * __nv_drm_plane_create_alpha_blending_properties().
+                 */
+                WARN_ON("Unsupported blending mode");
+                break;
+
+        }
+
+    } else {
+        req_config->compParams.compMode =
+            nv_plane->defaultCompositionMode;
+    }
+#else
+    req_config->compParams.compMode = nv_plane->defaultCompositionMode;
+#endif
+
+    /*
+     * Unconditionally mark the surface as changed, even if nothing changed,
+     * so that we always get a flip event: a DRM client may flip with
+     * the same surface and wait for a flip event.
+     */
+    req_config->flags.surfaceChanged = NV_TRUE;
+
+    if (old_config.surface == NULL &&
+        old_config.surface != req_config->surface) {
+        req_config->flags.dstXYChanged = NV_TRUE;
+        return;
+    }
+
+    req_config->flags.dstXYChanged =
+        old_config.dstX != req_config->dstX ||
+        old_config.dstY != req_config->dstY;
+}
+
+static int
 plane_req_config_update(struct drm_plane *plane,
                         struct drm_plane_state *plane_state,
                         struct NvKmsKapiLayerRequestedConfig *req_config)
 {
+    struct nv_drm_plane *nv_plane = to_nv_plane(plane);
     struct NvKmsKapiLayerConfig old_config = req_config->config;
+    struct nv_drm_device *nv_dev = to_nv_device(plane->dev);
+    struct nv_drm_plane_state *nv_drm_plane_state =
+        to_nv_drm_plane_state(plane_state);
+    int ret = 0;
 
     if (plane_state->fb == NULL) {
         plane_req_config_disable(req_config);
-        return;
+        return 0;
     }
 
     *req_config = (struct NvKmsKapiLayerRequestedConfig) {
@@ -132,11 +249,11 @@ plane_req_config_update(struct drm_plane *plane,
         switch (plane_state->pixel_blend_mode) {
             case DRM_MODE_BLEND_PREMULTI:
                 req_config->config.compParams.compMode =
-                    NVKMS_COMPOSITION_MODE_PREMULT_SURFACE_ALPHA;
+                    NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_SURFACE_ALPHA;
                 break;
             case DRM_MODE_BLEND_COVERAGE:
                 req_config->config.compParams.compMode =
-                    NVKMS_COMPOSITION_MODE_NON_PREMULT_SURFACE_ALPHA;
+                    NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_SURFACE_ALPHA;
                 break;
             default:
                 /*
@@ -158,11 +275,11 @@ plane_req_config_update(struct drm_plane *plane,
         switch (plane_state->pixel_blend_mode) {
             case DRM_MODE_BLEND_PREMULTI:
                 req_config->config.compParams.compMode =
-                    NVKMS_COMPOSITION_MODE_PREMULT_ALPHA;
+                    NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_ALPHA;
                 break;
             case DRM_MODE_BLEND_COVERAGE:
                 req_config->config.compParams.compMode =
-                    NVKMS_COMPOSITION_MODE_NON_PREMULT_ALPHA;
+                    NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_ALPHA;
                 break;
             default:
                 /*
@@ -176,9 +293,45 @@ plane_req_config_update(struct drm_plane *plane,
 
         }
 
+    } else {
+        req_config->config.compParams.compMode =
+            nv_plane->defaultCompositionMode;
     }
+#else
+    req_config->config.compParams.compMode =
+        nv_plane->defaultCompositionMode;
 #endif
 
+    req_config->config.syncptParams.preSyncptSpecified = false;
+    req_config->config.syncptParams.postSyncptRequested = false;
+
+    if (plane_state->fence != NULL || nv_drm_plane_state->fd_user_ptr) {
+        if (!nv_dev->supportsSyncpts) {
+            return -1;
+        }
+
+#if defined(NV_LINUX_NVHOST_H_PRESENT) && defined(CONFIG_TEGRA_GRHOST)
+#if defined(NV_NVHOST_DMA_FENCE_UNPACK_PRESENT)
+        if (plane_state->fence != NULL) {
+            ret = nvhost_dma_fence_unpack(
+                      plane_state->fence,
+                      &req_config->config.syncptParams.preSyncptId,
+                      &req_config->config.syncptParams.preSyncptValue);
+            if (ret != 0) {
+                return ret;
+            }
+            req_config->config.syncptParams.preSyncptSpecified = true;
+        }
+#endif
+
+        if (nv_drm_plane_state->fd_user_ptr) {
+            req_config->config.syncptParams.postSyncptRequested = true;
+        }
+#else
+        return -1;
+#endif
+    }
+
     /*
      * Unconditionally mark the surface as changed, even if nothing changed,
      * so that we always get a flip event: a DRM client may flip with
@@ -192,7 +345,7 @@ plane_req_config_update(struct drm_plane *plane,
         req_config->flags.srcWHChanged = NV_TRUE;
         req_config->flags.dstXYChanged = NV_TRUE;
         req_config->flags.dstWHChanged = NV_TRUE;
-        return;
+        return 0;
     }
 
     req_config->flags.srcXYChanged =
@@ -210,6 +363,8 @@ plane_req_config_update(struct drm_plane *plane,
     req_config->flags.dstWHChanged =
         old_config.dstWidth != req_config->config.dstWidth ||
         old_config.dstHeight != req_config->config.dstHeight;
+
+    return 0;
 }
 
 static bool __is_async_flip_requested(const struct drm_plane *plane,
@@ -226,24 +381,68 @@ static bool __is_async_flip_requested(const struct drm_plane *plane,
     return false;
 }
 
-static int nv_drm_plane_atomic_check(struct drm_plane *plane,
-                                     struct drm_plane_state *plane_state)
+static int __nv_drm_cursor_atomic_check(struct drm_plane *plane,
+                                        struct drm_plane_state *plane_state)
 {
+    struct nv_drm_plane *nv_plane = to_nv_plane(plane);
     int i;
     struct drm_crtc *crtc;
     struct drm_crtc_state *crtc_state;
-    NvKmsKapiLayerType type;
 
-    if (NV_DRM_WARN(!drm_plane_type_to_nvkms_plane_type(plane->type, &type))) {
-        goto done;
+    WARN_ON(nv_plane->layer_idx != NVKMS_KAPI_LAYER_INVALID_IDX);
+
+    nv_drm_for_each_crtc_in_state(plane_state->state, crtc, crtc_state, i) {
+        struct nv_drm_crtc_state *nv_crtc_state = to_nv_crtc_state(crtc_state);
+        struct NvKmsKapiHeadRequestedConfig *head_req_config =
+            &nv_crtc_state->req_config;
+        struct NvKmsKapiCursorRequestedConfig *cursor_req_config =
+            &head_req_config->cursorRequestedConfig;
+
+        if (plane->state->crtc == crtc &&
+            plane->state->crtc != plane_state->crtc) {
+            cursor_req_config_disable(cursor_req_config);
+            continue;
+        }
+
+        if (plane_state->crtc == crtc) {
+            cursor_plane_req_config_update(plane, plane_state,
+                                           cursor_req_config);
+        }
     }
 
+    return 0;
+}
+
+#if defined(NV_DRM_PLANE_ATOMIC_CHECK_HAS_ATOMIC_STATE_ARG)
+static int nv_drm_plane_atomic_check(struct drm_plane *plane,
+                                     struct drm_atomic_state *state)
+#else
+static int nv_drm_plane_atomic_check(struct drm_plane *plane,
+                                     struct drm_plane_state *plane_state)
+#endif
+{
+    struct nv_drm_plane *nv_plane = to_nv_plane(plane);
+#if defined(NV_DRM_PLANE_ATOMIC_CHECK_HAS_ATOMIC_STATE_ARG)
+    struct drm_plane_state *plane_state =
+        drm_atomic_get_new_plane_state(state, plane);
+#endif
+    int i;
+    struct drm_crtc *crtc;
+    struct drm_crtc_state *crtc_state;
+    int ret;
+
+    if (plane->type == DRM_PLANE_TYPE_CURSOR) {
+        return __nv_drm_cursor_atomic_check(plane, plane_state);
+    }
+
+    WARN_ON(nv_plane->layer_idx == NVKMS_KAPI_LAYER_INVALID_IDX);
+
     nv_drm_for_each_crtc_in_state(plane_state->state, crtc, crtc_state, i) {
         struct nv_drm_crtc_state *nv_crtc_state = to_nv_crtc_state(crtc_state);
         struct NvKmsKapiHeadRequestedConfig *head_req_config =
             &nv_crtc_state->req_config;
         struct NvKmsKapiLayerRequestedConfig *plane_requested_config =
-            &head_req_config->layerRequestedConfig[type];
+            &head_req_config->layerRequestedConfig[nv_plane->layer_idx];
 
         if (plane->state->crtc == crtc &&
             plane->state->crtc != plane_state->crtc) {
@@ -252,9 +451,12 @@ static int nv_drm_plane_atomic_check(struct drm_plane *plane,
         }
 
         if (plane_state->crtc == crtc) {
-            plane_req_config_update(plane,
-                                    plane_state,
-                                    plane_requested_config);
+            ret = plane_req_config_update(plane,
+                                          plane_state,
+                                          plane_requested_config);
+            if (ret != 0) {
+                return ret;
+            }
 
             if (__is_async_flip_requested(plane, crtc_state)) {
                 /*
@@ -271,20 +473,9 @@ static int nv_drm_plane_atomic_check(struct drm_plane *plane,
         }
     }
 
-done:
     return 0;
 }
 
-static void nv_drm_plane_atomic_update(struct drm_plane *plane,
-                                       struct drm_plane_state *old_state)
-{
-}
-
-static void nv_drm_plane_atomic_disable(struct drm_plane *plane,
-                                        struct drm_plane_state *old_state)
-{
-}
-
 #if defined(NV_DRM_UNIVERSAL_PLANE_INIT_HAS_FORMAT_MODIFIERS_ARG)
 static bool nv_drm_plane_format_mod_supported(struct drm_plane *plane,
                                               uint32_t format,
@@ -295,13 +486,90 @@ static bool nv_drm_plane_format_mod_supported(struct drm_plane *plane,
 }
 #endif
 
+
+static int nv_drm_plane_atomic_set_property(
+    struct drm_plane *plane,
+    struct drm_plane_state *state,
+    struct drm_property *property,
+    uint64_t val)
+{
+    struct nv_drm_device *nv_dev = to_nv_device(plane->dev);
+    struct nv_drm_plane_state *nv_drm_plane_state =
+        to_nv_drm_plane_state(state);
+
+    if (property == nv_dev->nv_out_fence_property) {
+#if defined(NV_LINUX_NVHOST_H_PRESENT) && defined(CONFIG_TEGRA_GRHOST)
+        nv_drm_plane_state->fd_user_ptr = u64_to_user_ptr(val);
+#endif
+        return 0;
+    } else {
+        return -EINVAL;
+    }
+}
+
+static int nv_drm_plane_atomic_get_property(
+    struct drm_plane *plane,
+    const struct drm_plane_state *state,
+    struct drm_property *property,
+    uint64_t *val)
+{
+    struct nv_drm_device *nv_dev = to_nv_device(plane->dev);
+
+    if (property == nv_dev->nv_out_fence_property) {
+        return 0;
+    } else {
+        return -EINVAL;
+    }
+}
+
+static struct drm_plane_state *
+nv_drm_plane_atomic_duplicate_state(struct drm_plane *plane)
+{
+    struct nv_drm_plane_state *nv_old_plane_state =
+        to_nv_drm_plane_state(plane->state);
+    struct nv_drm_plane_state *nv_plane_state =
+        nv_drm_calloc(1, sizeof(*nv_plane_state));
+
+    if (nv_plane_state == NULL) {
+        return NULL;
+    }
+
+    __drm_atomic_helper_plane_duplicate_state(plane, &nv_plane_state->base);
+
+    nv_plane_state->fd_user_ptr = nv_old_plane_state->fd_user_ptr;
+
+    return &nv_plane_state->base;
+}
+
+static inline void __nv_drm_plane_atomic_destroy_state(
+    struct drm_plane *plane,
+    struct drm_plane_state *state)
+{
+#if defined(NV_DRM_ATOMIC_HELPER_PLANE_DESTROY_STATE_HAS_PLANE_ARG)
+    __drm_atomic_helper_plane_destroy_state(plane, state);
+#else
+    __drm_atomic_helper_plane_destroy_state(state);
+#endif
+}
+
+static void nv_drm_plane_atomic_destroy_state(
+    struct drm_plane *plane,
+    struct drm_plane_state *state)
+{
+    __nv_drm_plane_atomic_destroy_state(plane, state);
+
+    nv_drm_free(to_nv_drm_plane_state(state));
+}
+
 static const struct drm_plane_funcs nv_plane_funcs = {
     .update_plane           = drm_atomic_helper_update_plane,
     .disable_plane          = drm_atomic_helper_disable_plane,
     .destroy                = nv_drm_plane_destroy,
     .reset                  = drm_atomic_helper_plane_reset,
-    .atomic_duplicate_state = drm_atomic_helper_plane_duplicate_state,
-    .atomic_destroy_state   = drm_atomic_helper_plane_destroy_state,
+    .atomic_get_property    = nv_drm_plane_atomic_get_property,
+    .atomic_set_property    = nv_drm_plane_atomic_set_property,
+    .atomic_duplicate_state = nv_drm_plane_atomic_duplicate_state,
+    .atomic_destroy_state   = nv_drm_plane_atomic_destroy_state,
 #if defined(NV_DRM_UNIVERSAL_PLANE_INIT_HAS_FORMAT_MODIFIERS_ARG)
     .format_mod_supported   = nv_drm_plane_format_mod_supported,
 #endif
@@ -309,8 +577,6 @@ static const struct drm_plane_funcs nv_plane_funcs = {
 
 static const struct drm_plane_helper_funcs nv_plane_helper_funcs = {
     .atomic_check   = nv_drm_plane_atomic_check,
-    .atomic_update  = nv_drm_plane_atomic_update,
-    .atomic_disable = nv_drm_plane_atomic_disable,
 };
 
 static void nv_drm_crtc_destroy(struct drm_crtc *crtc)
@@ -529,24 +795,35 @@ static const struct drm_crtc_helper_funcs nv_crtc_helper_funcs = {
     .mode_fixup = nv_drm_crtc_mode_fixup,
 };
 
+static void nv_drm_plane_install_properties(
+    struct drm_plane *plane)
+{
+    struct nv_drm_device *nv_dev = to_nv_device(plane->dev);
+
+    if (nv_dev->nv_out_fence_property) {
+        drm_object_attach_property(
+            &plane->base, nv_dev->nv_out_fence_property, 0);
+    }
+}
+
 static void
 __nv_drm_plane_create_alpha_blending_properties(struct drm_plane *plane,
                                                  NvU32 validCompModes)
 {
 #if defined(NV_DRM_ALPHA_BLENDING_AVAILABLE)
     if ((validCompModes &
-         NVBIT(NVKMS_COMPOSITION_MODE_PREMULT_SURFACE_ALPHA)) != 0x0 &&
+         NVBIT(NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_SURFACE_ALPHA)) != 0x0 &&
         (validCompModes &
-         NVBIT(NVKMS_COMPOSITION_MODE_NON_PREMULT_SURFACE_ALPHA)) != 0x0) {
+         NVBIT(NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_SURFACE_ALPHA)) != 0x0) {
 
         drm_plane_create_alpha_property(plane);
         drm_plane_create_blend_mode_property(plane,
                                              NVBIT(DRM_MODE_BLEND_PREMULTI) |
                                              NVBIT(DRM_MODE_BLEND_COVERAGE));
     } else if ((validCompModes &
-                NVBIT(NVKMS_COMPOSITION_MODE_PREMULT_ALPHA)) != 0x0 &&
+                NVBIT(NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_ALPHA)) != 0x0 &&
                (validCompModes &
-                NVBIT(NVKMS_COMPOSITION_MODE_NON_PREMULT_ALPHA)) != 0x0) {
+                NVBIT(NVKMS_COMPOSITION_BLENDING_MODE_NON_PREMULT_ALPHA)) != 0x0) {
 
         drm_plane_create_blend_mode_property(plane,
                                              NVBIT(DRM_MODE_BLEND_PREMULTI) |
@@ -614,10 +891,9 @@ __nv_drm_plane_create_rotation_property(struct drm_plane *plane,
 static struct drm_plane*
 nv_drm_plane_create(struct drm_device *dev,
                     enum drm_plane_type plane_type,
+                    uint32_t layer_idx,
                     NvU32 head,
-                    long unsigned int nvkms_formats_mask,
-                    NvU32 validCompositionModes,
-                    NvU16 validLayerRRTransforms)
+                    const struct NvKmsKapiDeviceResourcesInfo *pResInfo)
 {
 #if defined(NV_DRM_UNIVERSAL_PLANE_INIT_HAS_FORMAT_MODIFIERS_ARG)
     struct nv_drm_device *nv_dev = to_nv_device(dev);
@@ -626,10 +902,34 @@ nv_drm_plane_create(struct drm_device *dev,
         DRM_FORMAT_MOD_INVALID,
     };
 #endif
+    enum NvKmsCompositionBlendingMode defaultCompositionMode;
+    struct nv_drm_plane *nv_plane = NULL;
+    struct nv_drm_plane_state *nv_plane_state = NULL;
     struct drm_plane *plane = NULL;
     int ret = -ENOMEM;
     uint32_t *formats = NULL;
     unsigned int formats_count = 0;
+    const NvU32 validCompositionModes =
+        (plane_type == DRM_PLANE_TYPE_CURSOR) ?
+            pResInfo->caps.validCursorCompositionModes :
+            pResInfo->caps.layer[layer_idx].validCompositionModes;
+    const long unsigned int nvkms_formats_mask =
+        (plane_type == DRM_PLANE_TYPE_CURSOR) ?
+            pResInfo->caps.supportedCursorSurfaceMemoryFormats :
+            pResInfo->supportedSurfaceMemoryFormats[layer_idx];
+    const NvU16 validLayerRRTransforms =
+        (plane_type == DRM_PLANE_TYPE_CURSOR) ?
+            0x0 : pResInfo->caps.layer[layer_idx].validRRTransforms;
+
+    if ((validCompositionModes &
+         NVBIT(NVKMS_COMPOSITION_BLENDING_MODE_OPAQUE)) != 0x0) {
+        defaultCompositionMode = NVKMS_COMPOSITION_BLENDING_MODE_OPAQUE;
+    } else if ((validCompositionModes &
+                NVBIT(NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_ALPHA)) != 0x0) {
+        defaultCompositionMode = NVKMS_COMPOSITION_BLENDING_MODE_PREMULT_ALPHA;
+    } else {
+        goto failed;
+    }
 
     formats =
         nv_drm_format_array_alloc(&formats_count,
@@ -638,15 +938,20 @@ nv_drm_plane_create(struct drm_device *dev,
         goto failed;
     }
 
-    if ((plane = nv_drm_calloc(1, sizeof(*plane))) == NULL) {
+    if ((nv_plane = nv_drm_calloc(1, sizeof(*nv_plane))) == NULL) {
         goto failed_plane_alloc;
     }
+    plane = &nv_plane->base;
 
-    plane->state = nv_drm_calloc(1, sizeof(*plane->state));
-    if (plane->state == NULL) {
+    nv_plane->defaultCompositionMode = defaultCompositionMode;
+    nv_plane->layer_idx = layer_idx;
+
+    if ((nv_plane_state =
+            nv_drm_calloc(1, sizeof(*nv_plane_state))) == NULL) {
         goto failed_state_alloc;
     }
 
+    plane->state = &nv_plane_state->base;
     plane->state->plane = plane;
 
     /*
@@ -677,6 +982,10 @@ nv_drm_plane_create(struct drm_device *dev,
 
     drm_plane_helper_add(plane, &nv_plane_helper_funcs);
 
+    if (plane_type != DRM_PLANE_TYPE_CURSOR) {
+        nv_drm_plane_install_properties(plane);
+    }
+
     __nv_drm_plane_create_alpha_blending_properties(
             plane,
             validCompositionModes);
@@ -688,10 +997,10 @@ nv_drm_plane_create(struct drm_device *dev,
     return plane;
 
 failed_plane_init:
-    nv_drm_free(plane->state);
+    nv_drm_free(nv_plane_state);
 
 failed_state_alloc:
-    nv_drm_free(plane);
+    nv_drm_free(nv_plane);
 
 failed_plane_alloc:
     nv_drm_free(formats);
@@ -766,58 +1075,56 @@ void nv_drm_enumerate_crtcs_and_planes(
 {
     unsigned int i;
 
-    NvU32 validCursorCompositionModes =
-        pResInfo->caps.validCursorCompositionModes;
-    NvU32 validOverlayCompositionModes =
-        pResInfo->caps.validOverlayCompositionModes;
-    long unsigned int nvkms_primary_formats_mask =
-        pResInfo->supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_PRIMARY];
-    long unsigned int nvkms_cursor_formats_mask =
-        pResInfo->supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_CURSOR];
-    long unsigned int nvkms_overlay_formats_mask =
-        pResInfo->supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_OVERLAY];
-    NvU16 validLayerRRTransforms = pResInfo->caps.validLayerRRTransforms;
-
     for (i = 0; i < pResInfo->numHeads; i++) {
         struct drm_plane *primary_plane = NULL, *cursor_plane = NULL;
-        struct drm_plane *overlay_plane = NULL;
+        NvU32 layer;
+
+        if (pResInfo->numLayers[i] <= NVKMS_KAPI_LAYER_PRIMARY_IDX) {
+            continue;
+        }
+
+        primary_plane =
+            nv_drm_plane_create(nv_dev->dev,
+                                DRM_PLANE_TYPE_PRIMARY,
+                                NVKMS_KAPI_LAYER_PRIMARY_IDX,
+                                i,
+                                pResInfo);
 
-        primary_plane = nv_drm_plane_create(nv_dev->dev,
-                                            DRM_PLANE_TYPE_PRIMARY,
-                                            i,
-                                            nvkms_primary_formats_mask,
-                                            0,
-                                            validLayerRRTransforms);
         if (IS_ERR(primary_plane)) {
             NV_DRM_DEV_LOG_ERR(
                 nv_dev,
                 "Failed to create primary plane for head %u, error = %ld",
                 i, PTR_ERR(primary_plane));
+            continue;
         }
 
-        /* nvkms doesn't support rrTransforms for cursor layer */
-        cursor_plane = nv_drm_plane_create(nv_dev->dev,
-                                           DRM_PLANE_TYPE_CURSOR,
-                                           i,
-                                           nvkms_cursor_formats_mask,
-                                           validCursorCompositionModes,
-                                           0 /* rrTransforms */);
+        cursor_plane =
+            nv_drm_plane_create(nv_dev->dev,
+                                DRM_PLANE_TYPE_CURSOR,
+                                NVKMS_KAPI_LAYER_INVALID_IDX,
+                                i,
+                                pResInfo);
         if (IS_ERR(cursor_plane)) {
             NV_DRM_DEV_LOG_ERR(
                 nv_dev,
                 "Failed to create cursor plane for head %u, error = %ld",
                 i, PTR_ERR(cursor_plane));
+            cursor_plane = NULL;
         }
 
-        if (primary_plane != NULL) {
+        /* Create crtc with the primary and cursor planes */
+        {
             struct drm_crtc *crtc =
                 __nv_drm_crtc_create(nv_dev,
                                      primary_plane, cursor_plane,
                                      i);
-
             if (IS_ERR(crtc)) {
                 nv_drm_plane_destroy(primary_plane);
-                nv_drm_plane_destroy(cursor_plane);
+
+                if (cursor_plane != NULL) {
+                    nv_drm_plane_destroy(cursor_plane);
+                }
+
                 NV_DRM_DEV_LOG_ERR(
                     nv_dev,
                     "Failed to add DRM CRTC for head %u, error = %ld",
@@ -826,18 +1133,26 @@ void nv_drm_enumerate_crtcs_and_planes(
             }
         }
 
-        overlay_plane = nv_drm_plane_create(nv_dev->dev,
-                                            DRM_PLANE_TYPE_OVERLAY,
-                                            i,
-                                            nvkms_overlay_formats_mask,
-                                            validOverlayCompositionModes,
-                                            validLayerRRTransforms);
-        if (IS_ERR(overlay_plane)) {
-            NV_DRM_DEV_LOG_ERR(
-                nv_dev,
-                "Failed to create overlay plane for head %u, error = %ld",
-                i, PTR_ERR(overlay_plane));
+        for (layer = 0; layer < pResInfo->numLayers[i]; layer++) {
+            if (layer == NVKMS_KAPI_LAYER_PRIMARY_IDX) {
+                continue;
+            }
+
+            struct drm_plane *overlay_plane =
+                nv_drm_plane_create(nv_dev->dev,
+                                    DRM_PLANE_TYPE_OVERLAY,
+                                    layer,
+                                    i,
+                                    pResInfo);
+
+            if (IS_ERR(overlay_plane)) {
+                NV_DRM_DEV_LOG_ERR(
+                    nv_dev,
+                    "Failed to create plane for layer-%u of head %u, error = %ld",
+                    layer, i, PTR_ERR(overlay_plane));
+            }
         }
+
     }
 }
 /*
