@@ -319,6 +319,35 @@ static void nv_drm_enumerate_encoders_and_connectors
 
 #endif /* NV_DRM_ATOMIC_MODESET_AVAILABLE */
 
+/*!
+ * 'NV_DRM_OUT_FENCE_PTR' is an atomic per-plane property that clients can use
+ * to request an out-fence fd for a particular plane that's being flipped.
+ * 'NV_DRM_OUT_FENCE_PTR' does NOT have the same behavior as the standard
+ * 'OUT_FENCE_PTR' property - the fd that's returned via 'NV_DRM_OUT_FENCE_PTR'
+ * will only be signaled once the buffers in the corresponding flip are flipped
+ * away from.
+ * In order to use this property, client needs to call set property function
+ * with user mode pointer as value. Once driver have post syncpt fd from flip reply,
+ * it will copy post syncpt fd at location pointed by user mode pointer.
+ */
+static int nv_drm_create_properties(struct nv_drm_device *nv_dev)
+{
+#if defined(NV_LINUX_NVHOST_H_PRESENT) && defined(CONFIG_TEGRA_GRHOST)
+    if (!nv_dev->supportsSyncpts) {
+        return 0;
+    }
+
+    nv_dev->nv_out_fence_property =
+        drm_property_create_range(nv_dev->dev, DRM_MODE_PROP_ATOMIC,
+            "NV_DRM_OUT_FENCE_PTR", 0, U64_MAX);
+    if (nv_dev->nv_out_fence_property == NULL) {
+        return -ENOMEM;
+    }
+#endif
+
+    return 0;
+}
+
 static int nv_drm_load(struct drm_device *dev, unsigned long flags)
 {
 #if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
@@ -332,6 +361,7 @@ static int nv_drm_load(struct drm_device *dev, unsigned long flags)
     NvU64 gen;
     int i;
 #endif
+    int ret;
 
     struct nv_drm_device *nv_dev = to_nv_device(dev);
 
@@ -382,8 +412,17 @@ static int nv_drm_load(struct drm_device *dev, unsigned long flags)
     nv_dev->hasVideoMemory = resInfo.caps.hasVideoMemory;
 
     nv_dev->genericPageKind = resInfo.caps.genericPageKind;
+
+    // Fermi-Volta use generation 0, Turing+ uses generation 2.
+    nv_dev->pageKindGeneration = (nv_dev->genericPageKind == 0x06) ? 2 : 0;
+
+    // Desktop GPUs and mobile GPUs Xavier and later use the same sector layout
+    nv_dev->sectorLayout = 1;
+
+    nv_dev->supportsSyncpts = resInfo.caps.supportsSyncpts;
+
 #if defined(NV_DRM_FORMAT_MODIFIERS_PRESENT)
-    gen = (nv_dev->genericPageKind == 0x06) ? 2 : 0;
+    gen = nv_dev->pageKindGeneration;
     kind = nv_dev->genericPageKind;
 
     for (i = 0; i <= 5; i++) {
@@ -405,6 +444,11 @@ static int nv_drm_load(struct drm_device *dev, unsigned long flags)
 
     nv_drm_init_mode_config(nv_dev, &resInfo);
 
+    ret = nv_drm_create_properties(nv_dev);
+    if (ret < 0) {
+        return -ENODEV;
+    }
+
     if (!nvKms->declareEventInterest(
             nv_dev->pDevice,
             ((1 << NVKMS_EVENT_TYPE_DPY_CHANGED) |
@@ -421,7 +465,9 @@ static int nv_drm_load(struct drm_device *dev, unsigned long flags)
 
     nv_drm_enumerate_encoders_and_connectors(nv_dev);
 
+#if !defined(NV_DRM_CRTC_STATE_HAS_NO_VBLANK)
     drm_vblank_init(dev, dev->mode_config.num_crtc);
+#endif
 
     /*
      * Trigger hot-plug processing, to update connection status of
@@ -527,7 +573,9 @@ static int nv_drm_master_set(struct drm_device *dev,
 static void nv_drm_master_set(struct drm_device *dev,
                               struct drm_file *file_priv, bool from_open)
 {
-     WARN_ON(__nv_drm_master_set(dev, file_priv, from_open) != 0);
+    if (__nv_drm_master_set(dev, file_priv, from_open) != 0) {
+        NV_DRM_DEV_LOG_ERR(to_nv_device(dev), "Failed to grab modeset ownership");
+    }
 }
 #endif
 
@@ -606,6 +654,15 @@ static int nv_drm_get_dev_info_ioctl(struct drm_device *dev,
 
     params->gpu_id = nv_dev->gpu_info.gpu_id;
     params->primary_index = dev->primary->index;
+#if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
+    params->generic_page_kind = nv_dev->genericPageKind;
+    params->page_kind_generation = nv_dev->pageKindGeneration;
+    params->sector_layout = nv_dev->sectorLayout;
+#else
+    params->generic_page_kind = 0;
+    params->page_kind_generation = 0;
+    params->sector_layout = 0;
+#endif
 
     return 0;
 }
@@ -735,6 +792,12 @@ static const struct drm_ioctl_desc nv_drm_ioctls[] = {
     DRM_IOCTL_DEF_DRV(NVIDIA_GEM_ALLOC_NVKMS_MEMORY,
                       nv_drm_gem_alloc_nvkms_memory_ioctl,
                       DRM_RENDER_ALLOW|DRM_UNLOCKED),
+    DRM_IOCTL_DEF_DRV(NVIDIA_GEM_EXPORT_DMABUF_MEMORY,
+                      nv_drm_gem_export_dmabuf_memory_ioctl,
+                      DRM_RENDER_ALLOW|DRM_UNLOCKED),
+    DRM_IOCTL_DEF_DRV(NVIDIA_GEM_IDENTIFY_OBJECT,
+                      nv_drm_gem_identify_object_ioctl,
+                      DRM_RENDER_ALLOW|DRM_UNLOCKED),
 #endif /* NV_DRM_ATOMIC_MODESET_AVAILABLE */
 };
 
@@ -865,9 +928,12 @@ static void nv_drm_register_drm_device(const nv_gpu_info_t *gpu_info)
 
     dev->dev_private = nv_dev;
     nv_dev->dev = dev;
+
+#if defined(NV_DRM_DEVICE_HAS_PDEV)
     if (device->bus == &pci_bus_type) {
         dev->pdev = to_pci_dev(device);
     }
+#endif
 
     /* Register DRM device to DRM sub-system */
 
