@@ -378,7 +378,7 @@ int nv_drm_gem_export_nvkms_memory_ioctl(struct drm_device *dev,
         ret = -EINVAL;
         NV_DRM_DEV_LOG_ERR(
             nv_dev,
-            "Failed to lookup gem object for export: 0x%08x",
+            "Failed to lookup NVKMS gem object for export: 0x%08x",
             p->handle);
         goto done;
     }
@@ -390,7 +390,7 @@ int nv_drm_gem_export_nvkms_memory_ioctl(struct drm_device *dev,
         ret = -EINVAL;
         NV_DRM_DEV_LOG_ERR(
             nv_dev,
-            "Failed to export memory from GEM object: 0x%08x", p->handle);
+            "Failed to export memory from NVKMS GEM object: 0x%08x", p->handle);
         goto done;
     }
 
