@@ -47,6 +47,8 @@
 #include "nv-retpoline.h"
 #endif
 
+#include <linux/backlight.h>
+
 #define NVKMS_LOG_PREFIX "nvidia-modeset: "
 
 /* These parameters are used for fault injection tests.  Normally the defaults
@@ -170,7 +172,6 @@ static atomic_t nvkms_alloc_called_count;
 
 
 
-
 
 
 #ifdef NVKMS_SYNCPT_STUBS_NEEDED
@@ -706,6 +707,12 @@ static void nvkms_kthread_q_callback(void *arg)
         del_timer_sync(&timer->kernel_timer);
     }
 
+    /*
+     * Block the kthread during system suspend & resume in order to defer
+     * handling of events such as DP_IRQ and hotplugs until after resume.
+     */
+    nvkms_read_lock_pm_lock();
+
     down(&nvkms_lock);
 
     if (timer->isRefPtr) {
@@ -732,6 +739,8 @@ static void nvkms_kthread_q_callback(void *arg)
     }
 
     up(&nvkms_lock);
+
+    nvkms_read_unlock_pm_lock();
 }
 
 static void nvkms_queue_work(nv_kthread_q_t *q, nv_kthread_q_item_t *q_item)
@@ -864,11 +873,6 @@ void nvkms_free_timer(nvkms_timer_handle_t *handle)
     timer->cancel = NV_TRUE;
 }
 
-void nvkms_get_random(void *ptr, size_t size)
-{
-    get_random_bytes(ptr, size);
-}
-
 void* nvkms_get_per_open_data(int fd)
 {
     struct file *filp = fget(fd);
@@ -992,6 +996,146 @@ NvBool nvkms_allow_write_combining(void)
     return __rm_ops.system_info.allow_write_combining;
 }
 
+/*************************************************************************
+ * Implementation of sysfs interface to control backlight
+ *************************************************************************/
+
+#if defined(NV_BACKLIGHT_DEVICE_REGISTER_PRESENT)
+
+struct nvkms_backlight_device {
+    NvU32 gpu_id;
+    NvU32 display_id;
+
+    void *drv_priv;
+
+    struct backlight_device * dev;
+};
+
+static int nvkms_update_backlight_status(struct backlight_device *bd)
+{
+    struct nvkms_backlight_device *nvkms_bd = bl_get_data(bd);
+    NvBool status;
+    int ret;
+
+    ret = down_interruptible(&nvkms_lock);
+
+    if (ret != 0) {
+        return ret;
+    }
+
+    status = nvKmsSetBacklight(nvkms_bd->display_id, nvkms_bd->drv_priv,
+                               bd->props.brightness);
+
+    up(&nvkms_lock);
+
+    return status ? 0 : -EINVAL;
+}
+
+static int nvkms_get_backlight_brightness(struct backlight_device *bd)
+{
+    struct nvkms_backlight_device *nvkms_bd = bl_get_data(bd);
+    NvU32 brightness = 0;
+    NvBool status;
+    int ret;
+
+    ret = down_interruptible(&nvkms_lock);
+
+    if (ret != 0) {
+        return ret;
+    }
+
+    status = nvKmsGetBacklight(nvkms_bd->display_id, nvkms_bd->drv_priv,
+                               &brightness);
+
+    up(&nvkms_lock);
+
+    return  status ? brightness : -1;
+}
+
+static const struct backlight_ops nvkms_backlight_ops = {
+    .update_status = nvkms_update_backlight_status,
+    .get_brightness = nvkms_get_backlight_brightness,
+};
+#endif
+
+
+struct nvkms_backlight_device*
+nvkms_register_backlight(NvU32 gpu_id, NvU32 display_id, void *drv_priv,
+                         NvU32 current_brightness)
+{
+#if defined(NV_BACKLIGHT_DEVICE_REGISTER_PRESENT)
+    char name[11];
+    struct backlight_properties props = {
+        .brightness = current_brightness,
+        .max_brightness = 100,
+# if defined(NV_BACKLIGHT_PROPERTIES_TYPE_PRESENT)
+        .type = BACKLIGHT_RAW,
+# endif
+    };
+    nv_gpu_info_t *gpu_info = NULL;
+    NvU32 gpu_count = 0;
+    struct nvkms_backlight_device *nvkms_bd = NULL;
+    int i;
+
+    gpu_info = nvkms_alloc(NV_MAX_GPUS * sizeof(*gpu_info), NV_TRUE);
+    if (gpu_info == NULL) {
+        return NULL;
+    }
+
+    gpu_count = __rm_ops.enumerate_gpus(gpu_info);
+    if (gpu_count == 0) {
+        goto done;
+    }
+
+    for (i = 0; i < gpu_count; i++) {
+        if (gpu_info[i].gpu_id == gpu_id) {
+            break;
+        }
+    }
+
+    if (i == gpu_count) {
+        goto done;
+    }
+
+    nvkms_bd = nvkms_alloc(sizeof(*nvkms_bd), NV_TRUE);
+    if (nvkms_bd == NULL) {
+        goto done;
+    }
+
+    snprintf(name, sizeof(name), "nvidia_%d", i);
+    name[sizeof(name) - 1] = '\0';
+
+    nvkms_bd->gpu_id = gpu_id;
+    nvkms_bd->display_id = display_id;
+    nvkms_bd->drv_priv = drv_priv;
+
+    nvkms_bd->dev =
+        backlight_device_register(name,
+                                  gpu_info[i].os_device_ptr,
+                                  nvkms_bd,
+                                  &nvkms_backlight_ops,
+                                  &props);
+
+done:
+    nvkms_free(gpu_info, NV_MAX_GPUS * sizeof(*gpu_info));
+
+    return nvkms_bd;
+#else
+    return NULL;
+#endif
+}
+
+void nvkms_unregister_backlight(struct nvkms_backlight_device *nvkms_bd)
+{
+#if defined(NV_BACKLIGHT_DEVICE_REGISTER_PRESENT)
+    if (nvkms_bd->dev) {
+        backlight_device_unregister(nvkms_bd->dev);
+    }
+
+    nvkms_free(nvkms_bd, sizeof(*nvkms_bd));
+#endif
+}
+
 /*************************************************************************
  * Common to both user-space and kapi NVKMS interfaces
  *************************************************************************/
@@ -1093,6 +1237,31 @@ void nvkms_close_common(struct nvkms_per_open *popen)
     nvkms_free(popen, sizeof(*popen));
 }
 
+static void nvkms_close_deferred(void *data)
+{
+    struct nvkms_per_open *popen = data;
+
+    nvkms_read_lock_pm_lock();
+
+    nvkms_close_common(popen);
+
+    nvkms_read_unlock_pm_lock();
+}
+
+static void nvkms_close_popen(struct nvkms_per_open *popen)
+{
+    if (nvkms_read_trylock_pm_lock() == 0) {
+        nvkms_close_common(popen);
+        nvkms_read_unlock_pm_lock();
+    } else {
+        nv_kthread_q_item_init(&popen->deferred_close_q_item,
+                               nvkms_close_deferred,
+                               popen);
+        nvkms_queue_work(&nvkms_deferred_close_kthread_q,
+                         &popen->deferred_close_q_item);
+    }
+}
+
 int nvkms_ioctl_common
 (
     struct nvkms_per_open *popen,
@@ -1128,12 +1297,18 @@ struct nvkms_per_open* nvkms_open_from_kapi
 )
 {
     int status = 0;
-    return nvkms_open_common(NVKMS_CLIENT_KERNEL_SPACE, device, &status);
+    struct nvkms_per_open *ret;
+
+    nvkms_read_lock_pm_lock();
+    ret = nvkms_open_common(NVKMS_CLIENT_KERNEL_SPACE, device, &status);
+    nvkms_read_unlock_pm_lock();
+
+    return ret;
 }
 
 void nvkms_close_from_kapi(struct nvkms_per_open *popen)
 {
-    nvkms_close_common(popen);
+    nvkms_close_popen(popen);
 }
 
 NvBool nvkms_ioctl_from_kapi
@@ -1142,9 +1317,15 @@ NvBool nvkms_ioctl_from_kapi
     NvU32 cmd, void *params_address, const size_t param_size
 )
 {
-    return nvkms_ioctl_common(popen,
-                              cmd,
-                              (NvU64)(NvUPtr)params_address, param_size) == 0;
+    NvBool ret;
+
+    nvkms_read_lock_pm_lock();
+    ret = nvkms_ioctl_common(popen,
+                             cmd,
+                             (NvU64)(NvUPtr)params_address, param_size) == 0;
+    nvkms_read_unlock_pm_lock();
+
+    return ret;
 }
 
 /*************************************************************************
@@ -1353,38 +1534,15 @@ static int nvkms_open(struct inode *inode, struct file *filp)
     return status;
 }
 
-static void nvkms_close_deferred(void *data)
-{
-    struct nvkms_per_open *popen = data;
-
-    nvkms_read_lock_pm_lock();
-
-    nvkms_close_common(popen);
-
-    nvkms_read_unlock_pm_lock();
-}
-
 static int nvkms_close(struct inode *inode, struct file *filp)
 {
-    int status;
     struct nvkms_per_open *popen = filp->private_data;
 
     if (popen == NULL) {
         return -EINVAL;
     }
 
-    status = nvkms_read_trylock_pm_lock();
-    if (status == 0) {
-        nvkms_close_common(popen);
-        nvkms_read_unlock_pm_lock();
-    } else {
-        nv_kthread_q_item_init(&popen->deferred_close_q_item,
-                               nvkms_close_deferred,
-                               popen);
-        nvkms_queue_work(&nvkms_deferred_close_kthread_q,
-                         &popen->deferred_close_q_item);
-    }
-
+    nvkms_close_popen(popen);
     return 0;
 }
 
@@ -1565,7 +1723,7 @@ restart:
             if (del_timer_sync(&timer->kernel_timer) == 1) {
                 /*  We've deactivated timer so we need to clean after it */
                 list_del(&timer->timers_list);
-                
+
                 /* We need to unlock spinlock because we are freeing memory which
                  * may sleep */
                 spin_unlock_irqrestore(&nvkms_timers.lock, flags);
@@ -1601,7 +1759,11 @@ module_init(nvkms_init);
 module_exit(nvkms_exit);
 
 #if defined(MODULE_LICENSE)
+
+
+
   MODULE_LICENSE("NVIDIA");
+
 #endif
 #if defined(MODULE_INFO)
   MODULE_INFO(supported, "external");
