@@ -16,12 +16,15 @@
 #if !defined(_NVIDIA_MODESET_OS_INTERFACE_H_)
 #define _NVIDIA_MODESET_OS_INTERFACE_H_
 
-#include <stddef.h>  /* size_t */
-#include <stdarg.h>  /* va_list */
-
+#if defined(NV_KERNEL_INTERFACE_LAYER) && defined(NV_LINUX)
+#include <linux/stddef.h>  /* size_t */
+#else
+#include <stddef.h>        /* size_t */
+#endif
 #include "nvtypes.h" /* NvU8 */
 
 #include "nvkms.h"
+#include "nv_stdarg.h"
 
 enum NvKmsSyncPtOp {
     NVKMS_SYNCPT_OP_ALLOC,
@@ -269,11 +272,6 @@ void
 nvkms_event_queue_changed(nvkms_per_open_handle_t *pOpenKernel,
                           NvBool eventsAvailable);
 
-/*!
- * Get some random data.
- */
-void nvkms_get_random(void *ptr, size_t size);
-
 
 /*!
  * Get the "per-open" data (the pointer returned by nvKmsOpen())
@@ -337,5 +335,16 @@ void nvkms_sema_free     (nvkms_sema_handle_t *sema);
 void nvkms_sema_down     (nvkms_sema_handle_t *sema);
 void nvkms_sema_up       (nvkms_sema_handle_t *sema);
 
+/*!
+ * APIs to register/unregister backlight device.
+ */
+struct nvkms_backlight_device;
+
+struct nvkms_backlight_device*
+nvkms_register_backlight(NvU32 gpu_id, NvU32 display_id, void *drv_priv,
+                         NvU32 current_brightness);
+
+void nvkms_unregister_backlight(struct nvkms_backlight_device *nvkms_bd);
+
 #endif /* _NVIDIA_MODESET_OS_INTERFACE_H_ */
 
