@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -46,7 +46,7 @@ static LIST_HEAD(g_cpu_service_block_context_list);
 
 static uvm_spinlock_t g_cpu_service_block_context_list_lock;
 
-static int alloc_cpu_service_block_context_list(void)
+NV_STATUS uvm_service_block_context_init(void)
 {
     unsigned num_preallocated_contexts = 4;
 
@@ -56,15 +56,15 @@ static int alloc_cpu_service_block_context_list(void)
     while (num_preallocated_contexts-- > 0) {
         uvm_service_block_context_t *service_context = uvm_kvmalloc(sizeof(*service_context));
         if (!service_context)
-            return -ENOMEM;
+            return NV_ERR_NO_MEMORY;
 
         list_add(&service_context->cpu_fault.service_context_list, &g_cpu_service_block_context_list);
     }
 
-    return 0;
+    return NV_OK;
 }
 
-static void free_cpu_service_block_context_list(void)
+void uvm_service_block_context_exit(void)
 {
     uvm_service_block_context_t *service_context, *service_context_tmp;
 
@@ -78,7 +78,7 @@ static void free_cpu_service_block_context_list(void)
 
 // Get a fault service context from the global list or allocate a new one if there are no
 // available entries
-static uvm_service_block_context_t *get_cpu_fault_service_context(void)
+static uvm_service_block_context_t *uvm_service_block_context_cpu_alloc(void)
 {
     uvm_service_block_context_t *service_context;
 
@@ -99,7 +99,7 @@ static uvm_service_block_context_t *get_cpu_fault_service_context(void)
 }
 
 // Put a fault service context in the global list
-static void put_cpu_fault_service_context(uvm_service_block_context_t *service_context)
+static void uvm_service_block_context_cpu_free(uvm_service_block_context_t *service_context)
 {
     uvm_spin_lock(&g_cpu_service_block_context_list_lock);
 
@@ -514,7 +514,7 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
         goto convert_error;
     }
 
-    service_context = get_cpu_fault_service_context();
+    service_context = uvm_service_block_context_cpu_alloc();
     if (!service_context) {
         status = NV_ERR_NO_MEMORY;
         goto unlock;
@@ -556,7 +556,7 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
         if (do_sleep)
             uvm_tools_record_throttling_end(va_space, fault_addr, UVM_ID_CPU);
 
-        status = uvm_va_block_find_create(va_space, fault_addr, &va_block);
+        status = uvm_va_block_find_create(va_space, NULL, fault_addr, &va_block);
         if (status != NV_OK) {
             UVM_ASSERT_MSG(status == NV_ERR_NO_MEMORY, "status: %s\n", nvstatusToString(status));
             break;
@@ -602,7 +602,7 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
     // If any pages were DMA'ed between the GPU and host memory, that makes it a major fault.
     // A process can also get statistics for major and minor faults by calling readproc().
     major_fault = service_context->cpu_fault.did_migrate;
-    put_cpu_fault_service_context(service_context);
+    uvm_service_block_context_cpu_free(service_context);
 
 unlock:
     // TODO: Bug 2583279: See the comment above the matching lock acquisition
@@ -824,7 +824,7 @@ static int uvm_mmap(struct file *filp, struct vm_area_struct *vma)
     }
     vma_wrapper_allocated = true;
 
-    // The kernel has taken mmap_sem in write mode, but that doesn't prevent
+    // The kernel has taken mmap_lock in write mode, but that doesn't prevent
     // this va_space from being modified by the GPU fault path or from the ioctl
     // path where we don't have this mm for sure, so we have to lock the VA
     // space directly.
@@ -989,11 +989,11 @@ NV_STATUS uvm_test_register_unload_state_buffer(UVM_TEST_REGISTER_UNLOAD_STATE_B
     struct page *page;
     NV_STATUS status = NV_OK;
 
-    if (!IS_ALIGNED(params->unload_state_buf, sizeof(NvU64))) 
+    if (!IS_ALIGNED(params->unload_state_buf, sizeof(NvU64)))
         return NV_ERR_INVALID_ADDRESS;
 
-    // Hold mmap_sem to call get_user_pages(), the UVM locking helper functions
-    // are not used because unload_state_buf may be a managed memory pointer and 
+    // Hold mmap_lock to call get_user_pages(), the UVM locking helper functions
+    // are not used because unload_state_buf may be a managed memory pointer and
     // therefore a locking assertion from the CPU fault handler could be fired.
     nv_mmap_read_lock(current->mm);
     ret = NV_GET_USER_PAGES(params->unload_state_buf, 1, write, force, &page, NULL);
@@ -1029,13 +1029,8 @@ static void uvm_test_unload_state_exit(void)
     }
 }
 
-static int uvm_init(void)
+static int uvm_chardev_create(void)
 {
-    bool allocated_dev = false;
-    bool initialized_globals = false;
-    bool added_device = false;
-    bool initialized_tools = false;
-    NV_STATUS status;
     dev_t uvm_dev;
 
     int ret = alloc_chrdev_region(&g_uvm_base_dev,
@@ -1044,12 +1039,34 @@ static int uvm_init(void)
                                   NVIDIA_UVM_DEVICE_NAME);
     if (ret != 0) {
         UVM_ERR_PRINT("alloc_chrdev_region failed: %d\n", ret);
-        goto error;
+        return ret;
     }
     uvm_dev = MKDEV(MAJOR(g_uvm_base_dev), NVIDIA_UVM_PRIMARY_MINOR_NUMBER);
-    allocated_dev = true;
 
-    status = uvm_global_init();
+    uvm_init_character_device(&g_uvm_cdev, &uvm_fops);
+    ret = cdev_add(&g_uvm_cdev, uvm_dev, 1);
+    if (ret != 0) {
+        UVM_ERR_PRINT("cdev_add (major %u, minor %u) failed: %d\n", MAJOR(uvm_dev), MINOR(uvm_dev), ret);
+        unregister_chrdev_region(g_uvm_base_dev, NVIDIA_UVM_NUM_MINOR_DEVICES);
+        return ret;
+    }
+
+    return 0;
+}
+
+static void uvm_chardev_exit(void)
+{
+    cdev_del(&g_uvm_cdev);
+    unregister_chrdev_region(g_uvm_base_dev, NVIDIA_UVM_NUM_MINOR_DEVICES);
+}
+
+static int uvm_init(void)
+{
+    bool initialized_globals = false;
+    bool added_device = false;
+    int ret;
+
+    NV_STATUS status = uvm_global_init();
     if (status != NV_OK) {
         UVM_ERR_PRINT("uvm_global_init() failed: %s\n", nvstatusToString(status));
         ret = -ENODEV;
@@ -1057,10 +1074,9 @@ static int uvm_init(void)
     }
     initialized_globals = true;
 
-    uvm_init_character_device(&g_uvm_cdev, &uvm_fops);
-    ret = cdev_add(&g_uvm_cdev, uvm_dev, 1);
+    ret = uvm_chardev_create();
     if (ret != 0) {
-        UVM_ERR_PRINT("cdev_add (major %u, minor %u) failed: %d\n", MAJOR(uvm_dev), MINOR(uvm_dev), ret);
+        UVM_ERR_PRINT("uvm_chardev_create failed: %d\n", ret);
         goto error;
     }
     added_device = true;
@@ -1070,38 +1086,32 @@ static int uvm_init(void)
         UVM_ERR_PRINT("uvm_tools_init() failed: %d\n", ret);
         goto error;
     }
-    initialized_tools = true;
-
-    ret = alloc_cpu_service_block_context_list();
-    if (ret != 0) {
-        UVM_ERR_PRINT("alloc_cpu_service_block_context_list failed: %d\n", ret);
-        goto error;
-    }
-
-    uvm_hmm_init();
 
     pr_info("Loaded the UVM driver, major device number %d.\n", MAJOR(g_uvm_base_dev));
 
     if (uvm_enable_builtin_tests)
         pr_info("Built-in UVM tests are enabled. This is a security risk.\n");
 
+
+
+
+
+
+
+
+
+
+
+
     return 0;
 
 error:
-    free_cpu_service_block_context_list();
-
-    if (initialized_tools)
-        uvm_tools_exit();
-
     if (added_device)
-        cdev_del(&g_uvm_cdev);
+        uvm_chardev_exit();
 
     if (initialized_globals)
         uvm_global_exit();
 
-    if (allocated_dev)
-        unregister_chrdev_region(g_uvm_base_dev, NVIDIA_UVM_NUM_MINOR_DEVICES);
-
     UVM_ERR_PRINT("uvm init failed: %d\n", ret);
 
     return ret;
@@ -1114,14 +1124,11 @@ static int __init uvm_init_entry(void)
 
 static void uvm_exit(void)
 {
-    free_cpu_service_block_context_list();
     uvm_tools_exit();
-    cdev_del(&g_uvm_cdev);
+    uvm_chardev_exit();
 
     uvm_global_exit();
 
-    unregister_chrdev_region(g_uvm_base_dev, NVIDIA_UVM_NUM_MINOR_DEVICES);
-
     uvm_test_unload_state_exit();
 
     pr_info("Unloaded the UVM driver.\n");
