@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2018-2020 NVIDIA Corporation
+    Copyright (c) 2018-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -29,8 +29,6 @@
 
 void uvm_hal_ampere_arch_init_properties(uvm_parent_gpu_t *parent_gpu)
 {
-    parent_gpu->big_page.swizzling = false;
-
     parent_gpu->tlb_batch.va_invalidate_supported = true;
 
     parent_gpu->tlb_batch.va_range_invalidate_supported = true;
@@ -60,6 +58,8 @@ void uvm_hal_ampere_arch_init_properties(uvm_parent_gpu_t *parent_gpu)
     // access channel buffers.
     parent_gpu->max_channel_va = 1ULL << 40;
 
+    parent_gpu->max_host_va = 1ULL << 40;
+
     // Ampere can map sysmem with any page size
     parent_gpu->can_map_sysmem_with_large_pages = true;
 
@@ -79,8 +79,6 @@ void uvm_hal_ampere_arch_init_properties(uvm_parent_gpu_t *parent_gpu)
 
     parent_gpu->scoped_atomics_supported = true;
 
-    parent_gpu->has_pulse_based_interrupts = true;
-
     parent_gpu->has_clear_faulted_channel_sw_method = true;
 
     parent_gpu->has_clear_faulted_channel_method = false;
@@ -91,9 +89,6 @@ void uvm_hal_ampere_arch_init_properties(uvm_parent_gpu_t *parent_gpu)
 
     UVM_ASSERT(parent_gpu->rm_info.gpuArch == NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GA100);
     if (parent_gpu->rm_info.gpuImplementation == NV2080_CTRL_MC_ARCH_INFO_IMPLEMENTATION_GA100 ||
-
-
-
         parent_gpu->rm_info.gpuImplementation == NV2080_CTRL_MC_ARCH_INFO_IMPLEMENTATION_GA000)
         parent_gpu->map_remap_larger_page_promotion = true;
     else
