@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2019 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -33,7 +33,7 @@
 #include "uvm_kvmalloc.h"
 
 #include "nv_uvm_interface.h"
-#include "cla06f.h"
+#include "clb06f.h"
 
 #define UVM_CHANNEL_NUM_GPFIFO_ENTRIES_DEFAULT 1024
 #define UVM_CHANNEL_NUM_GPFIFO_ENTRIES_MIN 32
@@ -136,14 +136,19 @@ NvU32 uvm_channel_update_progress_all(uvm_channel_t *channel)
 NvU32 uvm_channel_manager_update_progress(uvm_channel_manager_t *channel_manager)
 {
     NvU32 pending_gpfifos = 0;
-    uvm_channel_t *channel;
-    uvm_for_each_channel(channel, channel_manager)
-        pending_gpfifos += uvm_channel_update_progress(channel);
+    uvm_channel_pool_t *pool;
+
+    uvm_for_each_pool(pool, channel_manager) {
+        uvm_channel_t *channel;
+
+        uvm_for_each_channel_in_pool(channel, pool)
+            pending_gpfifos += uvm_channel_update_progress(channel);
+    }
 
     return pending_gpfifos;
 }
 
-static bool is_channel_available(uvm_channel_t *channel)
+static bool channel_is_available(uvm_channel_t *channel)
 {
     NvU32 next_put;
 
@@ -160,7 +165,7 @@ static bool try_claim_channel(uvm_channel_t *channel)
 
     uvm_spin_lock(&channel->pool->lock);
 
-    if (is_channel_available(channel)) {
+    if (channel_is_available(channel)) {
         ++channel->current_pushes_count;
         claimed = true;
     }
@@ -170,16 +175,13 @@ static bool try_claim_channel(uvm_channel_t *channel)
     return claimed;
 }
 
-// Reserve a channel in the given manager index range
-static NV_STATUS channel_reserve_in_range(uvm_channel_manager_t *manager,
-                                          unsigned start,
-                                          unsigned end,
-                                          uvm_channel_t **channel_out)
+// Reserve a channel in the specified pool
+static NV_STATUS channel_reserve_in_pool(uvm_channel_pool_t *pool, uvm_channel_t **channel_out)
 {
     uvm_channel_t *channel;
     uvm_spin_loop_t spin;
 
-    uvm_for_each_channel_in_range(channel, manager, start, end) {
+    uvm_for_each_channel_in_pool(channel, pool) {
         // TODO: Bug 1764953: Prefer idle/less busy channels
         if (try_claim_channel(channel)) {
             *channel_out = channel;
@@ -189,7 +191,7 @@ static NV_STATUS channel_reserve_in_range(uvm_channel_manager_t *manager,
 
     uvm_spin_loop_init(&spin);
     while (1) {
-        uvm_for_each_channel_in_range(channel, manager, start, end) {
+        uvm_for_each_channel_in_pool(channel, pool) {
             NV_STATUS status;
 
             uvm_channel_update_progress(channel);
@@ -211,51 +213,23 @@ static NV_STATUS channel_reserve_in_range(uvm_channel_manager_t *manager,
     return NV_ERR_GENERIC;
 }
 
-static bool channel_manager_uses_ce(uvm_channel_manager_t *manager, NvU32 ce_index)
+NV_STATUS uvm_channel_reserve_type(uvm_channel_manager_t *manager, uvm_channel_type_t type, uvm_channel_t **channel_out)
 {
-    return manager->channel_pools[ce_index].manager != NULL;
+    return channel_reserve_in_pool(manager->pool_to_use.default_for_type[type], channel_out);
 }
 
-static NV_STATUS channel_reserve_ce(uvm_channel_manager_t *channel_manager,
-                                    NvU32 ce_index,
-                                    uvm_channel_t **channel_out)
-{
-    unsigned start, end;
-
-    UVM_ASSERT(ce_index < UVM_COPY_ENGINE_COUNT_MAX);
-    UVM_ASSERT(channel_manager_uses_ce(channel_manager, ce_index));
-
-    start = channel_manager->channel_pools[ce_index].channel_index;
-    end = start + UVM_CHANNELS_PER_COPY_ENGINE;
-
-    return channel_reserve_in_range(channel_manager, start, end, channel_out);
-}
-
-NV_STATUS uvm_channel_reserve_type(uvm_channel_manager_t *channel_manager,
-                                   uvm_channel_type_t type,
-                                   uvm_channel_t **channel_out)
-{
-    unsigned ce_index;
-
-    if (type == UVM_CHANNEL_TYPE_ANY)
-        return channel_reserve_in_range(channel_manager, 0, channel_manager->num_channels, channel_out);
-
-    ce_index = channel_manager->ce_to_use.default_for_type[type];
-    return channel_reserve_ce(channel_manager, ce_index, channel_out);
-}
-
-NV_STATUS uvm_channel_reserve_gpu_to_gpu(uvm_channel_manager_t *channel_manager,
+NV_STATUS uvm_channel_reserve_gpu_to_gpu(uvm_channel_manager_t *manager,
                                          uvm_gpu_t *dst_gpu,
                                          uvm_channel_t **channel_out)
 {
     const NvU32 dst_gpu_index = uvm_id_gpu_index(dst_gpu->id);
-    NvU32 ce_index = channel_manager->ce_to_use.gpu_to_gpu[dst_gpu_index];
+    uvm_channel_pool_t *pool = manager->pool_to_use.gpu_to_gpu[dst_gpu_index];
 
-    // No recommended CE for the given pair, use default
-    if (ce_index == UVM_COPY_ENGINE_COUNT_MAX)
-        ce_index = channel_manager->ce_to_use.default_for_type[UVM_CHANNEL_TYPE_GPU_TO_GPU];
+    // If there is no recommended pool for the given GPU pair, use default
+    if (pool == NULL)
+        pool = manager->pool_to_use.default_for_type[UVM_CHANNEL_TYPE_GPU_TO_GPU];
 
-    return channel_reserve_ce(channel_manager, ce_index, channel_out);
+    return channel_reserve_in_pool(pool, channel_out);
 }
 
 NV_STATUS uvm_channel_manager_wait(uvm_channel_manager_t *manager)
@@ -318,8 +292,8 @@ void uvm_channel_end_push(uvm_push_t *push)
     uvm_channel_manager_t *channel_manager = channel->pool->manager;
     uvm_gpu_t *gpu = channel_manager->gpu;
     uvm_pushbuffer_t *pushbuffer = channel_manager->pushbuffer;
-    uvm_gpu_semaphore_t *semaphore = &channel->tracking_sem.semaphore;
     uvm_gpfifo_entry_t *entry;
+    NvU64 semaphore_va;
     NvU64 new_tracking_value;
     NvU32 new_payload;
     NvU32 push_size;
@@ -328,14 +302,15 @@ void uvm_channel_end_push(uvm_push_t *push)
     NvU32 new_cpu_put;
     NvU64 *gpfifo_entry;
 
-    BUILD_BUG_ON(sizeof(*gpfifo_entry) != NVA06F_GP_ENTRY__SIZE);
+    BUILD_BUG_ON(sizeof(*gpfifo_entry) != NVB06F_GP_ENTRY__SIZE);
 
     uvm_spin_lock(&channel->pool->lock);
 
     new_tracking_value = ++channel->tracking_sem.queued_value;
     new_payload = (NvU32)new_tracking_value;
 
-    gpu->parent->ce_hal->semaphore_release(push, semaphore, new_payload);
+    semaphore_va = uvm_channel_tracking_semaphore_get_gpu_va(channel);
+    gpu->parent->ce_hal->semaphore_release(push, semaphore_va, new_payload);
 
     push_size = uvm_push_get_size(push);
     UVM_ASSERT_MSG(push_size <= UVM_MAX_PUSH_SIZE, "push size %u\n", push_size);
@@ -475,26 +450,25 @@ NV_STATUS uvm_channel_check_errors(uvm_channel_t *channel)
 
 NV_STATUS uvm_channel_manager_check_errors(uvm_channel_manager_t *channel_manager)
 {
+    uvm_channel_pool_t *pool;
     NV_STATUS status = uvm_global_get_status();
-    uvm_channel_t *channel;
 
     if (status != NV_OK)
         return status;
 
-    uvm_for_each_channel(channel, channel_manager) {
-        status = uvm_channel_check_errors(channel);
-        if (status != NV_OK)
-            return status;
+    uvm_for_each_pool(pool, channel_manager) {
+        uvm_channel_t *channel;
+
+        uvm_for_each_channel_in_pool(channel, pool) {
+            status = uvm_channel_check_errors(channel);
+            if (status != NV_OK)
+                return status;
+        }
     }
 
     return status;
 }
 
-uvm_gpu_semaphore_t *uvm_channel_get_tracking_semaphore(uvm_channel_t *channel)
-{
-    return &channel->tracking_sem.semaphore;
-}
-
 bool uvm_channel_is_value_completed(uvm_channel_t *channel, NvU64 value)
 {
     return uvm_gpu_tracking_semaphore_is_value_completed(&channel->tracking_sem, value);
@@ -505,26 +479,55 @@ NvU64 uvm_channel_update_completed_value(uvm_channel_t *channel)
     return uvm_gpu_tracking_semaphore_update_completed_value(&channel->tracking_sem);
 }
 
-static void destroy_channel(uvm_channel_manager_t *manager, uvm_channel_t *channel);
-
-static unsigned channel_pool_get_ce_index(const uvm_channel_pool_t *pool)
+static void channel_destroy(uvm_channel_pool_t *pool, uvm_channel_t *channel)
 {
-    return pool - pool->manager->channel_pools;
+    UVM_ASSERT(pool->num_channels > 0);
+
+    if (channel->tracking_sem.queued_value > 0) {
+        // The channel should have been idled before being destroyed, unless an
+        // error was triggered. We need to check both error cases (global and
+        // channel) to handle the UVM_TEST_CHANNEL_SANITY unit test.
+        if (uvm_global_get_status() == NV_OK && uvm_channel_get_status(channel) == NV_OK)
+            UVM_ASSERT(uvm_gpu_tracking_semaphore_is_completed(&channel->tracking_sem));
+
+        // Remove all remaining GPFIFOs from their pushbuffer chunk, since the
+        // pushbuffer has a longer lifetime.
+        channel_update_progress_all(channel, UVM_CHANNEL_UPDATE_MODE_FORCE_ALL);
+    }
+
+    uvm_procfs_destroy_entry(channel->procfs.pushes);
+    uvm_procfs_destroy_entry(channel->procfs.info);
+    uvm_procfs_destroy_entry(channel->procfs.dir);
+
+    uvm_kvfree(channel->push_acquire_infos);
+    uvm_kvfree(channel->push_infos);
+
+    uvm_kvfree(channel->gpfifo_entries);
+
+    if (channel->handle)
+        uvm_rm_locked_call_void(nvUvmInterfaceChannelDestroy(channel->handle));
+
+    uvm_gpu_tracking_semaphore_free(&channel->tracking_sem);
+
+    UVM_ASSERT(list_empty(&channel->tools.channel_list_node));
+    UVM_ASSERT(channel->tools.pending_event_count == 0);
+
+    pool->num_channels--;
 }
 
-static NV_STATUS create_channel(uvm_channel_pool_t *pool, bool with_procfs, uvm_channel_t *channel)
+static NV_STATUS channel_create(uvm_channel_pool_t *pool, bool with_procfs, uvm_channel_t *channel)
 {
     NV_STATUS status;
     uvm_channel_manager_t *manager = pool->manager;
     uvm_gpu_t *gpu = manager->gpu;
     UvmGpuChannelAllocParams channel_alloc_params;
     unsigned int i;
-    const unsigned ce_index = channel_pool_get_ce_index(pool);
+    uvmGpuAddressSpaceHandle rm_address_space;
 
     UVM_ASSERT(channel != NULL);
 
     channel->pool = pool;
-    manager->num_channels++;
+    pool->num_channels++;
     INIT_LIST_HEAD(&channel->available_push_infos);
     channel->tools.pending_event_count = 0;
     INIT_LIST_HEAD(&channel->tools.channel_list_node);
@@ -541,9 +544,14 @@ static NV_STATUS create_channel(uvm_channel_pool_t *pool, bool with_procfs, uvm_
     channel_alloc_params.numGpFifoEntries = manager->conf.num_gpfifo_entries;
     channel_alloc_params.gpFifoLoc        = manager->conf.gpfifo_loc;
     channel_alloc_params.gpPutLoc         = manager->conf.gpput_loc;
-    channel_alloc_params.engineIndex      = ce_index;
+    channel_alloc_params.engineIndex      = pool->ce_index;
 
-    status = uvm_rm_locked_call(nvUvmInterfaceChannelAllocate(gpu->rm_address_space,
+    if (uvm_channel_is_proxy(channel))
+        rm_address_space = gpu->rm_proxy_address_space;
+    else
+        rm_address_space = gpu->rm_address_space;
+
+    status = uvm_rm_locked_call(nvUvmInterfaceChannelAllocate(rm_address_space,
                                                               &channel_alloc_params,
                                                               &channel->handle,
                                                               &channel->channel_info));
@@ -556,9 +564,14 @@ static NV_STATUS create_channel(uvm_channel_pool_t *pool, bool with_procfs, uvm_
 
     channel->num_gpfifo_entries = manager->conf.num_gpfifo_entries;
 
-    snprintf(channel->name, sizeof(channel->name), "ID %u (0x%x) CE %u",
-            channel->channel_info.hwChannelId, channel->channel_info.hwChannelId,
-            ce_index);
+    snprintf(channel->name,
+             sizeof(channel->name),
+             "ID %u:%u (0x%x:0x%x) CE %u",
+             channel->channel_info.hwRunlistId,
+             channel->channel_info.hwChannelId,
+             channel->channel_info.hwRunlistId,
+             channel->channel_info.hwChannelId,
+             pool->ce_index);
 
     channel->gpfifo_entries = uvm_kvmalloc_zero(sizeof(*channel->gpfifo_entries) * channel->num_gpfifo_entries);
     if (channel->gpfifo_entries == NULL) {
@@ -592,46 +605,18 @@ static NV_STATUS create_channel(uvm_channel_pool_t *pool, bool with_procfs, uvm_
     return status;
 
 error:
-    destroy_channel(manager, channel);
+    channel_destroy(pool, channel);
 
     return status;
 }
 
-static void destroy_channel(uvm_channel_manager_t *manager, uvm_channel_t *channel)
+NvU64 uvm_channel_tracking_semaphore_get_gpu_va_in_channel(uvm_channel_t *semaphore_channel,
+                                                           uvm_channel_t *access_channel)
 {
-    UVM_ASSERT(manager->num_channels > 0);
-    UVM_ASSERT(uvm_channel_get_index(manager, channel) == manager->num_channels - 1);
+    uvm_gpu_semaphore_t *semaphore = &semaphore_channel->tracking_sem.semaphore;
+    uvm_gpu_t *gpu = uvm_channel_get_gpu(access_channel);
 
-    if (channel->tracking_sem.queued_value > 0) {
-        // The channel should have been idled before being destroyed, unless an
-        // error was triggered. We need to check both error cases (global and
-        // channel) to handle the UVM_TEST_CHANNEL_SANITY unit test.
-        if (uvm_global_get_status() == NV_OK && uvm_channel_get_status(channel) == NV_OK)
-            UVM_ASSERT(uvm_gpu_tracking_semaphore_is_completed(&channel->tracking_sem));
-
-        // Remove all remaining GPFIFOs from their pushbuffer chunk, since the
-        // pushbuffer has a longer lifetime.
-        channel_update_progress_all(channel, UVM_CHANNEL_UPDATE_MODE_FORCE_ALL);
-    }
-
-    uvm_procfs_destroy_entry(channel->procfs.pushes);
-    uvm_procfs_destroy_entry(channel->procfs.info);
-    uvm_procfs_destroy_entry(channel->procfs.dir);
-
-    uvm_kvfree(channel->push_acquire_infos);
-    uvm_kvfree(channel->push_infos);
-
-    uvm_kvfree(channel->gpfifo_entries);
-
-    if (channel->handle)
-        uvm_rm_locked_call_void(nvUvmInterfaceChannelDestroy(channel->handle));
-
-    uvm_gpu_tracking_semaphore_free(&channel->tracking_sem);
-
-    UVM_ASSERT(list_empty(&channel->tools.channel_list_node));
-    UVM_ASSERT(channel->tools.pending_event_count == 0);
-
-    manager->num_channels--;
+    return uvm_gpu_semaphore_get_gpu_va(semaphore, gpu, uvm_channel_is_proxy(access_channel));
 }
 
 static NV_STATUS init_channel(uvm_channel_t *channel)
@@ -654,53 +639,77 @@ static NV_STATUS init_channel(uvm_channel_t *channel)
     return status;
 }
 
-static NV_STATUS create_channel_pool(uvm_channel_manager_t *channel_manager, unsigned ce_index, bool with_procfs)
+static bool channel_manager_uses_proxy_pool(uvm_channel_manager_t *manager)
 {
+    return uvm_gpu_is_virt_mode_sriov_heavy(manager->gpu);
+}
+
+// Number of channels to create in the given pool.
+//
+// TODO: Bug 1764958: Tweak this function after benchmarking real workloads.
+static unsigned channel_pool_num_channels(uvm_channel_pool_t *pool)
+{
+    // The vGPU plugin implementation supports a single proxy channel
+    if (pool->is_proxy)
+        return 1;
+
+    return 2;
+}
+
+static void channel_pool_destroy(uvm_channel_pool_t *pool)
+{
+    UVM_ASSERT(pool->manager->num_channel_pools > 0);
+
+    while (pool->num_channels > 0)
+        channel_destroy(pool, pool->channels + pool->num_channels - 1);
+
+    uvm_kvfree(pool->channels);
+    pool->manager->num_channel_pools--;
+}
+
+static NV_STATUS channel_pool_create(uvm_channel_manager_t *channel_manager,
+                                     unsigned ce_index,
+                                     bool is_proxy,
+                                     bool with_procfs,
+                                     uvm_channel_pool_t *pool)
+{
+    NV_STATUS status;
     unsigned i;
-    unsigned start, end;
-    uvm_channel_pool_t *pool = channel_manager->channel_pools + ce_index;
-    uvm_channel_t *channel;
+    unsigned num_channels;
+
+    UVM_ASSERT(pool != NULL);
 
     pool->manager = channel_manager;
-    pool->channel_index = channel_manager->num_channels;
+    pool->ce_index = ce_index;
+    pool->is_proxy = is_proxy;
 
     uvm_spin_lock_init(&pool->lock, UVM_LOCK_ORDER_CHANNEL);
 
-    start = pool->channel_index;
-    end = start + UVM_CHANNELS_PER_COPY_ENGINE;
+    num_channels = channel_pool_num_channels(pool);
 
-    // Avoid uvm_for_each_channel_in_range because channel creation modifies the number of channels
-    for (i = start; i < end; i++) {
-        NV_STATUS status;
+    pool->channels = uvm_kvmalloc_zero(sizeof(*pool->channels) * num_channels);
+    if (!pool->channels)
+        return NV_ERR_NO_MEMORY;
 
-        channel = channel_manager->channels + i;
-        status = create_channel(pool, with_procfs, channel);
+    channel_manager->num_channel_pools++;
+
+    for (i = 0; i < num_channels; i++) {
+        uvm_channel_t *channel = pool->channels + i;
+
+        status = channel_create(pool, with_procfs, channel);
         if (status != NV_OK)
-            return status;
+            goto error;
 
         status = init_channel(channel);
         if (status != NV_OK)
-            return status;
+            goto error;
     }
 
     return NV_OK;
-}
 
-uvm_channel_t *uvm_channel_manager_find_available_channel(uvm_channel_manager_t *channel_manager)
-{
-    uvm_channel_t *channel;
-
-    uvm_for_each_channel(channel, channel_manager) {
-        bool available;
-
-        uvm_spin_lock(&channel->pool->lock);
-        available = is_channel_available(channel);
-        uvm_spin_unlock(&channel->pool->lock);
-
-        if (available)
-            return channel;
-    }
-    return NULL;
+ error:
+    channel_pool_destroy(pool);
+    return status;
 }
 
 static bool ce_usable_for_channel_type(uvm_channel_type_t type, const UvmGpuCopyEngineCaps *cap)
@@ -723,13 +732,29 @@ static bool ce_usable_for_channel_type(uvm_channel_type_t type, const UvmGpuCopy
     }
 }
 
+static unsigned ce_usage_count(NvU32 ce, const unsigned *preferred_ce)
+{
+    unsigned i;
+    unsigned count = 0;
+
+    UVM_ASSERT(ce < UVM_COPY_ENGINE_COUNT_MAX);
+
+    for (i = 0; i < UVM_CHANNEL_TYPE_COUNT; i++) {
+        if (ce == preferred_ce[i])
+            count++;
+    }
+
+    return count;
+}
+
 // Returns negative if the first CE should be considered better than the second
 static int compare_ce_for_channel_type(const UvmGpuCopyEngineCaps *ce_caps,
                                        uvm_channel_type_t type,
                                        NvU32 ce_index0,
                                        NvU32 ce_index1,
-                                       NvU32 *usage_count)
+                                       NvU32 *preferred_ce)
 {
+    unsigned ce0_usage, ce1_usage;
     const UvmGpuCopyEngineCaps *cap0 = ce_caps + ce_index0;
     const UvmGpuCopyEngineCaps *cap1 = ce_caps + ce_index1;
 
@@ -803,8 +828,11 @@ static int compare_ce_for_channel_type(const UvmGpuCopyEngineCaps *ce_caps,
     }
 
     // By default, prefer less used CEs (within the UVM driver at least)
-    if (usage_count[ce_index0] != usage_count[ce_index1])
-        return usage_count[ce_index0] - usage_count[ce_index1];
+    ce0_usage = ce_usage_count(ce_index0, preferred_ce);
+    ce1_usage = ce_usage_count(ce_index1, preferred_ce);
+
+    if (ce0_usage != ce1_usage)
+        return ce0_usage - ce1_usage;
 
     // And CEs that don't share PCEs
     if (cap0->shared != cap1->shared)
@@ -814,35 +842,31 @@ static int compare_ce_for_channel_type(const UvmGpuCopyEngineCaps *ce_caps,
     return ce_index0 - ce_index1;
 }
 
-// Pick default CE for the given channel type, and increment its usage
-// count. This function also sets the i-th bit in usable_ce_mask if the
-// CE with index i can be used for transfers of the specified type.
-//
-// Both the usage count array and the CE mask must be initialized prior
-// to calling this function
+// Identify usable CEs, and select the preferred CE for a given channel type.
 static NV_STATUS pick_ce_for_channel_type(uvm_channel_manager_t *manager,
                                           const UvmGpuCopyEngineCaps *ce_caps,
                                           uvm_channel_type_t type,
-                                          NvU32 *usage_count,
-                                          long unsigned *usable_ce_mask)
+                                          unsigned *preferred_ce)
 {
     NvU32 i;
     NvU32 best_ce = UVM_COPY_ENGINE_COUNT_MAX;
 
+    UVM_ASSERT(type < UVM_CHANNEL_TYPE_COUNT);
+
     for (i = 0; i < UVM_COPY_ENGINE_COUNT_MAX; ++i) {
         const UvmGpuCopyEngineCaps *cap = ce_caps + i;
 
         if (!ce_usable_for_channel_type(type, cap))
             continue;
 
-        __set_bit(i, usable_ce_mask);
+        __set_bit(i, manager->ce_mask);
 
         if (best_ce == UVM_COPY_ENGINE_COUNT_MAX) {
             best_ce = i;
             continue;
         }
 
-        if (compare_ce_for_channel_type(ce_caps, type, i, best_ce, usage_count) < 0)
+        if (compare_ce_for_channel_type(ce_caps, type, i, best_ce, preferred_ce) < 0)
             best_ce = i;
     }
 
@@ -851,18 +875,15 @@ static NV_STATUS pick_ce_for_channel_type(uvm_channel_manager_t *manager,
         return NV_ERR_NOT_SUPPORTED;
     }
 
-    ++usage_count[best_ce];
-    manager->ce_to_use.default_for_type[type] = best_ce;
-
+    preferred_ce[type] = best_ce;
     return NV_OK;
 }
 
-static NV_STATUS channel_manager_pick_copy_engines(uvm_channel_manager_t *manager, long unsigned *usable_ce_mask)
+static NV_STATUS channel_manager_pick_copy_engines(uvm_channel_manager_t *manager, unsigned *preferred_ce)
 {
     NV_STATUS status;
     unsigned i;
     UvmGpuCopyEnginesCaps ces_caps;
-    NvU32 ce_usage_count[UVM_COPY_ENGINE_COUNT_MAX] = {0};
     uvm_channel_type_t types[] = {UVM_CHANNEL_TYPE_CPU_TO_GPU,
                                   UVM_CHANNEL_TYPE_GPU_TO_CPU,
                                   UVM_CHANNEL_TYPE_GPU_INTERNAL,
@@ -870,7 +891,7 @@ static NV_STATUS channel_manager_pick_copy_engines(uvm_channel_manager_t *manage
                                   UVM_CHANNEL_TYPE_MEMOPS};
 
     memset(&ces_caps, 0, sizeof(ces_caps));
-    status = uvm_rm_locked_call(nvUvmInterfaceQueryCopyEnginesCaps(manager->gpu->rm_address_space, &ces_caps));
+    status = uvm_rm_locked_call(nvUvmInterfaceQueryCopyEnginesCaps(uvm_gpu_device_handle(manager->gpu), &ces_caps));
     if (status != NV_OK)
         return status;
 
@@ -879,8 +900,7 @@ static NV_STATUS channel_manager_pick_copy_engines(uvm_channel_manager_t *manage
    // MEMOPS has the least priority as it only cares about low usage of the
    // CE to improve latency
     for (i = 0; i < ARRAY_SIZE(types); ++i) {
-        status = pick_ce_for_channel_type(manager, ces_caps.copyEngineCaps, types[i], ce_usage_count, usable_ce_mask);
-
+        status = pick_ce_for_channel_type(manager, ces_caps.copyEngineCaps, types[i], preferred_ce);
         if (status != NV_OK)
             return status;
     }
@@ -888,24 +908,33 @@ static NV_STATUS channel_manager_pick_copy_engines(uvm_channel_manager_t *manage
     return NV_OK;
 }
 
-static void channel_manager_init_p2p_ces(uvm_channel_manager_t *manager)
+// Return the pool corresponding to the given CE index
+//
+// This function cannot be used to access the proxy pool in SR-IOV heavy.
+static uvm_channel_pool_t *channel_manager_ce_pool(uvm_channel_manager_t *manager, NvU32 ce)
 {
-    NvU32 i;
+    uvm_channel_pool_t *pool;
 
-    for (i = 0; i < ARRAY_SIZE(manager->ce_to_use.gpu_to_gpu); i++)
-        manager->ce_to_use.gpu_to_gpu[i] = UVM_COPY_ENGINE_COUNT_MAX;
+    UVM_ASSERT(test_bit(ce, manager->ce_mask));
+
+    // The index of the pool associated with 'ce' is the number of usable CEs
+    // in [0, ce)
+    pool = manager->channel_pools + bitmap_weight(manager->ce_mask, ce);
+
+    UVM_ASSERT(!pool->is_proxy);
+    UVM_ASSERT(pool->ce_index == ce);
+
+    return pool;
 }
 
-void uvm_channel_manager_set_p2p_ce(uvm_channel_manager_t *manager,
-                                    uvm_gpu_t *peer,
-                                    NvU32 optimal_ce)
+void uvm_channel_manager_set_p2p_ce(uvm_channel_manager_t *manager, uvm_gpu_t *peer, NvU32 optimal_ce)
 {
     const NvU32 peer_gpu_index = uvm_id_gpu_index(peer->id);
 
     UVM_ASSERT(manager->gpu != peer);
     UVM_ASSERT(optimal_ce < UVM_COPY_ENGINE_COUNT_MAX);
 
-    manager->ce_to_use.gpu_to_gpu[peer_gpu_index] = optimal_ce;
+    manager->pool_to_use.gpu_to_gpu[peer_gpu_index] = channel_manager_ce_pool(manager, optimal_ce);
 }
 
 static bool is_string_valid_location(const char *loc)
@@ -1049,17 +1078,72 @@ static void init_channel_manager_conf(uvm_channel_manager_t *manager)
     }
 }
 
+// A pool is created for each usable CE, even if it has not been selected as the
+// preferred CE for any type, because as more information is discovered (for
+// example, a pair of peer GPUs is added) we may start using the previously idle
+// channels.
+static NV_STATUS channel_manager_create_pools(uvm_channel_manager_t *manager, bool with_procfs)
+{
+    NV_STATUS status;
+    unsigned ce, type;
+    unsigned num_channel_pools;
+    bool is_proxy = false;
+    unsigned preferred_ce[UVM_CHANNEL_TYPE_COUNT];
+
+    for (type = 0; type < ARRAY_SIZE(preferred_ce); type++)
+        preferred_ce[type] = UVM_COPY_ENGINE_COUNT_MAX;
+
+    status = channel_manager_pick_copy_engines(manager, preferred_ce);
+    if (status != NV_OK)
+        return status;
+
+    num_channel_pools = bitmap_weight(manager->ce_mask, UVM_COPY_ENGINE_COUNT_MAX);
+
+    if (uvm_gpu_uses_proxy_channel_pool(manager->gpu))
+        num_channel_pools++;
+
+    manager->channel_pools = uvm_kvmalloc_zero(sizeof(*manager->channel_pools) * num_channel_pools);
+    if (!manager->channel_pools)
+        return NV_ERR_NO_MEMORY;
+
+    for_each_set_bit(ce, manager->ce_mask, UVM_COPY_ENGINE_COUNT_MAX) {
+        uvm_channel_pool_t *pool = manager->channel_pools + manager->num_channel_pools;
+
+        status = channel_pool_create(manager, ce, is_proxy, with_procfs, pool);
+        if (status != NV_OK)
+            return status;
+    }
+
+    // Assign channel types to pools
+    for (type = 0; type < ARRAY_SIZE(preferred_ce); type++) {
+        unsigned ce = preferred_ce[type];
+
+        UVM_ASSERT(test_bit(ce, manager->ce_mask));
+
+        manager->pool_to_use.default_for_type[type] = channel_manager_ce_pool(manager, ce);
+    }
+
+    // In SR-IOV heavy, add an additional, single-channel, pool that is
+    // dedicated to the MEMOPS type.
+    if (uvm_gpu_uses_proxy_channel_pool(manager->gpu)) {
+        uvm_channel_pool_t *pool = manager->channel_pools + manager->num_channel_pools;
+
+        is_proxy = true;
+
+        status = channel_pool_create(manager, preferred_ce[UVM_CHANNEL_TYPE_MEMOPS], is_proxy, with_procfs, pool);
+        if (status != NV_OK)
+            return status;
+
+        manager->pool_to_use.default_for_type[UVM_CHANNEL_TYPE_MEMOPS] = pool;
+    }
+
+    return NV_OK;
+}
+
 NV_STATUS uvm_channel_manager_create_common(uvm_gpu_t *gpu, bool with_procfs, uvm_channel_manager_t **channel_manager_out)
 {
     NV_STATUS status = NV_OK;
     uvm_channel_manager_t *channel_manager;
-    NvU32 i;
-
-    // Mask containing the indexes of the Copy Engines that can be used for at
-    // least one channel type
-    long unsigned usable_ce_mask = 0;
-
-    BUILD_BUG_ON(UVM_COPY_ENGINE_COUNT_MAX > sizeof(usable_ce_mask) * 8);
 
     channel_manager = uvm_kvmalloc_zero(sizeof(*channel_manager));
     if (!channel_manager)
@@ -1077,22 +1161,10 @@ NV_STATUS uvm_channel_manager_create_common(uvm_gpu_t *gpu, bool with_procfs, uv
             goto error;
     }
 
-    status = channel_manager_pick_copy_engines(channel_manager, &usable_ce_mask);
+    status = channel_manager_create_pools(channel_manager, with_procfs);
     if (status != NV_OK)
         goto error;
 
-    // We create a channel for every CE that is usable for at least one channel
-    // type, even if it has not been selected as the default CE for any type.
-    // As more information is discovered (for example, a pair of peer GPUs is
-    // added) we may start using the previously idle channels.
-    for_each_set_bit(i, &usable_ce_mask, UVM_COPY_ENGINE_COUNT_MAX) {
-        status = create_channel_pool(channel_manager, i, with_procfs);
-        if (status != NV_OK)
-            goto error;
-    }
-
-    channel_manager_init_p2p_ces(channel_manager);
-
     if (with_procfs) {
         status = manager_create_procfs(channel_manager);
         if (status != NV_OK)
@@ -1108,6 +1180,14 @@ error:
     return status;
 }
 
+static void channel_manager_destroy_pools(uvm_channel_manager_t *manager)
+{
+    while (manager->num_channel_pools > 0)
+        channel_pool_destroy(manager->channel_pools + manager->num_channel_pools - 1);
+
+    uvm_kvfree(manager->channel_pools);
+}
+
 void uvm_channel_manager_destroy(uvm_channel_manager_t *channel_manager)
 {
     if (channel_manager == NULL)
@@ -1115,8 +1195,7 @@ void uvm_channel_manager_destroy(uvm_channel_manager_t *channel_manager)
 
     uvm_procfs_destroy_entry(channel_manager->procfs.pending_pushes);
 
-    while (channel_manager->num_channels)
-        destroy_channel(channel_manager, channel_manager->channels + channel_manager->num_channels - 1);
+    channel_manager_destroy_pools(channel_manager);
 
     uvm_procfs_destroy_entry(channel_manager->procfs.channels_dir);
 
@@ -1127,7 +1206,7 @@ void uvm_channel_manager_destroy(uvm_channel_manager_t *channel_manager)
 
 const char *uvm_channel_type_to_string(uvm_channel_type_t channel_type)
 {
-    BUILD_BUG_ON(UVM_CHANNEL_TYPE_COUNT != 6);
+    BUILD_BUG_ON(UVM_CHANNEL_TYPE_COUNT != 5);
 
     switch (channel_type) {
         UVM_ENUM_STRING_CASE(UVM_CHANNEL_TYPE_CPU_TO_GPU);
@@ -1135,7 +1214,6 @@ const char *uvm_channel_type_to_string(uvm_channel_type_t channel_type)
         UVM_ENUM_STRING_CASE(UVM_CHANNEL_TYPE_GPU_INTERNAL);
         UVM_ENUM_STRING_CASE(UVM_CHANNEL_TYPE_MEMOPS);
         UVM_ENUM_STRING_CASE(UVM_CHANNEL_TYPE_GPU_TO_GPU);
-        UVM_ENUM_STRING_CASE(UVM_CHANNEL_TYPE_ANY);
         UVM_ENUM_STRING_DEFAULT();
     }
 }
@@ -1154,8 +1232,7 @@ static void uvm_channel_print_info(uvm_channel_t *channel, struct seq_file *s)
     UVM_SEQ_OR_DBG_PRINT(s, "GPPUT location     %s\n", buffer_location_to_string(manager->conf.gpput_loc));
     UVM_SEQ_OR_DBG_PRINT(s, "get                %u\n", channel->gpu_get);
     UVM_SEQ_OR_DBG_PRINT(s, "put                %u\n", channel->cpu_put);
-    UVM_SEQ_OR_DBG_PRINT(s, "Semaphore GPU VA   0x%llx\n", uvm_gpu_semaphore_get_gpu_va(&channel->tracking_sem.semaphore,
-                                                                                        uvm_channel_get_gpu(channel)));
+    UVM_SEQ_OR_DBG_PRINT(s, "Semaphore GPU VA   0x%llx\n", uvm_channel_tracking_semaphore_get_gpu_va(channel));
 
     uvm_spin_unlock(&channel->pool->lock);
 }
@@ -1175,9 +1252,10 @@ static void channel_print_push_acquires(uvm_push_acquire_info_t *push_acquire_in
 
     for (i = 0; i < valid_entries; ++i) {
         UVM_SEQ_OR_DBG_PRINT(seq,
-                             "%s gpu%u:channel%u:%llu",
+                             "%s gpu%u:channel%u:%u:value%llu",
                              i == 0? " acquiring values" : "",
                              uvm_id_value(push_acquire_info->values[i].gpu_id),
+                             push_acquire_info->values[i].runlist_id,
                              push_acquire_info->values[i].channel_id,
                              push_acquire_info->values[i].value);
     }
@@ -1243,12 +1321,16 @@ void uvm_channel_print_pending_pushes(uvm_channel_t *channel)
 
 static void channel_manager_print_pending_pushes(uvm_channel_manager_t *manager, struct seq_file *seq)
 {
-    uvm_channel_t *channel;
+    uvm_channel_pool_t *pool;
 
-    uvm_for_each_channel(channel, manager) {
-        UVM_SEQ_OR_DBG_PRINT(seq, "Channel %s, pending pushes:\n", channel->name);
+    uvm_for_each_pool(pool, manager) {
+        uvm_channel_t *channel;
 
-        channel_print_pushes(channel, 0, seq);
+        uvm_for_each_channel_in_pool(channel, pool) {
+            UVM_SEQ_OR_DBG_PRINT(seq, "Channel %s, pending pushes:\n", channel->name);
+
+            channel_print_pushes(channel, 0, seq);
+        }
     }
 }
 
@@ -1358,7 +1440,11 @@ static NV_STATUS channel_create_procfs(uvm_channel_t *channel)
     if (!uvm_procfs_is_debug_enabled())
         return NV_OK;
 
-    snprintf(channel_dirname, sizeof(channel_dirname), "%u", channel->channel_info.hwChannelId);
+    snprintf(channel_dirname,
+             sizeof(channel_dirname),
+             "%u:%u",
+             channel->channel_info.hwRunlistId,
+             channel->channel_info.hwChannelId);
 
     channel->procfs.dir = NV_CREATE_PROC_DIR(channel_dirname, manager->procfs.channels_dir);
     if (channel->procfs.dir == NULL)
@@ -1369,7 +1455,7 @@ static NV_STATUS channel_create_procfs(uvm_channel_t *channel)
         return NV_ERR_OPERATING_SYSTEM;
 
     channel->procfs.pushes = NV_CREATE_PROC_FILE("pushes", channel->procfs.dir, channel_pushes_entry, channel);
-    if (channel->procfs.info == NULL)
+    if (channel->procfs.pushes == NULL)
         return NV_ERR_OPERATING_SYSTEM;
 
     return NV_OK;
