@@ -38,7 +38,6 @@
 // Schedule pushes one after another on all GPUs and channel types that copy and
 // increment a counter into an adjacent memory location in a buffer. And then
 // verify that all the values are correct on the CPU.
-// GK110+ is required for the CE semaphore reduction method.
 static NV_STATUS test_ordering(uvm_va_space_t *va_space)
 {
     NV_STATUS status;
@@ -49,7 +48,6 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
     uvm_push_t push;
     NvU64 gpu_va;
     uvm_tracker_t tracker = UVM_TRACKER_INIT();
-    bool at_least_one_gk110_plus = false;
     NvU32 value = 0;
     const NvU32 iters_per_channel_type_per_gpu = g_uvm_global.num_simulated_devices > 0?
                                                      TEST_ORDERING_ITERS_PER_CHANNEL_TYPE_PER_GPU_EMU:
@@ -57,14 +55,6 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
     const NvU32 values_count = iters_per_channel_type_per_gpu;
     const size_t buffer_size = sizeof(NvU32) * values_count;
 
-    for_each_va_space_gpu(gpu, va_space) {
-        if (uvm_gpu_is_gk110_plus(gpu))
-            at_least_one_gk110_plus = true;
-    }
-
-    if (!at_least_one_gk110_plus)
-        return NV_OK;
-
     gpu = uvm_va_space_find_first_gpu(va_space);
     TEST_CHECK_RET(gpu != NULL);
 
@@ -77,7 +67,7 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
     status = uvm_push_begin(gpu->channel_manager, UVM_CHANNEL_TYPE_GPU_TO_CPU, &push, "Initial memset");
     TEST_CHECK_GOTO(status == NV_OK, done);
 
-    gpu_va = uvm_rm_mem_get_gpu_va(mem, gpu);
+    gpu_va = uvm_rm_mem_get_gpu_va(mem, gpu, uvm_channel_is_proxy(push.channel));
 
     // Semaphore release as part of uvm_push_end() will do the membar
     uvm_push_set_flag(&push, UVM_PUSH_FLAG_CE_NEXT_MEMBAR_NONE);
@@ -85,8 +75,7 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
 
     uvm_push_end(&push);
 
-    status = uvm_tracker_add_push(&tracker, &push);
-    TEST_CHECK_GOTO(status == NV_OK, done);
+    TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, &push), done);
 
     for (i = 0; i < iters_per_channel_type_per_gpu; ++i) {
         for (j = 0; j < UVM_CHANNEL_TYPE_COUNT; ++j) {
@@ -96,10 +85,6 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
                 NvU64 gpu_va_dst;
                 uvm_channel_type_t channel_type = j;
 
-                // semaphore reduction below requires GK110+
-                if (!uvm_gpu_is_gk110_plus(gpu))
-                    continue;
-
                 status = uvm_push_begin_acquire(gpu->channel_manager,
                                                 channel_type,
                                                 &tracker,
@@ -108,7 +93,7 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
                                                 value + 1);
                 TEST_CHECK_GOTO(status == NV_OK, done);
 
-                gpu_va_base = uvm_rm_mem_get_gpu_va(mem, gpu);
+                gpu_va_base = uvm_rm_mem_get_gpu_va(mem, gpu, uvm_channel_is_proxy(push.channel));
                 gpu_va_src = gpu_va_base + (value % values_count) * sizeof(NvU32);
                 gpu_va_dst = gpu_va_base + ((value + 1) % values_count) * sizeof(NvU32);
 
@@ -124,8 +109,7 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
                 uvm_push_end(&push);
 
                 uvm_tracker_clear(&tracker);
-                status = uvm_tracker_add_push(&tracker, &push);
-                TEST_CHECK_GOTO(status == NV_OK, done);
+                TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, &push), done);
             }
         }
     }
@@ -159,7 +143,7 @@ done:
 static NV_STATUS uvm_test_rc_for_channel_manager(uvm_channel_manager_t *manager)
 {
     uvm_push_t push;
-    uvm_channel_t *channel;
+    uvm_channel_pool_t *pool;
     uvm_gpfifo_entry_t *fatal_entry;
     uvm_push_info_t *push_info;
     int fatal_line;
@@ -171,13 +155,17 @@ static NV_STATUS uvm_test_rc_for_channel_manager(uvm_channel_manager_t *manager)
     // Submit a bunch of successful pushes on each channel first so that the
     // fatal one is behind a bunch of work (notably more than
     // uvm_channel_update_progress() completes by default).
-    uvm_for_each_channel(channel, manager) {
-        NvU32 i;
-        for (i = 0; i < 512; ++i) {
-            status = uvm_push_begin_on_channel(channel, &push, "Non-faulting push");
-            TEST_CHECK_RET(status == NV_OK);
+    uvm_for_each_pool(pool, manager) {
+        uvm_channel_t *channel;
 
-            uvm_push_end(&push);
+        uvm_for_each_channel_in_pool(channel, pool) {
+            NvU32 i;
+            for (i = 0; i < 512; ++i) {
+                status = uvm_push_begin_on_channel(channel, &push, "Non-faulting push");
+                TEST_CHECK_RET(status == NV_OK);
+
+                uvm_push_end(&push);
+            }
         }
     }
 
@@ -275,13 +263,18 @@ static void snapshot_counter(uvm_push_t *push,
                              NvU32 counters_count)
 {
     uvm_gpu_t *gpu = uvm_push_get_gpu(push);
-    NvU64 counter_gpu_va = uvm_rm_mem_get_gpu_va(counter_mem, gpu);
-    NvU64 snapshot_gpu_va = uvm_rm_mem_get_gpu_va(snapshot_mem, gpu) + index * 2 * sizeof(NvU32);
+    NvU64 counter_gpu_va;
+    NvU64 snapshot_gpu_va;
+    bool is_proxy_channel;
     NvU32 last_counter_offset = (counters_count - 1) * sizeof(NvU32);
 
     if (counters_count == 0)
         return;
 
+    is_proxy_channel = uvm_channel_is_proxy(push->channel);
+    counter_gpu_va = uvm_rm_mem_get_gpu_va(counter_mem, gpu, is_proxy_channel);
+    snapshot_gpu_va = uvm_rm_mem_get_gpu_va(snapshot_mem, gpu, is_proxy_channel) + index * 2 * sizeof(NvU32);
+
     // Copy the last and first counter to a snapshot for later verification.
 
     // Membar will be done by uvm_push_end()
@@ -301,7 +294,12 @@ static void snapshot_counter(uvm_push_t *push,
 static void set_counter(uvm_push_t *push, uvm_rm_mem_t *counter_mem, NvU32 value, NvU32 count)
 {
     uvm_gpu_t *gpu = uvm_push_get_gpu(push);
-    NvU64 counter_gpu_va = uvm_rm_mem_get_gpu_va(counter_mem, gpu);
+    NvU64 counter_gpu_va;
+    bool is_proxy_channel;
+
+    is_proxy_channel = uvm_channel_is_proxy(push->channel);
+    counter_gpu_va = uvm_rm_mem_get_gpu_va(counter_mem, gpu, is_proxy_channel);
+
     gpu->parent->ce_hal->memset_v_4(push, counter_gpu_va, value, count * sizeof(NvU32));
 }
 
@@ -336,7 +334,7 @@ static void test_memset_rm_mem(uvm_push_t *push, uvm_rm_mem_t *rm_mem, NvU32 val
     UVM_ASSERT(rm_mem->size % 4 == 0);
 
     gpu = uvm_push_get_gpu(push);
-    gpu_va = uvm_rm_mem_get_gpu_va(rm_mem, gpu);
+    gpu_va = uvm_rm_mem_get_gpu_va(rm_mem, gpu, uvm_channel_is_proxy(push->channel));
 
     gpu->parent->ce_hal->memset_v_4(push, gpu_va, value, rm_mem->size);
 }
@@ -487,9 +485,7 @@ static NV_STATUS stress_test_all_gpus_in_va(uvm_va_space_t *va_space,
 
             uvm_push_end(&stream->push);
             uvm_tracker_clear(&stream->tracker);
-            status = uvm_tracker_add_push(&stream->tracker, &stream->push);
-            if (status != NV_OK)
-                goto done;
+            TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&stream->tracker, &stream->push), done);
         }
 
         // Randomly schedule other threads
