@@ -139,7 +139,11 @@ bool uvm_debug_prints_enabled(void);
         UVM_IGNORE_EXPR(expr2);         \
     } while (0)
 
+// NO-OP function to break on_uvm_test_fail - that is just to set a breakpoint
+void on_uvm_test_fail(void);
+
 // NO-OP function to break on_uvm_assert - that is just to set a breakpoint
+// Unlike on_uvm_test_fail it provides 'panic' coverity semantics
 void on_uvm_assert(void);
 
 // UVM_ASSERT_RELEASE and UVM_ASSERT_MSG_RELEASE are always enabled, even on
@@ -164,8 +168,8 @@ void on_uvm_assert(void);
         UVM_NO_PRINT(fmt, ##__VA_ARGS__);       \
     } while (0)
 
-// UVM_ASSERT and UVM_ASSERT_MSG are only enabled on non-release builds
-#if UVM_IS_DEBUG()
+// UVM_ASSERT and UVM_ASSERT_MSG are only enabled on non-release and Coverity builds
+#if UVM_IS_DEBUG() || defined __COVERITY__
     #define UVM_ASSERT_MSG                  UVM_ASSERT_MSG_RELEASE
     #define UVM_ASSERT                      UVM_ASSERT_RELEASE
 #else
@@ -239,10 +243,9 @@ static inline void kmem_cache_destroy_safe(struct kmem_cache **ppCache)
 {
     if (ppCache)
     {
-        if (*ppCache) {
+        if (*ppCache)
             kmem_cache_destroy(*ppCache);
-            NV_KMEM_CACHE_DESTROY_FLUSH();
-        }
+
         *ppCache = NULL;
     }
 }
