@@ -33,6 +33,9 @@
 #include "uvm_lock.h"
 #include "uvm_ats_ibm.h"
 
+
+
+
 // Global state of the uvm driver
 struct uvm_global_struct
 {
@@ -144,6 +147,10 @@ struct uvm_global_struct
         NvU64 *ptr;
         struct page *page;
     } unload_state;
+
+
+
+
 };
 
 // Initialize global uvm state
@@ -312,4 +319,10 @@ void uvm_global_mask_release(const uvm_global_processor_mask_t *mask);
 // Notably this check cannot be performed where it's not safe to call into RM.
 NV_STATUS uvm_global_mask_check_ecc_error(uvm_global_processor_mask_t *gpus);
 
+// Pre-allocate fault service contexts.
+NV_STATUS uvm_service_block_context_init(void);
+
+// Release fault service contexts if any exist.
+void uvm_service_block_context_exit(void);
+
 #endif // __UVM_GLOBAL_H__
