@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -494,6 +494,10 @@ typedef enum
     UVM_GPU_LINK_NVLINK_1,
     UVM_GPU_LINK_NVLINK_2,
     UVM_GPU_LINK_NVLINK_3,
+
+
+
+
     UVM_GPU_LINK_MAX
 } uvm_gpu_link_type_t;
 
@@ -559,27 +563,6 @@ struct uvm_gpu_struct
         // Notably it may be different than the big page size used by a user's VA
         // space in general.
         NvU32 internal_size;
-
-        // Big page self identity mapping
-        // Used only on GPUs that have big page swizzling enabled. Notably
-        // that's only Kepler which only supports one big page size at a time
-        // and hence only a single mapping is needed.
-        uvm_gpu_identity_mapping_t identity_mapping;
-
-        struct
-        {
-            // These fields are only used on GPUs that have big page swizzling
-            // enabled.
-
-            // Staging memory used for converting between formats
-            uvm_gpu_chunk_t *chunk;
-
-            // Tracker for all staging operations performed using chunk
-            uvm_tracker_t tracker;
-
-            // Lock protecting tracker
-            uvm_mutex_t lock;
-        } staging;
     } big_page;
 
     // Mapped registers needed to obtain the current GPU timestamp
@@ -614,6 +597,9 @@ struct uvm_gpu_struct
 
     // RM address space handle used in many of the UVM/RM APIs
     // Represents a GPU VA space within rm_device.
+    //
+    // In SR-IOV heavy, proxy channels are not associated with this address
+    // space.
     uvmGpuAddressSpaceHandle rm_address_space;
 
     // Page tree used for the internal UVM VA space shared with RM
@@ -623,6 +609,13 @@ struct uvm_gpu_struct
     // to the address_space_tree.
     bool rm_address_space_moved_to_page_tree;
 
+    // RM handle of the VA space associated with the proxy pool channels.
+    //
+    // TODO: Bug 3325177: this field will be removed in the final SR-IOV heavy
+    // implementation. UVM will not have access to the vGPU-owned VA space of
+    // the proxy channels.
+    uvmGpuAddressSpaceHandle rm_proxy_address_space;
+
     uvm_gpu_semaphore_pool_t *semaphore_pool;
 
     uvm_channel_manager_t *channel_manager;
@@ -667,6 +660,8 @@ struct uvm_gpu_struct
         uvmGpuSessionHandle rm_session_handle;
 
         // RM device handle used in many of the UVM/RM APIs.
+        //
+        // Do not read this field directly, use uvm_gpu_device_handle instead.
         uvmGpuDeviceHandle rm_device;
     } smc;
 
@@ -741,6 +736,8 @@ struct uvm_parent_gpu_struct
     int closest_cpu_numa_node;
 
     // RM device handle used in many of the UVM/RM APIs.
+    //
+    // Do not read this field directly, use uvm_gpu_device_handle instead.
     uvmGpuDeviceHandle rm_device;
 
     // The physical address range addressable by the GPU
@@ -764,22 +761,6 @@ struct uvm_parent_gpu_struct
     uvm_fault_buffer_hal_t *fault_buffer_hal;
     uvm_access_counter_buffer_hal_t *access_counter_buffer_hal;
 
-    struct
-    {
-        // VA where the identity mapping should be mapped in the internal VA
-        // space managed by uvm_gpu_t.address_space_tree (see below).
-        NvU64 identity_mapping_base;
-
-        // Whether big page mappings have the physical memory swizzled.
-        // On some architectures (Kepler) physical memory mapped with a big page
-        // size doesn't follow the common 1-1 mapping where each offset within the
-        // big page maps to the same offset within the mapped physical memory.
-        // Swizzling makes physically accessing memory mapped with big pages
-        // infeasible and has to be worked around by creating virtual identity
-        // mappings (see uvm_gpu_struct's big_page.identity_mapping).
-        bool swizzling;
-    } big_page;
-
     uvm_gpu_peer_copy_mode_t peer_copy_mode;
 
     // Virtualization mode of the GPU.
@@ -805,8 +786,6 @@ struct uvm_parent_gpu_struct
     // True if the GPU has hardware support for scoped atomics
     bool scoped_atomics_supported;
 
-    bool has_pulse_based_interrupts;
-
     // If true, a HW method can be used to clear a faulted channel.
     // If false, then the GPU supports clearing faulted channels using registers
     // instead of a HW method.
@@ -852,6 +831,9 @@ struct uvm_parent_gpu_struct
     // Largest VA (exclusive) which can be used for channel buffer mappings
     NvU64 max_channel_va;
 
+    // Largest VA (exclusive) which Host can operate.
+    NvU64 max_host_va;
+
     // Indicates whether the GPU can map sysmem with pages larger than 4k
     bool can_map_sysmem_with_large_pages;
 
@@ -862,7 +844,7 @@ struct uvm_parent_gpu_struct
     // On Pascal a single top level PDE covers 128 TB of VA and given that
     // semaphores and other allocations limited to 40bit are currently allocated
     // through RM, RM needs to control the [0, 128TB) VA range at least for now.
-    // On Kepler and Maxwell limit RMs VA to [0, 128GB) that should easily fit
+    // On Maxwell, limit RMs VA to [0, 128GB) that should easily fit
     // all RM allocations and leave enough space for UVM.
     NvU64 rm_va_base;
     NvU64 rm_va_size;
@@ -971,10 +953,6 @@ struct uvm_parent_gpu_struct
         atomic64_t              num_pages_in;
     } stats;
 
-#if UVM_IS_CONFIG_HMM()
-    uvm_hmm_gpu_t hmm_gpu;
-#endif
-
     // Structure to hold nvswitch specific information. In an nvswitch
     // environment, rather than using the peer-id field of the PTE (which can
     // only address 8 gpus), all gpus are assigned a 47-bit physical address
@@ -1044,7 +1022,7 @@ struct uvm_gpu_peer_struct
     // than a PEER aperture
     NvU8 is_indirect_peer : 1;
 
-    // The link type between the peer GPUs, currently either PCIe or NVLINK1/2.
+    // The link type between the peer GPUs, currently either PCIe or NVLINK.
     // This field is used to determine the when this peer struct has been
     // initialized (link_type != UVM_GPU_LINK_INVALID). NVLink peers are
     // initialized at GPU registration time. PCIe peers are initialized when
@@ -1258,11 +1236,6 @@ static void uvm_gpu_unmap_cpu_page(uvm_gpu_t *gpu, NvU64 dma_address)
     uvm_gpu_unmap_cpu_pages(gpu, dma_address, PAGE_SIZE);
 }
 
-static bool uvm_gpu_is_gk110_plus(uvm_gpu_t *gpu)
-{
-    return gpu->parent->rm_info.gpuArch >= NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK110;
-}
-
 // Returns whether the given address is within the GPU's maximum addressable VA
 // range. Warning: This only checks whether the GPU's MMU can support the given
 // address. Some HW units on that GPU might only support a smaller range.
@@ -1276,11 +1249,25 @@ static bool uvm_gpu_supports_eviction(uvm_gpu_t *gpu)
     return gpu->parent->replayable_faults_supported;
 }
 
-// Returns true if the virtualization mode is SRIOV-Heavy or SRIOV-Standard.
-static bool uvm_gpu_is_virt_mode_sriov(uvm_gpu_t *gpu)
+static bool uvm_gpu_is_virt_mode_sriov_heavy(const uvm_gpu_t *gpu)
 {
-    return (gpu->parent->virt_mode == UVM_VIRT_MODE_SRIOV_HEAVY) ||
-           (gpu->parent->virt_mode == UVM_VIRT_MODE_SRIOV_STANDARD);
+    return gpu->parent->virt_mode == UVM_VIRT_MODE_SRIOV_HEAVY;
+}
+
+static bool uvm_gpu_is_virt_mode_sriov_standard(const uvm_gpu_t *gpu)
+{
+    return gpu->parent->virt_mode == UVM_VIRT_MODE_SRIOV_STANDARD;
+}
+
+// Returns true if the virtualization mode is SR-IOV heavy or SR-IOV standard.
+static bool uvm_gpu_is_virt_mode_sriov(const uvm_gpu_t *gpu)
+{
+    return uvm_gpu_is_virt_mode_sriov_heavy(gpu) || uvm_gpu_is_virt_mode_sriov_standard(gpu);
+}
+
+static bool uvm_gpu_uses_proxy_channel_pool(const uvm_gpu_t *gpu)
+{
+    return uvm_gpu_is_virt_mode_sriov_heavy(gpu);
 }
 
 // Debug print of GPU properties
@@ -1308,23 +1295,6 @@ NV_STATUS uvm_gpu_access_counter_entry_to_va_space(uvm_gpu_t *gpu,
                                                    uvm_access_counter_buffer_entry_t *entry,
                                                    uvm_va_space_t **out_va_space);
 
-typedef enum
-{
-    UVM_GPU_SWIZZLE_OP_SWIZZLE,
-    UVM_GPU_SWIZZLE_OP_DESWIZZLE,
-    UVM_GPU_SWIZZLE_OP_COUNT
-} uvm_gpu_swizzle_op_t;
-
-// Convert the data format of the given big page physical address. The tracker
-// parameter may be NULL. If not, it is an in/out parameter: the swizzle
-// operation will acquire it, then replace it.
-//
-// This will only fail due to a global error.
-NV_STATUS uvm_gpu_swizzle_phys(uvm_gpu_t *gpu,
-                               NvU64 big_page_phys_address,
-                               uvm_gpu_swizzle_op_t op,
-                               uvm_tracker_t *tracker);
-
 typedef enum
 {
     UVM_GPU_BUFFER_FLUSH_MODE_CACHED_PUT,
