@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -24,10 +24,7 @@
 #include "uvm_hal.h"
 #include "uvm_kvmalloc.h"
 
-#include "cla06f.h"
-#include "cla06fsubch.h"
 #include "cla16f.h"
-#include "cla0b5.h"
 #include "clb069.h"
 #include "clb06f.h"
 #include "clb0b5.h"
@@ -43,14 +40,12 @@
 #include "clc5b5.h"
 #include "clc6b5.h"
 #include "clc56f.h"
-
 #include "clc7b5.h"
 
 
 
 
 
-
 #define CE_OP_COUNT (sizeof(uvm_ce_hal_t) / sizeof(void *))
 #define HOST_OP_COUNT (sizeof(uvm_host_hal_t) / sizeof(void *))
 #define ARCH_OP_COUNT (sizeof(uvm_arch_hal_t) / sizeof(void *))
@@ -64,34 +59,32 @@
 // The parent class must appear earlier in the array than the child.
 static uvm_hal_class_ops_t ce_table[] =
 {
-    {
-        .id = KEPLER_DMA_COPY_A,
-        .u.ce_ops = {
-            .init = uvm_hal_kepler_ce_init,
-            .semaphore_release = uvm_hal_kepler_ce_semaphore_release,
-            .semaphore_timestamp = uvm_hal_kepler_ce_semaphore_timestamp,
-            .semaphore_reduction_inc = uvm_hal_kepler_ce_semaphore_reduction_inc,
-            .offset_out = uvm_hal_kepler_ce_offset_out,
-            .offset_in_out = uvm_hal_kepler_ce_offset_in_out,
-            .phys_mode = uvm_hal_kepler_ce_phys_mode,
-            .plc_mode = uvm_hal_kepler_ce_plc_mode,
-            .memcopy = uvm_hal_kepler_ce_memcopy,
-            .memcopy_v_to_v = uvm_hal_kepler_ce_memcopy_v_to_v,
-            .memset_1 = uvm_hal_kepler_ce_memset_1,
-            .memset_4 = uvm_hal_kepler_ce_memset_4,
-            .memset_8 = uvm_hal_kepler_ce_memset_8,
-            .memset_v_4 = uvm_hal_kepler_ce_memset_v_4,
-        }
-    },
     {
         .id = MAXWELL_DMA_COPY_A,
-        .parent_id = KEPLER_DMA_COPY_A,
-        .u.ce_ops = {}
+        .u.ce_ops = {
+            .init = uvm_hal_maxwell_ce_init,
+            .semaphore_release = uvm_hal_maxwell_ce_semaphore_release,
+            .semaphore_timestamp = uvm_hal_maxwell_ce_semaphore_timestamp,
+            .semaphore_reduction_inc = uvm_hal_maxwell_ce_semaphore_reduction_inc,
+            .offset_out = uvm_hal_maxwell_ce_offset_out,
+            .offset_in_out = uvm_hal_maxwell_ce_offset_in_out,
+            .phys_mode = uvm_hal_maxwell_ce_phys_mode,
+            .plc_mode = uvm_hal_maxwell_ce_plc_mode,
+            .memcopy = uvm_hal_maxwell_ce_memcopy,
+            .memcopy_v_to_v = uvm_hal_maxwell_ce_memcopy_v_to_v,
+            .memset_1 = uvm_hal_maxwell_ce_memset_1,
+            .memset_4 = uvm_hal_maxwell_ce_memset_4,
+            .memset_8 = uvm_hal_maxwell_ce_memset_8,
+            .memset_v_4 = uvm_hal_maxwell_ce_memset_v_4,
+        }
     },
     {
         .id = PASCAL_DMA_COPY_A,
         .parent_id = MAXWELL_DMA_COPY_A,
         .u.ce_ops = {
+            .semaphore_release = uvm_hal_pascal_ce_semaphore_release,
+            .semaphore_timestamp = uvm_hal_pascal_ce_semaphore_timestamp,
+            .semaphore_reduction_inc = uvm_hal_pascal_ce_semaphore_reduction_inc,
             .offset_out = uvm_hal_pascal_ce_offset_out,
             .offset_in_out = uvm_hal_pascal_ce_offset_in_out,
         }
@@ -118,8 +111,6 @@ static uvm_hal_class_ops_t ce_table[] =
             .phys_mode = uvm_hal_ampere_ce_phys_mode,
         },
     },
-
-    // We assume GLOBAL_GPU_FAMILY_GA10X -> GLOBAL_ARCH_AMPERE
     {
         .id = AMPERE_DMA_COPY_B,
         .parent_id = AMPERE_DMA_COPY_A,
@@ -135,52 +126,53 @@ static uvm_hal_class_ops_t ce_table[] =
 
 
 
+
+
+
+
+
 };
 
 // Table for GPFIFO functions.  Same idea as the copy engine table.
 static uvm_hal_class_ops_t host_table[] =
 {
     {
-        .id = KEPLER_CHANNEL_GPFIFO_A,
-        .u.host_ops = {
-            .init = uvm_hal_kepler_host_init_noop,
-            .wait_for_idle = uvm_hal_kepler_host_wait_for_idle_a06f,
-            .membar_sys = uvm_hal_kepler_host_membar_sys,
-            // No MEMBAR GPU until Pascal, just do a MEMBAR SYS.
-            .membar_gpu = uvm_hal_kepler_host_membar_sys,
-            .noop = uvm_hal_kepler_host_noop,
-            .interrupt = uvm_hal_kepler_host_interrupt,
-            .semaphore_acquire = uvm_hal_kepler_host_semaphore_acquire,
-            .semaphore_release = uvm_hal_kepler_host_semaphore_release,
-            .set_gpfifo_entry = uvm_hal_kepler_host_set_gpfifo_entry,
-            .write_gpu_put = uvm_hal_kepler_host_write_gpu_put,
-            .tlb_invalidate_all = uvm_hal_kepler_host_tlb_invalidate_all,
-            .tlb_invalidate_va = uvm_hal_kepler_host_tlb_invalidate_va,
-            .tlb_invalidate_test = uvm_hal_kepler_host_tlb_invalidate_test,
-            .replay_faults = uvm_hal_kepler_replay_faults_unsupported,
-            .cancel_faults_targeted = uvm_hal_kepler_cancel_faults_targeted_unsupported,
-            .cancel_faults_va = uvm_hal_kepler_cancel_faults_va_unsupported,
-            .clear_faulted_channel_sw_method = uvm_hal_kepler_host_clear_faulted_channel_sw_method_unsupported,
-            .clear_faulted_channel_method = uvm_hal_kepler_host_clear_faulted_channel_method_unsupported,
-            .clear_faulted_channel_register = uvm_hal_kepler_host_clear_faulted_channel_register_unsupported,
-            .access_counter_clear_all = uvm_hal_kepler_access_counter_clear_all_unsupported,
-            .access_counter_clear_type = uvm_hal_kepler_access_counter_clear_type_unsupported,
-            .access_counter_clear_targeted = uvm_hal_kepler_access_counter_clear_targeted_unsupported,
-            .get_time = uvm_hal_kepler_get_time,
-        }
-    },
-    {
+        // This host class is reported for GM10x
         .id = KEPLER_CHANNEL_GPFIFO_B,
-        .parent_id = KEPLER_CHANNEL_GPFIFO_A,
         .u.host_ops = {
-            .wait_for_idle = uvm_hal_kepler_host_wait_for_idle_a16f,
+            .init = uvm_hal_maxwell_host_init_noop,
+            .wait_for_idle = uvm_hal_maxwell_host_wait_for_idle,
+            .membar_sys = uvm_hal_maxwell_host_membar_sys,
+            // No MEMBAR GPU until Pascal, just do a MEMBAR SYS.
+            .membar_gpu = uvm_hal_maxwell_host_membar_sys,
+            .noop = uvm_hal_maxwell_host_noop,
+            .interrupt = uvm_hal_maxwell_host_interrupt,
+            .semaphore_acquire = uvm_hal_maxwell_host_semaphore_acquire,
+            .semaphore_release = uvm_hal_maxwell_host_semaphore_release,
+            .set_gpfifo_entry = uvm_hal_maxwell_host_set_gpfifo_entry,
+            .write_gpu_put = uvm_hal_maxwell_host_write_gpu_put,
+            .tlb_invalidate_all = uvm_hal_maxwell_host_tlb_invalidate_all_a16f,
+            .tlb_invalidate_va = uvm_hal_maxwell_host_tlb_invalidate_va,
+            .tlb_invalidate_test = uvm_hal_maxwell_host_tlb_invalidate_test,
+            .replay_faults = uvm_hal_maxwell_replay_faults_unsupported,
+            .cancel_faults_global = uvm_hal_maxwell_cancel_faults_global_unsupported,
+            .cancel_faults_targeted = uvm_hal_maxwell_cancel_faults_targeted_unsupported,
+            .cancel_faults_va = uvm_hal_maxwell_cancel_faults_va_unsupported,
+            .clear_faulted_channel_sw_method = uvm_hal_maxwell_host_clear_faulted_channel_sw_method_unsupported,
+            .clear_faulted_channel_method = uvm_hal_maxwell_host_clear_faulted_channel_method_unsupported,
+            .clear_faulted_channel_register = uvm_hal_maxwell_host_clear_faulted_channel_register_unsupported,
+            .access_counter_clear_all = uvm_hal_maxwell_access_counter_clear_all_unsupported,
+            .access_counter_clear_type = uvm_hal_maxwell_access_counter_clear_type_unsupported,
+            .access_counter_clear_targeted = uvm_hal_maxwell_access_counter_clear_targeted_unsupported,
+            .get_time = uvm_hal_maxwell_get_time,
         }
     },
     {
+        // This host class is reported for GM20x
         .id = MAXWELL_CHANNEL_GPFIFO_A,
         .parent_id = KEPLER_CHANNEL_GPFIFO_B,
         .u.host_ops = {
-            .tlb_invalidate_all = uvm_hal_maxwell_host_tlb_invalidate_all,
+            .tlb_invalidate_all = uvm_hal_maxwell_host_tlb_invalidate_all_b06f,    
         }
     },
     {
@@ -194,6 +186,7 @@ static uvm_hal_class_ops_t host_table[] =
             .tlb_invalidate_va = uvm_hal_pascal_host_tlb_invalidate_va,
             .tlb_invalidate_test = uvm_hal_pascal_host_tlb_invalidate_test,
             .replay_faults = uvm_hal_pascal_replay_faults,
+            .cancel_faults_global = uvm_hal_pascal_cancel_faults_global,
             .cancel_faults_targeted = uvm_hal_pascal_cancel_faults_targeted,
         }
     },
@@ -227,6 +220,9 @@ static uvm_hal_class_ops_t host_table[] =
         .u.host_ops = {
             .clear_faulted_channel_sw_method = uvm_hal_ampere_host_clear_faulted_channel_sw_method,
             .clear_faulted_channel_register = uvm_hal_ampere_host_clear_faulted_channel_register,
+            .tlb_invalidate_all = uvm_hal_ampere_host_tlb_invalidate_all,
+            .tlb_invalidate_va = uvm_hal_ampere_host_tlb_invalidate_va,
+            .tlb_invalidate_test = uvm_hal_ampere_host_tlb_invalidate_test,
         }
     },
 
@@ -236,36 +232,26 @@ static uvm_hal_class_ops_t host_table[] =
 
 
 
+
+
+
+
+
+
+
 };
 
 static uvm_hal_class_ops_t arch_table[] =
 {
-    {
-        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK100,
-        .u.arch_ops = {
-            .init_properties = uvm_hal_kepler_arch_init_properties,
-            .mmu_mode_hal = uvm_hal_mmu_mode_kepler,
-            .enable_prefetch_faults = uvm_hal_kepler_mmu_enable_prefetch_faults_unsupported,
-            .disable_prefetch_faults = uvm_hal_kepler_mmu_disable_prefetch_faults_unsupported,
-            .mmu_engine_id_to_type = uvm_hal_kepler_mmu_engine_id_to_type_unsupported,
-            .mmu_client_id_to_utlb_id = uvm_hal_kepler_mmu_client_id_to_utlb_id_unsupported,
-        }
-    },
-    {
-        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK110,
-        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK100,
-        .u.arch_ops = {}
-    },
-    {
-        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK200,
-        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK110,
-        .u.arch_ops = {}
-    },
     {
         .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM000,
-        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK200,
         .u.arch_ops = {
             .init_properties = uvm_hal_maxwell_arch_init_properties,
+            .mmu_mode_hal = uvm_hal_mmu_mode_maxwell,
+            .enable_prefetch_faults = uvm_hal_maxwell_mmu_enable_prefetch_faults_unsupported,
+            .disable_prefetch_faults = uvm_hal_maxwell_mmu_disable_prefetch_faults_unsupported,
+            .mmu_engine_id_to_type = uvm_hal_maxwell_mmu_engine_id_to_type_unsupported,
+            .mmu_client_id_to_utlb_id = uvm_hal_maxwell_mmu_client_id_to_utlb_id_unsupported,
         }
     },
     {
@@ -325,12 +311,60 @@ static uvm_hal_class_ops_t arch_table[] =
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 };
 
 static uvm_hal_class_ops_t fault_buffer_table[] =
 {
     {
-        .id = MAXWELL_FAULT_BUFFER_A,
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM000,
+        .u.fault_buffer_ops = {
+            .enable_replayable_faults  = uvm_hal_maxwell_enable_replayable_faults_unsupported,
+            .disable_replayable_faults = uvm_hal_maxwell_disable_replayable_faults_unsupported,
+            .clear_replayable_faults = uvm_hal_maxwell_clear_replayable_faults_unsupported,
+            .read_put = uvm_hal_maxwell_fault_buffer_read_put_unsupported,
+            .read_get = uvm_hal_maxwell_fault_buffer_read_get_unsupported,
+            .write_get = uvm_hal_maxwell_fault_buffer_write_get_unsupported,
+            .get_ve_id = uvm_hal_maxwell_fault_buffer_get_ve_id_unsupported,
+            .parse_entry = uvm_hal_maxwell_fault_buffer_parse_entry_unsupported,
+            .entry_is_valid = uvm_hal_maxwell_fault_buffer_entry_is_valid_unsupported,
+            .entry_clear_valid = uvm_hal_maxwell_fault_buffer_entry_clear_valid_unsupported,
+            .entry_size = uvm_hal_maxwell_fault_buffer_entry_size_unsupported,
+            .parse_non_replayable_entry = uvm_hal_maxwell_fault_buffer_parse_non_replayable_entry_unsupported,
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM200,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM000,
+        .u.fault_buffer_ops = {}
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GP100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM200,
         .u.fault_buffer_ops = {
             .enable_replayable_faults  = uvm_hal_pascal_enable_replayable_faults,
             .disable_replayable_faults = uvm_hal_pascal_disable_replayable_faults,
@@ -342,26 +376,89 @@ static uvm_hal_class_ops_t fault_buffer_table[] =
             .entry_is_valid = uvm_hal_pascal_fault_buffer_entry_is_valid,
             .entry_clear_valid = uvm_hal_pascal_fault_buffer_entry_clear_valid,
             .entry_size = uvm_hal_pascal_fault_buffer_entry_size,
-            .parse_non_replayable_entry = uvm_hal_pascal_fault_buffer_parse_non_replayable_entry_unsupported,
         }
     },
     {
-        .id = MMU_FAULT_BUFFER,
-        .parent_id = MAXWELL_FAULT_BUFFER_A,
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GP100,
         .u.fault_buffer_ops = {
             .read_put = uvm_hal_volta_fault_buffer_read_put,
             .read_get = uvm_hal_volta_fault_buffer_read_get,
             .write_get = uvm_hal_volta_fault_buffer_write_get,
+            .get_ve_id = uvm_hal_volta_fault_buffer_get_ve_id,
             .parse_entry = uvm_hal_volta_fault_buffer_parse_entry,
             .parse_non_replayable_entry = uvm_hal_volta_fault_buffer_parse_non_replayable_entry,
-        },
-    }
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_TU100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
+        .u.fault_buffer_ops = {
+            .disable_replayable_faults = uvm_hal_turing_disable_replayable_faults,
+            .clear_replayable_faults = uvm_hal_turing_clear_replayable_faults,
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GA100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_TU100,
+        .u.fault_buffer_ops = {}
+    },
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 };
 
 static uvm_hal_class_ops_t access_counter_buffer_table[] =
 {
     {
-        .id = ACCESS_COUNTER_NOTIFY_BUFFER,
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM000,
+        .u.access_counter_buffer_ops = {
+            .enable_access_counter_notifications  = uvm_hal_maxwell_enable_access_counter_notifications_unsupported,
+            .disable_access_counter_notifications = uvm_hal_maxwell_disable_access_counter_notifications_unsupported,
+            .clear_access_counter_notifications = uvm_hal_maxwell_clear_access_counter_notifications_unsupported,
+            .parse_entry = uvm_hal_maxwell_access_counter_buffer_parse_entry_unsupported,
+            .entry_is_valid = uvm_hal_maxwell_access_counter_buffer_entry_is_valid_unsupported,
+            .entry_clear_valid = uvm_hal_maxwell_access_counter_buffer_entry_clear_valid_unsupported,
+            .entry_size = uvm_hal_maxwell_access_counter_buffer_entry_size_unsupported,
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM200,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM000,
+        .u.access_counter_buffer_ops = {}
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GP100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM200,
+        .u.access_counter_buffer_ops = {}
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GP100,
         .u.access_counter_buffer_ops = {
             .enable_access_counter_notifications  = uvm_hal_volta_enable_access_counter_notifications,
             .disable_access_counter_notifications = uvm_hal_volta_disable_access_counter_notifications,
@@ -370,8 +467,46 @@ static uvm_hal_class_ops_t access_counter_buffer_table[] =
             .entry_is_valid = uvm_hal_volta_access_counter_buffer_entry_is_valid,
             .entry_clear_valid = uvm_hal_volta_access_counter_buffer_entry_clear_valid,
             .entry_size = uvm_hal_volta_access_counter_buffer_entry_size,
-        },
-    }
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_TU100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
+        .u.access_counter_buffer_ops = {
+            .disable_access_counter_notifications = uvm_hal_turing_disable_access_counter_notifications,
+            .clear_access_counter_notifications = uvm_hal_turing_clear_access_counter_notifications,
+        }
+    },
+    {
+        .id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GA100,
+        .parent_id = NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_TU100,
+        .u.access_counter_buffer_ops = {}
+    },
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 };
 
 static inline uvm_hal_class_ops_t *ops_find_by_id(uvm_hal_class_ops_t *table, NvU32 row_count, NvU32 id)
@@ -459,7 +594,9 @@ NV_STATUS uvm_hal_init_table(void)
         return status;
     }
 
-    status = ops_init_from_parent(fault_buffer_table, ARRAY_SIZE(fault_buffer_table), FAULT_BUFFER_OP_COUNT,
+    status = ops_init_from_parent(fault_buffer_table,
+                                  ARRAY_SIZE(fault_buffer_table),
+                                  FAULT_BUFFER_OP_COUNT,
                                   offsetof(uvm_hal_class_ops_t, u.fault_buffer_ops));
     if (status != NV_OK) {
         UVM_ERR_PRINT("ops_init_from_parent(fault_buffer_table) failed: %s\n", nvstatusToString(status));
@@ -507,40 +644,24 @@ NV_STATUS uvm_hal_init_gpu(uvm_parent_gpu_t *parent_gpu)
 
     parent_gpu->arch_hal = &class_ops->u.arch_ops;
 
-    // Initialize the fault buffer hal only for GPUs supporting faults (with
-    // non-0 fault buffer class).
-    if (gpu_info->faultBufferClass != 0) {
-        class_ops = ops_find_by_id(fault_buffer_table, ARRAY_SIZE(fault_buffer_table), gpu_info->faultBufferClass);
-        if (class_ops == NULL) {
-            UVM_ERR_PRINT("Unsupported fault buffer class: 0x%X, GPU %s\n",
-                          gpu_info->faultBufferClass,
-                          parent_gpu->name);
-            return NV_ERR_INVALID_CLASS;
-        }
-        parent_gpu->fault_buffer_hal = &class_ops->u.fault_buffer_ops;
-    }
-    else {
-        parent_gpu->fault_buffer_hal = NULL;
+    class_ops = ops_find_by_id(fault_buffer_table, ARRAY_SIZE(fault_buffer_table), gpu_info->gpuArch);
+    if (class_ops == NULL) {
+        UVM_ERR_PRINT("Fault buffer HAL not found, GPU %s, arch: 0x%X\n", parent_gpu->name, gpu_info->gpuArch);
+        return NV_ERR_INVALID_CLASS;
     }
 
-    // Initialize the access counter buffer hal only for GPUs supporting access
-    // counters (with non-0 access counter buffer class).
-    if (gpu_info->accessCounterBufferClass != 0) {
-        class_ops = ops_find_by_id(access_counter_buffer_table,
-                                   ARRAY_SIZE(access_counter_buffer_table),
-                                   gpu_info->accessCounterBufferClass);
-        if (class_ops == NULL) {
-            UVM_ERR_PRINT("Unsupported access counter buffer class: 0x%X, GPU %s\n",
-                          gpu_info->accessCounterBufferClass,
-                          parent_gpu->name);
-            return NV_ERR_INVALID_CLASS;
-        }
-        parent_gpu->access_counter_buffer_hal = &class_ops->u.access_counter_buffer_ops;
-    }
-    else {
-        parent_gpu->access_counter_buffer_hal = NULL;
+    parent_gpu->fault_buffer_hal = &class_ops->u.fault_buffer_ops;
+
+    class_ops = ops_find_by_id(access_counter_buffer_table,
+                               ARRAY_SIZE(access_counter_buffer_table),
+                               gpu_info->gpuArch);
+    if (class_ops == NULL) {
+        UVM_ERR_PRINT("Access counter HAL not found, GPU %s, arch: 0x%X\n", parent_gpu->name, gpu_info->gpuArch);
+        return NV_ERR_INVALID_CLASS;
     }
 
+    parent_gpu->access_counter_buffer_hal = &class_ops->u.access_counter_buffer_ops;
+
     return NV_OK;
 }
 
