@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2013-2020 NVIDIA Corporation
+    Copyright (c) 2013-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -32,7 +32,6 @@
 #ifndef _UVM_LINUX_H
 #define _UVM_LINUX_H
 
-#include "nv-misc.h"
 #include "nvtypes.h"
 
 #include "nv-time.h"
@@ -89,17 +88,15 @@
 
 #include "nv-kthread-q.h"
 
-#if defined(NV_BUILD_MAY_USE_GPL_SYMBOLS)
-#define UVM_MAY_USE_GPL_SYMBOLS() 1
-#else
-#define UVM_MAY_USE_GPL_SYMBOLS() 0
-#endif
 
-#if UVM_MAY_USE_GPL_SYMBOLS() == 1 && NV_KTHREAD_Q_SUPPORTS_AFFINITY() == 1 && defined(NV_CPUMASK_OF_NODE_PRESENT)
-#define UVM_THREAD_AFFINITY_SUPPORTED() 1
-#else
-#define UVM_THREAD_AFFINITY_SUPPORTED() 0
-#endif
+
+
+
+
+
+
+    #define UVM_THREAD_AFFINITY_SUPPORTED() 0
+
 
 // The ARM arch lacks support for cpumask_of_node() until kernel 4.7. It was
 // added via commit1a2db300348b ("arm64, numa: Add NUMA support for arm64
@@ -115,21 +112,17 @@ static inline const struct cpumask *uvm_cpumask_of_node(int node)
 #endif
 }
 
-// TODO: Bug 1772628: remove the "defined(NV_BUILD_SUPPORTS_HMM)" part,
-// once the HMM (Heterogeneous Memory Management Linux kernel feature) patch
-// gets submitted to the Linux kernel.
-//
-// Until HMM is part of the upstream kernel, NV_BUILD_SUPPORTS_HMM will not be
-// defined in the source code. However, our kernel module build allows you to
-// specify this, via:
-//     "make modules NV_BUILD_SUPPORTS_HMM=1"
-//
-#if defined(CONFIG_HMM) && defined(NV_BUILD_SUPPORTS_HMM)
-    #include <linux/hmm.h>
-    #define UVM_IS_CONFIG_HMM() 1
-#else
+
+
+
+
+
+
+
+
+
     #define UVM_IS_CONFIG_HMM() 0
-#endif
+
 
 // Various issues prevent us from using mmu_notifiers in older kernels. These
 // include:
@@ -146,12 +139,20 @@ static inline const struct cpumask *uvm_cpumask_of_node(int node)
 //
 // The callback was added in commit 0f0a327fa12cd55de5e7f8c05a70ac3d047f405e,
 // v3.19 (2014-11-13).
-#if defined(NV_MMU_NOTIFIER_OPS_HAS_INVALIDATE_RANGE) && \
-  (UVM_MAY_USE_GPL_SYMBOLS() == 1 || (defined(NVCPU_PPC64LE) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0)))
-    #define UVM_CAN_USE_MMU_NOTIFIERS() 1
-#else
-    #define UVM_CAN_USE_MMU_NOTIFIERS() 0
-#endif
+
+
+
+
+
+
+
+    #if defined(NV_MMU_NOTIFIER_OPS_HAS_INVALIDATE_RANGE) && \
+        defined(NVCPU_PPC64LE) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0)
+        #define UVM_CAN_USE_MMU_NOTIFIERS() 1
+    #else
+        #define UVM_CAN_USE_MMU_NOTIFIERS() 0
+    #endif
+
 
 // See bug 1707453 for further details about setting the minimum kernel version.
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 32)
