@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2019 NVIDIA Corporation
+    Copyright (c) 2016-2020 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -404,6 +404,9 @@ NV_STATUS uvm_va_range_map_rm_allocation(uvm_va_range_t *va_range,
         return NV_ERR_INVALID_ADDRESS;
 
     if (va_range->type == UVM_VA_RANGE_TYPE_EXTERNAL) {
+        // We should be never called with ext_gpu_map == NULL
+        // and UVM_VA_RANGE_TYPE_EXTERNAL
+        UVM_ASSERT(ext_gpu_map != NULL);
         node = &ext_gpu_map->node;
         pt_range_vec = &ext_gpu_map->pt_range_vec;
     }
@@ -489,31 +492,60 @@ out:
 
 static bool uvm_api_mapping_type_invalid(UvmGpuMappingType map_type)
 {
+    BUILD_BUG_ON((int)UvmGpuMappingTypeDefault != (int)UvmRmGpuMappingTypeDefault);
+    BUILD_BUG_ON((int)UvmGpuMappingTypeReadWriteAtomic != (int)UvmRmGpuMappingTypeReadWriteAtomic);
+    BUILD_BUG_ON((int)UvmGpuMappingTypeReadWrite != (int)UvmRmGpuMappingTypeReadWrite);
+    BUILD_BUG_ON((int)UvmGpuMappingTypeReadOnly != (int)UvmRmGpuMappingTypeReadOnly);
+    BUILD_BUG_ON((int)UvmGpuMappingTypeCount != (int)UvmRmGpuMappingTypeCount);
+
     switch (map_type) {
         case UvmGpuMappingTypeDefault:
         case UvmGpuMappingTypeReadWriteAtomic:
         case UvmGpuMappingTypeReadWrite:
         case UvmGpuMappingTypeReadOnly:
             return false;
+        default:
+            return true;
     }
-    return true;
 }
 
 static bool uvm_api_caching_type_invalid(UvmGpuCachingType cache_type)
 {
+    BUILD_BUG_ON((int)UvmGpuCachingTypeDefault != (int)UvmRmGpuCachingTypeDefault);
+    BUILD_BUG_ON((int)UvmGpuCachingTypeForceUncached != (int)UvmRmGpuCachingTypeForceUncached);
+    BUILD_BUG_ON((int)UvmGpuCachingTypeForceCached != (int)UvmRmGpuCachingTypeForceCached);
+    BUILD_BUG_ON((int)UvmGpuCachingTypeCount != (int)UvmRmGpuCachingTypeCount);
+
     switch (cache_type) {
         case UvmGpuCachingTypeDefault:
         case UvmGpuCachingTypeForceUncached:
         case UvmGpuCachingTypeForceCached:
             return false;
+        default:
+            return true;
     }
-    return true;
 }
 
 static bool uvm_api_kind_type_invalid(UvmGpuFormatType format_type,
                                       UvmGpuFormatElementBits element_bits,
                                       UvmGpuCompressionType compression_type)
 {
+    BUILD_BUG_ON((int)UvmGpuFormatTypeDefault != (int)UvmRmGpuFormatTypeDefault);
+    BUILD_BUG_ON((int)UvmGpuFormatTypeBlockLinear != (int)UvmRmGpuFormatTypeBlockLinear);
+    BUILD_BUG_ON((int)UvmGpuFormatTypeCount != (int)UvmRmGpuFormatTypeCount);
+
+    BUILD_BUG_ON((int)UvmGpuFormatElementBitsDefault != (int)UvmRmGpuFormatElementBitsDefault);
+    BUILD_BUG_ON((int)UvmGpuFormatElementBits8 != (int)UvmRmGpuFormatElementBits8);
+    BUILD_BUG_ON((int)UvmGpuFormatElementBits16 != (int)UvmRmGpuFormatElementBits16);
+    BUILD_BUG_ON((int)UvmGpuFormatElementBits32 != (int)UvmRmGpuFormatElementBits32);
+    BUILD_BUG_ON((int)UvmGpuFormatElementBits64 != (int)UvmRmGpuFormatElementBits64);
+    BUILD_BUG_ON((int)UvmGpuFormatElementBits128 != (int)UvmRmGpuFormatElementBits128);
+    BUILD_BUG_ON((int)UvmGpuFormatElementBitsCount != (int)UvmRmGpuFormatElementBitsCount);
+
+    BUILD_BUG_ON((int)UvmGpuCompressionTypeDefault != (int)UvmRmGpuCompressionTypeDefault);
+    BUILD_BUG_ON((int)UvmGpuCompressionTypeEnabledNoPlc != (int)UvmRmGpuCompressionTypeEnabledNoPlc);
+    BUILD_BUG_ON((int)UvmGpuCompressionTypeCount != (int)UvmRmGpuCompressionTypeCount);
+
     if (compression_type >= UvmGpuCompressionTypeCount)
         return true;
 
@@ -552,7 +584,8 @@ static void uvm_release_rm_handle(struct nv_kref *ref)
     if (mem_handle->rm_handle) {
         NV_STATUS status;
 
-        status = uvm_rm_locked_call(nvUvmInterfaceFreeDupedHandle(mem_handle->gpu->rm_address_space, mem_handle->rm_handle));
+        status = uvm_rm_locked_call(nvUvmInterfaceFreeDupedHandle(uvm_gpu_device_handle(mem_handle->gpu),
+                                                                  mem_handle->rm_handle));
         UVM_ASSERT(status == NV_OK);
     }
     uvm_kvfree(mem_handle);
@@ -868,7 +901,7 @@ static NV_STATUS uvm_map_external_allocation_on_gpu(uvm_va_range_t *va_range,
     // Dup the memory. This verifies the input handles, takes a ref count on the
     // physical allocation so it can't go away under us, and returns us the
     // allocation info.
-    status = uvm_rm_locked_call(nvUvmInterfaceDupMemory(mapping_gpu->rm_address_space,
+    status = uvm_rm_locked_call(nvUvmInterfaceDupMemory(uvm_gpu_device_handle(mapping_gpu),
                                                         user_rm_mem->user_client,
                                                         user_rm_mem->user_object,
                                                         &ext_gpu_map->mem_handle->rm_handle,
