@@ -80,14 +80,10 @@ static NV_STATUS check_accessible_from_gpu(uvm_gpu_t *gpu, uvm_mem_t *mem)
                 mem_gpu_phys.aperture == UVM_APERTURE_SYS ? "sys" : "vid", mem_gpu_phys.address, mem->chunk_size);
         TEST_CHECK_GOTO(status == NV_OK, done);
 
-        if (uvm_mem_is_vidmem(mem) && mem->chunk_size == gpu->big_page.internal_size)
-            mem_gpu_phys = uvm_mmu_gpu_address_for_big_page_physical(mem_gpu_phys, gpu);
-
         gpu->parent->ce_hal->memcopy(&push, mem_gpu_phys, sys_mem_gpu_virt, size_this_time);
 
         uvm_push_end(&push);
-        status = uvm_tracker_add_push(&tracker, &push);
-        TEST_CHECK_GOTO(status == NV_OK, done);
+        TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, &push), done);
     }
 
     status = uvm_tracker_wait(&tracker);
@@ -110,8 +106,7 @@ static NV_STATUS check_accessible_from_gpu(uvm_gpu_t *gpu, uvm_mem_t *mem)
         gpu->parent->ce_hal->memcopy(&push, sys_mem_gpu_phys, mem_gpu_virt, size_this_time);
 
         uvm_push_end(&push);
-        status = uvm_tracker_add_push(&tracker, &push);
-        TEST_CHECK_GOTO(status == NV_OK, done);
+        TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, &push), done);
     }
 
     status = uvm_tracker_wait(&tracker);
@@ -222,7 +217,7 @@ static NV_STATUS test_alloc_sysmem(uvm_va_space_t *va_space, NvU32 page_size, si
     TEST_CHECK_GOTO(test_map_cpu(mem) == NV_OK, error);
 
     for_each_va_space_gpu(gpu, va_space)
-        TEST_CHECK_GOTO(test_map_gpu(mem, gpu) == NV_OK, error);
+        TEST_NV_CHECK_GOTO(test_map_gpu(mem, gpu), error);
 
     *mem_out = mem;
 
@@ -253,7 +248,7 @@ static NV_STATUS test_alloc_vidmem(uvm_gpu_t *gpu, NvU32 page_size, size_t size,
             TEST_CHECK_GOTO(mem->chunk_size == UVM_PAGE_SIZE_4K || mem->chunk_size <= size, error);
     }
 
-    TEST_CHECK_GOTO(test_map_gpu(mem, gpu) == NV_OK, error);
+    TEST_NV_CHECK_GOTO(test_map_gpu(mem, gpu), error);
 
     if (gpu->parent->numa_info.enabled && (page_size == UVM_PAGE_SIZE_DEFAULT || page_size >= PAGE_SIZE))
         TEST_CHECK_GOTO(test_map_cpu(mem) == NV_OK, error);
@@ -442,8 +437,8 @@ static NV_STATUS test_basic(uvm_va_space_t *va_space)
 
 static NV_STATUS tests(uvm_va_space_t *va_space)
 {
-    TEST_CHECK_RET(test_basic(va_space) == NV_OK);
-    TEST_CHECK_RET(test_all(va_space) == NV_OK);
+    TEST_NV_CHECK_RET(test_basic(va_space));
+    TEST_NV_CHECK_RET(test_all(va_space));
 
     return NV_OK;
 }
