@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2018-2019 NVIDIA Corporation
+    Copyright (c) 2018-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -34,98 +34,12 @@
 #include "uvm_migrate_pageable.h"
 #include "uvm_populate_pageable.h"
 
-#include <linux/migrate.h>
-
-#if defined(CONFIG_MIGRATE_VMA_HELPER)
+#ifdef UVM_MIGRATE_VMA_SUPPORTED
 
 static struct kmem_cache *g_uvm_migrate_vma_state_cache __read_mostly;
 
 static const gfp_t g_migrate_vma_gfp_flags = NV_UVM_GFP_FLAGS | GFP_HIGHUSER_MOVABLE | __GFP_THISNODE;
 
-// The calls to migrate_vma are capped at 32MB to set an upper bound on the
-// amount of metadata that needs to be allocated for the operation. This number
-// was chosen because performance seems to plateau at this size.
-#define UVM_MIGRATE_VMA_MAX_SIZE (32UL * 1024 * 1024)
-#define UVM_MIGRATE_VMA_MAX_PAGES (UVM_MIGRATE_VMA_MAX_SIZE / PAGE_SIZE)
-
-typedef struct
-{
-    // Input parameters
-    //
-    uvm_va_space_t     *va_space;
-    uvm_processor_id_t  dst_id;
-    int                 dst_node_id;
-
-    // Output parameters
-    //
-    // Error code. This only signals errors in internal UVM operations.
-    // Pages that failed allocation or could not be populated are communicated
-    // using the fields below.
-    NV_STATUS           status;
-
-    // Whether the masks below must be examined by the caller
-    bool                unpopulated_pages : 1;
-    bool                allocation_failed  : 1;
-
-    // Mask of pages that couldn't be made resident on the destination because
-    // (a) they are backed with data but pages are not populated (e.g. in swap),
-    // (b) pages are not backed with any data yet but were not populated
-    // due to the vma not being READ_WRITE, as it would not charge the pages to
-    // the process properly.
-    DECLARE_BITMAP(unpopulated_pages_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-
-    // Mask of pages that failed allocation on the destination
-    DECLARE_BITMAP(allocation_failed_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-
-    // Global state managed by the caller
-    //
-    // These are scratch masks that can be used by the migrate_vma caller to
-    // save output page masks and orchestrate the migrate_vma
-    // retries/population calls if needed.
-    DECLARE_BITMAP(scratch1_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-    DECLARE_BITMAP(scratch2_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-
-    // Arrays used by migrate_vma to store the src/dst pfns
-    unsigned long dst_pfn_array[UVM_MIGRATE_VMA_MAX_PAGES];
-    unsigned long src_pfn_array[UVM_MIGRATE_VMA_MAX_PAGES];
-
-    // Internal state
-    //
-    uvm_tracker_t tracker;
-
-    DECLARE_BITMAP(pending_page_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-
-    struct {
-        // Array of page IOMMU mappings created during allocate_and_copy.
-        // Required when using SYS aperture. They are freed in
-        // finalize_and_map. Also keep an array with the GPUs for which the
-        // mapping was created.
-        NvU64              addrs[UVM_MIGRATE_VMA_MAX_PAGES];
-        uvm_gpu_t    *addrs_gpus[UVM_MIGRATE_VMA_MAX_PAGES];
-
-        // Mask of pages with entries in the dma address arrays above
-        DECLARE_BITMAP(page_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-
-        // Number of pages for which IOMMU mapping were created
-        unsigned  long num_pages;
-    } dma;
-
-    // Processors where pages are resident before calling migrate_vma
-    uvm_processor_mask_t src_processors;
-
-    // Array of per-processor page masks with the pages that are resident
-    // before calling migrate_vma.
-    struct {
-        DECLARE_BITMAP(page_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-    } processors[UVM_ID_MAX_PROCESSORS];
-
-    // Number of pages in the migrate_vma call
-    unsigned long num_pages;
-
-    // Number of pages that are directly populated on the destination
-    unsigned long num_populate_anon_pages;
-} migrate_vma_state_t;
-
 // Compute the address needed for copying_gpu to access the given page,
 // resident on resident_id.
 static NV_STATUS migrate_vma_page_copy_address(struct page *page,
@@ -135,7 +49,7 @@ static NV_STATUS migrate_vma_page_copy_address(struct page *page,
                                                migrate_vma_state_t *state,
                                                uvm_gpu_address_t *gpu_addr)
 {
-    uvm_va_space_t *va_space = state->va_space;
+    uvm_va_space_t *va_space = state->uvm_migrate_args->va_space;
     uvm_gpu_t *owning_gpu = UVM_ID_IS_CPU(resident_id)? NULL: uvm_va_space_get_gpu(va_space, resident_id);
     const bool can_copy_from = uvm_processor_mask_test(&va_space->can_copy_from[uvm_id_value(copying_gpu->id)],
                                                        resident_id);
@@ -297,6 +211,8 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
 {
     unsigned long i;
     const bool is_rw = vma->vm_flags & VM_WRITE;
+    uvm_migrate_args_t *uvm_migrate_args = state->uvm_migrate_args;
+    uvm_processor_id_t dst_id = uvm_migrate_args->dst_id;
 
     UVM_ASSERT(vma_is_anonymous(vma));
 
@@ -335,9 +251,9 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
                 // Populate PROT_WRITE vmas in migrate_vma so we can use the
                 // GPU's copy engines
                 if (state->num_populate_anon_pages++ == 0)
-                    bitmap_zero(state->processors[uvm_id_value(state->dst_id)].page_mask, state->num_pages);
+                    bitmap_zero(state->processors[uvm_id_value(dst_id)].page_mask, state->num_pages);
 
-                __set_bit(i, state->processors[uvm_id_value(state->dst_id)].page_mask);
+                __set_bit(i, state->processors[uvm_id_value(dst_id)].page_mask);
             }
             else {
                 __clear_bit(i, state->pending_page_mask);
@@ -356,22 +272,22 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
         src_nid = page_to_nid(src_page);
 
         // Already at destination
-        if (src_nid == state->dst_node_id) {
+        if (src_nid == uvm_migrate_args->dst_node_id) {
             __clear_bit(i, state->pending_page_mask);
             continue;
         }
 
         // Already resident on a CPU node, don't move
-        if (UVM_ID_IS_CPU(state->dst_id) && node_state(src_nid, N_CPU)) {
+        if (UVM_ID_IS_CPU(dst_id) && node_state(src_nid, N_CPU)) {
             __clear_bit(i, state->pending_page_mask);
             continue;
         }
 
-        src_gpu = get_gpu_from_node_id(state->va_space, src_nid);
+        src_gpu = get_gpu_from_node_id(uvm_migrate_args->va_space, src_nid);
 
         // Already resident on a node with no CPUs that doesn't belong to a
         // GPU, don't move
-        if (UVM_ID_IS_CPU(state->dst_id) && !src_gpu) {
+        if (UVM_ID_IS_CPU(dst_id) && !src_gpu) {
             __clear_bit(i, state->pending_page_mask);
             continue;
         }
@@ -395,13 +311,14 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
 static struct page *migrate_vma_alloc_page(migrate_vma_state_t *state)
 {
     struct page *dst_page;
-    uvm_va_space_t *va_space = state->va_space;
+    uvm_migrate_args_t *uvm_migrate_args = state->uvm_migrate_args;
+    uvm_va_space_t *va_space = uvm_migrate_args->va_space;
 
     if (uvm_enable_builtin_tests && atomic_dec_if_positive(&va_space->test.migrate_vma_allocation_fail_nth) == 0) {
         dst_page = NULL;
     }
     else {
-        dst_page = alloc_pages_node(state->dst_node_id, g_migrate_vma_gfp_flags, 0);
+        dst_page = alloc_pages_node(uvm_migrate_args->dst_node_id, g_migrate_vma_gfp_flags, 0);
 
         // TODO: Bug 2399573: Linux commit
         // 183f6371aac2a5496a8ef2b0b0a68562652c3cdb introduced a bug that makes
@@ -410,7 +327,7 @@ static struct page *migrate_vma_alloc_page(migrate_vma_state_t *state)
         // whether the flag has been honored and abort the allocation,
         // otherwise. Remove this check when the fix is deployed on all
         // production systems.
-        if (dst_page && page_to_nid(dst_page) != state->dst_node_id) {
+        if (dst_page && page_to_nid(dst_page) != uvm_migrate_args->dst_node_id) {
             __free_page(dst_page);
             dst_page = NULL;
         }
@@ -426,11 +343,13 @@ static NV_STATUS migrate_vma_populate_anon_pages(struct vm_area_struct *vma,
                                                  migrate_vma_state_t *state)
 {
     NV_STATUS status = NV_OK;
-    unsigned long i;
-    unsigned long *page_mask = state->processors[uvm_id_value(state->dst_id)].page_mask;
-    uvm_push_t push;
+    uvm_migrate_args_t *uvm_migrate_args = state->uvm_migrate_args;
+    uvm_processor_id_t dst_id = uvm_migrate_args->dst_id;
+    unsigned long *page_mask = state->processors[uvm_id_value(dst_id)].page_mask;
     uvm_gpu_t *copying_gpu = NULL;
-    uvm_va_space_t *va_space = state->va_space;
+    uvm_va_space_t *va_space = uvm_migrate_args->va_space;
+    uvm_push_t push;
+    unsigned long i;
 
     // Nothing to do
     if (state->num_populate_anon_pages == 0)
@@ -454,18 +373,18 @@ static NV_STATUS migrate_vma_populate_anon_pages(struct vm_area_struct *vma,
         if (!copying_gpu) {
             // Try to get a GPU attached to the node being populated. If there
             // is none, use any of the GPUs registered in the VA space.
-            if (UVM_ID_IS_CPU(state->dst_id)) {
-                copying_gpu = uvm_va_space_find_first_gpu_attached_to_cpu_node(va_space, state->dst_node_id);
+            if (UVM_ID_IS_CPU(dst_id)) {
+                copying_gpu = uvm_va_space_find_first_gpu_attached_to_cpu_node(va_space, uvm_migrate_args->dst_node_id);
                 if (!copying_gpu)
                     copying_gpu = uvm_va_space_find_first_gpu(va_space);
             }
             else {
-                copying_gpu = uvm_va_space_get_gpu(va_space, state->dst_id);
+                copying_gpu = uvm_va_space_get_gpu(va_space, dst_id);
             }
 
             UVM_ASSERT(copying_gpu);
 
-            status = migrate_vma_zero_begin_push(va_space, state->dst_id, copying_gpu, start, outer - 1, &push);
+            status = migrate_vma_zero_begin_push(va_space, dst_id, copying_gpu, start, outer - 1, &push);
             if (status != NV_OK) {
                 __free_page(dst_page);
                 return status;
@@ -475,7 +394,7 @@ static NV_STATUS migrate_vma_populate_anon_pages(struct vm_area_struct *vma,
             uvm_push_set_flag(&push, UVM_PUSH_FLAG_CE_NEXT_PIPELINED);
         }
 
-        status = migrate_vma_page_copy_address(dst_page, i, state->dst_id, copying_gpu, state, &dst_address);
+        status = migrate_vma_page_copy_address(dst_page, i, dst_id, copying_gpu, state, &dst_address);
         if (status != NV_OK) {
             __free_page(dst_page);
             break;
@@ -513,10 +432,12 @@ static NV_STATUS migrate_vma_copy_pages_from(struct vm_area_struct *vma,
 {
     NV_STATUS status = NV_OK;
     uvm_push_t push;
-    uvm_gpu_t *copying_gpu = NULL;
     unsigned long i;
+    uvm_gpu_t *copying_gpu = NULL;
+    uvm_migrate_args_t *uvm_migrate_args = state->uvm_migrate_args;
+    uvm_processor_id_t dst_id = uvm_migrate_args->dst_id;
     unsigned long *page_mask = state->processors[uvm_id_value(src_id)].page_mask;
-    uvm_va_space_t *va_space = state->va_space;
+    uvm_va_space_t *va_space = uvm_migrate_args->va_space;
 
     UVM_ASSERT(!bitmap_empty(page_mask, state->num_pages));
 
@@ -540,7 +461,7 @@ static NV_STATUS migrate_vma_copy_pages_from(struct vm_area_struct *vma,
         }
 
         if (!copying_gpu) {
-            status = migrate_vma_copy_begin_push(va_space, state->dst_id, src_id, start, outer - 1, &push);
+            status = migrate_vma_copy_begin_push(va_space, dst_id, src_id, start, outer - 1, &push);
             if (status != NV_OK) {
                 __free_page(dst_page);
                 return status;
@@ -556,7 +477,7 @@ static NV_STATUS migrate_vma_copy_pages_from(struct vm_area_struct *vma,
         // the second call can't overwrite a dma addr set up by the first call.
         status = migrate_vma_page_copy_address(src_page, i, src_id, copying_gpu, state, &src_address);
         if (status == NV_OK)
-            status = migrate_vma_page_copy_address(dst_page, i, state->dst_id, copying_gpu, state, &dst_address);
+            status = migrate_vma_page_copy_address(dst_page, i, dst_id, copying_gpu, state, &dst_address);
 
         if (status != NV_OK) {
             __free_page(dst_page);
@@ -609,15 +530,12 @@ static NV_STATUS migrate_vma_copy_pages(struct vm_area_struct *vma,
     return NV_OK;
 }
 
-static void migrate_vma_alloc_and_copy(struct vm_area_struct *vma,
-                                       const unsigned long *src,
-                                       unsigned long *dst,
-                                       unsigned long start,
-                                       unsigned long outer,
-                                       void *private)
+void uvm_migrate_vma_alloc_and_copy(struct migrate_vma *args, migrate_vma_state_t *state)
 {
+    struct vm_area_struct *vma = args->vma;
+    unsigned long start = args->start;
+    unsigned long outer = args->end;
     NV_STATUS tracker_status;
-    migrate_vma_state_t *state = (migrate_vma_state_t *)private;
 
     uvm_tracker_init(&state->tracker);
 
@@ -626,12 +544,12 @@ static void migrate_vma_alloc_and_copy(struct vm_area_struct *vma,
     state->unpopulated_pages = false;
     state->allocation_failed = false;
 
-    migrate_vma_compute_masks(vma, src, state);
+    migrate_vma_compute_masks(vma, args->src, state);
 
-    state->status = migrate_vma_populate_anon_pages(vma, dst, start, outer, state);
+    state->status = migrate_vma_populate_anon_pages(vma, args->dst, start, outer, state);
 
     if (state->status == NV_OK)
-        state->status = migrate_vma_copy_pages(vma, src, dst, start, outer, state);
+        state->status = migrate_vma_copy_pages(vma, args->src, args->dst, start, outer, state);
 
     // Wait for tracker since all copies must have completed before returning
     tracker_status = uvm_tracker_wait_deinit(&state->tracker);
@@ -640,15 +558,27 @@ static void migrate_vma_alloc_and_copy(struct vm_area_struct *vma,
         state->status = tracker_status;
 }
 
-static void migrate_vma_finalize_and_map(struct vm_area_struct *vma,
-                                         const unsigned long *src,
-                                         const unsigned long *dst,
-                                         unsigned long start,
-                                         unsigned long outer,
-                                         void *private)
+void uvm_migrate_vma_alloc_and_copy_helper(struct vm_area_struct *vma,
+                                const unsigned long *src,
+                                unsigned long *dst,
+                                unsigned long start,
+                                unsigned long end,
+                                void *private)
 {
-    migrate_vma_state_t *state = (migrate_vma_state_t *)private;
+    struct migrate_vma args =
+    {
+        .vma = vma,
+        .dst = dst,
+        .src = (unsigned long *) src,
+        .start = start,
+        .end = end,
+    };
 
+    uvm_migrate_vma_alloc_and_copy(&args, (migrate_vma_state_t *) private);
+}
+
+void uvm_migrate_vma_finalize_and_map(struct migrate_vma *args, migrate_vma_state_t *state)
+{
     // Pages could fail migration if the page state could not be migrated by the
     // kernel. However, when this happens this likely means that the page is
     // populated somewhere, so we do nothing about it.
@@ -672,11 +602,60 @@ static void migrate_vma_finalize_and_map(struct vm_area_struct *vma,
                                   state->num_pages));
 }
 
-static struct migrate_vma_ops g_migrate_vma_ops =
+void uvm_migrate_vma_finalize_and_map_helper(struct vm_area_struct *vma,
+                                             const unsigned long *src,
+                                             const unsigned long *dst,
+                                             unsigned long start,
+                                             unsigned long end,
+                                             void *private)
 {
-    .alloc_and_copy = migrate_vma_alloc_and_copy,
-    .finalize_and_map = migrate_vma_finalize_and_map,
-};
+    struct migrate_vma args =
+    {
+        .vma = vma,
+        .dst = (unsigned long *) dst,
+        .src = (unsigned long *) src,
+        .start = start,
+        .end = end,
+    };
+
+    uvm_migrate_vma_finalize_and_map(&args, (migrate_vma_state_t *) private);
+}
+
+static NV_STATUS nv_migrate_vma(struct migrate_vma *args, migrate_vma_state_t *state)
+{
+    int ret;
+
+#if defined(CONFIG_MIGRATE_VMA_HELPER)
+    static const struct migrate_vma_ops uvm_migrate_vma_ops =
+    {
+        .alloc_and_copy = uvm_migrate_vma_alloc_and_copy_helper,
+        .finalize_and_map = uvm_migrate_vma_finalize_and_map_helper,
+    };
+
+    ret = migrate_vma(&uvm_migrate_vma_ops, args->vma, args->start, args->end, args->src, args->dst, state);
+    if (ret < 0)
+        return errno_to_nv_status(ret);
+#else // CONFIG_MIGRATE_VMA_HELPER
+
+#if defined(NV_MIGRATE_VMA_FLAGS_PRESENT)
+    args->flags = MIGRATE_VMA_SELECT_SYSTEM;
+#endif // NV_MIGRATE_VMA_FLAGS_PRESENT
+
+    ret = migrate_vma_setup(args);
+    if (ret < 0)
+        return errno_to_nv_status(ret);
+
+    uvm_migrate_vma_alloc_and_copy(args, state);
+    if (state->status == NV_OK) {
+        migrate_vma_pages(args);
+        uvm_migrate_vma_finalize_and_map(args, state);
+    }
+
+    migrate_vma_finalize(args);
+#endif // CONFIG_MIGRATE_VMA_HELPER
+
+    return state->status;
+}
 
 static NV_STATUS migrate_pageable_vma_populate_mask(struct vm_area_struct *vma,
                                                     unsigned long start,
@@ -709,23 +688,25 @@ static NV_STATUS migrate_pageable_vma_migrate_mask(struct vm_area_struct *vma,
                                                    const unsigned long *mask,
                                                    migrate_vma_state_t *migrate_vma_state)
 {
+    NV_STATUS status;
     const unsigned long num_pages = (outer - start) / PAGE_SIZE;
     unsigned long subregion_first = find_first_bit(mask, num_pages);
+    struct migrate_vma args =
+    {
+        .vma = vma,
+        .src = migrate_vma_state->src_pfn_array,
+        .dst = migrate_vma_state->dst_pfn_array,
+    };
 
     while (subregion_first < num_pages) {
         unsigned long subregion_outer = find_next_zero_bit(mask, num_pages, subregion_first + 1);
-        int ret = migrate_vma(&g_migrate_vma_ops,
-                              vma,
-                              start + subregion_first * PAGE_SIZE,
-                              start + subregion_outer * PAGE_SIZE,
-                              migrate_vma_state->src_pfn_array,
-                              migrate_vma_state->dst_pfn_array,
-                              migrate_vma_state);
-        if (ret < 0)
-            return errno_to_nv_status(ret);
 
-        if (migrate_vma_state->status != NV_OK)
-            return migrate_vma_state->status;
+        args.start = start + subregion_first * PAGE_SIZE;
+        args.end = start + subregion_outer * PAGE_SIZE;
+
+        status = nv_migrate_vma(&args, migrate_vma_state);
+        if (status != NV_OK)
+            return status;
 
         // We ignore allocation failure here as we are just retrying migration,
         // but pages must have already been populated by the caller
@@ -746,9 +727,17 @@ static NV_STATUS migrate_pageable_vma_region(struct vm_area_struct *vma,
     NV_STATUS status;
     const unsigned long num_pages = (outer - start) / PAGE_SIZE;
     struct mm_struct *mm = vma->vm_mm;
-    int ret;
     bool unpopulated_pages;
     bool allocation_failed;
+    uvm_migrate_args_t *uvm_migrate_args = migrate_vma_state->uvm_migrate_args;
+    struct migrate_vma args =
+    {
+        .vma = vma,
+        .src = migrate_vma_state->src_pfn_array,
+        .dst = migrate_vma_state->dst_pfn_array,
+        .start = start,
+        .end = outer,
+    };
 
     UVM_ASSERT(PAGE_ALIGNED(start));
     UVM_ASSERT(PAGE_ALIGNED(outer));
@@ -757,20 +746,11 @@ static NV_STATUS migrate_pageable_vma_region(struct vm_area_struct *vma,
     UVM_ASSERT(outer <= vma->vm_end);
     UVM_ASSERT(outer - start <= UVM_MIGRATE_VMA_MAX_SIZE);
     uvm_assert_mmap_lock_locked(mm);
-    uvm_assert_rwsem_locked(&migrate_vma_state->va_space->lock);
+    uvm_assert_rwsem_locked(&uvm_migrate_args->va_space->lock);
 
-    ret = migrate_vma(&g_migrate_vma_ops,
-                      vma,
-                      start,
-                      outer,
-                      migrate_vma_state->src_pfn_array,
-                      migrate_vma_state->dst_pfn_array,
-                      migrate_vma_state);
-    if (ret < 0)
-        return errno_to_nv_status(ret);
-
-    if (migrate_vma_state->status != NV_OK)
-        return migrate_vma_state->status;
+    status = nv_migrate_vma(&args, migrate_vma_state);
+    if (status != NV_OK)
+        return status;
 
     unpopulated_pages = migrate_vma_state->unpopulated_pages;
     allocation_failed = migrate_vma_state->allocation_failed;
@@ -802,7 +782,7 @@ static NV_STATUS migrate_pageable_vma_region(struct vm_area_struct *vma,
         // If the destination is the CPU, signal user-space to retry with a
         // different node. Otherwise, just try to populate anywhere in the
         // system
-        if (UVM_ID_IS_CPU(migrate_vma_state->dst_id)) {
+        if (UVM_ID_IS_CPU(uvm_migrate_args->dst_id)) {
             *next_addr = start + find_first_bit(migrate_vma_state->scratch2_mask, num_pages) * PAGE_SIZE;
             return NV_ERR_MORE_PROCESSING_REQUIRED;
         }
@@ -824,7 +804,8 @@ static NV_STATUS migrate_pageable_vma(struct vm_area_struct *vma,
 {
     NV_STATUS status = NV_OK;
     struct mm_struct *mm = vma->vm_mm;
-    uvm_va_space_t *va_space = migrate_vma_state->va_space;
+    uvm_migrate_args_t *uvm_migrate_args = migrate_vma_state->uvm_migrate_args;
+    uvm_va_space_t *va_space = uvm_migrate_args->va_space;
 
     UVM_ASSERT(PAGE_ALIGNED(start));
     UVM_ASSERT(PAGE_ALIGNED(outer));
@@ -849,11 +830,7 @@ static NV_STATUS migrate_pageable_vma(struct vm_area_struct *vma,
     while (start < outer) {
         const size_t region_size = min(outer - start, UVM_MIGRATE_VMA_MAX_SIZE);
 
-        status = migrate_pageable_vma_region(vma,
-                                             start,
-                                             start + region_size,
-                                             migrate_vma_state,
-                                             next_addr);
+        status = migrate_pageable_vma_region(vma, start, start + region_size, migrate_vma_state, next_addr);
         if (status == NV_ERR_MORE_PROCESSING_REQUIRED) {
             UVM_ASSERT(*next_addr >= start);
             UVM_ASSERT(*next_addr < outer);
@@ -868,17 +845,18 @@ static NV_STATUS migrate_pageable_vma(struct vm_area_struct *vma,
     return status;
 }
 
-static NV_STATUS migrate_pageable(struct mm_struct *mm,
-                                  const unsigned long start,
-                                  const unsigned long length,
-                                  migrate_vma_state_t * migrate_vma_state,
-                                  NvU64 *user_space_start,
-                                  NvU64 *user_space_length)
+static NV_STATUS migrate_pageable(migrate_vma_state_t *state)
 {
-    struct vm_area_struct *vma;
-    const unsigned long outer = start + length;
+    uvm_migrate_args_t *uvm_migrate_args = state->uvm_migrate_args;
+    uvm_va_space_t *va_space = uvm_migrate_args->va_space;
+    unsigned long start = uvm_migrate_args->start;
+    const unsigned long length = uvm_migrate_args->length;
+    unsigned long outer = start + length;
     unsigned long prev_outer = outer;
-    uvm_va_space_t *va_space = migrate_vma_state->va_space;
+    NvU64 *user_space_start = uvm_migrate_args->user_space_start;
+    NvU64 *user_space_length = uvm_migrate_args->user_space_length;
+    struct mm_struct *mm = uvm_migrate_args->mm;
+    struct vm_area_struct *vma;
 
     UVM_ASSERT(PAGE_ALIGNED(start));
     UVM_ASSERT(PAGE_ALIGNED(length));
@@ -890,17 +868,25 @@ static NV_STATUS migrate_pageable(struct mm_struct *mm,
     // on one vma at a time
     for (; vma && (vma->vm_start <= prev_outer); vma = vma->vm_next) {
         unsigned long next_addr = 0;
-        NV_STATUS status = migrate_pageable_vma(vma,
-                                                start,
-                                                outer,
-                                                migrate_vma_state,
-                                                &next_addr);
+        NV_STATUS status = migrate_pageable_vma(vma, start, outer, state, &next_addr);
         if (status == NV_WARN_NOTHING_TO_DO) {
             NV_STATUS populate_status = NV_OK;
 
             UVM_ASSERT(!vma_is_anonymous(vma) || uvm_processor_mask_empty(&va_space->registered_gpus));
 
-            populate_status = uvm_populate_pageable_vma(vma, start, outer - start, 0);
+            // We can't use migrate_vma to move the pages as desired. Normally
+            // this fallback path is supposed to populate the memory then inform
+            // user mode that it should call move_pages, but that move_pages
+            // call won't work as expected if the caller is in the wrong
+            // process. Make that failure explicit so the caller is aware that
+            // move_pages won't behave as expected.
+            //
+            // If the caller is a kernel thread, such as the GPU BH, continue
+            // with population since there's no move_pages fallback.
+            if (current->mm != mm && !(current->flags & PF_KTHREAD))
+                return NV_ERR_NOT_SUPPORTED;
+
+            populate_status = uvm_populate_pageable_vma(vma, start, length, 0);
             if (populate_status == NV_OK) {
                 *user_space_start = max(vma->vm_start, start);
                 *user_space_length = min(vma->vm_end, outer) - *user_space_start;
@@ -912,7 +898,7 @@ static NV_STATUS migrate_pageable(struct mm_struct *mm,
         else if (status == NV_ERR_MORE_PROCESSING_REQUIRED) {
             UVM_ASSERT(next_addr >= start);
             UVM_ASSERT(next_addr < outer);
-            UVM_ASSERT(UVM_ID_IS_CPU(migrate_vma_state->dst_id));
+            UVM_ASSERT(UVM_ID_IS_CPU(uvm_migrate_args->dst_id));
 
             *user_space_start = next_addr;
         }
@@ -930,43 +916,43 @@ static NV_STATUS migrate_pageable(struct mm_struct *mm,
     return NV_ERR_INVALID_ADDRESS;
 }
 
-NV_STATUS uvm_migrate_pageable(uvm_va_space_t *va_space,
-                               struct mm_struct *mm,
-                               const unsigned long start,
-                               const unsigned long length,
-                               uvm_processor_id_t dst_id,
-                               int dst_cpu_node_id,
-                               NvU64 *user_space_start,
-                               NvU64 *user_space_length)
+NV_STATUS uvm_migrate_pageable(uvm_migrate_args_t *uvm_migrate_args)
 {
     migrate_vma_state_t *migrate_vma_state = NULL;
     NV_STATUS status;
+    uvm_va_space_t *va_space = uvm_migrate_args->va_space;
+    uvm_processor_id_t dst_id = uvm_migrate_args->dst_id;
+    int dst_node_id = uvm_migrate_args->dst_node_id;
 
-    UVM_ASSERT(PAGE_ALIGNED(start));
-    UVM_ASSERT(PAGE_ALIGNED(length));
-    uvm_assert_mmap_lock_locked(mm);
+    UVM_ASSERT(PAGE_ALIGNED(uvm_migrate_args->start));
+    UVM_ASSERT(PAGE_ALIGNED(uvm_migrate_args->length));
+    uvm_assert_mmap_lock_locked(uvm_migrate_args->mm);
 
-    // We only check that dst_cpu_node_id is a valid node in the system and it
-    // doesn't correspond to a GPU node. This is fine because alloc_pages_node
-    // will clamp the allocation to cpuset_current_mems_allowed, and
-    // uvm_migrate_pageable is only called from the process context
-    // (uvm_migrate). However, this would need to change if we wanted to call
-    // this function from a bottom half.
-    if (UVM_ID_IS_CPU(dst_id) &&
-        (!nv_numa_node_has_memory(dst_cpu_node_id) || get_gpu_from_node_id(va_space, dst_cpu_node_id) != NULL))
-        return NV_ERR_INVALID_ARGUMENT;
+    if (UVM_ID_IS_CPU(dst_id)) {
+        // We only check that dst_node_id is a valid node in the system and it
+        // doesn't correspond to a GPU node. This is fine because
+        // alloc_pages_node will clamp the allocation to
+        // cpuset_current_mems_allowed, and uvm_migrate_pageable is only called
+        // from process context (uvm_migrate). However, this would need to
+        // change if we wanted to call this function from a bottom half, so
+        // assert that we're in a user thread.
+        UVM_ASSERT(!(current->flags & PF_KTHREAD));
+
+        if (!nv_numa_node_has_memory(dst_node_id) || get_gpu_from_node_id(va_space, dst_node_id) != NULL)
+            return NV_ERR_INVALID_ARGUMENT;
+    }
+    else {
+        // Incoming dst_node_id is only valid if dst_id belongs to the CPU. Use
+        // dst_node_id as the GPU node id if dst_id doesn't belong to the CPU.
+        uvm_migrate_args->dst_node_id = uvm_gpu_numa_info(uvm_va_space_get_gpu(va_space, dst_id))->node_id;
+    }
 
     migrate_vma_state = kmem_cache_alloc(g_uvm_migrate_vma_state_cache, NV_UVM_GFP_FLAGS);
     if (!migrate_vma_state)
         return NV_ERR_NO_MEMORY;
 
-    migrate_vma_state->va_space    = va_space;
-    migrate_vma_state->dst_id      = dst_id;
-    migrate_vma_state->dst_node_id = UVM_ID_IS_CPU(dst_id)?
-                                         dst_cpu_node_id :
-                                         uvm_gpu_numa_info(uvm_va_space_get_gpu(va_space, dst_id))->node_id;
-
-    status = migrate_pageable(mm, start, length, migrate_vma_state, user_space_start, user_space_length);
+    migrate_vma_state->uvm_migrate_args = uvm_migrate_args;
+    status = migrate_pageable(migrate_vma_state);
 
     kmem_cache_free(g_uvm_migrate_vma_state_cache, migrate_vma_state);
 
