@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -32,12 +32,18 @@
 #include "uvm_push.h"
 #include "uvm_mem.h"
 #include "uvm_va_space.h"
-#include <stdarg.h>
 
-// The page tree has 5 levels on pascal, and the root is never freed by a normal 'put' operation
+
+
+
+
+
+
+// The page tree has 5 levels on pascal, and the root is never freed by a normal 'put' operation 
 // which leaves a maximum of 4 levels
 #define MAX_OPERATION_DEPTH 4
 
+
 // Wrappers for push begin handling channel_manager not being there when running
 // the page tree unit test
 #define page_tree_begin_acquire(tree, tracker, push, format, ...) ({                                                            \
@@ -419,7 +425,7 @@ static uvm_chunk_sizes_mask_t allocation_sizes_for_big_page_size(uvm_parent_gpu_
 
         for_each_set_bit(page_size_log2, &page_sizes, BITS_PER_LONG) {
             NvU32 i;
-            NvU32 page_size = 1 << page_size_log2;
+            NvU32 page_size = (NvU32)(1ULL << page_size_log2);
             for (i = 0; i <= hal->page_table_depth(page_size); i++)
                 alloc_sizes |= hal->allocation_size(i, page_size);
         }
@@ -700,6 +706,7 @@ NV_STATUS uvm_page_tree_init(uvm_gpu_t *gpu,
     uvm_mutex_init(&tree->lock, UVM_LOCK_ORDER_PAGE_TREE);
     tree->hal = gpu->parent->arch_hal->mmu_mode_hal(big_page_size);
     UVM_ASSERT(tree->hal != NULL);
+    UVM_ASSERT(MAX_OPERATION_DEPTH >= tree->hal->page_table_depth(UVM_PAGE_SIZE_AGNOSTIC));
     tree->gpu = gpu;
     tree->type = type;
     tree->big_page_size = big_page_size;
@@ -786,6 +793,8 @@ void uvm_page_tree_put_ptes_async(uvm_page_tree_t *tree, uvm_page_table_range_t
     uvm_membar_t membar_after_pde_clears = UVM_MEMBAR_GPU;
     uvm_membar_t membar_after_invalidate = UVM_MEMBAR_GPU;
 
+    UVM_ASSERT(tree->hal->page_table_depth(range->page_size) <= MAX_OPERATION_DEPTH); 
+
     uvm_mutex_lock(&tree->lock);
 
     // release the range
@@ -833,9 +842,9 @@ void uvm_page_tree_put_ptes_async(uvm_page_tree_t *tree, uvm_page_table_range_t
         if (dir->host_parent->phys_alloc.addr.aperture == UVM_APERTURE_SYS)
             membar_after_pde_clears = UVM_MEMBAR_SYS;
 
-        // add this dir to the queue of directories that should be freed once the
-        // tracker value of the associated PDE writes is known
-        UVM_ASSERT(free_count < MAX_OPERATION_DEPTH);
+        // Add this dir to the queue of directories that should be freed once
+        // the tracker value of the associated PDE writes is known.
+        UVM_ASSERT(free_count < tree->hal->page_table_depth(range->page_size));
         free_queue[free_count++] = dir;
 
         dir = parent;
@@ -1317,7 +1326,7 @@ NV_STATUS uvm_page_table_range_vec_init(uvm_page_tree_t *tree,
     range_vec->ranges = uvm_kvmalloc_zero(sizeof(*range_vec->ranges) * range_vec->range_count);
     if (!range_vec->ranges) {
         status = NV_ERR_NO_MEMORY;
-        goto error;
+        goto out;
     }
 
     for (i = 0; i < range_vec->range_count; ++i) {
@@ -1336,13 +1345,15 @@ NV_STATUS uvm_page_table_range_vec_init(uvm_page_tree_t *tree,
             UVM_ERR_PRINT("Failed to get PTEs for subrange %zd [0x%llx, 0x%llx) size 0x%llx, part of [0x%llx, 0x%llx)\n",
                     i, range_start, range_start + range_size, range_size,
                     start, size);
-            goto error;
+            goto out;
         }
     }
-    return uvm_page_tree_wait(tree);
 
-error:
-    uvm_page_table_range_vec_deinit(range_vec);
+    status = uvm_page_tree_wait(tree);
+
+out:
+    if (status != NV_OK)
+        uvm_page_table_range_vec_deinit(range_vec);
     return status;
 }
 
@@ -1633,11 +1644,11 @@ typedef struct identity_mapping_pte_maker_data_struct
 static NvU64 identity_mapping_pte_maker(uvm_page_table_range_vec_t *range_vec, NvU64 offset, void *data)
 {
     identity_mapping_pte_maker_data_t *vpdata = (identity_mapping_pte_maker_data_t *)data;
-    bool is_vol = (vpdata->aperture != UVM_APERTURE_VID);
+    NvU64 pte_flags = vpdata->aperture == UVM_APERTURE_VID ? UVM_MMU_PTE_FLAGS_CACHED : UVM_MMU_PTE_FLAGS_NONE;
     return range_vec->tree->hal->make_pte(vpdata->aperture,
                                           offset + vpdata->phys_offset,
                                           UVM_PROT_READ_WRITE_ATOMIC,
-                                          is_vol);
+                                          pte_flags);
 }
 
 static NV_STATUS create_identity_mapping(uvm_gpu_t *gpu,
@@ -1680,36 +1691,6 @@ static NV_STATUS create_identity_mapping(uvm_gpu_t *gpu,
     return NV_OK;
 }
 
-static NV_STATUS create_big_page_identity_mapping(uvm_gpu_t *gpu)
-{
-    NvU64 base = gpu->parent->big_page.identity_mapping_base;
-    NvU32 big_page_size = gpu->big_page.internal_size;
-    NvU64 size = UVM_ALIGN_UP(gpu->mem_info.max_allocatable_address + 1, big_page_size);
-
-    UVM_ASSERT(gpu->parent->big_page.swizzling);
-    UVM_ASSERT(base);
-    UVM_ASSERT(size);
-    UVM_ASSERT(big_page_size);
-
-    return create_identity_mapping(gpu,
-                                   base,
-                                   size,
-                                   UVM_APERTURE_VID,
-                                   0ULL,
-                                   big_page_size,
-                                   &gpu->big_page.identity_mapping.range_vec);
-}
-
-NV_STATUS uvm_mmu_create_big_page_identity_mappings(uvm_gpu_t *gpu)
-{
-    NV_STATUS status = NV_OK;
-
-    if (gpu->parent->big_page.swizzling)
-        status = create_big_page_identity_mapping(gpu);
-
-    return status;
-}
-
 NV_STATUS uvm_mmu_create_peer_identity_mappings(uvm_gpu_t *gpu, uvm_gpu_t *peer)
 {
     NvU32 page_size;
@@ -1747,18 +1728,6 @@ NV_STATUS uvm_mmu_create_peer_identity_mappings(uvm_gpu_t *gpu, uvm_gpu_t *peer)
                                    &peer_mapping->range_vec);
 }
 
-void uvm_mmu_destroy_big_page_identity_mappings(uvm_gpu_t *gpu)
-{
-    if (gpu->big_page.identity_mapping.range_vec) {
-        // The self identity mappings point to local GPU memory and hence can
-        // use a GPU membar for the invalidates.
-        (void)uvm_page_table_range_vec_clear_ptes(gpu->big_page.identity_mapping.range_vec, UVM_MEMBAR_GPU);
-    }
-
-    uvm_page_table_range_vec_destroy(gpu->big_page.identity_mapping.range_vec);
-    gpu->big_page.identity_mapping.range_vec = NULL;
-}
-
 void uvm_mmu_destroy_peer_identity_mappings(uvm_gpu_t *gpu, uvm_gpu_t *peer)
 {
     if (gpu->parent->peer_copy_mode == UVM_GPU_PEER_COPY_MODE_VIRTUAL) {
@@ -1772,18 +1741,6 @@ void uvm_mmu_destroy_peer_identity_mappings(uvm_gpu_t *gpu, uvm_gpu_t *peer)
     }
 }
 
-uvm_gpu_address_t uvm_mmu_gpu_address_for_big_page_physical(uvm_gpu_address_t physical, uvm_gpu_t *gpu)
-{
-    UVM_ASSERT(!physical.is_virtual);
-    UVM_ASSERT(physical.aperture == UVM_APERTURE_VID);
-
-    if (gpu->parent->big_page.swizzling) {
-        UVM_ASSERT(gpu->big_page.identity_mapping.range_vec != NULL);
-        return uvm_gpu_address_virtual(gpu->parent->big_page.identity_mapping_base + physical.address);
-    }
-    return physical;
-}
-
 void uvm_mmu_init_gpu_chunk_sizes(uvm_parent_gpu_t *parent_gpu)
 {
     uvm_chunk_sizes_mask_t sizes = page_sizes_for_big_page_size(parent_gpu, UVM_PAGE_SIZE_64K)  |
@@ -1841,13 +1798,8 @@ NV_STATUS uvm_test_invalidate_tlb(UVM_TEST_INVALIDATE_TLB_PARAMS *params, struct
         return NV_ERR_INVALID_PARAMETER;
     }
 
-    if (params->target_pdb_mode < UvmTargetPdbModeAll ||
-        params->target_pdb_mode > UvmTargetPdbModeOne) {
-        return NV_ERR_INVALID_PARAMETER;
-    }
-
     if (params->page_table_level < UvmInvalidatePageTableLevelAll ||
-        params->page_table_level > UvmInvalidatePageTableLevelPde3) {
+        params->page_table_level > UvmInvalidatePageTableLevelPde4) {
         return NV_ERR_INVALID_PARAMETER;
     }
 
@@ -1867,11 +1819,8 @@ NV_STATUS uvm_test_invalidate_tlb(UVM_TEST_INVALIDATE_TLB_PARAMS *params, struct
                             &push,
                             "Pushing test invalidate, GPU %s",
                             uvm_gpu_name(gpu));
-    if (status == NV_OK) {
-        gpu->parent->host_hal->tlb_invalidate_test(&push,
-                                                   uvm_page_tree_pdb(&gpu_va_space->page_tables)->addr,
-                                                   params);
-    }
+    if (status == NV_OK)
+        gpu->parent->host_hal->tlb_invalidate_test(&push, uvm_page_tree_pdb(&gpu_va_space->page_tables)->addr, params);
 
 unlock_exit:
     // Wait for the invalidation to be performed
