@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -56,7 +56,6 @@
 
 
 
-
 
 
 //
@@ -97,26 +96,6 @@
 // |                |
 // +----------------+ 0 (rm_va_base)
 //
-// Kepler:
-// +----------------+ 1TB
-// |                |
-// |   (not used)   |
-// |                |
-// ------------------ 896GB
-// |uvm_mem_t(128GB)| (uvm_mem_va_size)
-// ------------------ 768GB (uvm_mem_va_base)
-// |                |
-// |   (not used)   |
-// |                |
-// ------------------ <= 160GB 
-// |big page ident. | (max_allocatable_address)
-// |map (<= 32GB)   |
-// ------------------ 128GB
-// |                |
-// | rm_mem(128GB)  | (rm_va_size)
-// |                |
-// +----------------+ 0 (rm_va_base)
-//
 // The size of VA that should be reserved per peer identity mapping.
 // This should be at least the maximum amount of memory of any GPU.
 #define UVM_PEER_IDENTITY_VA_SIZE (128ull * 1024 * 1024 * 1024)
@@ -171,10 +150,25 @@ struct uvm_page_directory_struct
     uvm_page_directory_t *entries[0];
 };
 
+enum
+{
+    UVM_MMU_PTE_FLAGS_NONE                     = 0,
+
+    // data from the page pointed by the PTE may be cached incoherently by the
+    // GPU L2. This option may result in accessing stale data. For vidmem
+    // aperture, however, it is safe to use _CACHED, i.e., no memory request
+    // accesses stale data because L2 is in the datapath of the GPU memory.
+    UVM_MMU_PTE_FLAGS_CACHED                   = (1 << 0),
+
+    // Disable access counters to the page pointed by the PTE.
+    UVM_MMU_PTE_FLAGS_ACCESS_COUNTERS_DISABLED = (1 << 1),
+    UVM_MMU_PTE_FLAGS_MASK                     = (1 << 2) - 1
+};
+
 struct uvm_mmu_mode_hal_struct
 {
-    // bit pattern of a valid PTE
-    NvU64 (*make_pte)(uvm_aperture_t aperture, NvU64 address, uvm_prot_t prot, bool vol);
+    // bit pattern of a valid PTE. flags is a bitwise-or of UVM_MMU_PTE_FLAGS_*.
+    NvU64 (*make_pte)(uvm_aperture_t aperture, NvU64 address, uvm_prot_t prot, NvU64 flags);
 
     // bit pattern of a sked reflected PTE
     NvU64 (*make_sked_reflected_pte)(void);
@@ -486,23 +480,12 @@ NV_STATUS uvm_page_table_range_vec_write_ptes(uvm_page_table_range_vec_t *range_
 // membar.
 NV_STATUS uvm_page_table_range_vec_clear_ptes(uvm_page_table_range_vec_t *range_vec, uvm_membar_t tlb_membar);
 
-// Create the necessary big page identity mappings for the GPU
-NV_STATUS uvm_mmu_create_big_page_identity_mappings(uvm_gpu_t *gpu);
-
-// Destroy the big page identity mappings
-void uvm_mmu_destroy_big_page_identity_mappings(uvm_gpu_t *gpu);
-
 // Create peer identity mappings
 NV_STATUS uvm_mmu_create_peer_identity_mappings(uvm_gpu_t *gpu, uvm_gpu_t *peer);
 
 // Destroy peer identity mappings
 void uvm_mmu_destroy_peer_identity_mappings(uvm_gpu_t *gpu, uvm_gpu_t *peer);
 
-// Translate a phyiscal vidmem address to a virtual one using the big page
-// identity mapping if big page swizzling is enabled for the GPU. Otherwise,
-// return the address as-is.
-uvm_gpu_address_t uvm_mmu_gpu_address_for_big_page_physical(uvm_gpu_address_t physical, uvm_gpu_t *gpu);
-
 static NvU64 uvm_mmu_page_tree_entries(uvm_page_tree_t *tree, NvU32 depth, NvU32 page_size)
 {
     return 1ull << tree->hal->index_bits(depth, page_size);
