@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -29,9 +29,9 @@
 #include "uvm_tlb_batch.h"
 #include "uvm_mmu.h"
 #include "uvm_kvmalloc.h"
-// KEPLER_*
-#include "cla0b5.h"
-#include "cla06f.h"
+// MAXWELL_*
+#include "cla16f.h"
+#include "clb0b5.h"
 // PASCAL_*
 #include "clb069.h" // MAXWELL_FAULT_BUFFER_A
 #include "clc0b5.h"
@@ -43,6 +43,11 @@
 // AMPERE_*
 #include "clc56f.h"
 #include "clc6b5.h"
+
+
+
+
+
 // ARCHITECTURE_*
 #include "ctrl2080mc.h"
 
@@ -384,6 +389,34 @@ static NV_STATUS alloc_64k_memory(uvm_gpu_t *gpu)
     return NV_OK;
 }
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 static NV_STATUS alloc_adjacent_64k_memory(uvm_gpu_t *gpu)
 {
     uvm_page_tree_t tree;
@@ -1351,7 +1384,7 @@ static NV_STATUS test_range_vec(uvm_gpu_t *gpu, NvU32 big_page_size, NvU32 page_
     return status;
 }
 
-static NV_STATUS alloc_64k_memory_kepler(uvm_gpu_t *gpu)
+static NV_STATUS alloc_64k_memory_maxwell(uvm_gpu_t *gpu)
 {
     uvm_page_tree_t tree;
     uvm_page_table_range_t range;
@@ -1372,7 +1405,7 @@ static NV_STATUS alloc_64k_memory_kepler(uvm_gpu_t *gpu)
     return NV_OK;
 }
 
-static NV_STATUS alloc_128k_memory_kepler(uvm_gpu_t *gpu)
+static NV_STATUS alloc_128k_memory_maxwell(uvm_gpu_t *gpu)
 {
     uvm_page_tree_t tree;
     uvm_page_table_range_t range;
@@ -1430,7 +1463,7 @@ size_t get_page_sizes(uvm_gpu_t *gpu, NvU32 *page_sizes)
     page_sizes_bitvec = hal->page_sizes();
 
     for_each_set_bit(page_size_log2, &page_sizes_bitvec, BITS_PER_LONG) {
-        NvU32 page_size = 1 << page_size_log2;
+        NvU32 page_size = (NvU32)(1ULL << page_size_log2);
         UVM_ASSERT(count < MAX_NUM_PAGE_SIZES);
         page_sizes[count++] = page_size;
     }
@@ -1461,9 +1494,24 @@ static NV_STATUS entry_test_page_size_ampere(uvm_gpu_t *gpu, size_t page_size)
     return entry_test_page_size_volta(gpu, page_size);
 }
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 typedef NV_STATUS (*entry_test_page_size_func)(uvm_gpu_t *gpu, size_t page_size);
 
-static NV_STATUS entry_test_kepler(uvm_gpu_t *gpu)
+static NV_STATUS entry_test_maxwell(uvm_gpu_t *gpu)
 {
     static const NvU32 big_page_sizes[] = {UVM_PAGE_SIZE_64K, UVM_PAGE_SIZE_128K};
     NvU64 pde_bits;
@@ -1504,47 +1552,48 @@ static NV_STATUS entry_test_kepler(uvm_gpu_t *gpu)
                 TEST_CHECK_RET(hal->unmapped_pte(page_size) == 0x2);
         }
 
-        // volatile
+        // uncached, i.e., the sysmem data is not cached in GPU's L2
+        // cache. Clear the volatile bit.
         TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                      0x9999999000LL,
                                      UVM_PROT_READ_WRITE_ATOMIC,
-                                     true) == 0x599999991LL);
+                                     UVM_MMU_PTE_FLAGS_NONE) == 0x599999991LL);
 
-        // change volatile to false
+        // change to cached, set the volatile bit.
         TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                      0x9999999000LL,
                                      UVM_PROT_READ_WRITE_ATOMIC,
-                                     false) == 0x499999991LL);
+                                     UVM_MMU_PTE_FLAGS_CACHED) == 0x499999991LL);
 
         // remove atomic
         TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                      0x9999999000LL,
                                      UVM_PROT_READ_WRITE,
-                                     false) == 0x499999991LL);
+                                     UVM_MMU_PTE_FLAGS_CACHED) == 0x499999991LL);
 
         // read only
         TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                      0x9999999000LL,
                                      UVM_PROT_READ_ONLY,
-                                     false) == 0x8000000499999995LL);
+                                     UVM_MMU_PTE_FLAGS_CACHED) == 0x8000000499999995LL);
 
         // local video
         TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_VID,
                                      0x1BBBBBB000LL,
                                      UVM_PROT_READ_ONLY,
-                                     false) == 0x800000001BBBBBB5LL);
+                                     UVM_MMU_PTE_FLAGS_CACHED) == 0x800000001BBBBBB5LL);
 
         // peer 0
         TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_PEER_0,
                                      0x1BBBBBB000LL,
                                      UVM_PROT_READ_ONLY,
-                                     false) == 0x800000021BBBBBB5LL);
+                                     UVM_MMU_PTE_FLAGS_CACHED) == 0x800000021BBBBBB5LL);
 
         // peer 7
         TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_PEER_7,
                                      0x1BBBBBB000LL,
                                      UVM_PROT_READ_ONLY,
-                                     false) == 0x80000002FBBBBBB5LL);
+                                     UVM_MMU_PTE_FLAGS_CACHED) == 0x80000002FBBBBBB5LL);
     }
 
     return NV_OK;
@@ -1592,46 +1641,47 @@ static NV_STATUS entry_test_pascal(uvm_gpu_t *gpu, entry_test_page_size_func ent
     hal->make_pde(pde_bits, phys_allocs, 3);
     TEST_CHECK_RET(pde_bits[0] == 0x1BBBBBBBA && pde_bits[1] == 0x3999999999990C);
 
-    // volatile
+    // uncached, i.e., the sysmem data is not cached in GPU's L2 cache. Clear
+    // the volatile bit.
     TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                  0x399999999999000LL,
                                  UVM_PROT_READ_WRITE_ATOMIC,
-                                 true) == 0x3999999999990D);
+                                 UVM_MMU_PTE_FLAGS_NONE) == 0x3999999999990D);
 
-    // change volatile to false
+    // change to cached, set the volatile bit.
     TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                  0x399999999999000LL,
                                  UVM_PROT_READ_WRITE_ATOMIC,
-                                 false) == 0x39999999999905);
+                                 UVM_MMU_PTE_FLAGS_CACHED) == 0x39999999999905);
 
     // remove atomic
     TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                  0x399999999999000LL,
                                  UVM_PROT_READ_WRITE,
-                                 false) == 0x39999999999985);
+                                 UVM_MMU_PTE_FLAGS_CACHED) == 0x39999999999985);
 
     // read only
     TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_SYS,
                                  0x399999999999000LL,
                                  UVM_PROT_READ_ONLY,
-                                 false) == 0x399999999999C5);
+                                 UVM_MMU_PTE_FLAGS_CACHED) == 0x399999999999C5);
 
     // local video
     TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_VID,
                                  0x1BBBBBB000LL,
                                  UVM_PROT_READ_ONLY,
-                                 false) == 0x1BBBBBBC1);
+                                 UVM_MMU_PTE_FLAGS_CACHED) == 0x1BBBBBBC1);
 
     // peer 0
     TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_PEER_0,
                                  0x1BBBBBB000LL,
                                  UVM_PROT_READ_ONLY,
-                                 false) == 0x1BBBBBBC3);
+                                 UVM_MMU_PTE_FLAGS_CACHED) == 0x1BBBBBBC3);
 
     num_page_sizes = get_page_sizes(gpu, page_sizes);
 
     for (i = 0; i < num_page_sizes; i++)
-        entry_test_page_size(gpu, page_sizes[i]);
+        TEST_NV_CHECK_RET(entry_test_page_size(gpu, page_sizes[i]));
 
     return NV_OK;
 }
@@ -1644,13 +1694,14 @@ static NV_STATUS entry_test_volta(uvm_gpu_t *gpu, entry_test_page_size_func entr
     uvm_mmu_page_table_alloc_t *phys_allocs[2] = {NULL, NULL};
     uvm_mmu_page_table_alloc_t alloc_sys = fake_table_alloc(UVM_APERTURE_SYS, 0x399999999999000LL);
     uvm_mmu_page_table_alloc_t alloc_vid = fake_table_alloc(UVM_APERTURE_VID, 0x1BBBBBB000LL);
+
     // big versions have [11:8] set as well to test the page table merging
     uvm_mmu_page_table_alloc_t alloc_big_sys = fake_table_alloc(UVM_APERTURE_SYS, 0x399999999999900LL);
     uvm_mmu_page_table_alloc_t alloc_big_vid = fake_table_alloc(UVM_APERTURE_VID, 0x1BBBBBBB00LL);
 
     uvm_mmu_mode_hal_t *hal = gpu->parent->arch_hal->mmu_mode_hal(UVM_PAGE_SIZE_64K);
 
-    // Make sure cleared PDEs  work as expected
+    // Make sure cleared PDEs work as expected
     hal->make_pde(pde_bits, phys_allocs, 0);
     TEST_CHECK_RET(pde_bits[0] == 0);
 
@@ -1690,12 +1741,12 @@ static NV_STATUS entry_test_volta(uvm_gpu_t *gpu, entry_test_page_size_func entr
     TEST_CHECK_RET(hal->make_pte(UVM_APERTURE_PEER_0,
                                  0x5BBBBBBBB000LL,
                                  UVM_PROT_READ_ONLY,
-                                 false) == 0x2DD1BBBBBBC3);
+                                 UVM_MMU_PTE_FLAGS_CACHED) == 0x2DD1BBBBBBC3);
 
     num_page_sizes = get_page_sizes(gpu, page_sizes);
 
     for (i = 0; i < num_page_sizes; i++)
-        entry_test_page_size(gpu, page_sizes[i]);
+        TEST_NV_CHECK_RET(entry_test_page_size(gpu, page_sizes[i]));
 
     return NV_OK;
 }
@@ -1708,12 +1759,117 @@ static NV_STATUS entry_test_ampere(uvm_gpu_t *gpu, entry_test_page_size_func ent
     num_page_sizes = get_page_sizes(gpu, page_sizes);
 
     for (i = 0; i < num_page_sizes; i++)
-        entry_test_page_size(gpu, page_sizes[i]);
+        TEST_NV_CHECK_RET(entry_test_page_size(gpu, page_sizes[i]));
 
     return NV_OK;
 }
 
-static NV_STATUS alloc_4k_kepler(uvm_gpu_t *gpu)
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+static NV_STATUS alloc_4k_maxwell(uvm_gpu_t *gpu)
 {
     uvm_page_tree_t tree;
     uvm_page_table_range_t range;
@@ -1885,7 +2041,7 @@ static uvm_ce_hal_t fake_ce_hal = {
         .memcopy = fake_ce_memcopy,
 };
 
-static NV_STATUS fake_gpu_init(NvU32 host_class, NvU32 ce_class, NvU32 architecture, NvU32 fault_buffer_class, uvm_gpu_t *fake_gpu)
+static NV_STATUS fake_gpu_init(NvU32 host_class, NvU32 ce_class, NvU32 architecture, uvm_gpu_t *fake_gpu)
 {
     uvm_parent_gpu_t *fake_parent_gpu = container_of(fake_gpu, uvm_parent_gpu_t, gpus[0]);
 
@@ -1896,15 +2052,15 @@ static NV_STATUS fake_gpu_init(NvU32 host_class, NvU32 ce_class, NvU32 architect
     fake_parent_gpu->rm_info.ceClass = ce_class;
     fake_parent_gpu->rm_info.hostClass = host_class;
     fake_parent_gpu->rm_info.gpuArch = architecture;
-    fake_parent_gpu->rm_info.faultBufferClass = fault_buffer_class;
 
     TEST_CHECK_RET(uvm_hal_init_gpu(fake_parent_gpu) == NV_OK);
 
     uvm_hal_init_properties(fake_parent_gpu);
 
-    // The PTE allocation code expects the address space tree HAL to be present (for example, when checking the
-    // addressing capabilities of a GPU).
-    // The selected page size (64K) should work across all supported GPU architectures.
+    // The PTE allocation code expects the address space tree HAL to be present
+    // (for example, when checking the addressing capabilities of a GPU).
+    // The selected page size (64K) should work across all supported GPU
+    // architectures.
     fake_gpu->address_space_tree.hal = fake_parent_gpu->arch_hal->mmu_mode_hal(UVM_PAGE_SIZE_64K);
 
     fake_parent_gpu->host_hal = &fake_host_hal;
@@ -1916,52 +2072,71 @@ static NV_STATUS fake_gpu_init(NvU32 host_class, NvU32 ce_class, NvU32 architect
     return NV_OK;
 }
 
-static NV_STATUS fake_gpu_init_kepler(uvm_gpu_t *fake_gpu)
+static NV_STATUS fake_gpu_init_maxwell(uvm_gpu_t *fake_gpu)
 {
-    return fake_gpu_init(KEPLER_CHANNEL_GPFIFO_A, KEPLER_DMA_COPY_A, NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GK100,
-                         0, fake_gpu);
+    // KEPLER_CHANNEL_GPFIFO_B host class is used for GM10x.
+    return fake_gpu_init(KEPLER_CHANNEL_GPFIFO_B,
+                         MAXWELL_DMA_COPY_A,
+                         NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GM000,
+                         fake_gpu);
 }
 
 static NV_STATUS fake_gpu_init_pascal(uvm_gpu_t *fake_gpu)
 {
-    return fake_gpu_init(PASCAL_CHANNEL_GPFIFO_A, PASCAL_DMA_COPY_A, NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GP100,
-                         MAXWELL_FAULT_BUFFER_A, fake_gpu);
+    return fake_gpu_init(PASCAL_CHANNEL_GPFIFO_A,
+                         PASCAL_DMA_COPY_A,
+                         NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GP100,
+                         fake_gpu);
 }
 
 static NV_STATUS fake_gpu_init_volta(uvm_gpu_t *fake_gpu)
 {
-    return fake_gpu_init(VOLTA_CHANNEL_GPFIFO_A, VOLTA_DMA_COPY_A, NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
-                         MMU_FAULT_BUFFER, fake_gpu);
+    return fake_gpu_init(VOLTA_CHANNEL_GPFIFO_A,
+                         VOLTA_DMA_COPY_A,
+                         NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GV100,
+                         fake_gpu);
 }
 
 static NV_STATUS fake_gpu_init_ampere(uvm_gpu_t *fake_gpu)
 {
-    return fake_gpu_init(AMPERE_CHANNEL_GPFIFO_A, AMPERE_DMA_COPY_A, NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GA100,
-                         MMU_FAULT_BUFFER, fake_gpu);
+    return fake_gpu_init(AMPERE_CHANNEL_GPFIFO_A,
+                         AMPERE_DMA_COPY_A,
+                         NV2080_CTRL_MC_ARCH_INFO_ARCHITECTURE_GA100,
+                         fake_gpu);
 }
 
-static NV_STATUS kepler_test_page_tree(uvm_gpu_t *kepler)
+
+
+
+
+
+
+
+
+
+
+static NV_STATUS maxwell_test_page_tree(uvm_gpu_t *maxwell)
 {
-    // create a fake Kepler GPU for this test.
+    // create a fake Maxwell GPU for this test.
     static const NvU32 big_page_sizes[] = {UVM_PAGE_SIZE_64K, UVM_PAGE_SIZE_128K};
     NvU32 i, j, big_page_size, page_size;
 
-    TEST_CHECK_RET(fake_gpu_init_kepler(kepler) == NV_OK);
+    TEST_CHECK_RET(fake_gpu_init_maxwell(maxwell) == NV_OK);
 
-    MEM_NV_CHECK_RET(allocate_root(kepler), NV_OK);
-    MEM_NV_CHECK_RET(alloc_64k_memory_kepler(kepler), NV_OK);
-    MEM_NV_CHECK_RET(alloc_128k_memory_kepler(kepler), NV_OK);
-    MEM_NV_CHECK_RET(alloc_4k_kepler(kepler), NV_OK);
-    TEST_CHECK_RET(entry_test_kepler(kepler) == NV_OK);
+    MEM_NV_CHECK_RET(allocate_root(maxwell), NV_OK);
+    MEM_NV_CHECK_RET(alloc_64k_memory_maxwell(maxwell), NV_OK);
+    MEM_NV_CHECK_RET(alloc_128k_memory_maxwell(maxwell), NV_OK);
+    MEM_NV_CHECK_RET(alloc_4k_maxwell(maxwell), NV_OK);
+    TEST_CHECK_RET(entry_test_maxwell(maxwell) == NV_OK);
 
     for (i = 0; i < ARRAY_SIZE(big_page_sizes); i++) {
         big_page_size = big_page_sizes[i];
         for (j = 0; j < 2; j++) {
             page_size = (j == 0) ? UVM_PAGE_SIZE_4K : big_page_size;
 
-            MEM_NV_CHECK_RET(shrink_test(kepler, big_page_size, page_size), NV_OK);
-            MEM_NV_CHECK_RET(get_upper_test(kepler, big_page_size, page_size), NV_OK);
-            MEM_NV_CHECK_RET(test_range_vec(kepler, big_page_size, page_size), NV_OK);
+            MEM_NV_CHECK_RET(shrink_test(maxwell, big_page_size, page_size), NV_OK);
+            MEM_NV_CHECK_RET(get_upper_test(maxwell, big_page_size, page_size), NV_OK);
+            MEM_NV_CHECK_RET(test_range_vec(maxwell, big_page_size, page_size), NV_OK);
         }
     }
 
@@ -2080,6 +2255,18 @@ static NV_STATUS ampere_test_page_tree(uvm_gpu_t *ampere)
     return NV_OK;
 }
 
+
+
+
+
+
+
+
+
+
+
+
+
 NV_STATUS uvm_test_page_tree(UVM_TEST_PAGE_TREE_PARAMS *params, struct file *filp)
 {
     NV_STATUS status = NV_OK;
@@ -2098,11 +2285,14 @@ NV_STATUS uvm_test_page_tree(UVM_TEST_PAGE_TREE_PARAMS *params, struct file *fil
     // calls.
     TEST_CHECK_GOTO(fake_tlb_invals_alloc() == NV_OK, done);
 
-    TEST_CHECK_GOTO(kepler_test_page_tree(&gpu->gpus[0]) == NV_OK, done);
+    TEST_CHECK_GOTO(maxwell_test_page_tree(&gpu->gpus[0]) == NV_OK, done);
     TEST_CHECK_GOTO(pascal_test_page_tree(&gpu->gpus[0]) == NV_OK, done);
     TEST_CHECK_GOTO(volta_test_page_tree(&gpu->gpus[0]) == NV_OK, done);
     TEST_CHECK_GOTO(ampere_test_page_tree(&gpu->gpus[0]) == NV_OK, done);
 
+
+
+
     fake_tlb_invals_free();
 
 done:
