@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2019 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -28,16 +28,18 @@
 #include "uvm_api.h"
 #include "uvm_tracker.h"
 #include "uvm_gpu.h"
+#include "uvm_va_space_mm.h"
 
 // Returns true if the interval [start, start + length - 1] is entirely covered
 // by vmas.
 //
-// Locking: current->mm->mmap_lock must be locked
+// Locking: mm->mmap_lock must be locked
 bool uvm_is_valid_vma_range(struct mm_struct *mm, NvU64 start, NvU64 length)
 {
     const NvU64 end = start + length;
     struct vm_area_struct *vma;
 
+    UVM_ASSERT(mm);
     uvm_assert_mmap_lock_locked(mm);
 
     vma = find_vma_intersection(mm, start, end);
@@ -57,6 +59,9 @@ NV_STATUS uvm_api_range_type_check(uvm_va_space_t *va_space, struct mm_struct *m
     uvm_va_range_t *va_range, *va_range_last;
     const NvU64 last_address = base + length - 1;
 
+    if (mm)
+        uvm_assert_mmap_lock_locked(mm);
+
     uvm_assert_rwsem_locked(&va_space->lock);
 
     if (uvm_api_range_invalid(base, length))
@@ -66,6 +71,7 @@ NV_STATUS uvm_api_range_type_check(uvm_va_space_t *va_space, struct mm_struct *m
     if (uvm_va_space_range_empty(va_space, base, last_address)) {
         if (g_uvm_global.ats.enabled &&
             uvm_va_space_pageable_mem_access_supported(va_space) &&
+            mm &&
             uvm_is_valid_vma_range(mm, base, length))
             return NV_WARN_NOTHING_TO_DO;
         else
@@ -226,7 +232,7 @@ NV_STATUS uvm_api_set_preferred_location(const UVM_SET_PREFERRED_LOCATION_PARAMS
     uvm_va_space_t *va_space = uvm_va_space_get(filp);
     uvm_va_range_t *va_range = NULL;
     uvm_va_range_t *first_va_range_to_migrate = NULL;
-    struct mm_struct *mm = current->mm;
+    struct mm_struct *mm;
     uvm_processor_id_t preferred_location_id;
     bool has_va_space_write_lock;
     const NvU64 start = params->requestedBase;
@@ -236,7 +242,7 @@ NV_STATUS uvm_api_set_preferred_location(const UVM_SET_PREFERRED_LOCATION_PARAMS
 
     UVM_ASSERT(va_space);
 
-    uvm_down_read_mmap_lock(mm);
+    mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
     has_va_space_write_lock = true;
 
@@ -309,7 +315,8 @@ done:
         uvm_va_space_up_write(va_space);
     else
         uvm_va_space_up_read(va_space);
-    uvm_up_read_mmap_lock(mm);
+
+    uvm_va_space_mm_or_current_release_unlock(va_space, mm);
 
     return status == NV_OK ? tracker_status : status;
 }
@@ -319,12 +326,12 @@ NV_STATUS uvm_api_unset_preferred_location(const UVM_UNSET_PREFERRED_LOCATION_PA
     NV_STATUS status;
     NV_STATUS tracker_status;
     uvm_va_space_t *va_space = uvm_va_space_get(filp);
-    struct mm_struct *mm = current->mm;
+    struct mm_struct *mm;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
 
     UVM_ASSERT(va_space);
 
-    uvm_down_read_mmap_lock(mm);
+    mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
 
     status = uvm_api_range_type_check(va_space, mm, params->requestedBase, params->length);
@@ -337,7 +344,7 @@ NV_STATUS uvm_api_unset_preferred_location(const UVM_UNSET_PREFERRED_LOCATION_PA
     tracker_status = uvm_tracker_wait_deinit(&local_tracker);
 
     uvm_va_space_up_write(va_space);
-    uvm_up_read_mmap_lock(mm);
+    uvm_va_space_mm_or_current_release_unlock(va_space, mm);
     return status == NV_OK ? tracker_status : status;
 }
 
@@ -411,7 +418,7 @@ static NV_STATUS accessed_by_set(uvm_va_space_t *va_space,
 {
     uvm_processor_id_t processor_id = UVM_ID_INVALID;
     uvm_va_range_t *va_range, *va_range_last;
-    struct mm_struct *mm = current->mm;
+    struct mm_struct *mm;
     const NvU64 last_address = base + length - 1;
     bool range_is_sysmem = false;
     accessed_by_split_params_t split_params;
@@ -421,7 +428,7 @@ static NV_STATUS accessed_by_set(uvm_va_space_t *va_space,
 
     UVM_ASSERT(va_space);
 
-    uvm_down_read_mmap_lock(mm);
+    mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
 
     status = uvm_api_range_type_check(va_space, mm, base, length);
@@ -487,7 +494,7 @@ done:
     tracker_status = uvm_tracker_wait_deinit(&local_tracker);
 
     uvm_va_space_up_write(va_space);
-    uvm_up_read_mmap_lock(mm);
+    uvm_va_space_mm_or_current_release_unlock(va_space, mm);
 
     return status == NV_OK ? tracker_status : status;
 }
@@ -663,7 +670,7 @@ static bool read_duplication_is_va_range_split_needed(uvm_va_range_t *va_range,
 static NV_STATUS read_duplication_set(uvm_va_space_t *va_space, NvU64 base, NvU64 length, bool enable)
 {
     uvm_va_range_t *va_range, *va_range_last;
-    struct mm_struct *mm = current->mm;
+    struct mm_struct *mm;
     const NvU64 last_address = base + length - 1;
     NV_STATUS status;
     uvm_read_duplication_policy_t new_policy;
@@ -671,7 +678,7 @@ static NV_STATUS read_duplication_set(uvm_va_space_t *va_space, NvU64 base, NvU6
     UVM_ASSERT(va_space);
 
     // We need mmap_lock as we may create CPU mappings
-    uvm_down_read_mmap_lock(mm);
+    mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
 
     status = uvm_api_range_type_check(va_space, mm, base, length);
@@ -725,7 +732,7 @@ static NV_STATUS read_duplication_set(uvm_va_space_t *va_space, NvU64 base, NvU6
 
 done:
     uvm_va_space_up_write(va_space);
-    uvm_up_read_mmap_lock(mm);
+    uvm_va_space_mm_or_current_release_unlock(va_space, mm);
     return status;
 }
 
