@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2018-2019 NVIDIA Corporation
+    Copyright (c) 2018-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -72,7 +72,7 @@ NV_STATUS uvm_populate_pageable_vma(struct vm_area_struct *vma,
     if (uvm_managed_vma)
         uvm_record_unlock_mmap_lock_read(mm);
 
-    ret = NV_GET_USER_PAGES(start, vma_num_pages, is_writable, 0, NULL, NULL);
+    ret = NV_GET_USER_PAGES_REMOTE(NULL, mm, start, vma_num_pages, is_writable, 0, NULL, NULL);
 
     if (uvm_managed_vma)
         uvm_record_lock_mmap_lock_read(mm);
@@ -158,7 +158,8 @@ NV_STATUS uvm_api_populate_pageable(const UVM_POPULATE_PAGEABLE_PARAMS *params,
         return NV_ERR_INVALID_ADDRESS;
 
     // mmap_lock is needed to traverse the vmas in the input range and call
-    // into get_user_pages
+    // into get_user_pages. Unlike most UVM APIs, this one is defined to only
+    // work on current->mm, not the mm associated with the VA space (if any).
     uvm_down_read_mmap_lock(current->mm);
 
     if (allow_managed || uvm_va_space_range_empty(va_space, params->base, params->base + params->length - 1))
