@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2020 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -29,10 +29,12 @@
 #include "uvm_channel.h"
 #include "nvtypes.h"
 #include "nvmisc.h"
-#include "cla06f.h"
 #include "cla06fsubch.h"
+#include "cla16f.h"
+#include "clb06f.h"
 
 #define HWMASK(d, r, f) DRF_MASK(NV ## d ## _ ## r ## _ ## f)
+#define HWSHIFTMASK(d, r, f) DRF_SHIFTMASK(NV ## d ## _ ## r ## _ ## f)
 #define HWSIZE(d, r, f) DRF_SIZE(NV ## d ## _ ## r ## _ ## f)
 #define HWCONST(d, r, f, c) DRF_DEF(d, _ ## r, _ ## f, _ ## c)
 #define HWVALUE(d, r, f, v) ({                                                                                      \
@@ -81,9 +83,7 @@
 // Starting with Kepler HW settled on using a fixed subchannel for CE.
 #define UVM_SUBCHANNEL_CE NVA06F_SUBCHANNEL_COPY_ENGINE
 
-#define UVM_SUBCHANNEL_A06F UVM_SUBCHANNEL_HOST
 #define UVM_SUBCHANNEL_A16F UVM_SUBCHANNEL_HOST
-#define UVM_SUBCHANNEL_A0B5 UVM_SUBCHANNEL_CE
 
 #define UVM_SUBCHANNEL_B06F UVM_SUBCHANNEL_HOST
 #define UVM_SUBCHANNEL_B0B5 UVM_SUBCHANNEL_CE
@@ -94,21 +94,35 @@
 #define UVM_SUBCHANNEL_C36F UVM_SUBCHANNEL_HOST
 #define UVM_SUBCHANNEL_C46F UVM_SUBCHANNEL_HOST
 
-#define UVM_SUBCHANNEL_C6B5 UVM_SUBCHANNEL_CE
 #define UVM_SUBCHANNEL_C56F UVM_SUBCHANNEL_HOST
+#define UVM_SUBCHANNEL_C6B5 UVM_SUBCHANNEL_CE
+
+
+
+
+
 
 // Channel for UVM SW methods. This is defined in nv_uvm_types.h. RM does not care about the specific number as long
 // as it's bigger than the largest HW value. For example, Kepler reserves subchannels 5-7 for software objects
 #define UVM_SUBCHANNEL_C076 UVM_SW_OBJ_SUBCHANNEL
 
 #define UVM_METHOD_SIZE 4
-#define UVM_METHOD_COUNT_MAX HWMASK(A06F, DMA, METHOD_COUNT)
+#define UVM_METHOD_COUNT_MAX HWMASK(B06F, DMA, INCR_COUNT)
+#if HWMASK(B06F, DMA, INCR_COUNT) != HWMASK(B06F, DMA, NONINCR_COUNT)
+#error "Unable to define UVM_METHOD_COUNT_MAX"
+#endif
 
-#define UVM_METHOD(op_type, subch, address, count)                          \
-                     (HWCONST(A06F, DMA, SEC_OP, op_type) |                 \
-                      HWVALUE(A06F, DMA, METHOD_ADDRESS, (address) >> 2) |  \
-                      HWVALUE(A06F, DMA, METHOD_SUBCHANNEL, (subch)) |      \
-                      HWVALUE(A06F, DMA, METHOD_COUNT, (count)))
+#define UVM_METHOD_INC(subch, address, count)                                   \
+                        (HWCONST(B06F, DMA, SEC_OP, INC_METHOD) |               \
+                         HWVALUE(B06F, DMA, INCR_ADDRESS, (address) >> 2) |     \
+                         HWVALUE(B06F, DMA, INCR_SUBCHANNEL, (subch)) |         \
+                         HWVALUE(B06F, DMA, INCR_COUNT, (count)))
+
+#define UVM_METHOD_NONINC(subch, address, count)                                    \
+                            (HWCONST(B06F, DMA, SEC_OP, NON_INC_METHOD) |           \
+                             HWVALUE(B06F, DMA, NONINCR_ADDRESS, (address) >> 2) |  \
+                             HWVALUE(B06F, DMA, NONINCR_SUBCHANNEL, (subch)) |      \
+                             HWVALUE(B06F, DMA, NONINCR_COUNT, (count)))
 
 #define __UVM_ASSERT_CONTIGUOUS_METHODS(a1, a2) BUILD_BUG_ON((a2) - (a1) != 4)
 
@@ -121,7 +135,7 @@
         UVM_ASSERT(uvm_push_get_size(push) + (count + 1) * 4 <= UVM_MAX_PUSH_SIZE);     \
         UVM_ASSERT_MSG(a1 % 4 == 0, "Address %u\n", a1);                                \
                                                                                         \
-        push->next[0] = UVM_METHOD(INC_METHOD, subch, a1, count);                       \
+        push->next[0] = UVM_METHOD_INC(subch, a1, count);                               \
         ++push->next;                                                                   \
     } while (0)
 
@@ -219,7 +233,7 @@
         UVM_ASSERT(!uvm_global_is_suspended());                                         \
         UVM_ASSERT(uvm_push_get_size(push) + (count + 1) * 4 <= UVM_MAX_PUSH_SIZE);     \
         UVM_ASSERT_MSG(address % 4 == 0, "Address %u\n", address);                      \
-        push->next[0] = UVM_METHOD(NON_INC_METHOD, subch, address, count);              \
+        push->next[0] = UVM_METHOD_NONINC(subch, address, count);                       \
         push->next += count + 1;                                                        \
     } while (0)
 
