@@ -211,8 +211,7 @@ static NV_STATUS test_concurrent_pushes(uvm_va_space_t *va_space)
         for (i = 0; i < UVM_PUSH_MAX_CONCURRENT_PUSHES; ++i) {
             uvm_push_t *push = &pushes[i];
             uvm_push_end(push);
-            status = uvm_tracker_add_push(&tracker, push);
-            TEST_CHECK_GOTO(status == NV_OK, done);
+            TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, push), done);
         }
         TEST_CHECK_GOTO(tracker.size != 0, done);
 
@@ -261,7 +260,10 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
     uvm_rm_mem_t *mem = NULL;
     atomic_t on_complete_counter = ATOMIC_INIT(0);
 
-    channel = uvm_channel_first(gpu->channel_manager);
+    channel = uvm_channel_any(gpu->channel_manager);
+    if (channel == NULL)
+        return NV_ERR_INVALID_STATE;
+
     if (channel->num_gpfifo_entries <= TEST_PUSH_INTERLEAVING_NUM_PAUSED_PUSHES) {
         UVM_TEST_PRINT("Insufficient number of gpfifo entries per channel to run this test. Expected at least %u "
                        "entries, but found %u\n",
@@ -280,7 +282,7 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
     status = uvm_rm_mem_alloc_and_map_cpu(gpu, UVM_RM_MEM_TYPE_SYS, size, &mem);
     TEST_CHECK_GOTO(status == NV_OK, done);
     host_va = (NvU32*)uvm_rm_mem_get_cpu_va(mem);
-    gpu_va = uvm_rm_mem_get_gpu_va(mem, gpu);
+    gpu_va = uvm_rm_mem_get_gpu_va(mem, gpu, uvm_channel_is_proxy(channel));
     memset(host_va, 0, size);
 
     // Begin a few pushes on the channel, but do not end them yet.
@@ -385,13 +387,10 @@ static NV_STATUS test_push_interleaving(uvm_va_space_t *va_space)
 
     BUILD_BUG_ON(TEST_PUSH_INTERLEAVING_NUM_PAUSED_PUSHES >= UVM_PUSH_MAX_CONCURRENT_PUSHES);
 
-    // GK110+ is required for the CE semaphore reduction method used in the test.
     for_each_va_space_gpu(gpu, va_space) {
-        if (uvm_gpu_is_gk110_plus(gpu)) {
-            status = test_push_interleaving_on_gpu(gpu);
-            if (status != NV_OK)
-                return status;
-        }
+        status = test_push_interleaving_on_gpu(gpu);
+        if (status != NV_OK)
+            return status;
     }
 
     return NV_OK;
@@ -407,15 +406,17 @@ static NV_STATUS test_push_exactly_max_push(uvm_gpu_t *gpu,
                                             NvU32 value)
 {
     NV_STATUS status;
+    NvU64 semaphore_gpu_va;
 
-    status = uvm_push_begin(gpu->channel_manager, UVM_CHANNEL_TYPE_ANY, push, "Test push");
+    status = uvm_push_begin(gpu->channel_manager, UVM_CHANNEL_TYPE_GPU_INTERNAL, push, "Test push");
     if (status != NV_OK)
         return status;
 
     TEST_CHECK_RET(uvm_push_has_space(push, UVM_MAX_PUSH_SIZE));
     TEST_CHECK_RET(!uvm_push_has_space(push, UVM_MAX_PUSH_SIZE + 1));
 
-    gpu->parent->host_hal->semaphore_acquire(push, sema_to_acquire, value);
+    semaphore_gpu_va = uvm_gpu_semaphore_get_gpu_va(sema_to_acquire, gpu, uvm_channel_is_proxy(push->channel));
+    gpu->parent->host_hal->semaphore_acquire(push, semaphore_gpu_va, value);
 
     // Push a noop leaving just UVM_PUSH_END_SIZE in the pushbuffer.
     gpu->parent->host_hal->noop(push, UVM_MAX_PUSH_SIZE - uvm_push_get_size(push) - UVM_PUSH_END_SIZE);
@@ -483,7 +484,7 @@ static NV_STATUS test_max_pushes_on_gpu(uvm_gpu_t *gpu)
         TEST_CHECK_GOTO(status == NV_OK, done);
 
         total_push_size += uvm_push_get_size(&push);
-        uvm_tracker_add_push(&tracker, &push);
+        TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, &push), done);
     }
 
     if (total_push_size != UVM_PUSHBUFFER_SIZE) {
@@ -517,8 +518,7 @@ static NV_STATUS test_max_pushes_on_gpu(uvm_gpu_t *gpu)
         status = test_push_exactly_max_push(gpu, &push, &sema, push_count);
         TEST_CHECK_GOTO(status == NV_OK, done);
 
-        status = uvm_tracker_add_push(&tracker, &push);
-        TEST_CHECK_GOTO(status == NV_OK, done);
+        TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, &push), done);
     }
 
 done:
@@ -554,14 +554,17 @@ static NV_STATUS test_idle_chunks_on_gpu(uvm_gpu_t *gpu)
     TEST_CHECK_GOTO(status == NV_OK, done);
 
     for (i = 0; i < UVM_PUSHBUFFER_CHUNKS; ++i) {
+        NvU64 semaphore_gpu_va;
         uvm_push_t push;
-        status = uvm_push_begin(gpu->channel_manager, UVM_CHANNEL_TYPE_ANY, &push, "Push using chunk %u", i);
+
+        status = uvm_push_begin(gpu->channel_manager, UVM_CHANNEL_TYPE_GPU_INTERNAL, &push, "Push using chunk %u", i);
         TEST_CHECK_GOTO(status == NV_OK, done);
 
-        gpu->parent->host_hal->semaphore_acquire(&push, &sema, i + 1);
+        semaphore_gpu_va = uvm_gpu_semaphore_get_gpu_va(&sema, gpu, uvm_channel_is_proxy(push.channel));
+        gpu->parent->host_hal->semaphore_acquire(&push, semaphore_gpu_va, i + 1);
         uvm_push_end(&push);
 
-        uvm_tracker_add_push(&tracker, &push);
+        TEST_NV_CHECK_GOTO(uvm_tracker_add_push(&tracker, &push), done);
 
         if (test_count_idle_chunks(gpu->channel_manager->pushbuffer) != UVM_PUSHBUFFER_CHUNKS - i - 1) {
             UVM_TEST_PRINT("Unexpected count of idle chunks in the pushbuffer %u instead of %u\n",
@@ -599,8 +602,8 @@ static NV_STATUS test_pushbuffer(uvm_va_space_t *va_space)
     uvm_gpu_t *gpu;
 
     for_each_va_space_gpu(gpu, va_space) {
-        TEST_CHECK_RET(test_max_pushes_on_gpu(gpu) == NV_OK);
-        TEST_CHECK_RET(test_idle_chunks_on_gpu(gpu) == NV_OK);
+        TEST_NV_CHECK_RET(test_max_pushes_on_gpu(gpu));
+        TEST_NV_CHECK_RET(test_idle_chunks_on_gpu(gpu));
     }
     return NV_OK;
 }
