@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVidia Corporation
+    Copyright (c) 2015-2021 NVidia Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -358,7 +358,8 @@ typedef struct
 #define UVM_TEST_CE_SANITY                               UVM_TEST_IOCTL_BASE(16)
 typedef struct
 {
-    NV_STATUS rmStatus; // Out
+    NvBool    skipTimestampTest;      // In
+    NV_STATUS rmStatus;               // Out
 } UVM_TEST_CE_SANITY_PARAMS;
 
 #define UVM_TEST_VA_BLOCK_INFO                           UVM_TEST_IOCTL_BASE(17)
@@ -519,13 +520,6 @@ typedef struct
     NV_STATUS rmStatus;
 } UVM_TEST_MEM_SANITY_PARAMS;
 
-#define UVM_TEST_MMU_SANITY                              UVM_TEST_IOCTL_BASE(31)
-typedef struct
-{
-    // Out params
-    NV_STATUS rmStatus;
-} UVM_TEST_MMU_SANITY_PARAMS;
-
 #define UVM_TEST_MAKE_CHANNEL_STOPS_IMMEDIATE            UVM_TEST_IOCTL_BASE(32)
 typedef struct
 {
@@ -673,15 +667,10 @@ typedef enum
     UvmInvalidatePageTableLevelPde0 = 3,
     UvmInvalidatePageTableLevelPde1 = 4,
     UvmInvalidatePageTableLevelPde2 = 5,
-    UvmInvalidatePageTableLevelPde3 = 6
+    UvmInvalidatePageTableLevelPde3 = 6,
+    UvmInvalidatePageTableLevelPde4 = 7,
 } UvmInvalidatePageTableLevel;
 
-typedef enum
-{
-    UvmTargetPdbModeAll = 1,
-    UvmTargetPdbModeOne = 2
-} UvmTargetPdbMode;
-
 typedef enum
 {
     UvmTargetVaModeAll      = 1,
@@ -692,11 +681,9 @@ typedef enum
 typedef struct
 {
     // In params
-
     NvProcessorUuid  gpu_uuid;
     NvU64            va NV_ALIGN_BYTES(8);
     NvU32            target_va_mode;           // UvmTargetVaMode
-    NvU32            target_pdb_mode;          // UvmTargetPdbMode
     NvU32            page_table_level;         // UvmInvalidatePageTableLevel
     NvU32            membar;                   // UvmInvalidateTlbMembarType
     NvBool           disable_gpc_invalidate;
@@ -1347,6 +1334,46 @@ typedef struct
     NV_STATUS                       rmStatus;                                           // Out
 } UVM_TEST_RB_TREE_RANDOM_PARAMS;
 
+#define UVM_TEST_HOST_SANITY                             UVM_TEST_IOCTL_BASE(88)
+typedef struct
+{
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_HOST_SANITY_PARAMS;
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+#define UVM_TEST_GET_USER_SPACE_END_ADDRESS              UVM_TEST_IOCTL_BASE(90)
+typedef struct
+{
+    NvU64                           user_space_end_address;                             // Out
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_GET_USER_SPACE_END_ADDRESS_PARAMS;
+
+#define UVM_TEST_HMM_SANITY                              UVM_TEST_IOCTL_BASE(91)
+typedef struct
+{
+    NvU64     hmm_address;                               // In
+    NvU64     hmm_length;                                // In
+    NvU64     uvm_address;                               // In
+    NvU64     uvm_length;                                // In
+    NV_STATUS rmStatus;                                  // Out
+} UVM_TEST_HMM_SANITY_PARAMS;
+
 #ifdef __cplusplus
 }
 #endif
