@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2019 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -25,7 +25,7 @@
 #define __UVM_VA_BLOCK_H__
 
 #include "uvm_forward_decl.h"
-#include "uvmtypes.h"
+#include "uvm_types.h"
 #include "uvm_linux.h"
 #include "nv-kref.h"
 #include "uvm_common.h"
@@ -42,9 +42,10 @@
 #include "nv-kthread-q.h"
 
 // VA blocks are the leaf nodes in the uvm_va_space tree for managed allocations
-// (VA ranges with type == UVM_VA_RANGE_TYPE_MMAP):
+// (VA ranges with type == UVM_VA_RANGE_TYPE_MANAGED):
 //
-//  uvm_va_space -> uvm_va_range -> uvm_va_block
+//  UVM: uvm_va_space -> uvm_va_range -> uvm_va_block
+//  HMM: uvm_va_space -> uvm_va_block
 //
 // Each VA block is contained within a single VA range, and contains state on
 // VAs covered by that block. Most importantly, the block tracks the current
@@ -106,18 +107,6 @@ typedef struct
     // Pages that have been evicted to sysmem
     uvm_page_mask_t evicted;
 
-    // For GPUs which have swizzled big pages, these bits track whether each
-    // allocated physical page on this GPU is currently in the swizzled format.
-    // These apply as long as the corresponding chunk is allocated, even if the
-    // corresponding pages are not resident (even though no data is present in
-    // that case, the bit will be used when making new pages resident on this
-    // GPU).
-    //
-    // If the GPU does not support swizzling, these can be ignored.
-    //
-    // See big_ptes for the layout of these PTEs within the block.
-    DECLARE_BITMAP(big_pages_swizzled, MAX_BIG_PAGES_PER_UVM_VA_BLOCK);
-
     // Array of addresses of CPU pages mapped for physical access by the GPU.
     //
     // The CPU pages are mapped eagerly on the GPU whenever a new CPU page is
@@ -252,7 +241,7 @@ typedef struct
 struct uvm_va_block_struct
 {
     // Reference count for this block. References are held by:
-    // - The parent VA range
+    // - The parent VA range for managed blocks or VA space for HMM blocks
     // - The reverse map
     // - The eviction path temporarily when attempting to evict a GPU page under
     //   this block
@@ -264,8 +253,10 @@ struct uvm_va_block_struct
     // Lock protecting the block. See the comment at the top of uvm.c.
     uvm_mutex_t lock;
 
-    // Parent VA range. If this is not set, the block is considered dead and is
-    // waiting for the last ref count to be removed.
+    // Parent VA range. UVM managed blocks have this set. HMM blocks will have
+    // va_range set to NULL and hmm.va_space set instead. Dead blocks that are
+    // waiting for the last ref count to be removed have va_range and
+    // hmm.va_space set to NULL (could be either type of block).
     //
     // TODO: Bug 1765193. Since the va_range contains the policy for this block
     //       (preferred location, accessed-by, etc) the parent va_range must be
@@ -452,6 +443,19 @@ struct uvm_va_block_struct
     nv_kthread_q_item_t eviction_mappings_q_item;
 
     uvm_perf_module_data_desc_t perf_modules_data[UVM_PERF_MODULE_TYPE_COUNT];
+
+#if UVM_IS_CONFIG_HMM()
+    struct
+    {
+
+
+
+
+
+        // Parent VA space pointer.
+        uvm_va_space_t *va_space;
+    } hmm;
+#endif
 };
 
 // We define additional per-VA Block fields for testing. When
@@ -519,7 +523,7 @@ void uvm_va_block_exit(void);
 //
 // The caller must be holding the VA space lock in at least read mode.
 //
-// The va_range must have type UVM_VA_RANGE_TYPE_MMAP.
+// The va_range must have type UVM_VA_RANGE_TYPE_MANAGED.
 NV_STATUS uvm_va_block_create(uvm_va_range_t *va_range,
                               NvU64 start,
                               NvU64 end,
@@ -552,6 +556,36 @@ static inline void uvm_va_block_release_no_destroy(uvm_va_block_t *va_block)
     UVM_ASSERT(!destroyed);
 }
 
+// Return true if the block is dead.
+// Locking: This can be called while holding either the block lock or just the
+// VA space lock in read mode, since it can only change when the VA space lock
+// is held in write mode.
+static inline bool uvm_va_block_is_dead(uvm_va_block_t *va_block)
+{
+    if (va_block->va_range)
+        return false;
+
+#if UVM_IS_CONFIG_HMM()
+    if (va_block->hmm.va_space)
+        return false;
+#endif
+
+    return true;
+}
+
+// Return the va_space pointer of the given block or NULL if the block is dead.
+// Locking: This can be called while holding either the block lock or just the
+// VA space lock in read mode, since it can only change when the VA space lock
+// is held in write mode.
+uvm_va_space_t *uvm_va_block_get_va_space_maybe_dead(uvm_va_block_t *va_block);
+
+// Return the va_space pointer of the given block assuming the block is not dead
+// (asserts that it is not dead and asserts va_space is not NULL).
+// Locking: This can be called while holding either the block lock or just the
+// VA space lock in read mode, since it can only change when the VA space lock
+// is held in write mode.
+uvm_va_space_t *uvm_va_block_get_va_space(uvm_va_block_t *va_block);
+
 // Dynamic cache-based allocation for uvm_va_block_context_t.
 //
 // See uvm_va_block_context_init() for a description of the mm parameter.
@@ -1102,7 +1136,7 @@ const uvm_page_mask_t *uvm_va_block_map_mask_get(uvm_va_block_t *block, uvm_proc
 // Finds the VA block containing addr, if any. The va_space->lock must be held
 // in at least read mode. Return values:
 // NV_ERR_INVALID_ADDRESS   addr is not covered by a va_range, or the va_range
-//                          does not have type UVM_VA_RANGE_TYPE_MMAP
+//                          does not have type UVM_VA_RANGE_TYPE_MANAGED
 //
 // NV_ERR_OBJECT_NOT_FOUND  addr is valid but no block has been allocated to
 //                          cover it yet
@@ -1112,7 +1146,10 @@ NV_STATUS uvm_va_block_find(uvm_va_space_t *va_space, NvU64 addr, uvm_va_block_t
 
 // Same as uvm_va_block_find except that the block is created if not already
 // present in the VA range.
-NV_STATUS uvm_va_block_find_create(uvm_va_space_t *va_space, NvU64 addr, uvm_va_block_t **out_block);
+NV_STATUS uvm_va_block_find_create(uvm_va_space_t *va_space,
+                                   struct mm_struct *mm,
+                                   NvU64 addr,
+                                   uvm_va_block_t **out_block);
 
 // Look up a chunk backing a specific address within the VA block. Returns NULL if none.
 uvm_gpu_chunk_t *uvm_va_block_lookup_gpu_chunk(uvm_va_block_t *va_block, uvm_gpu_t *gpu, NvU64 address);
