@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2019 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -161,6 +161,11 @@ NV_STATUS uvm_va_range_create_mmap(uvm_va_space_t *va_space,
     struct vm_area_struct *vma = vma_wrapper->vma;
     uvm_va_range_t *va_range = NULL;
 
+    // Check for no overlap with HMM blocks.
+    status = uvm_hmm_va_block_kill(va_space, vma->vm_start, vma->vm_end - 1);
+    if (status != NV_OK)
+        return status;
+
     // vma->vm_end is exclusive but va_range end is inclusive
     va_range = uvm_va_range_alloc_managed(va_space, vma->vm_start, vma->vm_end - 1);
     if (!va_range) {
@@ -193,6 +198,11 @@ NV_STATUS uvm_va_range_create_external(uvm_va_space_t *va_space,
     uvm_va_range_t *va_range = NULL;
     NvU32 i;
 
+    // Check for no overlap with HMM blocks.
+    status = uvm_hmm_va_block_kill(va_space, start, start + length - 1);
+    if (status != NV_OK)
+        return status;
+
     va_range = uvm_va_range_alloc(va_space, start, start + length - 1);
     if (!va_range) {
         status = NV_ERR_NO_MEMORY;
@@ -227,6 +237,11 @@ NV_STATUS uvm_va_range_create_channel(uvm_va_space_t *va_space,
     NV_STATUS status;
     uvm_va_range_t *va_range = NULL;
 
+    // Check for no overlap with HMM blocks.
+    status = uvm_hmm_va_block_kill(va_space, start, end);
+    if (status != NV_OK)
+        return status;
+
     va_range = uvm_va_range_alloc(va_space, start, end);
     if (!va_range) {
         status = NV_ERR_NO_MEMORY;
@@ -258,6 +273,11 @@ NV_STATUS uvm_va_range_create_sked_reflected(uvm_va_space_t *va_space,
     NV_STATUS status;
     uvm_va_range_t *va_range = NULL;
 
+    // Check for no overlap with HMM blocks.
+    status = uvm_hmm_va_block_kill(va_space, start, start + length - 1);
+    if (status != NV_OK)
+        return status;
+
     va_range = uvm_va_range_alloc(va_space, start, start + length - 1);
     if (!va_range) {
         status = NV_ERR_NO_MEMORY;
@@ -289,7 +309,7 @@ NV_STATUS uvm_va_range_create_semaphore_pool(uvm_va_space_t *va_space,
 {
     static const uvm_mem_gpu_mapping_attrs_t default_attrs = {
             .protection = UVM_PROT_READ_WRITE_ATOMIC,
-            .is_volatile = true
+            .is_cacheable = false
     };
 
     NV_STATUS status;
@@ -298,6 +318,11 @@ NV_STATUS uvm_va_range_create_semaphore_pool(uvm_va_space_t *va_space,
     NvU32 i;
     uvm_gpu_id_t gpu_id;
 
+    // Check for no overlap with HMM blocks.
+    status = uvm_hmm_va_block_kill(va_space, start, start + length - 1);
+    if (status != NV_OK)
+        return status;
+
     va_range = uvm_va_range_alloc(va_space, start, start + length - 1);
     if (!va_range) {
         status = NV_ERR_NO_MEMORY;
@@ -332,7 +357,7 @@ NV_STATUS uvm_va_range_create_semaphore_pool(uvm_va_space_t *va_space,
         status = uvm_mem_translate_gpu_attributes(&per_gpu_attrs[i], va_space, &gpu, &attrs);
         if (status != NV_OK)
             goto error;
-        if (!attrs.is_volatile) {
+        if (attrs.is_cacheable) {
             // At most 1 GPU can have this memory cached, in which case it is the 'owner' GPU.
             if (va_range->semaphore_pool.owner != NULL) {
                 UVM_DBG_PRINT("Caching of semaphore pool requested on >1 GPU.");
@@ -1725,12 +1750,6 @@ static NV_STATUS uvm_map_sked_reflected_range(uvm_va_space_t *va_space, UVM_MAP_
         goto done;
     }
 
-    // Only GK110+ supports CNP and SKED reflected mappings
-    if (!uvm_gpu_is_gk110_plus(gpu)) {
-        status = NV_ERR_INVALID_DEVICE;
-        goto done;
-    }
-
     // Check if the GPU can access the VA
     if (!uvm_gpu_can_address(gpu, params->base + params->length)) {
         status = NV_ERR_OUT_OF_RANGE;
