@@ -290,8 +290,6 @@ typedef struct
         NvU32 id;
     } tsg;
 
-    // TODO: Bug 1732494: RM should be able to figure this out on its own
-    bool is_global;
     NvU64 ref_count;
 
     // Storage in the corresponding uvm_gpu_va_space's channel_va_ranges list
