@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2020 NVIDIA Corporation
+    Copyright (c) 2015-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -36,6 +36,7 @@
 #include "uvm_ats_ibm.h"
 #include "uvm_gpu_access_counters.h"
 #include "uvm_hmm.h"
+#include "uvm_va_space_mm.h"
 #include "uvm_test.h"
 #include "uvm_common.h"
 #include "nv_uvm_interface.h"
@@ -113,6 +114,52 @@ static NV_STATUS register_gpu_nvlink_peers(uvm_va_space_t *va_space, uvm_gpu_t *
     return NV_OK;
 }
 
+static bool va_space_check_processors_masks(uvm_va_space_t *va_space)
+{
+    uvm_processor_id_t processor;
+    uvm_processor_mask_t processors;
+
+    uvm_assert_rwsem_locked_write(&va_space->lock);
+
+    uvm_processor_mask_copy(&processors, &va_space->registered_gpus);
+    uvm_processor_mask_set(&processors, UVM_ID_CPU);
+
+    for_each_id_in_mask(processor, &processors) {
+        uvm_processor_id_t other_processor;
+
+        UVM_ASSERT(processor_mask_array_test(va_space->can_access, processor, processor));
+        UVM_ASSERT(processor_mask_array_test(va_space->accessible_from, processor, processor));
+        UVM_ASSERT(processor_mask_array_test(va_space->can_copy_from, processor, processor));
+        UVM_ASSERT(processor_mask_array_test(va_space->can_copy_from, processor, UVM_ID_CPU));
+        UVM_ASSERT(processor_mask_array_test(va_space->can_copy_from, UVM_ID_CPU, processor));
+
+        // NVLINK
+        UVM_ASSERT(!processor_mask_array_test(va_space->has_nvlink, processor, processor));
+        UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_nvlink[uvm_id_value(processor)],
+                                             &va_space->can_copy_from[uvm_id_value(processor)]));
+
+        // Peers
+        UVM_ASSERT(!processor_mask_array_test(va_space->indirect_peers, processor, processor));
+        UVM_ASSERT(uvm_processor_mask_subset(&va_space->indirect_peers[uvm_id_value(processor)],
+                                             &va_space->has_native_atomics[uvm_id_value(processor)]));
+
+        // Atomics
+        UVM_ASSERT(processor_mask_array_test(va_space->has_native_atomics, processor, processor));
+        UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_native_atomics[uvm_id_value(processor)],
+                                             &va_space->can_copy_from[uvm_id_value(processor)]));
+        UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_native_atomics[uvm_id_value(processor)],
+                                             &va_space->can_access[uvm_id_value(processor)]));
+
+        for_each_id_in_mask(other_processor, &va_space->can_access[uvm_id_value(processor)])
+            UVM_ASSERT(processor_mask_array_test(va_space->accessible_from, other_processor, processor));
+
+        for_each_id_in_mask(other_processor, &va_space->accessible_from[uvm_id_value(processor)])
+            UVM_ASSERT(processor_mask_array_test(va_space->can_access, other_processor, processor));
+    }
+
+    return true;
+}
+
 NV_STATUS uvm_va_space_create(struct inode *inode, struct file *filp)
 {
     NV_STATUS status;
@@ -205,6 +252,8 @@ NV_STATUS uvm_va_space_create(struct inode *inode, struct file *filp)
     if (status != NV_OK)
         goto fail;
 
+    UVM_ASSERT(va_space_check_processors_masks(va_space));
+
     uvm_va_space_up_write(va_space);
 
     uvm_mutex_lock(&g_uvm_global.va_spaces.lock);
@@ -315,6 +364,8 @@ static void unregister_gpu(uvm_va_space_t *va_space,
             }
         }
     }
+
+    va_space_check_processors_masks(va_space);
 }
 
 static void gpu_va_space_stop_all_channels(uvm_gpu_va_space_t *gpu_va_space)
@@ -385,8 +436,6 @@ void uvm_va_space_destroy(uvm_va_space_t *va_space)
     // registered GPUs in the VA space, so those faults will be canceled.
     uvm_va_space_down_write(va_space);
 
-    uvm_hmm_mirror_unregister(va_space);
-
     uvm_va_space_global_gpus(va_space, &retained_gpus);
 
     bitmap_copy(va_space->enabled_peers_teardown, va_space->enabled_peers, UVM_MAX_UNIQUE_GPU_PAIRS);
@@ -403,6 +452,8 @@ void uvm_va_space_destroy(uvm_va_space_t *va_space)
         uvm_va_range_destroy(va_range, &deferred_free_list);
     }
 
+    uvm_hmm_va_space_destroy(va_space);
+
     uvm_range_group_radix_tree_destroy(va_space);
 
     // Unregister all GPUs in the VA space. Note that this does not release the
@@ -534,11 +585,9 @@ NV_STATUS uvm_va_space_initialize(uvm_va_space_t *va_space, NvU64 flags)
         if (status != NV_OK)
             goto out;
 
-        status = uvm_hmm_mirror_register(va_space);
-        if (status != NV_OK) {
-            uvm_va_space_mm_unregister(va_space);
-            goto out;
-        }
+        status = uvm_hmm_va_space_initialize(va_space);
+        if (status != NV_OK)
+            goto unreg;
 
         // Use release semantics to match the acquire semantics in
         // uvm_va_space_initialized. See that function for details. All
@@ -549,7 +598,14 @@ NV_STATUS uvm_va_space_initialize(uvm_va_space_t *va_space, NvU64 flags)
 out:
     uvm_va_space_up_write(va_space);
     uvm_up_write_mmap_lock(current->mm);
+    return status;
 
+unreg:
+    uvm_va_space_up_write(va_space);
+    uvm_up_write_mmap_lock(current->mm);
+    // See the comment in uvm_va_space_mm_unregister() for why this has to be
+    // called after releasing the locks.
+    uvm_va_space_mm_unregister(va_space);
     return status;
 }
 
@@ -699,6 +755,9 @@ NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
     // All GPUs have native atomics on their own memory
     processor_mask_array_set(va_space->has_native_atomics, gpu->id, gpu->id);
 
+
+
+
     if (gpu->parent->sysmem_link >= UVM_GPU_LINK_NVLINK_1) {
         processor_mask_array_set(va_space->has_nvlink, gpu->id, UVM_ID_CPU);
         processor_mask_array_set(va_space->has_nvlink, UVM_ID_CPU, gpu->id);
@@ -718,10 +777,19 @@ NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
     processor_mask_array_set(va_space->can_access, gpu->id, gpu->id);
     processor_mask_array_set(va_space->accessible_from, gpu->id, gpu->id);
 
+
+
+
+
+
+
+
+
     // All GPUs have direct access to sysmem
     processor_mask_array_set(va_space->can_access, gpu->id, UVM_ID_CPU);
     processor_mask_array_set(va_space->accessible_from, UVM_ID_CPU, gpu->id);
 
+
     processor_mask_array_set(va_space->can_copy_from, gpu->id, gpu->id);
     processor_mask_array_set(va_space->can_copy_from, gpu->id, UVM_ID_CPU);
     processor_mask_array_set(va_space->can_copy_from, UVM_ID_CPU, gpu->id);
@@ -767,6 +835,8 @@ NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
         *numa_node_id = -1;
     }
 
+    UVM_ASSERT(va_space_check_processors_masks(va_space));
+
 done:
     uvm_va_space_up_write(va_space);
 
@@ -787,7 +857,7 @@ NV_STATUS uvm_va_space_unregister_gpu(uvm_va_space_t *va_space, const NvProcesso
 {
     uvm_gpu_t *gpu;
     uvm_gpu_va_space_t *gpu_va_space;
-    struct mm_struct *mm = current->mm;
+    struct mm_struct *mm;
     uvm_global_gpu_id_t peer_gpu_id;
     uvm_global_processor_mask_t peers_to_release;
     LIST_HEAD(deferred_free_list);
@@ -837,9 +907,9 @@ NV_STATUS uvm_va_space_unregister_gpu(uvm_va_space_t *va_space, const NvProcesso
     if (gpu->parent->access_counters_supported)
         uvm_gpu_access_counters_disable(gpu, va_space);
 
-    // The mmap_lock lock is needed to establish CPU mappings to any pages
-    // evicted from the GPU if accessed by CPU is set for them.
-    uvm_down_read_mmap_lock(mm);
+    // mmap_lock is needed to establish CPU mappings to any pages evicted from
+    // the GPU if accessed by CPU is set for them.
+    mm = uvm_va_space_mm_or_current_retain_lock(va_space);
 
     uvm_va_space_down_write(va_space);
 
@@ -856,7 +926,7 @@ NV_STATUS uvm_va_space_unregister_gpu(uvm_va_space_t *va_space, const NvProcesso
     uvm_processor_mask_clear(&va_space->gpu_unregister_in_progress, gpu->id);
 
     uvm_va_space_up_write(va_space);
-    uvm_up_read_mmap_lock(mm);
+    uvm_va_space_mm_or_current_release_unlock(va_space, mm);
 
     uvm_deferred_free_object_list(&deferred_free_list);
 
@@ -884,9 +954,13 @@ static void disable_peers(uvm_va_space_t *va_space,
                           uvm_gpu_t *gpu1,
                           struct list_head *deferred_free_list)
 {
-    NvU32 table_index = uvm_gpu_peer_table_index(gpu0->id, gpu1->id);
+    NvU32 table_index;
     uvm_va_range_t *va_range;
 
+    uvm_assert_rwsem_locked_write(&va_space->lock);
+
+    table_index = uvm_gpu_peer_table_index(gpu0->id, gpu1->id);
+
     if (!test_bit(table_index, va_space->enabled_peers))
         return;
 
@@ -909,6 +983,8 @@ static void disable_peers(uvm_va_space_t *va_space,
     processor_mask_array_clear(va_space->has_native_atomics, gpu1->id, gpu0->id);
 
     __clear_bit(table_index, va_space->enabled_peers);
+
+    va_space_check_processors_masks(va_space);
 }
 
 static NV_STATUS enable_peers(uvm_va_space_t *va_space, uvm_gpu_t *gpu0, uvm_gpu_t *gpu1)
@@ -977,6 +1053,7 @@ static NV_STATUS enable_peers(uvm_va_space_t *va_space, uvm_gpu_t *gpu0, uvm_gpu
         }
     }
 
+    UVM_ASSERT(va_space_check_processors_masks(va_space));
     __set_bit(table_index, va_space->enabled_peers);
 
     uvm_for_each_va_range(va_range, va_space) {
@@ -1272,7 +1349,7 @@ NV_STATUS uvm_va_space_register_gpu_va_space(uvm_va_space_t *va_space,
     uvm_gpu_t *gpu;
     uvm_gpu_va_space_t *gpu_va_space;
     uvm_va_range_t *va_range;
-    struct mm_struct *mm = current->mm;
+    struct mm_struct *mm;
     LIST_HEAD(deferred_free_list);
 
     gpu = uvm_va_space_retain_gpu_by_uuid(va_space, gpu_uuid);
@@ -1285,14 +1362,17 @@ NV_STATUS uvm_va_space_register_gpu_va_space(uvm_va_space_t *va_space,
         return status;
     }
 
-    // uvm_ats_ibm_register_gpu_va_space() requires mmap_lock to be held in
-    // write mode if ATS support is provided through the kernel. Otherwise we
-    // only need mmap_lock in read mode to handle potential CPU mapping changes
-    // in uvm_va_range_add_gpu_va_space().
-    if (UVM_ATS_IBM_SUPPORTED_IN_KERNEL())
-        uvm_down_write_mmap_lock(mm);
-    else
-        uvm_down_read_mmap_lock(mm);
+    mm = uvm_va_space_mm_or_current_retain(va_space);
+    if (mm) {
+        // uvm_ats_ibm_register_gpu_va_space() requires mmap_lock to be held in
+        // write mode if ATS support is provided through the kernel. Otherwise
+        // we only need mmap_lock in read mode to handle potential CPU mapping
+        // changes in uvm_va_range_add_gpu_va_space().
+        if (UVM_ATS_IBM_SUPPORTED_IN_KERNEL())
+            uvm_down_write_mmap_lock(mm);
+        else
+            uvm_down_read_mmap_lock(mm);
+    }
 
     uvm_va_space_down_write(va_space);
 
@@ -1346,10 +1426,14 @@ NV_STATUS uvm_va_space_register_gpu_va_space(uvm_va_space_t *va_space,
 
     uvm_va_space_up_write(va_space);
 
-    if (UVM_ATS_IBM_SUPPORTED_IN_KERNEL())
-        uvm_up_write_mmap_lock(mm);
-    else
-        uvm_up_read_mmap_lock(mm);
+    if (mm) {
+        if (UVM_ATS_IBM_SUPPORTED_IN_KERNEL())
+            uvm_up_write_mmap_lock(mm);
+        else
+            uvm_up_read_mmap_lock(mm);
+
+        uvm_va_space_mm_or_current_release(va_space, mm);
+    }
 
     uvm_gpu_release(gpu);
     return NV_OK;
@@ -1367,10 +1451,14 @@ error:
 
     uvm_va_space_up_write(va_space);
 
-    if (UVM_ATS_IBM_SUPPORTED_IN_KERNEL())
-        uvm_up_write_mmap_lock(mm);
-    else
-        uvm_up_read_mmap_lock(mm);
+    if (mm) {
+        if (UVM_ATS_IBM_SUPPORTED_IN_KERNEL())
+            uvm_up_write_mmap_lock(mm);
+        else
+            uvm_up_read_mmap_lock(mm);
+
+        uvm_va_space_mm_or_current_release(va_space, mm);
+    }
 
     destroy_gpu_va_space(gpu_va_space);
 
@@ -1422,7 +1510,7 @@ NV_STATUS uvm_va_space_unregister_gpu_va_space(uvm_va_space_t *va_space, const N
     NV_STATUS status = NV_OK;
     uvm_gpu_t *gpu;
     uvm_gpu_va_space_t *gpu_va_space;
-    struct mm_struct *mm = current->mm;
+    struct mm_struct *mm;
     LIST_HEAD(deferred_free_list);
 
     // Stopping channels requires holding the VA space lock in read mode, so do
@@ -1447,7 +1535,7 @@ NV_STATUS uvm_va_space_unregister_gpu_va_space(uvm_va_space_t *va_space, const N
     uvm_gpu_retain(gpu);
     uvm_va_space_up_read_rm(va_space);
 
-    uvm_down_read_mmap_lock(mm);
+    mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
 
     // We dropped the lock so we have to re-verify that this gpu_va_space is
@@ -1465,7 +1553,7 @@ NV_STATUS uvm_va_space_unregister_gpu_va_space(uvm_va_space_t *va_space, const N
     }
 
     uvm_va_space_up_write(va_space);
-    uvm_up_read_mmap_lock(mm);
+    uvm_va_space_mm_or_current_release_unlock(va_space, mm);
 
     uvm_deferred_free_object_list(&deferred_free_list);
     uvm_gpu_va_space_release(gpu_va_space);
@@ -1525,13 +1613,9 @@ uvm_processor_id_t uvm_processor_mask_find_closest_id(uvm_va_space_t *va_space,
         return UVM_ID_IS_INVALID(id)? UVM_ID_CPU : id;
     }
 
-    // If the CPU would be a candidate, it would have been chosen by now since
-    // it is directly accessible from all processors
-    UVM_ASSERT(!uvm_processor_mask_test(candidates, UVM_ID_CPU));
-
-    // No GPUs with direct access, or the CPU, are in the mask. Just pick the
-    // first GPU in the mask, if any.
-    return uvm_processor_mask_find_first_gpu_id(candidates);
+    // No GPUs with direct access are in the mask. Just pick the first
+    // processor in the mask, if any.
+    return uvm_processor_mask_find_first_id(candidates);
 }
 
 static void uvm_deferred_free_object_channel(uvm_deferred_free_object_t *object, uvm_processor_mask_t *flushed_gpus)
