@@ -353,37 +353,17 @@ struct uvm_va_space_struct
     // The mm currently associated with this VA space, if any.
     uvm_va_space_mm_t va_space_mm;
 
-    struct
+    union
     {
-#if UVM_IBM_NPU_SUPPORTED()
-        // These are the active NPUs in this VA space, that is, all NPUs with
-        // GPUs that have GPU VA spaces registered in this VA space.
-        //
-        // If a bit is clear in npu_active_mask then the corresponding entry of
-        // npu_ref_counts is 0. If a bit is set then the corresponding entry of
-        // npu_ref_counts is greater than 0.
-        NvU32 npu_ref_counts[NV_MAX_NPUS];
-        DECLARE_BITMAP(npu_active_mask, NV_MAX_NPUS);
-#endif
+        uvm_ats_va_space_t ats;
 
-        // Lock protecting npu_ref_counts and npu_active_mask. Invalidations
-        // take this lock for read. GPU VA space register and unregister take
-        // this lock for write. Since all invalidations take the lock for read
-        // for the duration of the invalidate, taking the lock for write also
-        // flushes all invalidates.
-        //
-        // This is a spinlock because the invalidation code paths may be called
-        // with interrupts disabled, so those paths can't take the VA space
-        // lock. We could use a normal exclusive spinlock instead, but a reader/
-        // writer lock is preferred to allow concurrent invalidates in the same
-        // VA space.
-        uvm_rwlock_irqsave_t rwlock;
-    } ats;
 
-#if UVM_IS_CONFIG_HMM()
-    // HMM information about this VA space.
-    uvm_hmm_va_space_t hmm_va_space;
-#endif
+
+
+
+
+
+    };
 
     struct
     {
