@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2020 NVIDIA Corporation
+    Copyright (c) 2016-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -55,18 +55,19 @@ const static struct
 } nvswitch_status_map[] = {
     { NVL_ERR_GENERIC,                  -EIO        },
     { NVL_NO_MEM,                       -ENOMEM     },
-    { NVL_BAD_ARGS ,                    -EINVAL     },
+    { NVL_BAD_ARGS,                     -EINVAL     },
     { NVL_ERR_INVALID_STATE,            -EIO        },
     { NVL_ERR_NOT_SUPPORTED,            -EOPNOTSUPP },
-    { NVL_NOT_FOUND ,                   -EINVAL     },
+    { NVL_NOT_FOUND,                    -EINVAL     },
     { NVL_ERR_STATE_IN_USE,             -EBUSY      },
     { NVL_ERR_NOT_IMPLEMENTED,          -ENOSYS     },
-    { NVL_ERR_INSUFFICIENT_PERMISSIONS  -EPERM      },
-    { NVL_ERR_OPERATING_SYSTEM          -EIO        },
+    { NVL_ERR_INSUFFICIENT_PERMISSIONS, -EPERM      },
+    { NVL_ERR_OPERATING_SYSTEM,         -EIO        },
+    { NVL_MORE_PROCESSING_REQUIRED,     -EAGAIN     },
     { NVL_SUCCESS,                       0          },
 };
 
-static int
+int
 nvswitch_map_status
 (
     NvlStatus status
@@ -384,6 +385,75 @@ nvswitch_find_minor(void)
     return NVSWITCH_DEVICE_INSTANCE_MAX;
 }
 
+static int
+nvswitch_init_i2c_adapters
+(
+    NVSWITCH_DEV *nvswitch_dev
+)
+{
+    NvlStatus retval;
+    NvU32 i, valid_ports_mask;
+    struct i2c_adapter *adapter;
+    nvswitch_i2c_adapter_entry *adapter_entry;
+
+    if (!nvswitch_lib_is_i2c_supported(nvswitch_dev->lib_device))
+    {
+        return 0;
+    }
+
+    retval = nvswitch_lib_get_valid_ports_mask(nvswitch_dev->lib_device,
+                                               &valid_ports_mask);
+    if (retval != NVL_SUCCESS)
+    {
+        printk(KERN_ERR "Failed to get valid I2C ports mask.\n");
+        return -ENODEV;
+    }
+
+    FOR_EACH_INDEX_IN_MASK(32, i, valid_ports_mask)
+    {
+        adapter = nvswitch_i2c_add_adapter(nvswitch_dev, i);
+        if (adapter == NULL)
+        {
+            continue;
+        }
+
+        adapter_entry = nvswitch_os_malloc(sizeof(*adapter_entry));
+        if (adapter_entry == NULL)
+        {
+            printk(KERN_ERR "Failed to create I2C adapter entry.\n");
+            nvswitch_i2c_del_adapter(adapter);
+            continue;
+        }
+
+        adapter_entry->adapter = adapter;
+
+        list_add_tail(&adapter_entry->entry, &nvswitch_dev->i2c_adapter_list);
+    }
+    FOR_EACH_INDEX_IN_MASK_END;
+
+    return 0;
+}
+
+static void
+nvswitch_deinit_i2c_adapters
+(
+    NVSWITCH_DEV *nvswitch_dev
+)
+{
+    nvswitch_i2c_adapter_entry *curr;
+    nvswitch_i2c_adapter_entry *next;
+
+    list_for_each_entry_safe(curr,
+                             next,
+                             &nvswitch_dev->i2c_adapter_list,
+                             entry)
+    {
+        nvswitch_i2c_del_adapter(curr->adapter);
+        list_del(&curr->entry);
+        nvswitch_os_free(curr);
+    }
+}
+
 static int
 nvswitch_init_device
 (
@@ -394,6 +464,8 @@ nvswitch_init_device
     NvlStatus retval;
     int rc;
 
+    INIT_LIST_HEAD(&nvswitch_dev->i2c_adapter_list);
+
     retval = nvswitch_lib_register_device(NV_PCI_DOMAIN_NUMBER(pci_dev),
                                           NV_PCI_BUS_NUMBER(pci_dev),
                                           NV_PCI_SLOT_NUMBER(pci_dev),
@@ -472,10 +544,16 @@ nvswitch_post_init_device
     NVSWITCH_DEV *nvswitch_dev
 )
 {
+    int rc;
     NvlStatus retval;
 
-    retval = nvswitch_lib_post_init_device(nvswitch_dev->lib_device);
+    rc = nvswitch_init_i2c_adapters(nvswitch_dev);
+    if (rc < 0)
+    {
+       return rc;
+    }
 
+    retval = nvswitch_lib_post_init_device(nvswitch_dev->lib_device);
     if (retval != NVL_SUCCESS)
     {
         return -ENODEV;
@@ -636,9 +714,9 @@ nvswitch_device_poll
 {
     nvswitch_file_private_t *private = NVSWITCH_GET_FILE_PRIVATE(file);
     NVSWITCH_DEV *nvswitch_dev = private->nvswitch_dev;
-    NvU32 nonfatal = 0;
-    NvU32 fatal = 0;
     int rc = 0;
+    NvlStatus status;
+    struct NVSWITCH_CLIENT_EVENT *client_event;
 
     rc = mutex_lock_interruptible(&nvswitch_dev->device_mutex);
     if (rc)
@@ -653,12 +731,22 @@ nvswitch_device_poll
         goto done;
     }
 
-    poll_wait(file, &nvswitch_dev->wait_q_errors, wait);
+    status = nvswitch_lib_get_client_event(nvswitch_dev->lib_device,
+                                           (void *) private, &client_event);
+    if (status != NVL_SUCCESS)
+    {
+        printk(KERN_INFO "%s: no events registered for fd\n", nvswitch_dev->name);
+        rc = POLLERR;
+        goto done;
+    }
 
-    // Allow user to poll on either fatal, non-fatal or both types of errors.
-    nvswitch_lib_get_log_count(nvswitch_dev->lib_device, &fatal, &nonfatal);
-    rc = fatal ? POLLPRI : 0;
-    rc |= nonfatal ? POLLIN : 0;
+    poll_wait(file, &private->file_event.wait_q_event, wait);
+
+    if (private->file_event.event_pending)
+    {
+        rc = POLLPRI | POLLIN;
+        private->file_event.event_pending = NV_FALSE;
+    }
 
 done:
     mutex_unlock(&nvswitch_dev->device_mutex);
@@ -1367,6 +1455,10 @@ nvswitch_remove
 
     list_del(&nvswitch_dev->list_node);
 
+    nvswitch_deinit_i2c_adapters(nvswitch_dev);
+
+    WARN_ON(!list_empty(&nvswitch_dev->i2c_adapter_list));
+
     pci_set_drvdata(pci_dev, NULL);
 
     nvswitch_deinit_background_tasks(nvswitch_dev);
@@ -1744,7 +1836,7 @@ nvswitch_os_get_platform_time
 {
     struct timespec64 ts;
 
-    ktime_get_real_ts64(&ts);
+    ktime_get_raw_ts64(&ts);
     return (NvU64) timespec64_to_ns(&ts);
 }
 
@@ -2169,8 +2261,8 @@ nvswitch_os_sync_dma_region_for_device
     return NVL_SUCCESS;
 }
 
-void *
-nvswitch_os_malloc
+static inline void *
+_nvswitch_os_malloc
 (
     NvLength size
 )
@@ -2200,8 +2292,29 @@ nvswitch_os_malloc
     return ptr;
 }
 
-void
-nvswitch_os_free
+void *
+nvswitch_os_malloc_trace
+(
+    NvLength size,
+    const char *file,
+    NvU32 line
+)
+{
+#if defined(NV_MEM_LOGGER)
+    void *ptr = _nvswitch_os_malloc(size);
+    if (ptr)
+    {
+        nv_memdbg_add(ptr, size, file, line);
+    }
+
+    return ptr;
+#else
+    return _nvswitch_os_malloc(size);
+#endif
+}
+
+static inline void
+_nvswitch_os_free
 (
     void *ptr
 )
@@ -2219,6 +2332,24 @@ nvswitch_os_free
     }
 }
 
+void
+nvswitch_os_free
+(
+    void *ptr
+)
+{
+#if defined (NV_MEM_LOGGER)
+    if (ptr == NULL)
+        return;
+
+    nv_memdbg_remove(ptr, 0, NULL, 0);
+
+    return _nvswitch_os_free(ptr);
+#else
+    return _nvswitch_os_free(ptr);
+#endif
+}
+
 NvLength
 nvswitch_os_strlen
 (
