@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2020 NVIDIA Corporation
+    Copyright (c) 2016-2021 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -41,6 +41,14 @@
 
 #define NVSWITCH_KMALLOC_LIMIT (128 * 1024)
 
+#define nvswitch_os_malloc(_size)        nvswitch_os_malloc_trace(_size, __FILE__, __LINE__)
+
+typedef struct
+{
+    struct list_head entry;
+    struct i2c_adapter *adapter;
+} nvswitch_i2c_adapter_entry;
+
 // Per-chip driver state
 typedef struct
 {
@@ -66,12 +74,16 @@ typedef struct
     struct proc_dir_entry *procfs_dir;
 #endif
     NvU8 irq_mechanism;
+    struct list_head i2c_adapter_list;
 } NVSWITCH_DEV;
 
 
+int nvswitch_map_status(NvlStatus status);
 int nvswitch_procfs_init(void);
 void nvswitch_procfs_exit(void);
 int nvswitch_procfs_device_add(NVSWITCH_DEV *nvswitch_dev);
 void nvswitch_procfs_device_remove(NVSWITCH_DEV *nvswitch_dev);
+struct i2c_adapter *nvswitch_i2c_add_adapter(NVSWITCH_DEV *nvswitch_dev, NvU32 port);
+void nvswitch_i2c_del_adapter(struct i2c_adapter *adapter);
 
 #endif // LINUX_NVSWITCH_H
