@@ -1,16 +1,28 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
 
-#include "nv-misc.h"
 #include "os-interface.h"
 #include "nv-linux.h"
 #include "nv-reg.h"
@@ -37,8 +49,6 @@ static acpi_status nv_acpi_find_methods    (acpi_handle, u32, void *, void **);
 static NV_STATUS   nv_acpi_nvif_method     (NvU32, NvU32, void *, NvU16, NvU32 *, void *, NvU16 *);
 
 static NV_STATUS   nv_acpi_wmmx_method     (NvU32, NvU8 *, NvU16 *);
-static NV_STATUS   nv_acpi_mxmi_method     (NvU8 *, NvU16 *);
-static NV_STATUS   nv_acpi_mxms_method     (NvU8 *, NvU16 *);
 
 static const struct acpi_device_id nv_video_device_ids[] = {
     {
@@ -55,15 +65,18 @@ static struct acpi_driver *nv_acpi_driver;
 static acpi_handle nvif_handle = NULL;
 static acpi_handle nvif_parent_gpu_handle  = NULL;
 static acpi_handle wmmx_handle = NULL;
-static acpi_handle mxmi_handle = NULL;
-static acpi_handle mxms_handle = NULL;
-
 
 // Used for AC Power Source Hotplug Handling
 static acpi_handle psr_handle = NULL;
 static acpi_handle psr_device_handle = NULL;
 static nv_acpi_t  *psr_nv_acpi_object = NULL;
 
+static NvBool battery_present = NV_FALSE;
+
+#define BIX_BATTERY_TECHNOLOGY_OFFSET 0x4
+#define BIF_BATTERY_TECHNOLOGY_OFFSET 0x3
+#define BATTERY_RECHARGABLE           0x1
+
 /* Moved into acpi/video.h in Linux 4.10 */
 #ifndef ACPI_VIDEO_NOTIFY_PROBE
 #define ACPI_VIDEO_NOTIFY_PROBE    0x81
@@ -662,10 +675,6 @@ void NV_API_CALL nv_acpi_methods_init(NvU32 *handlesPresent)
 
     if (wmmx_handle)
         *handlesPresent = *handlesPresent | NV_ACPI_WMMX_HANDLE_PRESENT;
-    if (mxmi_handle)
-        *handlesPresent = *handlesPresent | NV_ACPI_MXMI_HANDLE_PRESENT;
-    if (mxms_handle)
-        *handlesPresent = *handlesPresent | NV_ACPI_MXMS_HANDLE_PRESENT;
     if (psr_handle)
     {
         // Since _PSR is not a per-GPU construct we only need to register a
@@ -705,16 +714,6 @@ acpi_status nv_acpi_find_methods(
         wmmx_handle = method_handle;
     }
 
-    if (!acpi_get_handle(handle, "MXMI", &method_handle))
-    {
-        mxmi_handle = method_handle;
-    }
-
-    if (!acpi_get_handle(handle, "MXMS", &method_handle))
-    {
-        mxms_handle = method_handle;
-    }
-
     if (!acpi_get_handle(handle, "_PSR", &method_handle))
     {
         psr_handle = method_handle;
@@ -730,8 +729,6 @@ void NV_API_CALL nv_acpi_methods_uninit(void)
 
     nvif_handle = NULL;
     wmmx_handle = NULL;
-    mxmi_handle = NULL;
-    mxms_handle = NULL;
 
     if (psr_nv_acpi_object != NULL)
     {
@@ -890,14 +887,6 @@ NV_STATUS NV_API_CALL nv_acpi_method(
                                          outDataSize);
             break;
 
-        case NV_EVAL_ACPI_METHOD_MXMI:
-            status = nv_acpi_mxmi_method(outData, outDataSize);
-            break;
-
-        case NV_EVAL_ACPI_METHOD_MXMS:
-            status = nv_acpi_mxms_method(outData, outDataSize);
-            break;
-
         case NV_EVAL_ACPI_METHOD_WMMX:
             status = nv_acpi_wmmx_method(function,  outData, outDataSize);
             break;
@@ -909,165 +898,6 @@ NV_STATUS NV_API_CALL nv_acpi_method(
     return status;
 }
 
-/*
- * This function executes an MXMI ACPI method.
- */
-static NV_STATUS nv_acpi_mxmi_method(
-    NvU8  *pInOut,
-    NvU16 *pSize
-)
-{
-    acpi_status status;
-    struct acpi_object_list input;
-    struct acpi_buffer output = { ACPI_ALLOCATE_BUFFER, NULL };
-    union acpi_object *mxmi = NULL;
-    union acpi_object mxmi_params[2];
-
-    if (!mxmi_handle)
-        return NV_ERR_NOT_SUPPORTED;
-
-    if ((pInOut == NULL) || (pSize == NULL))
-    {
-        nv_printf(NV_DBG_INFO,
-              "NVRM: nv_acpi_mxmi_method: invalid argument(s)!\n");
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    if (*pSize == 0)
-    {
-        *pSize = sizeof(NvU8);
-        return NV_ERR_BUFFER_TOO_SMALL;
-    }
-
-    if (!NV_MAY_SLEEP())
-    {
-#if defined(DEBUG)
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: nv_acpi_mxmi_method: invalid context!\n");
-#endif
-        return NV_ERR_NOT_SUPPORTED;
-    }
-
-    mxmi_params[0].integer.type   = ACPI_TYPE_INTEGER;
-    mxmi_params[0].integer.value  = 0;
-
-    mxmi_params[1].buffer.type    = ACPI_TYPE_BUFFER;
-    mxmi_params[1].buffer.pointer = pInOut;
-    mxmi_params[1].buffer.length  = sizeof(NvU8);
-
-    input.count = 2;
-    input.pointer = mxmi_params;
-
-    status = acpi_evaluate_object(mxmi_handle, NULL, &input, &output);
-    if (ACPI_FAILURE(status))
-    {
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: nv_acpi_mxmi_method: failed to get MXMI data!\n");
-        return NV_ERR_GENERIC;
-    }
-
-    mxmi = output.pointer;
-    if ((mxmi != NULL) && (mxmi->type == ACPI_TYPE_INTEGER))
-    {
-        *pInOut = mxmi->integer.value;
-    }
-    else
-    {
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: nv_acpi_mxmi_method: MXMI data invalid!\n");
-        kfree(output.pointer);
-        return NV_ERR_GENERIC;
-    }
-
-    kfree(output.pointer);
-    return NV_OK;
-}
-
-/*
- * This function executes an MXMS ACPI method.
- */
-static NV_STATUS nv_acpi_mxms_method(
-    NvU8  *pInOut,
-    NvU16 *pSize
-)
-{
-    acpi_status status;
-    struct acpi_object_list input;
-    struct acpi_buffer output = { ACPI_ALLOCATE_BUFFER, NULL };
-    union acpi_object *mxms = NULL;
-    union acpi_object mxms_params[2];
-
-    if (!mxms_handle)
-        return NV_ERR_NOT_SUPPORTED;
-
-    if ((pInOut == NULL) || (pSize == NULL))
-    {
-        nv_printf(NV_DBG_INFO,
-              "NVRM: nv_acpi_mxms_method: invalid argument(s)!\n");
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    if (*pSize == 0)
-    {
-        *pSize = sizeof(NvU8);
-        return NV_ERR_BUFFER_TOO_SMALL;
-    }
-
-    if (!NV_MAY_SLEEP())
-    {
-#if defined(DEBUG)
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: nv_acpi_mxms_method: invalid context!\n");
-#endif
-        return NV_ERR_NOT_SUPPORTED;
-    }
-
-    mxms_params[0].integer.type   = ACPI_TYPE_INTEGER;
-    mxms_params[0].integer.value  = 0;
-
-    mxms_params[1].buffer.type    = ACPI_TYPE_BUFFER;
-    mxms_params[1].buffer.pointer = pInOut;
-    mxms_params[1].buffer.length  = sizeof(NvU8);
-
-    input.count = 2;
-    input.pointer = mxms_params;
-
-    status = acpi_evaluate_object(mxms_handle, NULL, &input, &output);
-    if (ACPI_FAILURE(status))
-    {
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: nv_acpi_mxms_method: failed to get MXMS data!\n");
-        return NV_ERR_GENERIC;
-    }
-
-    mxms = output.pointer;
-    if ((mxms != NULL) && (mxms->type == ACPI_TYPE_BUFFER) &&
-        (mxms->buffer.length != 0))
-    {
-        if (*pSize >= mxms->buffer.length)
-        {
-            *pSize = NV_MIN(*pSize, mxms->buffer.length);
-            memcpy(pInOut, mxms->buffer.pointer, mxms->buffer.length);
-        }
-        else
-        {
-            *pSize = mxms->buffer.length;
-             kfree(output.pointer);
-            return NV_ERR_BUFFER_TOO_SMALL;
-        }
-    }
-    else
-    {
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: nv_acpi_mxms_method: MXMS data invalid!\n");
-        kfree(output.pointer);
-        return NV_ERR_GENERIC;
-    }
-
-    kfree(output.pointer);
-    return NV_OK;
-}
-
 /*
  * This function executes an NVIF ACPI method.
  */
@@ -1182,6 +1012,7 @@ NV_STATUS NV_API_CALL nv_acpi_dsm_method(
     nv_state_t *nv,
     NvU8  *pAcpiDsmGuid,
     NvU32 acpiDsmRev,
+    NvBool acpiNvpcfDsmFunction,
     NvU32 acpiDsmSubFunction,
     void  *pInParams,
     NvU16 inParamSize,
@@ -1208,7 +1039,7 @@ NV_STATUS NV_API_CALL nv_acpi_dsm_method(
 
     if ((!pInParams) || (inParamSize > MAX_INPUT_PARAM_SIZE) || (!pOutData) || (!pSize))
     {
-        nv_printf(NV_DBG_ERRORS,
+        nv_printf(NV_DBG_INFO,
                   "NVRM: %s: invalid argument(s)!\n", __FUNCTION__);
         return NV_ERR_INVALID_ARGUMENT;
     }
@@ -1250,7 +1081,20 @@ NV_STATUS NV_API_CALL nv_acpi_dsm_method(
     input.count = 4;
     input.pointer = dsm_params;
 
-    acpi_status = acpi_evaluate_object(dev_handle, "_DSM", &input, &output);
+    if (acpiNvpcfDsmFunction)
+    {
+        //
+        // acpi_evaluate_object() can operate with either valid object pathname or
+        // valid object handle. For NVPCF DSM function, use valid pathname as we do
+        // not have device handle for NVPCF device
+        //
+        dev_handle = NULL;
+        acpi_status = acpi_evaluate_object(dev_handle, "\\_SB.NPCF._DSM", &input, &output);
+    }
+    else
+    {
+        acpi_status = acpi_evaluate_object(dev_handle, "_DSM", &input, &output);
+    }
 
     if (ACPI_FAILURE(acpi_status))
     {
@@ -1804,6 +1648,108 @@ NV_STATUS NV_API_CALL nv_acpi_mux_method(
     return NV_OK;
 }
 
+static acpi_status nv_acpi_find_battery_info(
+    acpi_handle handle,
+    NvBool bUseBix
+)
+{
+    acpi_status status = AE_OK;
+    struct acpi_buffer buf = { ACPI_ALLOCATE_BUFFER, NULL };
+    union acpi_object *object_package;
+    NvU32 battery_technology_offset;
+
+    status = acpi_evaluate_object(handle, NULL, NULL, &buf);
+
+    if (ACPI_FAILURE(status))
+    {
+        nv_printf(NV_DBG_INFO, "NVRM: Failed to evaluate battery's object\n");
+        return AE_OK;
+    }
+
+    if (!buf.pointer)
+    {
+        nv_printf(NV_DBG_INFO, "NVRM: Battery object output buffer is null\n");
+        return AE_OK;
+    }
+
+    object_package = buf.pointer;
+
+    if (object_package->type != ACPI_TYPE_PACKAGE)
+    {
+        nv_printf(NV_DBG_INFO, "NVRM: Battery method output is not package\n");
+        return AE_OK;
+    }
+
+    if (bUseBix)
+    {
+        battery_technology_offset = BIX_BATTERY_TECHNOLOGY_OFFSET;
+    }
+    else
+    {
+        battery_technology_offset = BIF_BATTERY_TECHNOLOGY_OFFSET;
+    }
+
+    /*
+     * Only checking here for Battery technology type.
+     * Other fields like Battery Model/Serial number could also be checked but
+     * driver need to support the case where user has removed battery from the
+     * system.
+     * _STA method on the battery device handle couldn't be used due to the same
+     * reason.
+     * Hence just cheking if battery technology of slot is rechargable or not.
+     */
+    
+    if ((object_package->package.elements[battery_technology_offset].type != ACPI_TYPE_INTEGER) ||
+        (object_package->package.elements[battery_technology_offset].integer.value != BATTERY_RECHARGABLE))
+    {
+        return AE_OK;
+    }
+
+    battery_present = NV_TRUE;
+
+    /* Stop traversing acpi tree. */
+    return AE_CTRL_TERMINATE;
+}
+
+static acpi_status nv_acpi_find_battery_device(
+    acpi_handle handle,
+    u32 nest_level,
+    void *dummy1,
+    void **dummy2
+)
+{
+    acpi_handle bif_method_handle;
+    acpi_handle bix_method_handle;
+    acpi_status status = AE_OK;
+
+    // Find method Battery Information /Extended/ (_BIX or _BIF) and then Battery type.
+    if (!acpi_get_handle(handle, "_BIX", &bix_method_handle))
+    {
+        status = nv_acpi_find_battery_info(bix_method_handle, NV_TRUE/*bUseBix*/);
+    }
+
+    if ((battery_present == NV_FALSE) &&
+        !acpi_get_handle(handle, "_BIF", &bif_method_handle))
+    {
+        status = nv_acpi_find_battery_info(bif_method_handle, NV_FALSE/*bUseBix*/);
+    }
+
+    return status;
+}
+
+NvBool NV_API_CALL nv_acpi_is_battery_present(void)
+{
+    NV_ACPI_WALK_NAMESPACE(ACPI_TYPE_DEVICE, ACPI_ROOT_OBJECT, ACPI_UINT32_MAX,
+                           nv_acpi_find_battery_device, NULL, NULL);
+
+    if (battery_present == NV_TRUE)
+    {
+        return NV_TRUE;
+    }
+
+    return NV_FALSE;
+}
+
 #else // NV_LINUX_ACPI_EVENTS_SUPPORTED
 
 int nv_acpi_init(void)
@@ -1844,6 +1790,7 @@ NV_STATUS NV_API_CALL nv_acpi_dsm_method(
     nv_state_t *nv,
     NvU8  *pAcpiDsmGuid,
     NvU32 acpiDsmRev,
+    NvBool acpiNvpcfDsmFunction,
     NvU32 acpiDsmSubFunction,
     void  *pInParams,
     NvU16 inParamSize,
@@ -1914,4 +1861,9 @@ NV_STATUS NV_API_CALL nv_acpi_mux_method(
 {
     return NV_ERR_NOT_SUPPORTED;
 }
+
+NvBool NV_API_CALL nv_acpi_is_battery_present(void)
+{
+    return NV_FALSE;
+}
 #endif
