@@ -1,40 +1,31 @@
-/*******************************************************************************
-    Copyright (c) 2019-2021 NVIDIA Corporation
-
-    Permission is hereby granted, free of charge, to any person obtaining a copy
-    of this software and associated documentation files (the "Software"), to
-    deal in the Software without restriction, including without limitation the
-    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
-    sell copies of the Software, and to permit persons to whom the Software is
-    furnished to do so, subject to the following conditions:
-
-        The above copyright notice and this permission notice shall be
-        included in all copies or substantial portions of the Software.
-
-    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-    DEALINGS IN THE SOFTWARE.
-
-*******************************************************************************/
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ */
 
 #include "nv-linux.h"
 #include "nv-caps.h"
 #include "nv-procfs.h"
 #include "nv-hash.h"
 
-/*
- * This temporary module parameter will be nuked soon when devfs support
- * is enabled by default.
- */
-static int nv_cap_enable_devfs = 1;
-module_param(nv_cap_enable_devfs, int, S_IRUGO);
-MODULE_PARM_DESC(nv_cap_enable_devfs, "Enable (1) or disable (0) nv-caps " \
-                 "devfs support. Default: 1");
-
 extern int NVreg_ModifyDeviceFiles;
 
 /* sys_close() or __close_fd() */
@@ -166,8 +157,6 @@ typedef struct nv_cap_file_private
 {
     int minor;
     int permissions;
-    /* Will be used for fork() detection.*/
-    struct files_struct *files;
     int modify;
     char buffer[NV_CAP_PROCFS_WRITE_BUF_SIZE];
     off_t offset;
@@ -237,13 +226,13 @@ static int nv_procfs_read_mig_minors(struct seq_file *s, void *v)
     return 0;
 }
 
-NV_DEFINE_SINGLE_PROCFS_FILE(nvlink_minors,
-                             NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE,
-                             NV_READ_UNLOCK_SYSTEM_PM_LOCK);
+NV_DEFINE_SINGLE_PROCFS_FILE_READ_ONLY(nvlink_minors,
+                                       NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE,
+                                       NV_READ_UNLOCK_SYSTEM_PM_LOCK);
 
-NV_DEFINE_SINGLE_PROCFS_FILE(mig_minors,
-                             NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE,
-                             NV_READ_UNLOCK_SYSTEM_PM_LOCK);
+NV_DEFINE_SINGLE_PROCFS_FILE_READ_ONLY(mig_minors,
+                                       NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE,
+                                       NV_READ_UNLOCK_SYSTEM_PM_LOCK);
 
 static void nv_cap_procfs_exit(void)
 {
@@ -378,12 +367,9 @@ static int nv_cap_procfs_read(struct seq_file *s, void *v)
 {
     nv_cap_file_private_t *private = s->private;
 
-    if (nv_cap_enable_devfs)
-    {
-        seq_printf(s, "%s: %d\n", "DeviceFileMinor", private->minor);
-        seq_printf(s, "%s: %d\n", "DeviceFileMode", private->permissions);
-        seq_printf(s, "%s: %d\n", "DeviceFileModify", private->modify);
-    }
+    seq_printf(s, "%s: %d\n", "DeviceFileMinor", private->minor);
+    seq_printf(s, "%s: %d\n", "DeviceFileMode", private->permissions);
+    seq_printf(s, "%s: %d\n", "DeviceFileModify", private->modify);
 
     return 0;
 }
@@ -400,8 +386,6 @@ static int nv_cap_procfs_open(struct inode *inode, struct file *file)
         return -ENOMEM;
     }
 
-    /* Just copy over data for "fd" validation */
-    private->files = current->files;
     private->minor = cap->minor;
     private->permissions = cap->permissions;
     private->offset = 0;
@@ -481,9 +465,10 @@ int NV_API_CALL nv_cap_validate_and_dup_fd(const nv_cap_t *cap, int fd)
 {
     struct file *file;
     int dup_fd;
-    const nv_cap_file_private_t *private = NULL;
     struct inode *inode = NULL;
     dev_t rdev = 0;
+    struct files_struct *files = current->files;
+    struct fdtable *fdt;
 
     if (cap == NULL)
     {
@@ -502,42 +487,15 @@ int NV_API_CALL nv_cap_validate_and_dup_fd(const nv_cap_t *cap, int fd)
         goto err;
     }
 
-    if (nv_cap_enable_devfs)
+    /* Make sure the fd belongs to the nv-cap-drv */
+    if (file->f_op != &g_nv_cap_drv_fops)
     {
-        /* Make sure the fd belongs to the nv-cap-drv */
-        if (file->f_op != &g_nv_cap_drv_fops)
-        {
-            goto err;
-        }
-
-        /* Make sure the fd has the expected capability */
-        rdev = inode->i_rdev;
-        if (MINOR(rdev) != cap->minor)
-        {
-            goto err;
-        }
-
-        private = file->private_data;
-    }
-    else
-    {
-        /*
-         * Make sure the fd belongs to procfs and validate the associated
-         * capability; the NV_PDE_DATA(inode) check ensures this file was created
-         * by nv_cap_create_file_entry().
-         */
-        if ((strcmp(inode->i_sb->s_type->name, "proc") != 0) ||
-            (NV_PDE_DATA(inode) != cap))
-        {
-            goto err;
-        }
-
-        private = ((struct seq_file *)file->private_data)->private;
+        goto err;
     }
 
-    /* Make sure the fd is not duped during fork() */
-    if ((private == NULL) ||
-        (private->files != current->files))
+    /* Make sure the fd has the expected capability */
+    rdev = inode->i_rdev;
+    if (MINOR(rdev) != cap->minor)
     {
         goto err;
     }
@@ -545,11 +503,26 @@ int NV_API_CALL nv_cap_validate_and_dup_fd(const nv_cap_t *cap, int fd)
     dup_fd = NV_GET_UNUSED_FD_FLAGS(O_CLOEXEC);
     if (dup_fd < 0)
     {
-          dup_fd = NV_GET_UNUSED_FD();
-          if (dup_fd < 0)
-          {
-               goto err;
-          }
+        dup_fd = NV_GET_UNUSED_FD();
+        if (dup_fd < 0)
+        {
+            goto err;
+        }
+
+        /*
+         * Set CLOEXEC before installing the FD.
+         *
+         * If fork() happens in between, the opened unused FD will have
+         * a NULL struct file associated with it, which is okay.
+         *
+         * The only well known bug here is the race with dup(2), which is
+         * already documented in the kernel, see fd_install()'s description.
+         */
+
+        spin_lock(&files->file_lock);
+        fdt = files_fdtable(files);
+        NV_SET_CLOSE_ON_EXEC(dup_fd, fdt);
+        spin_unlock(&files->file_lock);
     }
 
     fd_install(dup_fd, file);
@@ -673,15 +646,7 @@ nv_cap_t* NV_API_CALL nv_cap_create_file_entry(nv_cap_t *parent_cap,
     cap->parent = parent_cap->entry;
     cap->permissions = mode;
 
-    /* Make proc entries world visible if devfs is enabled */
-    if (nv_cap_enable_devfs)
-    {
-        mode = (S_IFREG | S_IRUGO);
-    }
-    else
-    {
-        mode |= S_IFREG;
-    }
+    mode = (S_IFREG | S_IRUGO);
 
     minor = nv_cap_find_minor(cap->path);
     if (minor < 0)
@@ -718,15 +683,7 @@ nv_cap_t* NV_API_CALL nv_cap_create_dir_entry(nv_cap_t *parent_cap,
     cap->permissions = mode;
     cap->minor = -1;
 
-    /* Make proc entries world visible if devfs is enabled */
-    if (nv_cap_enable_devfs)
-    {
-        mode = (S_IFDIR | S_IRUGO | S_IXUGO);
-    }
-    else
-    {
-        mode |= S_IFDIR;
-    }
+    mode = (S_IFDIR | S_IRUGO | S_IXUGO);
 
     cap->entry = NV_PROC_MKDIR_MODE(name, mode, parent_cap->entry);
     if (cap->entry == NULL)
@@ -782,30 +739,11 @@ void NV_API_CALL nv_cap_destroy_entry(nv_cap_t *cap)
 
 static int nv_cap_drv_open(struct inode *inode, struct file *file)
 {
-    nv_cap_file_private_t *private = NULL;
-
-    NV_KMALLOC(private, sizeof(nv_cap_file_private_t));
-    if (private == NULL)
-    {
-        return -ENOMEM;
-    }
-
-    /* Just copy over data for "fd" validation */
-    private->files = current->files;
-    file->private_data = private;
-
     return 0;
 }
 
 static int nv_cap_drv_release(struct inode *inode, struct file *file)
 {
-    if (file->private_data != NULL)
-    {
-        NV_KFREE(file->private_data, sizeof(nv_cap_file_private_t));
-
-        file->private_data = NULL;
-    }
-
     return 0;
 }
 
@@ -822,12 +760,6 @@ int NV_API_CALL nv_cap_drv_init(void)
 
     nv_cap_tables_init();
 
-    if (!nv_cap_enable_devfs)
-    {
-        nv_printf(NV_DBG_INFO, "nv-caps-drv is disabled.\n");
-        return 0;
-    }
-
     if (g_nv_cap_drv.initialized)
     {
         nv_printf(NV_DBG_ERRORS, "nv-caps-drv is already initialized.\n");
