@@ -1,15 +1,27 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2015 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2015 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
-#include "nv-misc.h"
 
 #include "os-interface.h"
 #include "nv-linux.h"
@@ -141,6 +153,11 @@ NV_STATUS nv_create_dma_map_scatterlist(nv_dma_map_t *dma_map)
 
     WARN_ON(NvU64_HI32(num_submaps) != 0);
 
+    if (dma_map->import_sgt && (num_submaps != 1))
+    {
+        return -EINVAL;
+    }
+
     dma_map->mapping.discontig.submap_count = NvU64_LO32(num_submaps);
 
     status = os_alloc_mem((void **)&dma_map->mapping.discontig.submaps,
@@ -153,6 +170,16 @@ NV_STATUS nv_create_dma_map_scatterlist(nv_dma_map_t *dma_map)
     os_mem_set((void *)dma_map->mapping.discontig.submaps, 0,
         sizeof(nv_dma_submap_t) * dma_map->mapping.discontig.submap_count);
 
+    /* If we have an imported SGT, just use that directly. */
+    if (dma_map->import_sgt)
+    {
+        dma_map->mapping.discontig.submaps[0].page_count = dma_map->page_count;
+        dma_map->mapping.discontig.submaps[0].sgt = *dma_map->import_sgt;
+        dma_map->mapping.discontig.submaps[0].imported = NV_TRUE;
+
+        return status;
+    }
+
     NV_FOR_EACH_DMA_SUBMAP(dma_map, submap, i)
     {
         NvU64 submap_size = NV_MIN(NV_DMA_SUBMAP_MAX_PAGES << PAGE_SHIFT,
@@ -240,7 +267,7 @@ void nv_destroy_dma_map_scatterlist(nv_dma_map_t *dma_map)
 
     NV_FOR_EACH_DMA_SUBMAP(dma_map, submap, i)
     {
-        if (submap->page_count == 0)
+        if ((submap->page_count == 0) || submap->imported)
         {
             break;
         }
@@ -392,6 +419,84 @@ static void nv_dma_nvlink_addr_decompress
 #endif
 }
 
+NV_STATUS NV_API_CALL nv_dma_map_sgt(
+    nv_dma_device_t *dma_dev,
+    NvU64            page_count,
+    NvU64           *va_array,
+    NvU32            cache_type,
+    void           **priv
+)
+{
+    NV_STATUS status;
+    nv_dma_map_t *dma_map = NULL;
+
+    if (priv == NULL)
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    if (page_count > os_get_num_phys_pages())
+    {
+        NV_DMA_DEV_PRINTF(NV_DBG_ERRORS, dma_dev,
+                "DMA mapping request too large!\n");
+        return NV_ERR_INVALID_REQUEST;
+    }
+
+    status = os_alloc_mem((void **)&dma_map, sizeof(nv_dma_map_t));
+    if (status != NV_OK)
+    {
+        NV_DMA_DEV_PRINTF(NV_DBG_ERRORS, dma_dev,
+                "Failed to allocate nv_dma_map_t!\n");
+        return status;
+    }
+
+    dma_map->dev = dma_dev->dev;
+    dma_map->pages = NULL;
+    dma_map->import_sgt = (struct sg_table *) *priv;
+    dma_map->page_count = page_count;
+    dma_map->contiguous = NV_FALSE;
+    dma_map->cache_type = cache_type;
+
+    dma_map->mapping.discontig.submap_count = 0;
+    status = nv_dma_map_scatterlist(dma_dev, dma_map, va_array);
+
+    if (status != NV_OK)
+    {
+        os_free_mem(dma_map);
+    }
+    else
+    {
+        *priv = dma_map;
+        nv_dma_nvlink_addr_compress(dma_dev, va_array, dma_map->page_count,
+                                    dma_map->contiguous);
+    }
+
+    return status;
+}
+
+NV_STATUS NV_API_CALL nv_dma_unmap_sgt(
+    nv_dma_device_t *dma_dev,
+    void           **priv
+)
+{
+    nv_dma_map_t *dma_map;
+
+    if (priv == NULL)
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    dma_map = *priv;
+
+    *priv = NULL;
+
+    nv_dma_unmap_scatterlist(dma_map);
+
+    os_free_mem(dma_map);
+
+    return NV_OK;
+}
+
 NV_STATUS NV_API_CALL nv_dma_map_pages(
     nv_dma_device_t *dma_dev,
     NvU64            page_count,
@@ -430,6 +535,7 @@ NV_STATUS NV_API_CALL nv_dma_map_pages(
 
     dma_map->dev = dma_dev->dev;
     dma_map->pages = *priv;
+    dma_map->import_sgt = NULL;
     dma_map->page_count = page_count;
     dma_map->contiguous = contig;
     dma_map->cache_type = cache_type;
@@ -535,6 +641,19 @@ NV_STATUS NV_API_CALL nv_dma_map_alloc
     NvU32 cache_type = NV_MEMORY_CACHED;
     NvU64 pages_size = sizeof(struct page *) * (contig ? 1 : page_count);
 
+    /* If we have an imported SGT, just use that directly. */
+    if (at && at->import_sgt)
+    {
+        *priv = at->import_sgt;
+        status = nv_dma_map_sgt(dma_dev, page_count, va_array, at->cache_type,
+                                priv);
+        if (status != NV_OK)
+        {
+            *priv = at;
+        }
+        return status;
+    }
+
     /*
      * Convert the nv_alloc_t into a struct page * array for
      * nv_dma_map_pages().
@@ -604,22 +723,37 @@ NV_STATUS NV_API_CALL nv_dma_unmap_alloc
     void           **priv
 )
 {
-    NV_STATUS status = nv_dma_unmap_pages(dma_dev, page_count, va_array, priv);
-    if (status != NV_OK)
+    NV_STATUS status = NV_OK;
+    nv_dma_map_t *dma_map;
+
+    if (priv == NULL)
     {
-        /*
-         * If nv_dma_unmap_pages() fails, we hit an assert condition and the
-         * priv argument won't be the page array we allocated in
-         * nv_dma_map_alloc(), so we skip the free here. But note that since
-         * this is an assert condition it really should never happen.
-         */
-        return status;
+        return NV_ERR_NOT_SUPPORTED;
     }
 
-    /* Free the struct page * array allocated by nv_dma_map_alloc() */
-    os_free_mem(*priv);
+    dma_map = *priv;
 
-    return NV_OK;
+    if (!dma_map->import_sgt)
+    {
+        status = nv_dma_unmap_pages(dma_dev, page_count, va_array, priv);
+        if (status != NV_OK)
+        {
+            /*
+             * If nv_dma_unmap_pages() fails, we hit an assert condition and the
+             * priv argument won't be the page array we allocated in
+             * nv_dma_map_alloc(), so we skip the free here. But note that since
+             * this is an assert condition it really should never happen.
+             */
+            return status;
+        }
+
+        /* Free the struct page * array allocated by nv_dma_map_alloc() */
+        os_free_mem(*priv);
+    } else {
+        status = nv_dma_unmap_sgt(dma_dev, priv);
+    }
+
+    return status;
 }
 
 static NvBool nv_dma_is_map_resource_implemented
@@ -797,7 +931,7 @@ void NV_API_CALL nv_dma_cache_invalidate
     void *priv
 )
 {
-#if defined(NVCPU_FAMILY_ARM)
+#if defined(NVCPU_AARCH64)
     nv_dma_map_t *dma_map = priv;
 
     if (dma_map->contiguous)
@@ -832,24 +966,98 @@ void NV_API_CALL nv_dma_enable_nvlink
     dma_dev->nvlink = NV_TRUE;
 }
 
-#define IMPORT_STUBS_NEEDED 1
+#define IMPORT_DMABUF_STUBS_NEEDED 1
+#define IMPORT_SGT_STUBS_NEEDED 1
 
+#if defined(NV_LINUX_DMA_BUF_H_PRESENT)
 
+#if defined(NV_DRM_AVAILABLE) && defined(NV_DRM_DRM_GEM_H_PRESENT)
 
+#define IMPORT_SGT_STUBS_NEEDED 0
 
+/*
+ * drm_gem_object_{get/put}() added by commit
+ * e6b62714e87c8811d5564b6a0738dcde63a51774 (2017-02-28) and
+ * drm_gem_object_{reference/unreference}() removed by commit
+ * 3e70fd160cf0b1945225eaa08dd2cb8544f21cb8 (2018-11-15).
+ */
 
+static inline void
+nv_dma_gem_object_unreference_unlocked(struct drm_gem_object *gem)
+{
+#if defined(NV_DRM_GEM_OBJECT_GET_PRESENT)
 
+#if defined(NV_DRM_GEM_OBJECT_PUT_UNLOCK_PRESENT)
+    drm_gem_object_put_unlocked(gem);
+#else
+    drm_gem_object_put(gem);
+#endif
 
+#else
+    drm_gem_object_unreference_unlocked(gem);
+#endif
+}
 
+static inline void
+nv_dma_gem_object_reference(struct drm_gem_object *gem)
+{
+#if defined(NV_DRM_GEM_OBJECT_GET_PRESENT)
+    drm_gem_object_get(gem);
+#else
+    drm_gem_object_reference(gem);
+#endif
+}
 
+NV_STATUS NV_API_CALL nv_dma_import_sgt
+(
+    nv_dma_device_t *dma_dev,
+    struct sg_table *sgt,
+    struct drm_gem_object *gem
+)
+{
+    if ((dma_dev == NULL) ||
+        (sgt == NULL) ||
+        (gem == NULL))
+    {
+        NV_DMA_DEV_PRINTF(NV_DBG_ERRORS, dma_dev,
+                "Import arguments are NULL!\n");
+        return NV_ERR_INVALID_ARGUMENT;
+    }
 
+    // Prevent the kernel module controlling GEM from being unloaded
+    if (!try_module_get(gem->dev->driver->fops->owner))
+    {
+        NV_DMA_DEV_PRINTF(NV_DBG_ERRORS, dma_dev,
+                "Couldn't reference the GEM object's owner!\n");
+        return NV_ERR_INVALID_DEVICE;
+    }
 
+    // Do nothing with SGT, it is already mapped and pinned by the exporter
 
+    nv_dma_gem_object_reference(gem);
 
+    return NV_OK;
+}
 
+void NV_API_CALL nv_dma_release_sgt
+(
+    struct sg_table *sgt,
+    struct drm_gem_object *gem
+)
+{
+    if (gem == NULL)
+    {
+        return;
+    }
 
+    // Do nothing with SGT, it will be unmapped and unpinned by the exporter
+    WARN_ON(sgt == NULL);
 
+    nv_dma_gem_object_unreference_unlocked(gem);
 
+    module_put(gem->dev->driver->fops->owner);
+}
+#endif /* NV_DRM_AVAILABLE && NV_DRM_DRM_GEM_H_PRESENT */
 
 
 
@@ -1000,13 +1208,68 @@ void NV_API_CALL nv_dma_enable_nvlink
 
 
 
-#if IMPORT_STUBS_NEEDED == 1
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+#endif /* NV_LINUX_DMA_BUF_H_PRESENT */
+
+#if IMPORT_SGT_STUBS_NEEDED == 1
+NV_STATUS NV_API_CALL nv_dma_import_sgt
+(
+    nv_dma_device_t *dma_dev,
+    struct sg_table *sgt,
+    struct drm_gem_object *gem
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
+void NV_API_CALL nv_dma_release_sgt
+(
+    struct sg_table *sgt,
+    struct drm_gem_object *gem
+)
+{
+}
+#endif
+
+#if IMPORT_DMABUF_STUBS_NEEDED == 1
 NV_STATUS NV_API_CALL nv_dma_import_dma_buf
 (
     nv_dma_device_t *dma_dev,
     struct dma_buf *dma_buf,
     NvU32 *size,
     void **user_pages,
+    struct sg_table **sgt,
     nv_dma_buf_t **import_priv
 )
 {
@@ -1019,6 +1282,7 @@ NV_STATUS NV_API_CALL nv_dma_import_from_fd
     NvS32 fd,
     NvU32 *size,
     void **user_pages,
+    struct sg_table **sgt,
     nv_dma_buf_t **import_priv
 )
 {
