@@ -1,41 +1,38 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2005-2019 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2005-2019 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
 
 #include <linux/i2c.h>
 
-#include "nv-misc.h"
 #include "os-interface.h"
 #include "nv-linux.h"
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #if defined(CONFIG_I2C) || defined(CONFIG_I2C_MODULE)
 
-static int nv_i2c_algo_master_xfer(struct i2c_adapter *adap, struct i2c_msg msgs[], int num)
+static int nv_i2c_algo_master_xfer(struct i2c_adapter *adapter, struct i2c_msg msgs[], int num)
 {
-    nv_state_t *nv = (nv_state_t *)adap->algo_data;
+    nv_state_t *nv = (nv_state_t *)adapter->algo_data;
     unsigned int i = 0;
     int rc;
     NV_STATUS rmStatus = NV_OK;
@@ -66,7 +63,7 @@ static int nv_i2c_algo_master_xfer(struct i2c_adapter *adap, struct i2c_msg msgs
         }
         else
         {
-            rmStatus = rm_i2c_transfer(sp, nv, (void *)adap,
+            rmStatus = rm_i2c_transfer(sp, nv, (void *)adapter,
                                        (msgs[i].flags & I2C_M_RD) ?
                                            NV_I2C_CMD_READ : NV_I2C_CMD_WRITE,
                                        (NvU8)(msgs[i].addr & 0x7f), 0,
@@ -81,7 +78,7 @@ static int nv_i2c_algo_master_xfer(struct i2c_adapter *adap, struct i2c_msg msgs
 }
 
 static int nv_i2c_algo_smbus_xfer(
-    struct i2c_adapter *adap,
+    struct i2c_adapter *adapter,
     u16 addr,
     unsigned short flags,
     char read_write,
@@ -90,7 +87,7 @@ static int nv_i2c_algo_smbus_xfer(
     union i2c_smbus_data *data
 )
 {
-    nv_state_t *nv = (nv_state_t *)adap->algo_data;
+    nv_state_t *nv = (nv_state_t *)adapter->algo_data;
     int rc;
     NV_STATUS rmStatus = NV_OK;
     nvidia_stack_t *sp = NULL;
@@ -106,7 +103,7 @@ static int nv_i2c_algo_smbus_xfer(
     switch (size)
     {
         case I2C_SMBUS_QUICK:
-            rmStatus = rm_i2c_transfer(sp, nv, (void *)adap,
+            rmStatus = rm_i2c_transfer(sp, nv, (void *)adapter,
                                        (read_write == I2C_SMBUS_READ) ?
                                            NV_I2C_CMD_SMBUS_QUICK_READ :
                                            NV_I2C_CMD_SMBUS_QUICK_WRITE,
@@ -116,7 +113,7 @@ static int nv_i2c_algo_smbus_xfer(
         case I2C_SMBUS_BYTE:
             if (read_write == I2C_SMBUS_READ)
             {
-                rmStatus = rm_i2c_transfer(sp, nv, (void *)adap,
+                rmStatus = rm_i2c_transfer(sp, nv, (void *)adapter,
                                            NV_I2C_CMD_READ,
                                            (NvU8)(addr & 0x7f), 0, 1,
                                            (NvU8 *)&data->byte);
@@ -124,7 +121,7 @@ static int nv_i2c_algo_smbus_xfer(
             else
             {
                 u8 data = command;
-                rmStatus = rm_i2c_transfer(sp, nv, (void *)adap,
+                rmStatus = rm_i2c_transfer(sp, nv, (void *)adapter,
                                            NV_I2C_CMD_WRITE,
                                            (NvU8)(addr & 0x7f), 0, 1,
                                            (NvU8 *)&data);
@@ -132,7 +129,7 @@ static int nv_i2c_algo_smbus_xfer(
             break;
 
         case I2C_SMBUS_BYTE_DATA:
-            rmStatus = rm_i2c_transfer(sp, nv, (void *)adap,
+            rmStatus = rm_i2c_transfer(sp, nv, (void *)adapter,
                                        (read_write == I2C_SMBUS_READ) ?
                                            NV_I2C_CMD_SMBUS_READ :
                                            NV_I2C_CMD_SMBUS_WRITE,
@@ -147,7 +144,7 @@ static int nv_i2c_algo_smbus_xfer(
                 data->block[2] = (data->word >> 8);
             }
 
-            rmStatus = rm_i2c_transfer(sp, nv, (void *)adap,
+            rmStatus = rm_i2c_transfer(sp, nv, (void *)adapter,
                                        (read_write == I2C_SMBUS_READ) ?
                                            NV_I2C_CMD_SMBUS_READ :
                                            NV_I2C_CMD_SMBUS_WRITE,
@@ -162,7 +159,7 @@ static int nv_i2c_algo_smbus_xfer(
             break;
 
         case I2C_SMBUS_BLOCK_DATA:
-            rmStatus = rm_i2c_transfer(sp, nv, (void *)adap,
+            rmStatus = rm_i2c_transfer(sp, nv, (void *)adapter,
                                         (read_write == I2C_SMBUS_READ) ?
                                             NV_I2C_CMD_SMBUS_BLOCK_READ :
                                             NV_I2C_CMD_SMBUS_BLOCK_WRITE,
@@ -180,9 +177,9 @@ static int nv_i2c_algo_smbus_xfer(
     return (rmStatus != NV_OK) ? rc : 0;
 }
 
-static u32 nv_i2c_algo_functionality(struct i2c_adapter *adap)
+static u32 nv_i2c_algo_functionality(struct i2c_adapter *adapter)
 {
-    nv_state_t *nv = (nv_state_t *)adap->algo_data;
+    nv_state_t *nv = (nv_state_t *)adapter->algo_data;
     u32 ret = I2C_FUNC_I2C;
     nvidia_stack_t *sp = NULL;
 
@@ -191,7 +188,7 @@ static u32 nv_i2c_algo_functionality(struct i2c_adapter *adap)
         return 0;
     }
 
-    if (rm_i2c_is_smbus_capable(sp, nv, adap))
+    if (rm_i2c_is_smbus_capable(sp, nv, adapter))
     {
         ret |= (I2C_FUNC_SMBUS_QUICK |
                 I2C_FUNC_SMBUS_BYTE |
@@ -446,6 +443,15 @@ void NV_API_CALL nv_i2c_del_adapter(nv_state_t *nv, void *data)
 
 
 
+
+
+
+
+
+
+
+
+
 
 
 
