@@ -1,15 +1,27 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2019 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
-#include "nv-misc.h"
 
 #include "os-interface.h"
 #include "nv-linux.h"
@@ -156,7 +168,7 @@ nvidia_vma_access(
     else if (IS_FB_OFFSET(nv, offset, length))
     {
         addr = (offset & PAGE_MASK);
-        kernel_mapping = os_map_kernel_space(addr, PAGE_SIZE, NV_MEMORY_UNCACHED, NV_MEMORY_TYPE_SYSTEM);
+        kernel_mapping = os_map_kernel_space(addr, PAGE_SIZE, NV_MEMORY_UNCACHED);
         if (kernel_mapping == NULL)
             return -ENOMEM;
 
@@ -273,7 +285,7 @@ static struct vm_operations_struct nv_vm_ops = {
 int nv_encode_caching(
     pgprot_t *prot,
     NvU32     cache_type,
-    NvU32     memory_type
+    nv_memory_type_t memory_type
 )
 {
     pgprot_t tmp;
@@ -388,6 +400,7 @@ int static nvidia_mmap_sysmem(
         else
 #endif
         {
+            vma->vm_page_prot = nv_adjust_pgprot(vma->vm_page_prot, 0);
             ret = vm_insert_page(vma, start,
                                  NV_GET_PAGE_STRUCT(at->page_table[j]->phys_addr));
         }
@@ -403,7 +416,6 @@ int static nvidia_mmap_sysmem(
     return ret;
 }
 
-#if defined(NVCPU_PPC64LE)
 static int nvidia_mmap_numa(
     struct vm_area_struct *vma,
     const nv_alloc_mapping_context_t *mmap_context)
@@ -434,7 +446,6 @@ static int nvidia_mmap_numa(
 
     return 0;
 }
-#endif
 
 int nvidia_mmap_helper(
     nv_state_t *nv,
@@ -526,14 +537,15 @@ int nvidia_mmap_helper(
         if (nvl->safe_to_mmap)
         {
             nvl->all_mappings_revoked = NV_FALSE;
-#if defined(NVCPU_PPC64LE)
+
             //
             // This path is similar to the sysmem mapping code.
             // TODO: Refactor is needed as part of bug#2001704.
             // Use pfn_valid to determine whether the physical address has
             // backing struct page. This is used to isolate P8 from P9.
             //
-            if (!IS_REG_OFFSET(nv, access_start, access_len) &&
+            if ((nv_get_numa_status(nvl) == NV_NUMA_STATUS_ONLINE) &&
+                !IS_REG_OFFSET(nv, access_start, access_len) &&
                 (pfn_valid(PFN_DOWN(mmap_start))))
             {
                 ret = nvidia_mmap_numa(vma, mmap_context);
@@ -544,7 +556,6 @@ int nvidia_mmap_helper(
                 }
             }
             else
-#endif
             {
                 if (nv_io_remap_page_range(vma, mmap_start, mmap_length,
                         remap_prot_extra) != 0)
