@@ -1,11 +1,24 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2019-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include "nv-pci-table.h"
@@ -140,6 +153,72 @@ static void nv_init_dynamic_power_management
 }
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 /* find nvidia devices and set initial state */
 static int
 nv_pci_probe
@@ -155,6 +234,7 @@ nv_pci_probe
     nvidia_stack_t *sp = NULL;
     NvBool prev_nv_ats_supported = nv_ats_supported;
     NV_STATUS status;
+    NvBool last_bar_64bit = NV_FALSE;
 
     nv_printf(NV_DBG_SETUP, "NVRM: probing 0x%x 0x%x, class 0x%x\n",
         pci_dev->vendor, pci_dev->device, pci_dev->class);
@@ -274,6 +354,8 @@ nv_pci_probe
                 struct pci_dev *bridge = pci_dev->bus->self;
                 NvU32 base_upper, limit_upper;
 
+                last_bar_64bit = NV_TRUE;
+
                 if (bridge == NULL)
                     goto next_bar;
 
@@ -304,9 +386,33 @@ nv_pci_probe
             }
 
 next_bar:
+            //
+            // If we are here, then we have found a valid BAR -- 32 or 64-bit.
+            //
             j++;
             continue;
         }
+
+        //
+        // If last_bar_64bit is "true" then, we are looking at the 2nd (upper)
+        // half of the 64-bit BAR. This is typically all 0s which looks invalid
+        // but it's normal and not a problem and we can ignore it and continue.
+        //
+        if (last_bar_64bit)
+        {
+            last_bar_64bit = NV_FALSE;
+            continue;
+        }
+
+        // Invalid 32 or 64-bit BAR.
+        nv_printf(NV_DBG_ERRORS,
+            "NVRM: This PCI I/O region assigned to your NVIDIA device is invalid:\n"
+            "NVRM: BAR%d is %dM @ 0x%llx (PCI:%04x:%02x:%02x.%x)\n", i,
+            (NV_PCI_RESOURCE_SIZE(pci_dev, i) >> 20),
+            (NvU64)NV_PCI_RESOURCE_START(pci_dev, i),
+            NV_PCI_DOMAIN_NUMBER(pci_dev), NV_PCI_BUS_NUMBER(pci_dev),
+            NV_PCI_SLOT_NUMBER(pci_dev), PCI_FUNC(pci_dev->devfn));
+        goto failed;
     }
 
     if (!request_mem_region(NV_PCI_RESOURCE_START(pci_dev, NV_GPU_BAR_INDEX_REGS),
@@ -382,13 +488,35 @@ next_bar:
 
     nv->interrupt_line = pci_dev->irq;
 
+    NV_ATOMIC_SET(nvl->numa_info.status, NV_IOCTL_NUMA_STATUS_DISABLED);
+    nvl->numa_info.node_id = NUMA_NO_NODE;
+
     nv_init_ibmnpu_info(nv);
 
+
+
+
+
 #if defined(NVCPU_PPC64LE)
     // Use HW NUMA support as a proxy for ATS support. This is true in the only
-    // platform where ATS is currently supported (IBM P9).
-    nv_ats_supported &= nv_numa_info_valid(nvl);
+    // PPC64LE platform where ATS is currently supported (IBM P9).
+    nv_ats_supported &= nv_platform_supports_numa(nvl);
+#else
+
+
+
+
+
 #endif
+    if (nv_ats_supported)
+    {
+        NV_DEV_PRINTF(NV_DBG_INFO, nv, "ATS supported by this GPU!\n");
+    }
+    else
+    {
+        NV_DEV_PRINTF(NV_DBG_INFO, nv, "ATS not supported by this GPU. "
+                      "Disabling ATS support for all the GPUs in the system!\n");
+    }
 
     pci_set_master(pci_dev);
 
@@ -465,6 +593,8 @@ next_bar:
 
     nv_check_and_exclude_gpu(sp, nv);
 
+    rm_set_rm_firmware_requested(sp, nv);
+
     nv_init_dynamic_power_management(sp, pci_dev);
 
 #if defined(DPM_FLAG_NEVER_SKIP)
