@@ -1,15 +1,27 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
-#include "nv-misc.h"
 
 #include "os-interface.h"
 #include "nv-linux.h"
@@ -23,7 +35,7 @@
 #include "nv-ibmnpu.h"
 
 #define NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(name) \
-    NV_DEFINE_SINGLE_PROCFS_FILE(name, \
+    NV_DEFINE_SINGLE_PROCFS_FILE_READ_ONLY(name, \
                                  NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE, \
                                  NV_READ_UNLOCK_SYSTEM_PM_LOCK)
 
@@ -50,6 +62,7 @@ extern char *NVreg_RegistryDwordsPerDevice;
 extern char *NVreg_RmMsg;
 extern char *NVreg_GpuBlacklist;
 extern char *NVreg_TemporaryFilePath;
+extern char *NVreg_ExcludedGpus;
 
 static char nv_registry_keys[NV_MAX_REGISTRY_KEYS_LENGTH];
 
@@ -60,7 +73,18 @@ static nv_pm_action_depth_t nv_pm_action_depth = NV_PM_ACTION_DEPTH_DEFAULT;
 static int nv_procfs_read_registry(struct seq_file *s, void *v);
 
 #define NV_NUMA_STATUS_MSG_LEN      (32)
-#define NV_PROC_WRITE_BUFFER_SIZE   (128 * PAGE_SIZE)
+#define NV_PROC_WRITE_BUFFER_SIZE   (512 * PAGE_SIZE)
+
+typedef struct
+{
+    nvidia_stack_t *sp;
+    struct semaphore sp_lock;
+
+    nv_state_t *nv;
+
+    void *data;
+    off_t off;
+} nv_procfs_private_t;
 
 /*
  * Status messages directly corresponding to states in nv_numa_states_t.
@@ -85,11 +109,12 @@ nv_procfs_read_gpu_info(
     nv_state_t *nv = s->private;
     nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
     struct pci_dev *pci_dev = nvl->pci_dev;
-    char *type, *fmt;
+    char *type;
     const char *name;
     char *uuid;
-    NvU32 vbios_rev1, vbios_rev2, vbios_rev3, vbios_rev4, vbios_rev5;
+    char vbios_version[15];
     nvidia_stack_t *sp = NULL;
+    char firmware_version[64] = { 0 };
 
     if (nv_kmem_cache_alloc_stack(&sp) != 0)
     {
@@ -118,18 +143,9 @@ nv_procfs_read_gpu_info(
         uuid = NULL;
     }
 
-    if (rm_get_vbios_version(sp, nv, &vbios_rev1, &vbios_rev2,
-                             &vbios_rev3, &vbios_rev4,
-                             &vbios_rev5) != NV_OK)
-    {
-        seq_printf(s, "Video BIOS: \t ??.??.??.??.??\n");
-    }
-    else
-    {
-        fmt = "Video BIOS: \t %02x.%02x.%02x.%02x.%02x\n";
-        seq_printf(s, fmt, vbios_rev1, vbios_rev2, vbios_rev3, vbios_rev4,
-                   vbios_rev5);
-    }
+    rm_get_vbios_version(sp, nv, vbios_version);
+
+    seq_printf(s, "Video BIOS: \t %s\n", vbios_version);
 
     if (nv_find_pci_capability(pci_dev, PCI_CAP_ID_EXP))
         type = "PCIe";
@@ -145,6 +161,12 @@ nv_procfs_read_gpu_info(
                    nv->pci_info.slot, PCI_FUNC(pci_dev->devfn));
     seq_printf(s, "Device Minor: \t %u\n", nvl->minor_num);
 
+    rm_get_firmware_version(sp, nv, firmware_version, sizeof(firmware_version));
+    if (firmware_version[0] != '\0')
+    {
+        seq_printf(s, "GPU Firmware: \t %s\n", firmware_version);
+    }
+
 #if defined(DEBUG)
     do
     {
@@ -157,7 +179,7 @@ nv_procfs_read_gpu_info(
     } while (0);
 #endif
 
-    seq_printf(s, "Blacklisted:\t %s\n",
+    seq_printf(s, "GPU Excluded:\t %s\n",
                ((nv->flags & NV_FLAG_EXCLUDE) != 0) ? "Yes" : "No");
 
     rm_unref_dynamic_power(sp, nv, NV_DYNAMIC_PM_COARSE);
@@ -181,6 +203,8 @@ nv_procfs_read_power(
     const char *dynamic_power_status;
     const char *gc6_support;
     const char *gcoff_support;
+    NvU32 limitRated, limitCurr;
+    NV_STATUS status;
 
     if (nv_kmem_cache_alloc_stack(&sp) != 0)
     {
@@ -198,7 +222,20 @@ nv_procfs_read_power(
     seq_printf(s, " Video Memory Self Refresh: %s\n", gc6_support);
 
     gcoff_support = rm_get_gpu_gcx_support(sp, nv, NV_FALSE);
-    seq_printf(s, " Video Memory Off:          %s\n", gcoff_support);
+    seq_printf(s, " Video Memory Off:          %s\n\n", gcoff_support);
+
+    seq_printf(s, "Power Limits:\n");
+    status = rm_get_clientnvpcf_power_limits(sp, nv, &limitRated, &limitCurr);
+    if (status != NV_OK)
+    {
+        seq_printf(s, " Default:                   N/A milliwatts\n");
+        seq_printf(s, " GPU Boost:                 N/A milliwatts\n");
+    }
+    else
+    {
+        seq_printf(s, " Default:                   %u milliwatts\n", limitRated);
+        seq_printf(s, " GPU Boost:                 %u milliwatts\n", limitCurr);
+    }
 
     nv_kmem_cache_free_stack(sp);
     return 0;
@@ -222,45 +259,48 @@ NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(version);
 
 static void
 nv_procfs_close_file(
-    nv_linux_file_private_t *nvlfp
+    nv_procfs_private_t *nvpp
 )
 {
     nvidia_stack_t *sp;
 
-    if (nvlfp->data != NULL)
+    if (nvpp->data != NULL)
     {
-        os_free_mem(nvlfp->data);
+        os_free_mem(nvpp->data);
     }
 
-    sp  = nvlfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
+    sp  = nvpp->sp;
     if (sp != NULL)
     {
         nv_kmem_cache_free_stack(sp);
     }
 
-    nv_free_file_private(nvlfp);
+    NV_KFREE(nvpp, sizeof(*nvpp));
 }
 
 static int
 nv_procfs_open_file(
     struct inode *inode,
     struct file  *file,
-    nv_linux_file_private_t **nvlfpp
+    nv_procfs_private_t **pnvpp
 )
 {
     int retval = 0;
     NV_STATUS status;
-    nv_linux_file_private_t *nvlfp = NULL;
+    nv_procfs_private_t *nvpp = NULL;
     nvidia_stack_t *sp = NULL;
 
-    nvlfp = nv_alloc_file_private();
-    if (nvlfp == NULL)
+    NV_KMALLOC(nvpp, sizeof(nv_procfs_private_t));
+    if (nvpp == NULL)
     {
-        nv_printf(NV_DBG_ERRORS, "NVRM: failed to allocate file private!\n");
+        nv_printf(NV_DBG_ERRORS, "NVRM: failed to allocate procfs private!\n");
         return -ENOMEM;
     }
+    memset(nvpp, 0, sizeof(*nvpp));
 
-    nvlfp->proc_data = NV_PDE_DATA(inode);
+    NV_INIT_MUTEX(&nvpp->sp_lock);
+
+    nvpp->nv = NV_PDE_DATA(inode);
 
     if (0 == (file->f_mode & FMODE_WRITE))
         goto done;
@@ -271,24 +311,24 @@ nv_procfs_open_file(
         goto done;
     }
 
-    status = os_alloc_mem((void **)&nvlfp->data, NV_PROC_WRITE_BUFFER_SIZE);
+    status = os_alloc_mem((void **)&nvpp->data, NV_PROC_WRITE_BUFFER_SIZE);
     if (status != NV_OK)
     {
         retval = -ENOMEM;
         goto done;
     }
 
-    os_mem_set((void *)nvlfp->data, 0, NV_PROC_WRITE_BUFFER_SIZE);
-    nvlfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS] = sp;
+    os_mem_set((void *)nvpp->data, 0, NV_PROC_WRITE_BUFFER_SIZE);
+    nvpp->sp = sp;
 
 done:
     if (retval < 0)
     {
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
         return retval;
     }
 
-    *nvlfpp = nvlfp;
+    *pnvpp = nvpp;
 
     return 0;
 }
@@ -299,19 +339,19 @@ nv_procfs_open_registry(
     struct file  *file
 )
 {
-    nv_linux_file_private_t *nvlfp = NULL;
+    nv_procfs_private_t *nvpp = NULL;
     int retval;
 
-    retval = nv_procfs_open_file(inode, file, &nvlfp);
+    retval = nv_procfs_open_file(inode, file, &nvpp);
     if (retval < 0)
     {
         return retval;
     }
 
-    retval = single_open(file, nv_procfs_read_registry, nvlfp);
+    retval = single_open(file, nv_procfs_read_registry, nvpp);
     if (retval < 0)
     {
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
         return retval;
     }
 
@@ -319,7 +359,7 @@ nv_procfs_open_registry(
     if (retval < 0)
     {
         single_release(inode, file);
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
     }
 
     return retval;
@@ -332,22 +372,22 @@ nv_procfs_close_registry(
 )
 {
     struct seq_file *s = file->private_data;
-    nv_linux_file_private_t *nvlfp = s->private;
+    nv_procfs_private_t *nvpp = s->private;
     nv_state_t *nv;
     nv_linux_state_t *nvl = NULL;
-    nvidia_stack_t *sp = nvlfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
+    nvidia_stack_t *sp = nvpp->sp;
     char *key_name, *key_value, *registry_keys;
     size_t key_len, len;
     long count;
     NV_STATUS rm_status;
     int rc = 0;
 
-    if (0 != nvlfp->off)
+    if (0 != nvpp->off)
     {
-        nv = nvlfp->proc_data;
+        nv = nvpp->nv;
         if (nv != NULL)
             nvl = NV_GET_NVL_FROM_NV_STATE(nv);
-        key_value = (char *)nvlfp->data;
+        key_value = (char *)nvpp->data;
 
         key_name = strsep(&key_value, "=");
 
@@ -358,7 +398,7 @@ nv_procfs_close_registry(
         }
 
         key_len = (strlen(key_name) + 1);
-        count = (nvlfp->off - key_len);
+        count = (nvpp->off - key_len);
 
         if (count <= 0)
         {
@@ -393,7 +433,7 @@ done:
 
     single_release(inode, file);
 
-    nv_procfs_close_file(nvlfp);
+    nv_procfs_close_file(nvpp);
 
     return rc;
 }
@@ -420,6 +460,8 @@ nv_procfs_read_params(
                (NVreg_GpuBlacklist != NULL) ? NVreg_GpuBlacklist : "");
     seq_printf(s, "TemporaryFilePath: \"%s\"\n",
                (NVreg_TemporaryFilePath != NULL) ? NVreg_TemporaryFilePath : "");
+    seq_printf(s, "ExcludedGpus: \"%s\"\n",
+               (NVreg_ExcludedGpus != NULL) ? NVreg_ExcludedGpus : "");
 
     return 0;
 }
@@ -432,8 +474,8 @@ nv_procfs_read_registry(
     void *v
 )
 {
-    nv_linux_file_private_t *nvlfp = s->private;
-    nv_state_t *nv = nvlfp->proc_data;
+    nv_procfs_private_t *nvpp = s->private;
+    nv_state_t *nv = nvpp->nv;
     nv_linux_state_t *nvl = NULL;
     char *registry_keys;
 
@@ -456,13 +498,13 @@ nv_procfs_write_file(
 {
     int status = 0;
     struct seq_file *s = file->private_data;
-    nv_linux_file_private_t *nvlfp = s->private;
+    nv_procfs_private_t *nvpp = s->private;
     char *proc_buffer;
     unsigned long bytes_left;
 
-    down(&nvlfp->fops_sp_lock[NV_FOPS_STACK_INDEX_PROCFS]);
+    down(&nvpp->sp_lock);
 
-    bytes_left = (NV_PROC_WRITE_BUFFER_SIZE - nvlfp->off - 1);
+    bytes_left = (NV_PROC_WRITE_BUFFER_SIZE - nvpp->off - 1);
 
     if (count == 0)
     {
@@ -475,7 +517,7 @@ nv_procfs_write_file(
         goto done;
     }
 
-    proc_buffer = &((char *)nvlfp->data)[nvlfp->off];
+    proc_buffer = &((char *)nvpp->data)[nvpp->off];
 
     if (copy_from_user(proc_buffer, buffer, count))
     {
@@ -484,13 +526,13 @@ nv_procfs_write_file(
     }
     else
     {
-        nvlfp->off += count;
+        nvpp->off += count;
     }
 
-    *pos = nvlfp->off;
+    *pos = nvpp->off;
 
 done:
-    up(&nvlfp->fops_sp_lock[NV_FOPS_STACK_INDEX_PROCFS]);
+    up(&nvpp->sp_lock);
 
     return ((status < 0) ? status : (int)count);
 }
@@ -700,19 +742,19 @@ nv_procfs_open_exercise_error_forwarding(
     struct file  *file
 )
 {
-    nv_linux_file_private_t *nvlfp = NULL;
+    nv_procfs_private_t *nvpp = NULL;
     int retval;
 
-    retval = nv_procfs_open_file(inode, file, &nvlfp);
+    retval = nv_procfs_open_file(inode, file, &nvpp);
     if (retval < 0)
     {
         return retval;
     }
 
-    retval = single_open(file, nv_procfs_show_exercise_error_forwarding, nvlfp);
+    retval = single_open(file, nv_procfs_show_exercise_error_forwarding, nvpp);
     if (retval < 0)
     {
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
         return retval;
     }
 
@@ -720,7 +762,7 @@ nv_procfs_open_exercise_error_forwarding(
     if (retval < 0)
     {
         single_release(inode, file);
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
     }
 
     return retval;
@@ -733,10 +775,10 @@ nv_procfs_close_exercise_error_forwarding(
 )
 {
     struct seq_file *s = file->private_data;
-    nv_linux_file_private_t *nvlfp = s->private;
-    nv_state_t *nv = nvlfp->proc_data;
-    char *proc_buffer = &((char *)nvlfp->data)[0];
-    size_t count = nvlfp->off;
+    nv_procfs_private_t *nvpp = s->private;
+    nv_state_t *nv = nvpp->nv;
+    char *proc_buffer = &((char *)nvpp->data)[0];
+    size_t count = nvpp->off;
     int i = 0, status = 0;
     NvU32 xid = 0;
     const NvU8 MAX_XID_DIGITS = 3;
@@ -763,7 +805,7 @@ done:
 
     single_release(inode, file);
 
-    nv_procfs_close_file(nvlfp);
+    nv_procfs_close_file(nvpp);
 
     return status;
 }
@@ -781,8 +823,8 @@ nv_procfs_read_unbind_lock(
     void *v
 )
 {
-    nv_linux_file_private_t *nvlfp = s->private;
-    nv_state_t *nv = nvlfp->proc_data;
+    nv_procfs_private_t *nvpp = s->private;
+    nv_state_t *nv = nvpp->nv;
     nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
 
     down(&nvl->ldata_lock);
@@ -805,19 +847,19 @@ nv_procfs_open_unbind_lock(
     struct file  *file
 )
 {
-    nv_linux_file_private_t *nvlfp = NULL;
+    nv_procfs_private_t *nvpp = NULL;
     int retval;
 
-    retval = nv_procfs_open_file(inode, file, &nvlfp);
+    retval = nv_procfs_open_file(inode, file, &nvpp);
     if (retval < 0)
     {
         return retval;
     }
 
-    retval = single_open(file, nv_procfs_read_unbind_lock, nvlfp);
+    retval = single_open(file, nv_procfs_read_unbind_lock, nvpp);
     if (retval < 0)
     {
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
         return retval;
     }
 
@@ -825,7 +867,7 @@ nv_procfs_open_unbind_lock(
     if (retval < 0)
     {
         single_release(inode, file);
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
     }
 
     return retval;
@@ -838,25 +880,25 @@ nv_procfs_close_unbind_lock(
 )
 {
     struct seq_file *s = file->private_data;
-    nv_linux_file_private_t *nvlfp = s->private;
+    nv_procfs_private_t *nvpp = s->private;
     nv_state_t *nv;
-    nvidia_stack_t *sp = nvlfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
+    nvidia_stack_t *sp = nvpp->sp;
     int rc = 0;
     nv_linux_state_t * nvl;
     int value;
 
-    if (0 != nvlfp->off)
+    if (0 != nvpp->off)
     {
-        nv = nvlfp->proc_data;
+        nv = nvpp->nv;
         nvl = NV_GET_NVL_FROM_NV_STATE(nv);
 
-        if (NULL == nvlfp->data || NULL == nv)
+        if (NULL == nvpp->data || NULL == nv)
         {
             rc = -EINVAL;
             goto done;
         }
 
-        if (sscanf((char *)nvlfp->data, "%u\n", &value) != 1)
+        if (sscanf((char *)nvpp->data, "%u\n", &value) != 1)
         {
             rc = -EINVAL;
             goto done;
@@ -890,7 +932,7 @@ done:
 
     single_release(inode, file);
 
-    nv_procfs_close_file(nvlfp);
+    nv_procfs_close_file(nvpp);
 
     return rc;
 }
@@ -974,16 +1016,14 @@ numa_status_read(
      */
     if (!(nv->flags & NV_FLAG_OPEN))
     {
-#if defined(NVCPU_PPC64LE)
-        if (nv_numa_info_valid(nvl))
+        if (nv_platform_supports_numa(nvl))
         {
-            *nid = nvl->npu->numa_info.node_id;
+            *nid = nvl->numa_info.node_id;
             *status = nv_get_numa_status(nvl);
             *numa_mem_addr = 0;
             *numa_mem_size = 0;
             memset(list, 0x0, sizeof(*list));
         }
-#endif
 
         rm_status = NV_ERR_NOT_READY;
         goto done;
@@ -1011,9 +1051,9 @@ nv_procfs_read_offline_pages(
     int retval = 0;
     NV_STATUS rm_status;
     nv_ioctl_numa_info_t numa_info;
-    nv_linux_file_private_t *nvlfp = s->private;
-    nv_stack_t *sp = nvlfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
-    nv_state_t *nv = nvlfp->proc_data;
+    nv_procfs_private_t *nvpp = s->private;
+    nv_stack_t *sp = nvpp->sp;
+    nv_state_t *nv = nvpp->nv;
 
     rm_status = numa_status_read(nv, sp,
                                  &numa_info.nid,
@@ -1041,18 +1081,18 @@ nv_procfs_open_offline_pages(
 )
 {
     int retval;
-    nv_linux_file_private_t *nvlfp = NULL;
+    nv_procfs_private_t *nvpp = NULL;
 
-    retval = nv_procfs_open_file(inode, file, &nvlfp);
+    retval = nv_procfs_open_file(inode, file, &nvpp);
     if (retval < 0)
     {
         return retval;
     }
 
-    retval = single_open(file, nv_procfs_read_offline_pages, nvlfp);
+    retval = single_open(file, nv_procfs_read_offline_pages, nvpp);
     if (retval < 0)
     {
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
         return retval;
     }
 
@@ -1060,7 +1100,7 @@ nv_procfs_open_offline_pages(
     if (retval < 0)
     {
         single_release(inode, file);
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
     }
 
     return retval;
@@ -1073,13 +1113,13 @@ nv_procfs_close_offline_pages(
 )
 {
     struct seq_file *s = file->private_data;
-    nv_linux_file_private_t *nvlfp = s->private;
+    nv_procfs_private_t *nvpp = s->private;
 
     NV_READ_UNLOCK_SYSTEM_PM_LOCK();
 
     single_release(inode, file);
 
-    nv_procfs_close_file(nvlfp);
+    nv_procfs_close_file(nvpp);
 
     return 0;
 }
@@ -1093,9 +1133,9 @@ nv_procfs_read_numa_status(
     int retval = 0;
     NV_STATUS rm_status;
     nv_ioctl_numa_info_t numa_info;
-    nv_linux_file_private_t *nvlfp = s->private;
-    nv_stack_t *sp = nvlfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
-    nv_state_t *nv = nvlfp->proc_data;
+    nv_procfs_private_t *nvpp = s->private;
+    nv_stack_t *sp = nvpp->sp;
+    nv_state_t *nv = nvpp->nv;
 
     rm_status = numa_status_read(nv, sp,
                                  &numa_info.nid,
@@ -1127,18 +1167,18 @@ nv_procfs_open_numa_status(
 )
 {
     int retval;
-    nv_linux_file_private_t *nvlfp = NULL;
+    nv_procfs_private_t *nvpp = NULL;
 
-    retval = nv_procfs_open_file(inode, file, &nvlfp);
+    retval = nv_procfs_open_file(inode, file, &nvpp);
     if (retval < 0)
     {
         return retval;
     }
 
-    retval = single_open(file, nv_procfs_read_numa_status, nvlfp);
+    retval = single_open(file, nv_procfs_read_numa_status, nvpp);
     if (retval < 0)
     {
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
         return retval;
     }
 
@@ -1146,7 +1186,7 @@ nv_procfs_open_numa_status(
     if (retval < 0)
     {
         single_release(inode, file);
-        nv_procfs_close_file(nvlfp);
+        nv_procfs_close_file(nvpp);
     }
 
     return retval;
@@ -1160,18 +1200,18 @@ nv_procfs_close_numa_status(
 {
     int retval = 0;
     struct seq_file *s = file->private_data;
-    nv_linux_file_private_t *nvlfp = s->private;
-    nvidia_stack_t *sp = nvlfp->fops_sp[NV_FOPS_STACK_INDEX_PROCFS];
-    nv_state_t *nv = nvlfp->proc_data;
+    nv_procfs_private_t *nvpp = s->private;
+    nvidia_stack_t *sp = nvpp->sp;
+    nv_state_t *nv = nvpp->nv;
     nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
 
     const size_t MAX_STATES = ARRAY_SIZE(nv_numa_status_messages);
     nv_numa_status_t current_state = nv_get_numa_status(nvl);
-    char *cmd = nvlfp->data;
+    char *cmd = nvpp->data;
 
     down(&nvl->ldata_lock);
 
-    if (nvlfp->off != 0)
+    if (nvpp->off != 0)
     {
         NvU32 state;
         nv_numa_status_t requested = NV_NUMA_STATUS_DISABLED;
@@ -1229,7 +1269,7 @@ done:
 
     single_release(inode, file);
 
-    nv_procfs_close_file(nvlfp);
+    nv_procfs_close_file(nvpp);
 
     return retval;
 }
@@ -1272,25 +1312,6 @@ nv_procfs_add_text_file(
 {
     NV_CREATE_PROC_FILE(filename, parent, text_file, (void *)text);
 }
-
-void
-nv_procfs_unregister_all(struct proc_dir_entry *entry, struct proc_dir_entry *delimiter)
-{
-#if defined(NV_PROC_REMOVE_PRESENT)
-    proc_remove(entry);
-#else
-    while (entry)
-    {
-        struct proc_dir_entry *next = entry->next;
-        if (entry->subdir)
-            nv_procfs_unregister_all(entry->subdir, delimiter);
-        remove_proc_entry(entry->name, entry->parent);
-        if (entry == delimiter)
-            break;
-        entry = next;
-    }
-#endif
-}
 #endif
 
 void nv_procfs_add_warning(
