@@ -1,12 +1,24 @@
 /*
- * _NVRM_COPYRIGHT_BEGIN_
+ * SPDX-FileCopyrightText: Copyright (c) 2006-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2006-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #ifndef _RM_REG_H_
@@ -191,7 +203,7 @@
  * Description:
  *
  * This option accepts a semicolon-separated list of key=value pairs. Each
- * key name is checked agains the table of static options; if a match is
+ * key name is checked against the table of static options; if a match is
  * found, the static option value is overridden, but invalid options remain
  * invalid. Pairs that do not match an entry in the static option table
  * are passed on to the RM directly.
@@ -455,25 +467,6 @@
 #define __NV_ENABLE_STREAM_MEMOPS EnableStreamMemOPs
 #define NV_REG_ENABLE_STREAM_MEMOPS NV_REG_STRING(__NV_ENABLE_STREAM_MEMOPS)
 
-/*
- * Option: EnableBacklightHandler
- *
- * Description:
- *
- * When this option is enabled and the platform supports it, the NVIDIA driver
- * will register a backlight handler with the operating system.
- *
- * The backlight handler will be registered when the corresponding device is
- * initialized, and will be removed when the device is shut down.
- *
- * Possible Values:
- *
- *  0 = disable feature (default)
- *  1 = enable feature
- */
-#define __NV_ENABLE_BACKLIGHT_HANDLER EnableBacklightHandler
-#define NV_REG_ENABLE_BACKLIGHT_HANDLER NV_REG_STRING(__NV_ENABLE_BACKLIGHT_HANDLER)
-
 /*
  * Option: EnableUserNUMAManagement
  *
@@ -500,14 +493,28 @@
  *
  * Description:
  *
- * This option accepts a list of blacklisted GPUs, seperated by commas, that
+ * This option accepts a list of blacklisted GPUs, separated by commas, that
  * cannot be attached or used. Each blacklisted GPU is identified by a UUID in
  * the ASCII format with leading "GPU-". An exact match is required; no partial
- * UUIDs.
+ * UUIDs. This regkey is deprecated and will be removed in the future. Use
+ * NV_REG_EXCLUDED_GPUS instead.
  */
 #define __NV_GPU_BLACKLIST GpuBlacklist
 #define NV_REG_GPU_BLACKLIST NV_REG_STRING(__NV_GPU_BLACKLIST)
 
+/*
+ * Option: ExcludedGpus
+ *
+ * Description:
+ *
+ * This option accepts a list of excluded GPUs, separated by commas, that
+ * cannot be attached or used. Each excluded GPU is identified by a UUID in
+ * the ASCII format with leading "GPU-". An exact match is required; no partial
+ * UUIDs.
+ */
+#define __NV_EXCLUDED_GPUS ExcludedGpus
+#define NV_REG_EXCLUDED_GPUS NV_REG_STRING(__NV_EXCLUDED_GPUS)
+
 /*
  * Option: NvLinkDisable
  *
@@ -637,6 +644,9 @@
  *  0: Never allow the GPU to be powered down (default).
  *  1: Power down the GPU when it is not initialized.
  *  2: Power down the GPU after it has been inactive for some time.
+ *  3: (Default) Power down the GPU after a period of inactivity (i.e.,
+ *     mode 2) on Ampere or later notebooks.  Otherwise, do not power down
+ *     the GPU.
  */
 #define __NV_DYNAMIC_POWER_MANAGEMENT DynamicPowerManagement
 #define NV_REG_DYNAMIC_POWER_MANAGEMENT \
@@ -645,6 +655,7 @@
 #define NV_REG_DYNAMIC_POWER_MANAGEMENT_NEVER   0
 #define NV_REG_DYNAMIC_POWER_MANAGEMENT_COARSE  1
 #define NV_REG_DYNAMIC_POWER_MANAGEMENT_FINE    2
+#define NV_REG_DYNAMIC_POWER_MANAGEMENT_DEFAULT 3
 
 /*
  * Option: DynamicPowerManagementVideoMemoryThreshold
@@ -707,6 +718,62 @@
 #define NV_REG_ENABLE_PCIE_RELAXED_ORDERING_MODE \
     NV_REG_STRING(__NV_ENABLE_PCIE_RELAXED_ORDERING_MODE)
 
+/*
+ * Option: EnableGpuFirmware
+ *
+ * Description:
+ *
+ * When this option is enabled, the NVIDIA driver will enable use of GPU
+ * firmware.
+ *
+ * Possible mode values:
+ *  0 - Do not enable GPU firmware
+ *  1 - Enable GPU firmware
+ *  2 - (Default) Use the default enablement policy for GPU firmware
+ *
+ * Setting this to anything other than 2 will alter driver firmware-
+ * enablement policies, possibly disabling GPU firmware where it would
+ * have otherwise been enabled by default.
+ *
+ * If this key is set globally to the system, the driver may still attempt
+ * to apply some policies to maintain uniform firmware modes across all
+ * GPUS.  This may result in the driver failing initialization on some GPUs
+ * to maintain such a policy.
+ * 
+ * If this key is set using NVreg_RegistryDwordsPerDevice, then the driver 
+ * will attempt to honor whatever configuration is specified without applying
+ * additional policies.  This may also result in failed GPU initialzations if
+ * the configuration is not possible (for example if the firmware is missing 
+ * from the filesystem, or the GPU is not capable). 
+ *
+ * Policy bits:
+ *
+ * POLICY_ALLOW_FALLBACK:
+ *  As the normal behavior is to fail GPU initialization if this registry 
+ *  entry is set in such a way that results in an invalid configuration, if 
+ *  instead the user would like the driver to automatically try to fallback 
+ *  to initializing the failing GPU with firmware disabled, then this bit can 
+ *  be set (ex: 0x11 means try to enable GPU firmware but fall back if needed).
+ *  Note that this can result in a mixed mode configuration (ex: GPU0 has 
+ *  firmware enabled, but GPU1 does not).
+ *
+ */
+
+#define __NV_ENABLE_GPU_FIRMWARE  EnableGpuFirmware
+#define NV_REG_ENABLE_GPU_FIRMWARE NV_REG_STRING(__NV_ENABLE_GPU_FIRMWARE)
+
+#define NV_REG_ENABLE_GPU_FIRMWARE_MODE_MASK              0x0000000F
+#define NV_REG_ENABLE_GPU_FIRMWARE_MODE_DISABLED          0x00000000
+#define NV_REG_ENABLE_GPU_FIRMWARE_MODE_ENABLED           0x00000001
+#define NV_REG_ENABLE_GPU_FIRMWARE_MODE_DEFAULT           0x00000002
+
+#define NV_REG_ENABLE_GPU_FIRMWARE_POLICY_MASK            0x000000F0
+#define NV_REG_ENABLE_GPU_FIRMWARE_POLICY_ALLOW_FALLBACK  0x00000010
+
+#define NV_REG_ENABLE_GPU_FIRMWARE_DEFAULT_VALUE          0x00000012
+#define NV_REG_ENABLE_GPU_FIRMWARE_INVALID_VALUE          0xFFFFFFFF
+
+
 #if defined(NV_DEFINE_REGISTRY_KEY_TABLE)
 
 /*
@@ -726,15 +793,15 @@ NV_DEFINE_REG_ENTRY(__NV_ENABLE_PCIE_GEN3, 0);
 NV_DEFINE_REG_ENTRY(__NV_ENABLE_MSI, 1);
 NV_DEFINE_REG_ENTRY(__NV_TCE_BYPASS_MODE, NV_TCE_BYPASS_MODE_DEFAULT);
 NV_DEFINE_REG_ENTRY(__NV_ENABLE_STREAM_MEMOPS, 0);
-NV_DEFINE_REG_ENTRY(__NV_ENABLE_BACKLIGHT_HANDLER, 1);
 NV_DEFINE_REG_ENTRY(__NV_RM_PROFILING_ADMIN_ONLY_PARAMETER, 1);
 NV_DEFINE_REG_ENTRY(__NV_PRESERVE_VIDEO_MEMORY_ALLOCATIONS, 0);
 
 NV_DEFINE_REG_ENTRY(__NV_ENABLE_S0IX_POWER_MANAGEMENT, 0);
 NV_DEFINE_REG_ENTRY(__NV_S0IX_POWER_MANAGEMENT_VIDEO_MEMORY_THRESHOLD, 256);
 
-NV_DEFINE_REG_ENTRY(__NV_DYNAMIC_POWER_MANAGEMENT, 0);
+NV_DEFINE_REG_ENTRY(__NV_DYNAMIC_POWER_MANAGEMENT, 3);
 NV_DEFINE_REG_ENTRY(__NV_DYNAMIC_POWER_MANAGEMENT_VIDEO_MEMORY_THRESHOLD, 200);
+NV_DEFINE_REG_ENTRY(__NV_ENABLE_GPU_FIRMWARE, NV_REG_ENABLE_GPU_FIRMWARE_DEFAULT_VALUE);
 
 NV_DEFINE_REG_ENTRY_GLOBAL(__NV_ENABLE_USER_NUMA_MANAGEMENT, 1);
 NV_DEFINE_REG_ENTRY_GLOBAL(__NV_MEMORY_POOL_SIZE, 0);
@@ -754,6 +821,7 @@ NV_DEFINE_REG_STRING_ENTRY(__NV_REGISTRY_DWORDS_PER_DEVICE, NULL);
 NV_DEFINE_REG_STRING_ENTRY(__NV_RM_MSG, NULL);
 NV_DEFINE_REG_STRING_ENTRY(__NV_GPU_BLACKLIST, NULL);
 NV_DEFINE_REG_STRING_ENTRY(__NV_TEMPORARY_FILE_PATH, NULL);
+NV_DEFINE_REG_STRING_ENTRY(__NV_EXCLUDED_GPUS, NULL);
 
 /*
  *----------------registry database definition----------------------
@@ -784,7 +852,6 @@ nv_parm_t nv_parms[] = {
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_IGNORE_MMIO_CHECK),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_TCE_BYPASS_MODE),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_ENABLE_STREAM_MEMOPS),
-    NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_ENABLE_BACKLIGHT_HANDLER),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_ENABLE_USER_NUMA_MANAGEMENT),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_NVLINK_DISABLE),
     NV_DEFINE_PARAMS_TABLE_ENTRY_CUSTOM_NAME(__NV_RM_PROFILING_ADMIN_ONLY,
@@ -798,6 +865,7 @@ nv_parm_t nv_parms[] = {
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_DYNAMIC_POWER_MANAGEMENT_VIDEO_MEMORY_THRESHOLD),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_REGISTER_PCI_DRIVER),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_ENABLE_PCIE_RELAXED_ORDERING_MODE),
+    NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_ENABLE_GPU_FIRMWARE),
     {NULL, NULL}
 };
 
