@@ -1,14 +1,26 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2020 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
-#include "nv-misc.h"
 #include "os-interface.h"
 #include "nv.h"
 #include "nv-linux.h"
@@ -291,7 +303,7 @@ static NV_STATUS nv_alloc_coherent_pages(
     virt_addr = (unsigned long)dma_alloc_coherent(dev,
                                                   at->num_pages * PAGE_SIZE,
                                                   &bus_addr,
-                                                  gfp_mask | __GFP_COMP);
+                                                  gfp_mask);
     if (!virt_addr)
     {
         nv_printf(NV_DBG_MEMINFO,
@@ -315,7 +327,11 @@ static NV_STATUS nv_alloc_coherent_pages(
                                   NV_MEMORY_UNCACHED);
     }
 
-    at->is_coherent = NV_TRUE;
+    at->flags.coherent = NV_TRUE;
+
+    // dma_alloc_coherent() allocates unencrypted memory when AMD SEV is enabled.
+    if (os_sev_enabled)
+        at->flags.unencrypted = NV_TRUE;
 
     return NV_OK;
 }
@@ -356,7 +372,8 @@ NV_STATUS nv_alloc_contig_pages(
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %s: %u pages\n", __FUNCTION__, at->num_pages);
 
-    if (os_is_xen_dom0())
+    // TODO: This is temporary WAR, we will be removed after fixing bug 200732409.
+    if (os_is_xen_dom0() || os_sev_enabled)
         return nv_alloc_coherent_pages(nv, at);
 
     at->order = get_order(at->num_pages * PAGE_SIZE);
@@ -418,7 +435,7 @@ NV_STATUS nv_alloc_contig_pages(
                                   NV_MEMORY_UNCACHED);
     }
 
-    at->is_coherent = NV_FALSE;
+    at->flags.coherent = NV_FALSE;
 
     return NV_OK;
 
@@ -445,7 +462,7 @@ void nv_free_contig_pages(
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %s: %u pages\n", __FUNCTION__, at->num_pages);
 
-    if (at->is_coherent == NV_TRUE)
+    if (at->flags.coherent)
         return nv_free_coherent_pages(at);
 
     if (at->cache_type != NV_MEMORY_CACHED)
@@ -490,6 +507,7 @@ NV_STATUS nv_alloc_system_pages(
     unsigned long virt_addr = 0;
     NvU64 phys_addr;
     struct device *dev = at->dev;
+    dma_addr_t bus_addr;
 
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %u: %u pages\n", __FUNCTION__, at->num_pages);
@@ -498,7 +516,20 @@ NV_STATUS nv_alloc_system_pages(
 
     for (i = 0; i < at->num_pages; i++)
     {
-        if (at->flags.node0)
+        if (os_sev_enabled && (dev != NULL))
+        {
+            virt_addr = (unsigned long)dma_alloc_coherent(dev,
+                                                          PAGE_SIZE,
+                                                          &bus_addr,
+                                                          gfp_mask);
+            at->flags.coherent = NV_TRUE;
+            //
+            // dma_alloc_coherent() allocates unencrypted memory when AMD SEV
+            // is enabled.
+            //
+            at->flags.unencrypted = NV_TRUE;
+        }
+        else if (at->flags.node0)
         {
             NV_ALLOC_PAGES_NODE(virt_addr, 0, 0, gfp_mask);
         }
@@ -506,6 +537,7 @@ NV_STATUS nv_alloc_system_pages(
         {
             NV_GET_FREE_PAGES(virt_addr, 0, gfp_mask);
         }
+
         if (virt_addr == 0)
         {
             nv_printf(NV_DBG_MEMINFO,
@@ -546,9 +578,15 @@ NV_STATUS nv_alloc_system_pages(
         page_ptr->page_count = NV_GET_PAGE_COUNT(page_ptr);
         page_ptr->virt_addr = virt_addr;
 
-        if (dev)
+        //
+        // Use unencrypted dma_addr returned by dma_alloc_coherent() as
+        // nv_phys_to_dma() returns encrypted dma_addr when AMD SEV is enabled.
+        //
+        if (at->flags.coherent)
+            page_ptr->dma_addr = bus_addr;
+        else if (dev)
             page_ptr->dma_addr = nv_phys_to_dma(dev, page_ptr->phys_addr);
-	else
+        else
             page_ptr->dma_addr = page_ptr->phys_addr;
 
         NV_MAYBE_RESERVE_PAGE(page_ptr);
@@ -566,7 +604,15 @@ failed:
         {
             page_ptr = at->page_table[j];
             NV_MAYBE_UNRESERVE_PAGE(page_ptr);
-            NV_FREE_PAGES(page_ptr->virt_addr, 0);
+            if (at->flags.coherent)
+            {
+                dma_free_coherent(dev, PAGE_SIZE, (void *)page_ptr->virt_addr,
+                                  page_ptr->dma_addr);
+            }
+            else
+            {
+                NV_FREE_PAGES(page_ptr->virt_addr, 0);
+            }
         }
     }
 
@@ -579,6 +625,7 @@ void nv_free_system_pages(
 {
     nvidia_pte_t *page_ptr;
     unsigned int i;
+    struct device *dev = at->dev;
 
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %s: %u pages\n", __FUNCTION__, at->num_pages);
@@ -601,15 +648,25 @@ void nv_free_system_pages(
                     page_ptr->page_count);
             }
         }
+
         NV_MAYBE_UNRESERVE_PAGE(page_ptr);
-        NV_FREE_PAGES(page_ptr->virt_addr, 0);
+        if (at->flags.coherent)
+        {
+            dma_free_coherent(dev, PAGE_SIZE, (void *)page_ptr->virt_addr,
+                              page_ptr->dma_addr);
+        }
+        else
+        {
+            NV_FREE_PAGES(page_ptr->virt_addr, 0);
+        }
     }
 }
 
 NvUPtr nv_vm_map_pages(
     struct page **pages,
     NvU32 count,
-    NvBool cached
+    NvBool cached,
+    NvBool unencrypted
 )
 {
     NvUPtr virt_addr = 0;
@@ -623,7 +680,7 @@ NvUPtr nv_vm_map_pages(
         return virt_addr;
     }
 
-    virt_addr = nv_vmap(pages, count, cached);
+    virt_addr = nv_vmap(pages, count, cached, unencrypted);
     return virt_addr;
 }
 
