@@ -1,11 +1,24 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2013-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2013-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 /*
@@ -16,7 +29,6 @@
 
 #define  __NO_VERSION__
 
-#include "nv-misc.h"
 #include "os-interface.h"
 #include "nv-linux.h"
 
@@ -24,6 +36,7 @@
 
 #include "nv_uvm_interface.h"
 #include "nv_gpu_ops.h"
+#include "rm-gpu-ops.h"
 
 // This is really a struct UvmOpsUvmEvents *. It needs to be an atomic because
 // it can be read outside of the g_pNvUvmEventsLock. Use getUvmEvents and
@@ -344,8 +357,8 @@ NV_STATUS nvUvmInterfaceMemoryAllocSys(uvmGpuAddressSpaceHandle vaSpace,
 
 EXPORT_SYMBOL(nvUvmInterfaceMemoryAllocSys);
 
-NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
-                                   uvmGpuAddressSpaceHandle vaSpace2,
+NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuDeviceHandle device1,
+                                   uvmGpuDeviceHandle device2,
                                    UvmGpuP2PCapsParams * p2pCapsParams)
 {
     nvidia_stack_t *sp = NULL;
@@ -357,8 +370,8 @@ NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
     }
 
     status = rm_gpu_ops_get_p2p_caps(sp,
-                                     (gpuAddressSpaceHandle)vaSpace1,
-                                     (gpuAddressSpaceHandle)vaSpace2,
+                                     (gpuDeviceHandle)device1,
+                                     (gpuDeviceHandle)device2,
                                      p2pCapsParams);
     nv_kmem_cache_free_stack(sp);
     return status;
@@ -366,7 +379,7 @@ NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
 
 EXPORT_SYMBOL(nvUvmInterfaceGetP2PCaps);
 
-NV_STATUS nvUvmInterfaceGetPmaObject(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceGetPmaObject(uvmGpuDeviceHandle device,
                                      void **pPma,
                                      const UvmPmaStatistics **pPmaPubStats)
 {
@@ -378,7 +391,7 @@ NV_STATUS nvUvmInterfaceGetPmaObject(uvmGpuAddressSpaceHandle vaSpace,
         return NV_ERR_NO_MEMORY;
     }
 
-    status = rm_gpu_ops_get_pma_object(sp, (gpuAddressSpaceHandle)vaSpace, pPma, (const nvgpuPmaStatistics_t *)pPmaPubStats);
+    status = rm_gpu_ops_get_pma_object(sp, (gpuDeviceHandle)device, pPma, (const nvgpuPmaStatistics_t *)pPmaPubStats);
 
     nv_kmem_cache_free_stack(sp);
     return status;
@@ -591,7 +604,7 @@ NV_STATUS nvUvmInterfaceQueryCaps(uvmGpuDeviceHandle device,
 }
 EXPORT_SYMBOL(nvUvmInterfaceQueryCaps);
 
-NV_STATUS nvUvmInterfaceQueryCopyEnginesCaps(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceQueryCopyEnginesCaps(uvmGpuDeviceHandle device,
                                              UvmGpuCopyEnginesCaps *caps)
 {
     nvidia_stack_t *sp = NULL;
@@ -602,7 +615,7 @@ NV_STATUS nvUvmInterfaceQueryCopyEnginesCaps(uvmGpuAddressSpaceHandle vaSpace,
         return NV_ERR_NO_MEMORY;
     }
 
-    status = rm_gpu_ops_query_ces_caps(sp, (gpuAddressSpaceHandle)vaSpace, caps);
+    status = rm_gpu_ops_query_ces_caps(sp, (gpuDeviceHandle)device, caps);
 
     nv_kmem_cache_free_stack(sp);
     return status;
@@ -678,12 +691,10 @@ NV_STATUS nvUvmInterfaceUnsetPageDirectory(uvmGpuAddressSpaceHandle vaSpace)
 }
 EXPORT_SYMBOL(nvUvmInterfaceUnsetPageDirectory);
 
-NV_STATUS nvUvmInterfaceDupAllocation(NvHandle hPhysHandle,
-                                      uvmGpuAddressSpaceHandle srcVaspace,
+NV_STATUS nvUvmInterfaceDupAllocation(uvmGpuAddressSpaceHandle srcVaSpace,
                                       NvU64 srcAddress,
-                                      uvmGpuAddressSpaceHandle dstVaspace,
-                                      NvU64 *dstAddress,
-                                      NvBool bPhysHandleValid)
+                                      uvmGpuAddressSpaceHandle dstVaSpace,
+                                      NvU64 *dstAddress)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -694,19 +705,17 @@ NV_STATUS nvUvmInterfaceDupAllocation(NvHandle hPhysHandle,
     }
 
     status = rm_gpu_ops_dup_allocation(sp,
-                                      hPhysHandle,
-                                      (gpuAddressSpaceHandle)srcVaspace,
+                                      (gpuAddressSpaceHandle)srcVaSpace,
                                       srcAddress,
-                                      (gpuAddressSpaceHandle)dstVaspace,
-                                      dstAddress,
-                                      bPhysHandleValid);
+                                      (gpuAddressSpaceHandle)dstVaSpace,
+                                      dstAddress);
 
     nv_kmem_cache_free_stack(sp);
     return status;
 }
 EXPORT_SYMBOL(nvUvmInterfaceDupAllocation);
 
-NV_STATUS nvUvmInterfaceDupMemory(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceDupMemory(uvmGpuDeviceHandle device,
                                   NvHandle hClient,
                                   NvHandle hPhysMemory,
                                   NvHandle *hDupMemory,
@@ -721,7 +730,7 @@ NV_STATUS nvUvmInterfaceDupMemory(uvmGpuAddressSpaceHandle vaSpace,
     }
 
     status = rm_gpu_ops_dup_memory(sp,
-                                   (gpuAddressSpaceHandle)vaSpace,
+                                   (gpuDeviceHandle)device,
                                    hClient,
                                    hPhysMemory,
                                    hDupMemory,
@@ -733,14 +742,14 @@ NV_STATUS nvUvmInterfaceDupMemory(uvmGpuAddressSpaceHandle vaSpace,
 EXPORT_SYMBOL(nvUvmInterfaceDupMemory);
 
 
-NV_STATUS nvUvmInterfaceFreeDupedHandle(uvmGpuAddressSpaceHandle vaspace,
+NV_STATUS nvUvmInterfaceFreeDupedHandle(uvmGpuDeviceHandle device,
                                         NvHandle hPhysHandle)
 {
     nvidia_stack_t *sp = nvUvmGetSafeStack();
     NV_STATUS status;
 
     status = rm_gpu_ops_free_duped_handle(sp,
-                                         (gpuAddressSpaceHandle)vaspace,
+                                         (gpuDeviceHandle)device,
                                          hPhysHandle);
 
     nvUvmFreeSafeStack(sp);
@@ -748,7 +757,7 @@ NV_STATUS nvUvmInterfaceFreeDupedHandle(uvmGpuAddressSpaceHandle vaspace,
 }
 EXPORT_SYMBOL(nvUvmInterfaceFreeDupedHandle);
 
-NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuDeviceHandle device,
                                   UvmGpuFbInfo * fbInfo)
 {
     nvidia_stack_t *sp = NULL;
@@ -759,7 +768,7 @@ NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuAddressSpaceHandle vaSpace,
         return NV_ERR_NO_MEMORY;
     }
 
-    status = rm_gpu_ops_get_fb_info(sp, (gpuAddressSpaceHandle)vaSpace, fbInfo);
+    status = rm_gpu_ops_get_fb_info(sp, (gpuDeviceHandle)device, fbInfo);
 
     nv_kmem_cache_free_stack(sp);
 
@@ -767,7 +776,7 @@ NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuAddressSpaceHandle vaSpace,
 }
 EXPORT_SYMBOL(nvUvmInterfaceGetFbInfo);
 
-NV_STATUS nvUvmInterfaceGetEccInfo(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceGetEccInfo(uvmGpuDeviceHandle device,
                                    UvmGpuEccInfo * eccInfo)
 {
     nvidia_stack_t *sp = NULL;
@@ -778,7 +787,7 @@ NV_STATUS nvUvmInterfaceGetEccInfo(uvmGpuAddressSpaceHandle vaSpace,
         return NV_ERR_NO_MEMORY;
     }
 
-    status = rm_gpu_ops_get_ecc_info(sp, (gpuAddressSpaceHandle)vaSpace, eccInfo);
+    status = rm_gpu_ops_get_ecc_info(sp, (gpuDeviceHandle)device, eccInfo);
 
     nv_kmem_cache_free_stack(sp);
 
@@ -1146,8 +1155,8 @@ NV_STATUS nv_uvm_event_interrupt(const NvU8 *pUuid)
     return NV_ERR_NO_INTR_PENDING;
 }
 
-NV_STATUS nvUvmInterfaceP2pObjectCreate(uvmGpuAddressSpaceHandle vaSpace1,
-                                        uvmGpuAddressSpaceHandle vaSpace2,
+NV_STATUS nvUvmInterfaceP2pObjectCreate(uvmGpuDeviceHandle device1,
+                                        uvmGpuDeviceHandle device2,
                                         NvHandle *hP2pObject)
 {
     nvidia_stack_t *sp = NULL;
@@ -1158,8 +1167,8 @@ NV_STATUS nvUvmInterfaceP2pObjectCreate(uvmGpuAddressSpaceHandle vaSpace1,
     }
 
     status = rm_gpu_ops_p2p_object_create(sp,
-                                          (gpuAddressSpaceHandle)vaSpace1,
-                                          (gpuAddressSpaceHandle)vaSpace2,
+                                          (gpuDeviceHandle)device1,
+                                          (gpuDeviceHandle)device2,
                                           hP2pObject);
 
     nv_kmem_cache_free_stack(sp);
@@ -1296,19 +1305,107 @@ NV_STATUS nvUvmInterfaceGetChannelResourcePtes(uvmGpuAddressSpaceHandle vaSpace,
 }
 EXPORT_SYMBOL(nvUvmInterfaceGetChannelResourcePtes);
 
-NV_STATUS nvUvmInterfaceReportNonReplayableFault(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceReportNonReplayableFault(uvmGpuDeviceHandle device,
                                                  const void *pFaultPacket)
 {
     nvidia_stack_t *sp = nvUvmGetSafeStack();
     NV_STATUS status;
 
-    status = rm_gpu_ops_report_non_replayable_fault(sp, (gpuAddressSpaceHandle)vaSpace, pFaultPacket);
+    status = rm_gpu_ops_report_non_replayable_fault(sp, (gpuDeviceHandle)device, pFaultPacket);
 
     nvUvmFreeSafeStack(sp);
     return status;
 }
 EXPORT_SYMBOL(nvUvmInterfaceReportNonReplayableFault);
 
+NV_STATUS nvUvmInterfacePagingChannelAllocate(uvmGpuDeviceHandle device,
+                                              const UvmGpuPagingChannelAllocParams *allocParams,
+                                              UvmGpuPagingChannelHandle *channel,
+                                              UvmGpuPagingChannelInfo *channelInfo)
+{
+    nvidia_stack_t *sp = NULL;
+    nvidia_stack_t *pushSteamSp = NULL;
+    NV_STATUS status;
+
+    if (nv_kmem_cache_alloc_stack(&sp) != 0)
+        return NV_ERR_NO_MEMORY;
+
+    if (nv_kmem_cache_alloc_stack((nvidia_stack_t **)&pushSteamSp) != 0)
+        return NV_ERR_NO_MEMORY;
+
+    status = rm_gpu_ops_paging_channel_allocate(sp,
+                                                (gpuDeviceHandle)device,
+                                                allocParams,
+                                                (gpuPagingChannelHandle *)channel,
+                                                channelInfo);
+
+    if (status == NV_OK)
+        (*channel)->pushStreamSp = pushSteamSp;
+    else
+        nv_kmem_cache_free_stack(pushSteamSp);
+
+    nv_kmem_cache_free_stack(sp);
+
+    return status;
+}
+EXPORT_SYMBOL(nvUvmInterfacePagingChannelAllocate);
+
+void nvUvmInterfacePagingChannelDestroy(UvmGpuPagingChannelHandle channel)
+{
+    nvidia_stack_t *sp = nvUvmGetSafeStack();
+    nv_kmem_cache_free_stack(channel->pushStreamSp);
+    rm_gpu_ops_paging_channel_destroy(sp, (gpuPagingChannelHandle)channel);
+    nvUvmFreeSafeStack(sp);
+}
+EXPORT_SYMBOL(nvUvmInterfacePagingChannelDestroy);
+
+NV_STATUS nvUvmInterfacePagingChannelsMap(uvmGpuAddressSpaceHandle srcVaSpace,
+                                          UvmGpuPointer srcAddress,
+                                          uvmGpuDeviceHandle device,
+                                          NvU64 *dstAddress)
+{
+    nvidia_stack_t *sp = NULL;
+    NV_STATUS status;
+
+    if (nv_kmem_cache_alloc_stack(&sp) != 0)
+        return NV_ERR_NO_MEMORY;
+
+    status = rm_gpu_ops_paging_channels_map(sp,
+                                            (gpuAddressSpaceHandle)srcVaSpace,
+                                            (NvU64)srcAddress,
+                                            (gpuDeviceHandle)device,
+                                            dstAddress);
+
+    nv_kmem_cache_free_stack(sp);
+
+    return status;
+}
+EXPORT_SYMBOL(nvUvmInterfacePagingChannelsMap);
+
+void nvUvmInterfacePagingChannelsUnmap(uvmGpuAddressSpaceHandle srcVaSpace,
+                                       UvmGpuPointer srcAddress,
+                                       uvmGpuDeviceHandle device)
+{
+    nvidia_stack_t *sp = nvUvmGetSafeStack();
+    rm_gpu_ops_paging_channels_unmap(sp,
+                                     (gpuAddressSpaceHandle)srcVaSpace,
+                                     (NvU64)srcAddress,
+                                     (gpuDeviceHandle)device);
+    nvUvmFreeSafeStack(sp);
+}
+EXPORT_SYMBOL(nvUvmInterfacePagingChannelsUnmap);
+
+NV_STATUS nvUvmInterfacePagingChannelPushStream(UvmGpuPagingChannelHandle channel,
+                                                char *methodStream,
+                                                NvU32 methodStreamSize)
+{
+    return rm_gpu_ops_paging_channel_push_stream(channel->pushStreamSp,
+                                                 (gpuPagingChannelHandle)channel,
+                                                 methodStream,
+                                                 methodStreamSize);
+}
+EXPORT_SYMBOL(nvUvmInterfacePagingChannelPushStream);
+
 #else // NV_UVM_ENABLE
 
 NV_STATUS nv_uvm_suspend(void)
