@@ -69,7 +69,7 @@ static _nvlink_drvctx nvlink_drvctx;
 
 #if defined(CONFIG_PROC_FS)
 #define NV_DEFINE_SINGLE_NVLINK_PROCFS_FILE(name) \
-    NV_DEFINE_SINGLE_PROCFS_FILE(name, \
+    NV_DEFINE_SINGLE_PROCFS_FILE_READ_ONLY(name, \
                                  NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE, \
                                  NV_READ_UNLOCK_SYSTEM_PM_LOCK)
 #endif
