@@ -1,15 +1,27 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2019 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
-#include "nv-misc.h"
 
 #include "os-interface.h"
 #include "nv-linux.h"
@@ -22,6 +34,8 @@
 
 
 
+
+
 extern char *NVreg_TemporaryFilePath;
 
 #define MAX_ERROR_STRING 512
@@ -35,6 +49,8 @@ extern nv_kthread_q_t nv_kthread_q;
 NvU32 os_page_size  = PAGE_SIZE;
 NvU64 os_page_mask  = NV_PAGE_MASK;
 NvU8  os_page_shift = PAGE_SHIFT;
+NvU32 os_sev_status = 0;
+NvBool os_sev_enabled = 0;
 
 NV_STATUS NV_API_CALL os_disable_console_access(void)
 {
@@ -197,11 +213,12 @@ NV_STATUS NV_API_CALL os_cond_acquire_semaphore
 )
 {
     os_semaphore_t *os_sema = (os_semaphore_t *)pSema;
-    if (!NV_MAY_SLEEP())
-    {
-        return NV_ERR_INVALID_REQUEST;
-    }
-
+    //
+    // NOTE: down_trylock() is safe to call from IRQ, se we don't need an
+    // NV_MAY_SLEEP() check here. We do check it in os_cond_acquire_mutex(),
+    // even though it is also calling down_trylock(), since that keeps it
+    // in line with the kernel's 'struct mutex' API.
+    //
     if (down_trylock(os_sema))
     {
         return NV_ERR_TIMEOUT_RETRY;
@@ -623,7 +640,7 @@ NvU64 NV_API_CALL os_get_cpu_frequency(void)
     NvU64 cpu_hz = 0;
 #if defined(CONFIG_CPU_FREQ)
     cpu_hz = (cpufreq_get(0) * 1000);
-#elif (defined(NVCPU_X86) || defined(NVCPU_X86_64))
+#elif defined(NVCPU_X86_64)
     NvU64 tsc[2];
 
     tsc[0] = nv_rdtsc();
@@ -830,8 +847,7 @@ static NvBool NV_API_CALL xen_support_fully_virtualized_kernel(void)
 void* NV_API_CALL os_map_kernel_space(
     NvU64 start,
     NvU64 size_bytes,
-    NvU32 mode,
-    NvU32 memType
+    NvU32 mode
 )
 {
     void *vaddr;
@@ -856,16 +872,6 @@ void* NV_API_CALL os_map_kernel_space(
         return NULL;
     }
 
-#if defined(NVCPU_X86)
-    if (start > 0xffffffff)
-    {
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: os_map_kernel_space: can't map > 32-bit address 0x%0llx!\n", start);
-        os_dbg_breakpoint();
-        return NULL;
-    }
-#endif
-
     switch (mode)
     {
         case NV_MEMORY_CACHED:
@@ -910,12 +916,8 @@ NV_STATUS NV_API_CALL os_flush_cpu_cache(void)
 // flush the cache of all cpus
 NV_STATUS NV_API_CALL os_flush_cpu_cache_all(void)
 {
-#if defined(NVCPU_FAMILY_ARM)
+#if defined(NVCPU_AARCH64)
     CACHE_FLUSH_ALL();
-#if defined(NVCPU_ARM) && defined(NV_OUTER_FLUSH_ALL_PRESENT)
-    /* flush the external L2 cache in cortex-A9 and cortex-a15 */
-    OUTER_FLUSH_ALL();
-#endif
     return NV_OK;
 #endif
     return NV_ERR_NOT_SUPPORTED;
@@ -923,21 +925,18 @@ NV_STATUS NV_API_CALL os_flush_cpu_cache_all(void)
 
 NV_STATUS NV_API_CALL os_flush_user_cache(void)
 {
-#if defined(NVCPU_FAMILY_ARM)
+#if defined(NVCPU_AARCH64)
     if (!NV_MAY_SLEEP())
     {
         return NV_ERR_NOT_SUPPORTED;
     }
 
-#if defined(NVCPU_AARCH64)
     //
     // The Linux kernel does not export an interface for flushing a range,
     // although it is possible. For now, just flush the entire cache to be
     // safe.
     //
     CACHE_FLUSH_ALL();
-#endif
-
     return NV_OK;
 #else
     return NV_ERR_NOT_SUPPORTED;
@@ -1074,7 +1073,7 @@ void NV_API_CALL os_dbg_breakpoint(void)
 {
 #if defined(DEBUG)
   #if defined(CONFIG_X86_REMOTE_DEBUG) || defined(CONFIG_KGDB) || defined(CONFIG_XMON)
-    #if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+    #if defined(NVCPU_X86_64)
         __asm__ __volatile__ ("int $3");
     #elif defined(NVCPU_ARM)
         __asm__ __volatile__ (".word %c0" :: "i" (KGDB_COMPILED_BREAK));
@@ -1082,7 +1081,7 @@ void NV_API_CALL os_dbg_breakpoint(void)
         # warning "Need to implement os_dbg_breakpoint() for aarch64"
     #elif defined(NVCPU_PPC64LE)
         __asm__ __volatile__ ("trap");
-    #endif // NVCPU_X86 || NVCPU_X86_64
+    #endif // NVCPU_*
   #elif defined(CONFIG_KDB)
       KDB_ENTER();
   #endif // CONFIG_X86_REMOTE_DEBUG || CONFIG_KGDB || CONFIG_XMON
@@ -1116,32 +1115,30 @@ void NV_API_CALL os_get_screen_info(
     NvU16 *pFbWidth,
     NvU16 *pFbHeight,
     NvU16 *pFbDepth,
-    NvU16 *pFbPitch
+    NvU16 *pFbPitch,
+    NvU64 consoleBar1Address,
+    NvU64 consoleBar2Address
 )
 {
-#if NV_IS_EXPORT_SYMBOL_PRESENT_screen_info
-    //
-    // If there is not a framebuffer console, return 0 size.
-    //
-    // orig_video_isVGA is set to 1 during early Linux kernel
-    // initialization, and then will be set to a value, such as
-    // VIDEO_TYPE_VLFB or VIDEO_TYPE_EFI if an fbdev console is used.
-    //
-    if (screen_info.orig_video_isVGA <= 1)
-    {
-        *pPhysicalAddress = 0;
-        *pFbWidth = *pFbHeight = *pFbDepth = *pFbPitch = 0;
-        return;
-    }
+#if defined(CONFIG_FB)
+    int i;
+    *pPhysicalAddress = 0;
+    *pFbWidth = *pFbHeight = *pFbDepth = *pFbPitch = 0;
 
-    *pPhysicalAddress = screen_info.lfb_base;
-#if defined(VIDEO_CAPABILITY_64BIT_BASE)
-    *pPhysicalAddress |= (NvU64)screen_info.ext_lfb_base << 32;
-#endif
-    *pFbWidth = screen_info.lfb_width;
-    *pFbHeight = screen_info.lfb_height;
-    *pFbDepth = screen_info.lfb_depth;
-    *pFbPitch = screen_info.lfb_linelength;
+    for (i = 0; i < num_registered_fb; i++)
+    {
+        /* Make sure base address is mapped to GPU BAR */
+        if ((registered_fb[i]->fix.smem_start == consoleBar1Address) ||
+            (registered_fb[i]->fix.smem_start == consoleBar2Address))
+        {
+            *pPhysicalAddress = registered_fb[i]->fix.smem_start;
+            *pFbWidth = registered_fb[i]->var.xres;
+            *pFbHeight = registered_fb[i]->var.yres;
+            *pFbDepth = registered_fb[i]->var.bits_per_pixel;
+            *pFbPitch = registered_fb[i]->fix.line_length;
+            break;
+        }
+    }
 #else
     *pPhysicalAddress = 0;
     *pFbWidth = *pFbHeight = *pFbDepth = *pFbPitch = 0;
@@ -1150,9 +1147,7 @@ void NV_API_CALL os_get_screen_info(
 
 void NV_API_CALL os_dump_stack()
 {
-#if defined(DEBUG)
     dump_stack();
-#endif
 }
 
 typedef struct os_spinlock_s
@@ -1192,9 +1187,9 @@ NvU64 NV_API_CALL os_acquire_spinlock(void *pSpinlock)
     NV_SPIN_LOCK_IRQSAVE(&os_spinlock->lock, eflags);
     os_spinlock->eflags = eflags;
 
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+#if defined(NVCPU_X86_64)
     eflags &= X86_EFLAGS_IF;
-#elif defined(NVCPU_FAMILY_ARM)
+#elif defined(NVCPU_AARCH64)
     eflags &= PSR_I_BIT;
 #endif
     return eflags;
@@ -1210,30 +1205,6 @@ void NV_API_CALL os_release_spinlock(void *pSpinlock, NvU64 oldIrql)
     NV_SPIN_UNLOCK_IRQRESTORE(&os_spinlock->lock, eflags);
 }
 
-NV_STATUS NV_API_CALL os_get_address_space_info(
-    NvU64 *userStartAddress,
-    NvU64 *userEndAddress,
-    NvU64 *kernelStartAddress,
-    NvU64 *kernelEndAddress
-)
-{
-    NV_STATUS status = NV_OK;
-
-#if !defined(CONFIG_X86_4G)
-    *kernelStartAddress = PAGE_OFFSET;
-    *kernelEndAddress = (NvUPtr)0xffffffffffffffffULL;
-    *userStartAddress = 0;
-    *userEndAddress = TASK_SIZE;
-#else
-    *kernelStartAddress = 0;
-    *kernelEndAddress = 0;   /* invalid */
-    *userStartAddress = 0;
-    *userEndAddress = 0;     /* invalid */
-    status = NV_ERR_NOT_SUPPORTED;
-#endif
-    return status;
-}
-
 #define NV_KERNEL_RELEASE    ((LINUX_VERSION_CODE >> 16) & 0x0ff)
 #define NV_KERNEL_VERSION    ((LINUX_VERSION_CODE >> 8)  & 0x0ff)
 #define NV_KERNEL_SUBVERSION ((LINUX_VERSION_CODE)       & 0x0ff)
@@ -1368,8 +1339,7 @@ static NV_STATUS os_get_smbios_header_legacy(NvU64 *pSmbsAddr)
 
     pMappedAddr = (NvU8*)os_map_kernel_space(SMBIOS_LEGACY_BASE,
                                              SMBIOS_LEGACY_SIZE,
-                                             NV_MEMORY_CACHED,
-                                             NV_MEMORY_TYPE_SYSTEM);
+                                             NV_MEMORY_CACHED);
     if (pMappedAddr == NULL)
     {
         return NV_ERR_INSUFFICIENT_RESOURCES;
@@ -1414,8 +1384,7 @@ static NV_STATUS os_verify_smbios_header_uefi(NvU64 smbsAddr)
 
     pBufAddr = (NvU8*)os_map_kernel_space(start,
                                           size,
-                                          NV_MEMORY_CACHED,
-                                          NV_MEMORY_TYPE_SYSTEM);
+                                          NV_MEMORY_CACHED);
     if (pBufAddr == NULL)
     {
         return NV_ERR_INSUFFICIENT_RESOURCES;
@@ -1690,12 +1659,10 @@ NV_STATUS NV_API_CALL os_numa_memblock_size
     return NV_OK;
 }
 
-#if !(defined(NV_GRID_BUILD) && defined(NV_VGPU_HYPERV_BUILD))
 NV_STATUS NV_API_CALL os_call_nv_vmbus(NvU32 vmbus_cmd, void *input)
 {
     return NV_ERR_NOT_SUPPORTED;
 }
-#endif
 
 NV_STATUS NV_API_CALL os_open_temporary_file
 (
@@ -1704,6 +1671,8 @@ NV_STATUS NV_API_CALL os_open_temporary_file
 {
 #if defined(O_TMPFILE)
     struct file *file;
+    const char *default_path = "/tmp";
+    const int flags = O_TMPFILE | O_LARGEFILE | O_RDWR;
     const char *path = NVreg_TemporaryFilePath;
 
     /*
@@ -1717,9 +1686,23 @@ NV_STATUS NV_API_CALL os_open_temporary_file
     }
 
     if (!path)
-        path = "/tmp";
+    {
+        path = default_path;
+    }
+
+    file = filp_open(path, flags, 0);
+    if (IS_ERR(file))
+    {
+        if ((path != default_path) && (PTR_ERR(file) == -ENOENT))
+        {
+            nv_printf(NV_DBG_ERRORS,
+                      "NVRM: The temporary file path specified via the NVreg_TemporaryFilePath\n"
+                      "NVRM: module parameter does not exist. Defaulting to /tmp.\n");
+
+            file = filp_open(default_path, flags, 0);
+        }
+    }
 
-    file = filp_open(path, O_TMPFILE | O_LARGEFILE | O_RDWR, 0);
     if (IS_ERR(file))
     {
         return NV_ERR_OPERATING_SYSTEM;
@@ -1879,6 +1862,211 @@ NV_STATUS NV_API_CALL os_open_and_read_file
     return status;
 }
 
+NvBool NV_API_CALL os_is_nvswitch_present(void)
+{
+    struct pci_device_id nvswitch_pci_table[] = {
+        {
+            PCI_DEVICE(PCI_VENDOR_ID_NVIDIA, PCI_ANY_ID),
+            .class      = PCI_CLASS_BRIDGE_OTHER << 8,
+            .class_mask = PCI_ANY_ID
+        },
+        {0}
+    };
+
+    return !!pci_dev_present(nvswitch_pci_table);
+}
+
+void NV_API_CALL os_get_random_bytes
+(
+    NvU8 *bytes,
+    NvU16 numBytes
+)
+{
+    get_random_bytes(bytes, numBytes);
+}
+
+NV_STATUS NV_API_CALL os_alloc_wait_queue
+(
+    os_wait_queue **wq
+)
+{
+    NV_KMALLOC(*wq, sizeof(os_wait_queue));
+    if (*wq == NULL)
+        return NV_ERR_NO_MEMORY;
+
+    init_completion(&(*wq)->q);
+
+    return NV_OK;
+}
+
+void NV_API_CALL os_free_wait_queue
+(
+    os_wait_queue *wq
+)
+{
+    NV_KFREE(wq, sizeof(os_wait_queue));
+}
+
+void NV_API_CALL os_wait_uninterruptible
+(
+    os_wait_queue *wq
+)
+{
+    wait_for_completion(&wq->q);
+}
+
+void NV_API_CALL os_wake_up
+(
+    os_wait_queue *wq
+)
+{
+    complete_all(&wq->q);
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+nv_cap_t* NV_API_CALL os_nv_cap_init
+(
+    const char *path
+)
+{
+    return nv_cap_init(path);
+}
+
+nv_cap_t* NV_API_CALL os_nv_cap_create_dir_entry
+(
+    nv_cap_t *parent_cap,
+    const char *name,
+    int mode
+)
+{
+    return nv_cap_create_dir_entry(parent_cap, name, mode);
+}
+
+nv_cap_t* NV_API_CALL os_nv_cap_create_file_entry
+(
+    nv_cap_t *parent_cap,
+    const char *name,
+    int mode
+)
+{
+    return nv_cap_create_file_entry(parent_cap, name, mode);
+}
+
+void NV_API_CALL os_nv_cap_destroy_entry
+(
+    nv_cap_t *cap
+)
+{
+    nv_cap_destroy_entry(cap);
+}
+
+int NV_API_CALL os_nv_cap_validate_and_dup_fd
+(
+    const nv_cap_t *cap,
+    int fd
+)
+{
+    return nv_cap_validate_and_dup_fd(cap, fd);
+}
+
+void NV_API_CALL os_nv_cap_close_fd
+(
+    int fd
+)
+{
+    nv_cap_close_fd(fd);
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
